package gov.va.med.mhv.admin.persist.ldap;

import java.beans.PropertyDescriptor;
import java.io.Serializable;
import java.lang.reflect.Method;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;


@Component
public class LdapEnv implements Serializable {

	private static final long serialVersionUID = 3719104006793507494L;

	private Log log = LogFactory.getLog(getClass());

	@Value("${ldap.host}")
	private String host;

	@Value("${ldap.port}")
	private int port;

	@Value("${ldap.baseDn}")
	private String baseDn;

	@Value("${ldap.userBaseDn}")
	private String userBaseDn;

	@Value("${ldap.userNameAttribute}")
	private String userNameAttribute;

	@Value("${ldap.userPasswordAttribute}")
	private String userPasswordAttribute;

	@Value("${ldap.adminPrincipal}")
	private String adminPrincipal;

	@Value("${ldap.adminPassword}")
	private String adminPassword;

	@Value("${ldap.objectClassAttribute}")
	private String objectClassAttribute;

	@Value("${ldap.surNameAttribute}")
	private String surNameAttribute;

	@Value("${ldap.commonNameAttribute}")
	private String commonNameAttribute;

	@Value("${ldap.objectClassValueArray}")
	private String[] objectClassValues;

	@Value("${ad.initial.ctx.factory}")
	private String adIntialCtxFactory;

	@Value("${ad.security.auth}")
	private String adSecurityAuthentication;

	@Value("${ad.ldap.url}")
	private String adUrl;

	@Value("${ad.ldap.adminPrincipal}")
	private String adAdminPrincipal;

	@Value("${ad.ldap.adminPassword}")
	private String adAdminPassword;

	@Value("${ad.ldap.searchBase}")
	private String adSearchBase;

	public int getPort() {
		return port;
	}

	public String getAdminPrincipal() {
		return adminPrincipal;
	}

	public String getAdminPassword() {
		return adminPassword;
	}

	public String getBaseDn() {
		return baseDn;
	}

	public String getHost() {
		return host;
	}

	public String getUserNameAttribute() {
		return userNameAttribute;
	}

	public void setUserNameAttribute(String userCn) {
		this.userNameAttribute = userCn;
	}

	public String getUserBaseDn() {
		return userBaseDn;
	}

	public void setPort(int port) {
		this.port = port;
	}

	public void setBaseDn(String baseDn) {
		this.baseDn = baseDn;
	}

	public void setAdminPrincipal(String adminPrincipal) {
		this.adminPrincipal = adminPrincipal;
	}

	public void setAdminPassword(String adminPassword) {
		this.adminPassword = adminPassword;
	}

	public void setUserBaseDn(String userBaseDn) {
		this.userBaseDn = userBaseDn;
	}

	public void setHost(String host) {
		this.host = host;
	}

	public String getUserPasswordAttribute() {
		return userPasswordAttribute;
	}

	public void setUserPasswordAttribute(String userPasswordAttribute) {
		this.userPasswordAttribute = userPasswordAttribute;
	}

	public String getCommonNameAttribute() {
		return commonNameAttribute;
	}

	public void setCommonNameAttribute(String commonNameAttribute) {
		this.commonNameAttribute = commonNameAttribute;
	}

	public String getObjectClassAttribute() {
		return objectClassAttribute;
	}

	public void setObjectClassAttribute(String objectClassAttribute) {
		this.objectClassAttribute = objectClassAttribute;
	}

	public String getSurNameAttribute() {
		return surNameAttribute;
	}

	public void setSurNameAttribute(String surNameAttribute) {
		this.surNameAttribute = surNameAttribute;
	}

	public String[] getObjectClassValues() {
		return objectClassValues;
	}

	public void setObjectClassValues(String[] objectClassValues) {
		this.objectClassValues = objectClassValues;
	}

	public void setObjectClassValueArray(String delimited) {
		setObjectClassValues(delimited.split(","));
	}


	public String getAdIntialCtxFactory() {
		return adIntialCtxFactory;
	}


	public void setAdIntialCtxFactory(String adIntialCtxFactory) {
		this.adIntialCtxFactory = adIntialCtxFactory;
	}


	public String getAdSecurityAuthentication() {
		return adSecurityAuthentication;
	}


	public void setAdSecurityAuthentication(String adSecurityAuthentication) {
		this.adSecurityAuthentication = adSecurityAuthentication;
	}


	public String getAdUrl() {
		return adUrl;
	}


	public void setAdUrl(String adUrl) {
		this.adUrl = adUrl;
	}


	public String getAdAdminPrincipal() {
		return adAdminPrincipal;
	}


	public void setAdAdminPrincipal(String adAdminPrincipal) {
		this.adAdminPrincipal = adAdminPrincipal;
	}


	public String getAdAdminPassword() {
		return adAdminPassword;
	}


	public void setAdAdminPassword(String adAdminPassword) {
		this.adAdminPassword = adAdminPassword;
	}


	public String getAdSearchBase() {
		return adSearchBase;
	}


	public void setAdSearchBase(String adSearchBase) {
		this.adSearchBase = adSearchBase;
	}

	public String toString() {
		StringBuffer buf = new StringBuffer();

		if(log.isDebugEnabled()) {
			String sep = System.getProperty("line.separator");
			buf.append("=========================")
				.append(sep)
				.append("LdapEnv [" + hashCode() +"]")
				.append(sep);

			PropertyDescriptor[] pds = BeanUtils.getPropertyDescriptors(getClass());
			for(int i = 0; i < pds.length; i++) {
				PropertyDescriptor pd = pds[i];
				Method readMethod = pd.getReadMethod();

				if(readMethod != null) {
					Class returnType = readMethod.getReturnType();
					Class[] params = readMethod.getParameterTypes();
					String name = readMethod.getName();

					if(params.length == 0 && returnType != Class.class && name.startsWith("get")) {
						Object value = null;

						try {
							value = readMethod.invoke(this, null);
						}
						catch(Throwable t) {
							value = "!!error!!";
						}

						buf.append(name).append('=').append(value).append(sep);
					}
				}
			}
			buf.append("=========================");
		}
		return buf.toString();
	}
}