package gov.va.med.mhv.admin.repository;


import java.util.Date;
import java.util.List;

import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.Query;
import org.springframework.data.repository.query.Param;
import org.springframework.stereotype.Repository;


import gov.va.med.mhv.common.data.model.UserProfile;
import gov.va.med.mhv.common.data.model.UserProfileDeactivationReason;

@Repository
public interface UserProfileRepository extends JpaRepository<UserProfile, Long>{

	/**
	 *
	 * @param userName
	 * @return
	 */
	@Query("select u from UserProfile u where lower(u.userName) = lower(:userName)")
	UserProfile findByUserName(@Param("userName") String userName);


	/**
	 *
	 * @param userName
	 * @return
	 */
	@Query("select u from UserProfile u where u.id = :id")
	UserProfile getUserProfileById(@Param("id") Long id);


	/**
	 *
	 * @param userName
	 * @return
	 */
	@Query("Select u from UserProfile u where u.firstName like :userName")
	List <UserProfile>findByUserNameLike(@Param("userName") String userName);
	
	
	@Query("select u from UserProfileDeactivationReason u where u.id = :id")
	UserProfileDeactivationReason getUserProfileDeactivationReasonById(@Param("id") Long id);


	/**
	 * Execute the GetProfileByNameBirthDateSsnExcludeId query
	 *
	 * @param firstName The firstName
	 * @param lastName The lastName
	 * @param birthDate The birthDate
	 * @param ssn The ssn
	 * @param id The id
	 * @return List of UserProfiles
	 */
	@Query("Select u from UserProfile u where u.firstName = :firstName and u.lastName = :lastName and u.birthDate= :birthDate and u.ssn=:ssn and u.id=:id")
	List <UserProfile>getProfileByNameBirthDateSsnExcludeId(@Param("firstName") String firstName,@Param("lastName") String lastName,@Param("birthDate") Date birthDate,
			@Param("ssn") String ssn,@Param("id") Long id);
	
	@Query("Select u from UserProfile u where u.firstName = :firstName and u.lastName = :lastName and u.userName= :userName")
	List <UserProfile>testUser(@Param("firstName") String firstName,@Param("lastName") String lastName,@Param("userName") String userName);

	/**
	 *
	 * @param firstName
	 * @param lastName
	 * @param birthDate
	 * @param ssn
	 * @return
	 */
	@Query("Select u from UserProfile u where ((:firstName is null and u.firstName is null) or (:firstName = '%' and u.firstName is null) or (:firstName is not null "+
		   "and lower(u.firstName) = lower(:firstName)) and u.lastName like :lastName and u.birthDate = :birthDate and (:ssn is null and u.ssn is null) "+
		   "or (:ssn = '%' and u.ssn is null) or (:ssn is not null and u.ssn like :ssn ))")
	List<UserProfile>getProfileByNameBirthDateSsnIncludeDeleted1(@Param("firstName") String firstName,@Param("lastName") String lastName,@Param("birthDate") Date birthDate,
			@Param("ssn") String ssn);

	/**
	 * Execute the GetProfileByNameBirthDateSsn query
	 *
	 * @param firstName The firstName
	 * @param lastName The lastName
	 * @param birthDate The birthDate
	 * @param ssn The ssn
	 * @return List of UserProfiles
	 */
	@Query("Select u from UserProfile u where lower(u.firstName) =lower(:firstName) and lower(u.lastName)  = lower(:lastName) and u.birthDate= :birthDate and u.ssn=:ssn")
	List <UserProfile>getProfileByNameBirthDateSsn(@Param("firstName") String firstName,@Param("lastName") String lastName,@Param("birthDate") Date birthDate,
			@Param("ssn") String ssn);

	/**
	 * Execute the GetProfileByNameBirthDate query
	 *
	 * @param firstName The firstName
	 * @param lastName The lastName
	 * @param gender The gender
	 * @param birthDate The birthDate
	 * @return List of UserProfiles
	 */
	@Query("Select u from UserProfile u where lower(u.firstName) = lower(:firstName) and lower(u.lastName) = lower(:lastName) and u.gender= :gender and u.birthDate= :birthDate" )
	List <UserProfile>getProfileByNameBirthDate(@Param("firstName") String firstName, @Param("lastName") String lastName, @Param("gender") String gender, @Param("birthDate") Date birthDate);
	

	@Query("Select u from UserProfile u where lower(u.firstName) = lower(:firstName) and lower(u.lastName) = lower(:lastName) and u.gender= :gender and u.gender= :gender")
	List <UserProfile>getProfileByNameBirthDateForVAAFI(@Param("firstName") String firstName, @Param("lastName") String lastName, @Param("gender") String gender);

	@Query("Select u from UserProfile u where lower(u.firstName) = lower(:firstName) and lower(u.lastName) = lower(:lastName) and lower(u.userName) = lower(:userName) and u.gender= :gender and u.birthDate = :birthDate")
	List <UserProfile>getProfileByNameUserNameBirthDate(@Param("firstName") String firstName, @Param("lastName") String lastName,@Param("userName") String userName, @Param("gender") String gender, @Param("birthDate") Date birthDate);
	
	@Query("Select u from UserProfile u where u.birthDate= :birthDate and u.ssn=:ssn and lower(u.firstName) = lower(:firstName) "
			+ "and lower(u.lastName) = lower(:lastName) and u.deactivationReason is null")
	List <UserProfile>getActiveProfileByBirthDateSSN(@Param("birthDate") Date birthDate, @Param("ssn") String ssn, 
			@Param("firstName") String firstName, @Param("lastName") String lastName);
	
	//TODO:NOT BEING USED. DELETE THIS LATER
	
	@Query("Select profile from UserProfile profile "
		   + "where ((:firstName is null and profile.firstName is null)"
		   + " or (:firstName = '%' and profile.firstName is null)"
		   + " or (:firstName is not null  "
		   + " and lower(profile.firstName) like :firstName ))"
		   + " and (profile.isVeteran = 1 or profile.isPatient = 1)"
		   + " and lower(profile.lastName) like :lastName"
		   + " and lower(profile.mhvId) like :mhvId"
		   + " and ((:userName is null and profile.userName is null)"
		   + " or (:userName = '%' and profile.userName is null)"
		   + " or (:userName is not null"
		   + " and lower(profile.userName) like :userName ))"
		   + " and"
		   + " ((:ssn is null and profile.ssn is null)"
		   + " or (:ssn = '%' and profile.ssn is null)"
		   + " or (:ssn is not null"
		   + "  and profile.ssn like :ssn ))"
		   + " and profile.birthDate = :birthDate")
	List <UserProfile>getPatientVetProfileByNameMhvIdUserNameWithBirthDate(@Param("firstName") String firstName, @Param("lastName") String lastName,@Param("mhvId") String mhvId,@Param("userName") String userName, @Param("ssn") String ssn,@Param("birthDate") Date birthDate);	
	
	//TODO:NOT BEING USED. DELETE THIS LATER
	@Query("Select profile from UserProfile profile"
			+ "  where ((:firstName is null and profile.firstName is null)"
			+ " or (:firstName = '%' and profile.firstName is null)"
			+ " or (:firstName is not null "
			+ " and lower(profile.firstName) like :firstName ))"
			+ " and (profile.isVeteran = 1 or profile.isPatient = 1)"
			+ " and lower(profile.lastName) like :lastName  "
			+ " and lower(profile.mhvId) like :mhvId"
			+ " and ((:userName is null and profile.userName is null)"
			+ " or (:userName = '%' and profile.userName is null)"
			+ " or (:userName is not null"
			+ " and lower(profile.userName) like :userName ))"
			+ " and   "
			+ " ((:ssn is null and profile.ssn is null)"
			+ " or (:ssn = '%' and profile.ssn is null)"
			+ " or (:ssn is not null"
			+ " and profile.ssn like :ssn ))")
	List <UserProfile>getPatientVetProfileByNameMhvIdUserName(@Param("firstName") String firstName, @Param("lastName") String lastName,@Param("mhvId") String mhvId,@Param("userName") String userName, @Param("ssn") String ssn);	
	//TODO:NOT BEING USED. DELETE THIS LATER
		@Query("Select profile from UserProfile profile"
			+ " where ((:firstName is null and profile.firstName is null)"
		    + " or (:firstName = '%' and profile.firstName is null)"
		    + "  or (:firstName is not null"
		     + "  and lower(profile.firstName) like :firstName ))"
		     + "  and (profile.isVeteran = 1 or profile.isPatient = 1)"
		     + "  and lower(profile.lastName) like :lastName"
		     + "  and lower(profile.contactInfoEmail) like :email"
		     + "  and lower(profile.mhvId) like :mhvId"
		     + "  and ((:userName is null and profile.userName is null)"
		     + "  or (:userName = '%' and profile.userName is null)"
		     + "  or (:userName is not null"
		     + "  and lower(profile.userName) like :userName ))"
		     + "  and   "
		     + "  ((:ssn is null and profile.ssn is null)"
		     + "  or (:ssn = '%' and profile.ssn is null)"
		     + "  or (:ssn is not null"
		     + "  and profile.ssn like :ssn ))")
	List <UserProfile>getPatientVetProfileByNameEmailMhvIdUserName(@Param("firstName") String firstName, @Param("lastName") String lastName,@Param("email") String email,@Param("mhvId") String mhvId,@Param("userName") String userName, @Param("ssn") String ssn);	
		//TODO:NOT BEING USED. DELETE THIS LATER
		@Query("Select profile from UserProfile profile"
			 + "   where ((:firstName is null and profile.firstName is null)"
			 + "  or (:firstName = '%' and profile.firstName is null)"
			 + "  or (:firstName is not null"
			 + "  and lower(profile.firstName) like :firstName ))"
			 + "  and (profile.isVeteran = 1 or profile.isPatient = 1)"
			 + "  and lower(profile.lastName) like :lastName"
		      + "  and lower(profile.contactInfoEmail) like :email"
		      + "  and lower(profile.mhvId) like :mhvId"
		      + "  and ((:userName is null and profile.userName is null)"
		      + "  or (:userName = '%' and profile.userName is null)"
		      + "  or (:userName is not null"
		      + "  and lower(profile.userName) like :userName ))"
		      + "  and"
		      + "  ((:ssn is null and profile.ssn is null)"
		      + "  or (:ssn = '%' and profile.ssn is null)"
		      + "  or (:ssn is not null"
		      + "  and profile.ssn like :ssn ))"
		      + "  and profile.birthDate = :birthDate")
	List <UserProfile>getPatientVetProfileByNameEmailMhvIdUserNameWithBirthDate(@Param("firstName") String firstName, @Param("lastName") String lastName,@Param("email") String email,@Param("mhvId") String mhvId,@Param("userName") String userName, @Param("ssn") String ssn,@Param("birthDate") Date birthDate);
		
		/**
		 *
		 * @param userName
		 * @return
		 */
		@Query("Select reason from UserProfileDeactivationReason reason where lower(reason.name) = lower(:name)")
		UserProfileDeactivationReason findUserProfileDeactivationReasonByName(@Param("name") String name);
		
		
		@Query("Select profile from UserProfile profile"
				 + "    where ((:firstName is null and profile.firstName is null)"
				 + "    or (:firstName = '%' and profile.firstName is null)"
				 + "    or (:firstName is not null "
				 + "    and lower(profile.firstName) like :firstName ))"
				 + "    and lower(profile.lastName) like :lastName "
				 + "    and lower(profile.mhvId) like :mhvId "
				 + "    and ((:userName is null and profile.userName is null)"
				 + "    or (:userName = '%' and profile.userName is null)"
				 + "    or (:userName is not null "
				 + "    and lower(profile.userName) like :userName ))"
				 + "  	and  "
				 + "    ((:ssn is null and profile.ssn is null)"
				 + "    or (:ssn = '%' and profile.ssn is null)"
				 + "    or (:ssn is not null "
				 + "    and profile.ssn like :ssn ))"
				 + "  						 and profile.birthDate = :birthDate order by profile.lastName asc")
		List <UserProfile>getProfileByNameMhvIdUserNameWithBirthDate(@Param("firstName") String firstName, @Param("lastName") String lastName,@Param("mhvId") String mhvId,@Param("userName") String userName, @Param("ssn") String ssn,@Param("birthDate") Date birthDate);

		
		@Query("Select profile from UserProfile profile"
				 + "     where ((:firstName is null and profile.firstName is null)"
				 + "     or (:firstName = '%' and profile.firstName is null)"
				 + "     or (:firstName is not null "
				 + "     and lower(profile.firstName) like :firstName ))"
				 + "     and lower(profile.lastName) like :lastName "
				 + "     and lower(profile.mhvId) like :mhvId "
				 + "     and ((:userName is null and profile.userName is null)"
				 + "     or (:userName = '%' and profile.userName is null)"
				 + "     or (:userName is not null "
				 + "     and lower(profile.userName) like :userName ))"
				 + "     and  "
				 + "     ((:ssn is null and profile.ssn is null)"
				 + "     or (:ssn = '%' and profile.ssn is null)"
			  	 + "     or (:ssn is not null "
				        + "          and profile.ssn like :ssn )) order by profile.lastName asc")
		List <UserProfile>getProfileByNameMhvIdUserName(@Param("firstName") String firstName, @Param("lastName") String lastName,@Param("mhvId") String mhvId,@Param("userName") String userName, @Param("ssn") String ssn);
	
		
		
		@Query("Select profile from UserProfile profile"
				  	  + "  where ((:firstName is null and profile.firstName is null)"
				      + "  or (:firstName = '%' and profile.firstName is null)"
				      + "  or (:firstName is not null "
				      + "  and lower(profile.firstName) like :firstName ))"
				      + "  and lower(profile.lastName) like :lastName "
				      + "  and lower(profile.contactInfoEmail) like :email "
				      + "  and lower(profile.mhvId) like :mhvId "
				      + "  and ((:userName is null and profile.userName is null)"
				      + "  or (:userName = '%' and profile.userName is null)"
				      + "  or (:userName is not null "
				      + "  and lower(profile.userName) like :userName ))"
				      + "  and  "
				      + "  ((:ssn is null and profile.ssn is null)"
				      + "  or (:ssn = '%' and profile.ssn is null)"
				      + "  or (:ssn is not null "
				      + "  and profile.ssn like :ssn ))"
				      + "  and profile.birthDate = :birthDate order by profile.lastName asc")
		List <UserProfile>getProfileByNameEmailMhvIdUserNameWithBirthDate(@Param("firstName") String firstName, @Param("lastName") String lastName,@Param("email") String email,@Param("mhvId") String mhvId,@Param("userName") String userName, @Param("ssn") String ssn,@Param("birthDate") Date birthDate);
	
		
		@Query("Select profile from UserProfile profile"
					+ "  where ((:firstName is null and profile.firstName is null)"
				    + "  or (:firstName = '%' and profile.firstName is null)"
				    + "  or (:firstName is not null "
				    + "  and lower(profile.firstName) like :firstName ))"
				    + "  and lower(profile.lastName) like :lastName"
				    + "  and lower(profile.contactInfoEmail) like :email"
				    + "  and lower(profile.mhvId) like :mhvId "
				    + "  and ((:userName is null and profile.userName is null)"
				    + "  or (:userName = '%' and profile.userName is null)"
				    + "  or (:userName is not null"
				    + "  and lower(profile.userName) like :userName ))"
				    + "  and"
				    + "  ((:ssn is null and profile.ssn is null)"
				    + "  or (:ssn = '%' and profile.ssn is null)"
				    + "  or (:ssn is not null "
				    + " and profile.ssn like :ssn )) order by profile.lastName asc")
		List <UserProfile>getProfileByNameEmailMhvIdUserName(@Param("firstName") String firstName, @Param("lastName") String lastName,@Param("email") String email,@Param("mhvId") String mhvId,@Param("userName") String userName, @Param("ssn") String ssn);
	
}