package gov.va.med.mhv.admin.service.impl.mhvusers;

import java.sql.Timestamp;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

import javax.ws.rs.Consumes;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.core.MediaType;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import gov.va.med.mhv.admin.converter.ActivityDTOConverter;
import gov.va.med.mhv.admin.dto.ActivityDTO;
import gov.va.med.mhv.admin.enums.ActivityActionTypeEnumeration;
import gov.va.med.mhv.admin.enums.ActivityActorTypeEnumeration;
import gov.va.med.mhv.admin.enums.ActivityTypeEnumeration;
import gov.va.med.mhv.admin.model.Activity;
import gov.va.med.mhv.admin.repository.ActivityRepository;
import gov.va.med.mhv.admin.service.mhvusers.AccountActivityLogService;
import gov.va.med.mhv.admin.service.mhvusers.ActivityService;
import gov.va.med.mhv.common.api.dto.UserProfileDTO;
import gov.va.med.mhv.common.api.exception.MHVException;
import gov.va.med.mhv.common.api.util.ResponseUtil;

@Path("/")
@Service("accountActivityService")
@Component
public class ActivityServiceImpl implements AccountActivityLogService, ActivityService {

	protected static Logger log = LogManager.getLogger(ActivityServiceImpl.class);

	@Autowired
	private ActivityRepository activityRepository;

	@Autowired
	private ActivityDTOConverter activityDTOConverter;

	@Override
	@GET
	@Produces(MediaType.APPLICATION_JSON)
	@Path("/getAccountActvityBySearch/{userName}/{fromDate:.+}/{toDate:.+}/{performerType}")
	public List<ActivityDTO> getAccountActvityBySearch(@PathParam("userName") String userName,
			@PathParam("fromDate") Timestamp fromDate, @PathParam("toDate") Timestamp toDate,
			@PathParam("performerType") String performerType) throws MHVException {

		ResponseUtil<List<ActivityDTO>> response = new ResponseUtil<List<ActivityDTO>>();
		List<Activity> activities = null;
		System.out.println("ActivityServiceImpl>>>>>>>>>>>>>>>>>>>>>>>> userName" + userName);
		System.out.println("ActivityServiceImpl>>>>>>>>>>>>>>>>>>>>>>>> fromDate" + fromDate);
		System.out.println("ActivityServiceImpl>>>>>>>>>>>>>>>>>>>>>>>> toDate" + toDate);
		System.out.println("ActivityServiceImpl>>>>>>>>>>>>>>>>>>>>>>>> performerType" + performerType);

		//Date toDateTemp = stringToDate(toDate);
		//Date fromDateTemp = stringToDate(fromDate);

		//System.out.println("ActivityServiceImpl>>>>>>>>>>>>>>>>>>>>>>>> stringToDate To" + toDateTemp);
		//System.out.println("ActivityServiceImpl>>>>>>>>>>>>>>>>>>>>>>>> stringToDate From" + fromDateTemp);
		System.out.println("ActivityServiceImpl>>>>>>>>>>>>>>>>>>>>>>>> stringToDate Timestamp To" + toDate);
		System.out.println("ActivityServiceImpl>>>>>>>>>>>>>>>>>>>>>>>> stringToDate Timestamp From" + fromDate);		
		try {
			//activities = this.getActivitiesForSubject(userName, fromDateTemp, toDateTemp, performerType);
			activities = this.getActivitiesForSubject(userName, fromDate, toDate, performerType);
		} catch (Exception e) {
			throw new MHVException(e);
		}

		return activityDTOConverter.convertList(activities);
	}

	/*
	 * @Override
	 * 
	 * @GET
	 * 
	 * @Produces(MediaType.APPLICATION_JSON)
	 * 
	 * @Path("/getAccountActvity/{userName}") public List<ActivityDTO>
	 * getAccountActvity(@PathParam("userName") String userName) throws
	 * MHVException {
	 * 
	 * ResponseUtil<List<ActivityDTO>> response = new
	 * ResponseUtil<List<ActivityDTO>>(); List<Activity> activities = null; try
	 * { activities = activityRepository.getActivitiesforUser(userName); } catch
	 * (Exception e) { throw new MHVException(e); }
	 * 
	 * return activityDTOConverter.convertList(activities); }
	 */

	/**
	 * Audits Sign in partners activities. The audit call is in
	 * ResetPasswordController in method restrictSipAccess
	 *
	 * @param subjectId
	 *            Id of subject
	 */
	@Override
	@POST
	@Consumes(MediaType.APPLICATION_JSON)
	@Produces(MediaType.APPLICATION_JSON)
	@Transactional
	@Path("/auditRestrictSipAccessRestrictEvent/{subjectId}/{activityActor}")
	public void auditRestrictSipAccessRestrictEvent(@PathParam("subjectId") Long subjectId,
			@PathParam("activityActor") String activityActor) {
		try {
			Activity activity = new Activity();
			System.out.println("REACHED auditRestictSipAccessRestrictEvent >>>>>>>>>>>>>>>>>>>>>>>>>>> activityActor: "
					+ activityActor);
			activity.setAction(ActivityActionTypeEnumeration.BLOCK_ACCESS.getDescription());
			activity.setActivityType(ActivityTypeEnumeration.SIGN_IN_PARTNERS.getDescription());
			activity.setDetailValue(null);
			activity.setCompletionTime(new Timestamp(new Date().getTime()));
			activity.setPerformerType(activityActor);
			activity.setStatus(Boolean.TRUE);
			activity.setUserProfileId(subjectId);
			activityRepository.save(activity);
		} catch (Exception e) {
			log.error("Exception caught in auditing user profile: auditRestrictSipAccessRestrictEvent: ", e);
			// response.setSuccess(false);
			// response.setFailure(true);
			// response.setFailureMessage(e.getMessage());
		}
	}

	/**
	 * Audits Sign in partners activities. The audit call is in
	 * ResetPasswordController in method restrictSipAccess
	 *
	 * @param subjectId
	 *            Id of subject
	 */
	@Override
	@POST
	@Consumes(MediaType.APPLICATION_JSON)
	@Produces(MediaType.APPLICATION_JSON)
	@Transactional
	@Path("/auditRestrictSipAccessUnRestrictEvent/{subjectId}/{activityActor}")
	public void auditRestrictSipAccessUnRestrictEvent(@PathParam("subjectId") Long subjectId,
			@PathParam("activityActor") String activityActor) {

		try {
			System.out.println("******************************************** Inside AAL service call activityActor: "
					+ activityActor);
			Activity activity = new Activity();
			activity.setAction(ActivityActionTypeEnumeration.UNBLOCK_ACCESS.getDescription());
			activity.setActivityType(ActivityTypeEnumeration.SIGN_IN_PARTNERS.getDescription());
			activity.setDetailValue(null);
			activity.setCompletionTime(new Timestamp(new Date().getTime()));
			activity.setPerformerType(activityActor);
			activity.setStatus(Boolean.TRUE);
			activity.setUserProfileId(subjectId);
			activityRepository.save(activity);
		} catch (Exception e) {
			log.error("Exception caught in auditing user profile", e);
			// response.setSuccess(false);
			// response.setFailure(true);
			// response.setFailureMessage(e.getMessage());
		}
	}

	/**
	 * Audits Sign in partners activities. The audit call is in
	 * ResetPasswordController in method restrictSipAccess
	 *
	 * @param subjectId
	 *            Id of subject
	 */
	@Override
	@POST
	@Consumes(MediaType.APPLICATION_JSON)
	@Produces(MediaType.APPLICATION_JSON)
	@Transactional
	@Path("/auditGeneratePassEmailEvent/{subjectId}/{activityActor}")
	public void auditGeneratePassEmailEvent(@PathParam("subjectId") Long subjectId,
			@PathParam("activityActor") String activityActor) {
		try {
			Activity activity = new Activity();
			System.out.println(
					"REACHED auditGeneratePassEmailEvent >>>>>>>>>>>>>>>>>>>>>>>>>>> activityActor: " + activityActor);
			activity.setAction(ActivityActionTypeEnumeration.RESET.getDescription());
			activity.setActivityType(ActivityTypeEnumeration.PASSWORD.getDescription());
			activity.setDetailValue(null);
			activity.setCompletionTime(new Timestamp(new Date().getTime()));
			activity.setPerformerType(activityActor);
			activity.setStatus(Boolean.TRUE);
			activity.setUserProfileId(subjectId);
			activityRepository.save(activity);
		} catch (Exception e) {
			log.error("Exception caught in auditing user profile: auditGeneratePassEmailEvent: ", e);
			// response.setSuccess(false);
			// response.setFailure(true);
			// response.setFailureMessage(e.getMessage());
		}
	}

	@Transactional
	public void auditMviEvent(UserProfileDTO userProfile, ActivityActorTypeEnumeration actor, Boolean successful,
			ActivityActionTypeEnumeration activityActionType, ActivityTypeEnumeration activityType,
			ResponseUtil response) {
		try {
			Activity activity = new Activity();

			activity.setAction(activityActionType.getDescription());
			activity.setActivityType(activityType.getDescription());
			activity.setCompletionTime(new Timestamp(new Date().getTime()));
			activity.setPerformerType(actor.getDescription());
			activity.setUserProfileId(userProfile.getId());
			activity.setStatus(successful);
			activity.setDetailValue("Austin MHV");

			activityRepository.save(activity);
		} catch (Exception e) {
			log.error("Exception caught in auditing user profile", e);
			response.setSuccess(false);
			response.setFailure(true);
			response.setFailureMessage(e.getMessage());
		}
	}

	@Transactional
	public void auditAccountDeactivateEauthHashEvent(UserProfileDTO userProfile, ResponseUtil response) {
		try {
			Activity activity = new Activity();
			activity.setAction(ActivityActionTypeEnumeration.ACCOUNT_DISCONNECTED.getDescription());
			activity.setActivityType(ActivityTypeEnumeration.DSLOGON_CAC_USAA_ORC.getDescription());
			activity.setCompletionTime(new Timestamp(System.currentTimeMillis()));
			activity.setPerformerType(ActivityActorTypeEnumeration.SELF.getDescription());
			activity.setUserProfileId(userProfile.getId());
			activity.setDetailValue("Deactivation");
			activity.setStatus(Boolean.TRUE);

			activityRepository.save(activity);
		} catch (Exception e) {
			log.error("Exception caught in auditing Account Deactivate Auth Hash Event", e);
			response.setSuccess(false);
			response.setFailure(true);
			response.setFailureMessage(e.getMessage());
		}
	}

	public void auditAccountDeactivateEvent(UserProfileDTO userProfile, ActivityActorTypeEnumeration actor,
			Boolean successful, ResponseUtil response) {
		try {
			Activity activity = new Activity();
			System.out.println("#$@#$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$ auditAccountDeactivateEvent");
			activity.setAction(ActivityActionTypeEnumeration.DEACTIVATE.getDescription());
			activity.setActivityType(ActivityTypeEnumeration.ACCOUNT_MANAGEMENT.getDescription());
			activity.setCompletionTime(new Timestamp(new Date().getTime()));
			activity.setPerformerType(actor.getDescription());
			activity.setUserProfileId(userProfile.getId());
			activity.setStatus(successful);
			activity.setDetailValue(null);
			activityRepository.save(activity);
			System.out.println("#$@#$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$ auditAccountDeactivateEvent" + actor);
		} catch (Exception e) {
			log.error("Exception caught in auditing auditAccountDeactivateEvent", e);
			response.setSuccess(false);
			response.setFailure(true);
			response.setFailureMessage(e.getMessage());
		}
	}

	@Transactional
	public void auditAccountReactivateEvent(UserProfileDTO userProfile, ActivityActorTypeEnumeration actor,
			Boolean successful, ResponseUtil response) {
		try {
			Activity activity = new Activity();

			activity.setAction(ActivityActionTypeEnumeration.REACTIVATE.getDescription());
			activity.setActivityType(ActivityTypeEnumeration.ACCOUNT_MANAGEMENT.getDescription());
			activity.setCompletionTime(new Timestamp(new Date().getTime()));
			activity.setPerformerType(actor.getDescription());
			activity.setUserProfileId(userProfile.getId());
			activity.setStatus(successful);
			activity.setDetailValue(null);

			activityRepository.save(activity);
		} catch (Exception e) {
			log.error("Exception caught in auditing auditAccountDeactivateEvent", e);
			response.setSuccess(false);
			response.setFailure(true);
			response.setFailureMessage(e.getMessage());
		}
	}

	/**
	 * MHV_CodeCR1514 - US12.4 MVI Compliance Implementation - AAL impl -->
	 */
	@Transactional
	@Override
	public void auditMviUnauthenticateRequestEvent(UserProfileDTO userProfile, ActivityActorTypeEnumeration actor,
			Boolean successful, ResponseUtil response) {
		try {
			Activity activity = new Activity();

			activity.setAction("Unauthentication Request");
			activity.setActivityType("Account Upgrade");
			activity.setCompletionTime(new Timestamp(new Date().getTime()));
			activity.setPerformerType(actor.getDescription());
			activity.setUserProfileId(userProfile.getId());
			activity.setStatus(successful);
			activity.setDetailValue(null);

			activityRepository.save(activity);
		} catch (Exception e) {
			log.error("Exception caught in auditing auditMviUnauthenticateRequestEvent", e);
			response.setSuccess(false);
			response.setFailure(true);
			response.setFailureMessage(e.getMessage());
		}

	}

	/**
	 * MHV_CodeCR1514 - US12.4 MVI Compliance Implementation - AAL impl -->
	 */
	@Override
	@Transactional
	public void auditMviCorrelateEvent(UserProfileDTO userProfile, ActivityActorTypeEnumeration actor,
			Boolean successful, ResponseUtil response) {
		try {
			Activity activity = new Activity();

			activity.setAction("Connected to VA Record");
			activity.setActivityType("Account Upgrade");
			activity.setCompletionTime(new Timestamp(new Date().getTime()));
			activity.setPerformerType(actor.getDescription());
			activity.setUserProfileId(userProfile.getId());
			activity.setStatus(successful);
			activity.setDetailValue("Austin MHV");

			activityRepository.save(activity);
		} catch (Exception e) {
			log.error("Exception caught in auditing auditMviCorrelateEvent", e);
			response.setSuccess(false);
			response.setFailure(true);
			response.setFailureMessage(e.getMessage());
		}
	}

	@Override
	@Transactional
	public void auditUpdateProfileEvent(Long userProfileId, ActivityActorTypeEnumeration actor, ResponseUtil response) {

		try {
			Activity userProfileAudit = new Activity();

			userProfileAudit.setAction(ActivityActionTypeEnumeration.UPDATE.getDescription());
			userProfileAudit.setActivityType(ActivityTypeEnumeration.PROFILES.getDescription());
			userProfileAudit.setCompletionTime(new Timestamp(System.currentTimeMillis()));
			userProfileAudit.setPerformerType(actor.getDescription());
			userProfileAudit.setUserProfileId(userProfileId);
			userProfileAudit.setDetailValue("Update MHV User Profile");
			userProfileAudit.setStatus(Boolean.TRUE);

			activityRepository.save(userProfileAudit);
		} catch (Exception e) {
			log.error("Exception caught in auditing user profile", e);
			response.setSuccess(false);
			response.setFailure(true);
			response.setFailureMessage(e.getMessage());
		}
	}

	@Transactional
	private List<Activity> getActivitiesForSubject(String userName, Timestamp fromDate, Timestamp toDate, String performerType) {

		List<Activity> activities = null;
		System.out.println("TTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTT getActivitiesForSubject in Service" + userName);
		try {
			// response.setValidationErrors(new LinkedHashMap<String,
			// String>());
			// validator.activitySearchValidations(response, searchCriteria);
			System.out
					.println("ActivityServiceImpl>>>>>>>>>>>>>>>>>>>>>>>> getActivitiesForSubject userName" + userName);
			System.out
					.println("ActivityServiceImpl>>>>>>>>>>>>>>>>>>>>>>>> getActivitiesForSubject fromDate" + fromDate);
			System.out.println("ActivityServiceImpl>>>>>>>>>>>>>>>>>>>>>>>> getActivitiesForSubject toDate" + toDate);
			System.out.println("ActivityServiceImpl>>>>>>>>>>>>>>>>>>>>>>>> getActivitiesForSubject performerType"
					+ performerType);

			// EVERYONE
			if (performerType.equals(ActivityActorTypeEnumeration.EVERYONE.getDescription())) {
				activities = activityRepository.getActivitiesEveryoneTimestamp(fromDate, toDate, userName);
				if (activities != null) {
					System.out.println(
							"TTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTT getActivitiesEveryone in Service: " + activities.size());
				}
			} // EVERYONE BUT USER
			else if (performerType.equals(ActivityActorTypeEnumeration.EVERYONE_BUT_USER.getDescription())) {
				activities = activityRepository.getActivitiesEveryoneButUser(fromDate, toDate, userName);
				System.out.println(
						"TTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTT getActivitiesEveryoneButUser Service: " + activities.size());
			} // USER
			else if (performerType.equals(ActivityActorTypeEnumeration.USER.getDescription())) {
				activities = activityRepository.getActivitiesforUser(fromDate, toDate, userName);
				System.out.println(
						"TTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTT getActivitiesforUser Service: " + activities.size());
			} //  
			else {
				activities = activityRepository.getActivitiesForSubject(fromDate, toDate, performerType, userName);
				System.out.println(
						"TTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTT getActivitiesForSubject in Service" + activities.size());
			}

		} catch (Exception e) {
			log.error("Exception caught in get activities", e);
			// response.setSuccess(false);
			// response.setFailure(true);
			// response.setFailureMessage(e.getMessage());
		}
		return activities;
	}

	private static Date stringToDate(String dateStr) {
		Date date = null;
		try {
			if (dateStr != null) {
				SimpleDateFormat formatter = new SimpleDateFormat("dd-MMM-yyyy");
				date = formatter.parse(dateStr);
			}
		} catch (Exception e) {
			throw new RuntimeException("Invalid Date format");
		}
		return date;
	}

	/*
	 * @Transactional public boolean createLoginActivity(String userName, Long
	 * userProfileId,Boolean loginStatus) { boolean success = true;
	 *
	 * if ( null == userProfileId | null == loginStatus) { success = false;
	 * return success; } try { if(!loginStatus){ Activity
	 * loginAuditRow=getActivityInfo(userProfileId,loginStatus);
	 * activityRepository.save(loginAuditRow); }else{ List<Activity>
	 * loginActivities = activityRepository.getLoginActivities(userProfileId);
	 * if(loginActivities != null && loginActivities.size() > 0){ Activity
	 * loginActivity = loginActivities.get(0); if(!loginActivity.getStatus()){
	 * loginActivity.setStatus(loginStatus);
	 * activityRepository.save(loginActivity); }else{ Activity
	 * loginAuditRow=getActivityInfo(userProfileId,loginStatus);
	 * activityRepository.save(loginAuditRow); } }else{ Activity
	 * loginAuditRow=getActivityInfo(userProfileId,loginStatus);
	 * activityRepository.save(loginAuditRow); } } } catch (Exception e) {
	 * success = false; log.error("Exception caught creating Login Activity",
	 * e); } return success; }
	 *
	 * private Activity getActivityInfo(Long userProfileId, Boolean
	 * loginStatus){ Activity loginAuditRow = new Activity();
	 * loginAuditRow.setAction(ActivityActionTypeEnumeration.
	 * LOGIN_WITH_MY_HEALTHEVET.getDescription());
	 * loginAuditRow.setActivityType(ActivityActionTypeEnumeration.LOGIN.
	 * getDescription()); loginAuditRow.setUserProfileId(userProfileId);
	 * loginAuditRow.setStatus(loginStatus);
	 * loginAuditRow.setPerformerType(PerformerTypeEnumeration.valueOf("Self").
	 * toString()); loginAuditRow.setDetailValue("None");
	 * loginAuditRow.setCompletionTime(new Date()); return loginAuditRow; }
	 *
	 *
	 * @Override
	 *
	 * @Transactional public void auditOnlineAuthenticationEvent(Long subjectId,
	 * Boolean successful, String activityName, String actionName, String
	 * detail, ResponseUtil response){ try { Activity userProfileAudit = new
	 * Activity();
	 *
	 * userProfileAudit.setAction(actionName);
	 * userProfileAudit.setActivityType(activityName);
	 * userProfileAudit.setCompletionTime(new
	 * Timestamp(System.currentTimeMillis()));
	 * userProfileAudit.setPerformerType(ActivityActorTypeEnumeration.SELF.
	 * getDescription()); userProfileAudit.setUserProfileId(subjectId);
	 * if(detail != null) {
	 * userProfileAudit.setDetailValue(abbreviateString(detail,255)); }
	 * userProfileAudit.setStatus(successful);
	 *
	 * activityRepository.save(userProfileAudit); } catch (Exception e) {
	 * log.error("Exception caught in auditing user profile", e);
	 * response.setSuccess(false); response.setFailure(true);
	 * response.setFailureMessage(e.getMessage()); } }
	 *
	 *
	 *
	 *
	 *
	 * @Override
	 *
	 * @Transactional public void auditDSLogonMappingEvent(Long subjectId,
	 * Boolean successful, String actor, String activityType, String actionType,
	 * ResponseUtil response){ try { Activity userProfileAudit = new Activity();
	 *
	 * userProfileAudit.setAction(actionType);
	 * userProfileAudit.setActivityType(activityType);
	 * userProfileAudit.setCompletionTime(new
	 * Timestamp(System.currentTimeMillis()));
	 * userProfileAudit.setPerformerType(actor);
	 * userProfileAudit.setUserProfileId(subjectId);
	 * userProfileAudit.setDetailValue("");
	 * userProfileAudit.setStatus(successful);
	 *
	 * activityRepository.save(userProfileAudit); } catch (Exception e) {
	 * e.printStackTrace();
	 * log.error("Exception caught in auditing user profile", e);
	 * response.setSuccess(false); response.setFailure(true);
	 * response.setFailureMessage(e.getMessage()); } }
	 *
	 * @Override
	 *
	 * @Transactional public void auditDSLogonActivity(Long subjectId, Boolean
	 * successful, String actor, String actionType, String activityDetailValue,
	 * ResponseUtil response){ try { Activity userProfileAudit = new Activity();
	 *
	 * userProfileAudit.setAction(ActivityActionTypeEnumeration.
	 * LOGIN_WITH_DSLOGON.getDescription());
	 * userProfileAudit.setActivityType(actionType);
	 * userProfileAudit.setCompletionTime(new
	 * Timestamp(System.currentTimeMillis()));
	 * userProfileAudit.setPerformerType(actor);
	 * userProfileAudit.setUserProfileId(subjectId); if(activityDetailValue !=
	 * null) {
	 * userProfileAudit.setDetailValue(abbreviateString(activityDetailValue,255)
	 * ); } userProfileAudit.setStatus(successful);
	 *
	 * activityRepository.save(userProfileAudit); } catch (Exception e) {
	 * e.printStackTrace();
	 * log.error("Exception caught in auditing user profile", e);
	 * response.setSuccess(false); response.setFailure(true);
	 * response.setFailureMessage(e.getMessage()); } }
	 *
	 * @Override public void createAccountActivityLog(ActivityDTO activity,
	 * ResponseUtil response) throws MHVException {
	 * createAccountActivityLog(activity);
	 *
	 * }
	 *
	 *
	 *
	 * //JAZZ#32762 US02 The MHV System shall create an AAL entry for the
	 * onlineupgrade process.
	 *
	 * @Override
	 *
	 * @Transactional public void auditManualAuthenticationEvent(Long subjectId,
	 * Boolean successful, String actionName, String detail, ResponseUtil
	 * response){ try { Activity userProfileAudit = new Activity();
	 *
	 * userProfileAudit.setAction(actionName);
	 * userProfileAudit.setActivityType(ActivityTypeEnumeration.ACCOUNT_UPGRADE.
	 * getDescription()); userProfileAudit.setCompletionTime(new
	 * Timestamp(System.currentTimeMillis()));
	 * userProfileAudit.setPerformerType(ActivityActorTypeEnumeration.SELF.
	 * getDescription()); userProfileAudit.setUserProfileId(subjectId);
	 * if(detail != null) {
	 * userProfileAudit.setDetailValue(abbreviateString(detail,255)); }
	 * userProfileAudit.setStatus(true);
	 *
	 * activityRepository.save(userProfileAudit); } catch (Exception e) {
	 * log.error("Exception caught in auditing user profile", e);
	 * response.setSuccess(false); response.setFailure(true);
	 * response.setFailureMessage(e.getMessage()); } }
	 *
	 * @Override
	 *
	 * @Transactional public void auditUpgradeEvent(Long subjectId, String
	 * actor, String actionType, String activityDetailValue, ResponseUtil
	 * response){ try { Activity userProfileAudit = new Activity();
	 *
	 * userProfileAudit.setAction(ActivityActionTypeEnumeration.
	 * ACCOUNT_UPGRADE_REQUESTED.getDescription());
	 * userProfileAudit.setActivityType(actionType);
	 * userProfileAudit.setCompletionTime(new
	 * Timestamp(System.currentTimeMillis()));
	 * userProfileAudit.setPerformerType(actor);
	 * userProfileAudit.setUserProfileId(subjectId); if(activityDetailValue !=
	 * null) {
	 * userProfileAudit.setDetailValue(abbreviateString(activityDetailValue,255)
	 * ); } userProfileAudit.setStatus(true);
	 *
	 * activityRepository.save(userProfileAudit); } catch (Exception e) {
	 * log.error("Exception caught in auditing user profile", e);
	 * response.setSuccess(false); response.setFailure(true);
	 * response.setFailureMessage(e.getMessage()); } }
	 *
	 * @Override
	 *
	 * @Transactional public void auditUpdateProfileEvent(Long userProfileId,
	 * ActivityActorTypeEnumeration actor, ResponseUtil response) {
	 *
	 * try { Activity userProfileAudit = new Activity();
	 *
	 * userProfileAudit.setAction(ActivityActionTypeEnumeration.UPDATE.
	 * getDescription());
	 * userProfileAudit.setActivityType(ActivityTypeEnumeration.PROFILES.
	 * getDescription()); userProfileAudit.setCompletionTime(new
	 * Timestamp(System.currentTimeMillis()));
	 * userProfileAudit.setPerformerType(actor.getDescription());
	 * userProfileAudit.setUserProfileId(userProfileId);
	 * userProfileAudit.setDetailValue("Update MHV User Profile");
	 * userProfileAudit.setStatus(Boolean.TRUE);
	 *
	 * activityRepository.save(userProfileAudit); } catch (Exception e) {
	 * log.error("Exception caught in auditing user profile", e);
	 * response.setSuccess(false); response.setFailure(true);
	 * response.setFailureMessage(e.getMessage()); } }
	 *
	 * @Override
	 *
	 * @Transactional public void createUserActivity(Boolean status, String
	 * action, String activityType, ResponseUtil response, Long userProfileId){
	 *
	 * log.debug("Creating Account Activity Log - Forgot USERID and Password");
	 *
	 * try { Activity recoverActivity = new Activity();
	 * recoverActivity.setAction(action);
	 * recoverActivity.setActivityType(activityType);
	 * recoverActivity.setStatus(status);
	 * recoverActivity.setPerformerType("Self");
	 * recoverActivity.setCompletionTime(new Date());
	 * recoverActivity.setUserProfileId(userProfileId);
	 * activityRepository.save(recoverActivity); } catch (Exception e) {
	 * log.error("Exception caught in auditing Change Password", e);
	 * response.setSuccess(false); response.setFailure(true);
	 * response.setFailureMessage(e.getMessage()); } if (log.isDebugEnabled()) {
	 * log.debug("audir change password is successful..."); } }
	 *
	 *
	 *
	 * @Transactional public void auditRegistrationEvent(UserProfile
	 * userProfile, ResponseUtil response) { if (log.isDebugEnabled()) {
	 * log.debug("inside auditRegistrationEvent......"); }
	 *
	 * try { Activity activity = new Activity();
	 *
	 * activity.setAction(ActivityActionTypeEnumeration.ACCOUNT_CREATED.
	 * getDescription());
	 * activity.setActivityType(ActivityTypeEnumeration.REGISTRATION.
	 * getDescription()); activity.setCompletionTime(new
	 * Timestamp(System.currentTimeMillis()));
	 * activity.setPerformerType(ActivityActorTypeEnumeration.SELF.
	 * getDescription()); activity.setUserProfileId(userProfile.getId());
	 * activity.setDetailValue("User Registration");
	 * activity.setStatus(Boolean.TRUE);
	 *
	 * activityRepository.save(activity); } catch (Exception e) {
	 * log.error("Exception caught in auditing user profile", e);
	 * response.setSuccess(false); response.setFailure(true);
	 * response.setFailureMessage(e.getMessage()); } if (log.isDebugEnabled()) {
	 * log.debug("auditRegistrationEvent is successful..."); } }
	 *
	 *
	 *
	 * @Override
	 *
	 * @Transactional public void auditUpdateEmailProfileEvent(Long
	 * userProfileId, String actor, String action, ResponseUtil response) { try
	 * { Activity activity = new Activity(); activity.setAction(action);
	 * activity.setActivityType(ActivityTypeEnumeration.PROFILES.getDescription(
	 * )); activity.setCompletionTime(new Timestamp(new Date().getTime()));
	 * activity.setPerformerType(actor); activity.setStatus(Boolean.TRUE);
	 * activity.setUserProfileId(userProfileId);
	 *
	 * activityRepository.save(activity); } catch (Exception e) {
	 * log.error("Exception caught in auditing user profile", e);
	 * response.setSuccess(false); response.setFailure(true);
	 * response.setFailureMessage(e.getMessage()); } }
	 *
	 * @Override
	 *
	 * @Transactional public void auditUpdateEmailProfileEvent(Long
	 * userProfileId, String actor, String action, ResponseUtil response, String
	 * activityDetails) { try { Activity activity = new Activity();
	 * activity.setAction(action); if(activityDetails != null) {
	 * activity.setDetailValue(abbreviateString(activityDetails,255)); }
	 * activity.setActivityType(ActivityTypeEnumeration.PROFILES.getDescription(
	 * )); activity.setCompletionTime(new Timestamp(new Date().getTime()));
	 * activity.setPerformerType(actor); activity.setStatus(Boolean.TRUE);
	 * activity.setUserProfileId(userProfileId);
	 *
	 * activityRepository.save(activity); } catch (Exception e) {
	 * log.error("Exception caught in auditing user profile", e);
	 * response.setSuccess(false); response.setFailure(true);
	 * response.setFailureMessage(e.getMessage()); } }
	 *
	 * @Transactional public void auditMviEvent( UserProfileDTO userProfile,
	 * String actor, Boolean successful, ActivityActionTypeEnumeration
	 * activityActionType, ActivityTypeEnumeration activityType, ResponseUtil
	 * response) { auditMviEvent(userProfile,
	 * ActivityActorTypeEnumeration.fromString(actor), successful,
	 * activityActionType, activityType, response); }
	 *
	 *
	 *
	 *
	 * @Transactional public void auditMviLoginEvent( UserProfileDTO
	 * userProfile, ActivityActorTypeEnumeration actor, Boolean successful,
	 * ActivityActionTypeEnumeration activityActionType, ResponseUtil response)
	 * { try { Activity activity = new Activity();
	 *
	 * activity.setAction(activityActionType.getDescription());
	 * activity.setActivityType(ActivityTypeEnumeration.LOGIN_LOGOUT.
	 * getDescription()); activity.setCompletionTime(new
	 * Timestamp(System.currentTimeMillis()));
	 * activity.setPerformerType(actor.getDescription());
	 * activity.setUserProfileId(userProfile.getId());
	 * activity.setDetailValue("Austin MHV"); activity.setStatus(successful);
	 *
	 * activityRepository.save(activity); } catch (Exception e) {
	 * log.error("Exception caught in auditing auditMviLoginEvent", e);
	 * response.setSuccess(false); response.setFailure(true);
	 * response.setFailureMessage(e.getMessage()); } }
	 *
	 * @Transactional public void auditAccountUpgradeEvent(UserProfileDTO
	 * userProfile, Boolean successful, ActivityActorTypeEnumeration performer,
	 * ResponseUtil response) { try { Activity activity = new Activity();
	 *
	 * activity.setAction(ActivityActionTypeEnumeration.UPGRADE_TO_ADVANCED.
	 * getDescription());
	 * activity.setActivityType(ActivityTypeEnumeration.ACCOUNT_UPGRADE.
	 * getDescription()); activity.setCompletionTime(new Timestamp(new
	 * Date().getTime()));
	 * activity.setPerformerType(performer.getDescription());
	 * activity.setUserProfileId(userProfile.getId());
	 * activity.setStatus(successful); activity.setDetailValue("Austin MHV");
	 *
	 * activityRepository.save(activity); } catch (Exception e) {
	 * log.error("Exception caught in auditing auditAccountUpgradeEvent", e);
	 * response.setSuccess(false); response.setFailure(true);
	 * response.setFailureMessage(e.getMessage()); } }
	 *
	 *//**
		 * MHV_CodeCR1514 - US12.4 MVI Compliance Implementation - AAL impl -->
		 */
	/*
	 * @Transactional public void
	 * auditMviUnauthenticateRequestEvent(UserProfileDTO userProfile,
	 * ActivityActorTypeEnumeration actor, Boolean successful, ResponseUtil
	 * response) { try { Activity activity = new Activity();
	 *
	 * activity.setAction(ActivityActionTypeEnumeration.UNAUTHENTICATION_REQUEST
	 * .getDescription());
	 * activity.setActivityType(ActivityTypeEnumeration.ACCOUNT_UPGRADE.
	 * getDescription()); activity.setCompletionTime(new Timestamp(new
	 * Date().getTime())); activity.setPerformerType(actor.getDescription());
	 * activity.setUserProfileId(userProfile.getId());
	 * activity.setStatus(successful); activity.setDetailValue(null);
	 *
	 * activityRepository.save(activity); } catch (Exception e) { log.
	 * error("Exception caught in auditing auditMviUnauthenticateRequestEvent",
	 * e); response.setSuccess(false); response.setFailure(true);
	 * response.setFailureMessage(e.getMessage()); }
	 *
	 * }
	 *
	 *//**
		 * MHV_CodeCR1514 - US12.4 MVI Compliance Implementation - AAL impl -->
		 *//*
		 * @Transactional public void auditMviCorrelateEvent(UserProfileDTO
		 * userProfile, ActivityActorTypeEnumeration actor, Boolean successful,
		 * ResponseUtil response) { try { Activity activity = new Activity();
		 *
		 * activity.setAction(ActivityActionTypeEnumeration.
		 * CONNECTED_TO_VA_RECORD.getDescription());
		 * activity.setActivityType(ActivityTypeEnumeration.ACCOUNT_UPGRADE.
		 * getDescription()); activity.setCompletionTime(new Timestamp(new
		 * Date().getTime()));
		 * activity.setPerformerType(actor.getDescription());
		 * activity.setUserProfileId(userProfile.getId());
		 * activity.setStatus(successful);
		 * activity.setDetailValue("Austin MHV");
		 *
		 * activityRepository.save(activity); } catch (Exception e) {
		 * log.error("Exception caught in auditing auditMviCorrelateEvent", e);
		 * response.setSuccess(false); response.setFailure(true);
		 * response.setFailureMessage(e.getMessage()); } }
		 *
		 * @Transactional public void auditAccountDeactivateEvent(UserProfileDTO
		 * userProfile, ActivityActorTypeEnumeration actor, Boolean successful,
		 * ResponseUtil response) { try { Activity activity = new Activity();
		 *
		 * activity.setAction(ActivityActionTypeEnumeration.DEACTIVATE.
		 * getDescription());
		 * activity.setActivityType(ActivityTypeEnumeration.ACCOUNT_MANAGEMENT.
		 * getDescription()); activity.setCompletionTime(new Timestamp(new
		 * Date().getTime()));
		 * activity.setPerformerType(actor.getDescription());
		 * activity.setUserProfileId(userProfile.getId());
		 * activity.setStatus(successful); activity.setDetailValue(null);
		 *
		 * activityRepository.save(activity); } catch (Exception e) {
		 * log.error("Exception caught in auditing auditAccountDeactivateEvent",
		 * e); response.setSuccess(false); response.setFailure(true);
		 * response.setFailureMessage(e.getMessage()); } }
		 *
		 * @Override
		 *
		 * @Transactional public boolean createLogoutActivity(String userName,
		 * Long userProfileId) { boolean success = true;
		 *
		 * if (null == userName | null == userProfileId) { success = false;
		 * return success; } try { Activity loginAuditRow = new Activity();
		 * loginAuditRow.setAction(ActivityActionTypeEnumeration.
		 * LOGOUT_WITH_MY_HEALTHEVET.getDescription());
		 * loginAuditRow.setActivityType(ActivityActionTypeEnumeration.LOGOUT.
		 * getDescription()); loginAuditRow.setUserProfileId(userProfileId);
		 * loginAuditRow.setStatus(Boolean.TRUE);
		 * loginAuditRow.setPerformerType(PerformerTypeEnumeration.valueOf(
		 * "Self").toString()); loginAuditRow.setDetailValue("None");
		 * loginAuditRow.setCompletionTime(new Date());
		 * activityRepository.save(loginAuditRow); } catch (Exception e) {
		 * success = false;
		 * log.error("Exception caught creating Logout Activity", e); } return
		 * success;
		 *
		 * }
		 *
		 * @Override
		 *
		 * @Transactional public void auditDownloadDataRequestEvent(long
		 * userProfileId, ActivityActorTypeEnumeration actor, boolean
		 * successful, ActivityActionTypeEnumeration activityActionType,
		 * ResponseUtil response){ try { Activity activity = new Activity();
		 * activity.setAction(activityActionType.getDescription());
		 * activity.setActivityType(ActivityTypeEnumeration.DOWNLOAD.
		 * getDescription()); activity.setCompletionTime(new Timestamp(new
		 * Date().getTime()));
		 * activity.setPerformerType(actor.getDescription());
		 * activity.setUserProfileId(userProfileId);
		 * activity.setStatus(successful); activityRepository.save(activity); }
		 * catch (Exception e) { log.
		 * error("Exception caught in auditing auditDownloadDataRequestEvent",
		 * e); response.setSuccess(false); response.setFailure(true);
		 * response.setFailureMessage(e.getMessage()); } }
		 *
		 * @POST
		 *
		 * @Path("/activity")
		 *
		 * @Consumes(MediaType.APPLICATION_JSON) public void
		 * createAccountActivityLog(ActivityDTO activity) throws MHVException {
		 * try { System.out.println("activity Details::"+
		 * activity.getAction()+"::"+activity.getActivityType()+"::"+activity.
		 * getDetailValue()+"::"+activity.getDisplayStatus()+"::"+activity.
		 * getPerformerType()+"::"+activity.getUserProfileId());
		 * this.activityRepository.save(this.activityDTOConverter.convert(
		 * activity)); } catch (Exception ex) { String message =
		 * "Error creating activity log entry."; log.error(message, ex); throw
		 * new MHVException(message, ex); } }
		 *
		 *
		 *
		 * @Transactional public void auditShareDataWithHRAEvent(String
		 * userName) throws MHVException {
		 * log.debug(" DEBUG SERVER -- Create HRA Event Activity "); try {
		 * UserProfile profile = userProfileRepository.findByUserName(userName);
		 *
		 * if (null == profile) { throw new MHVException("UserProfile is null");
		 * }
		 *
		 * this.auditShareDataWithHRAEvent(profile.getId(),
		 * ActivityActorTypeEnumeration.SELF, true, true,
		 * ActivityActionTypeEnumeration.ACCESS_HEALTHELIVING_ASSESSMENT.
		 * getDescription());
		 *
		 * } catch (Exception e) {
		 * log.error("Exception Caught in logging HRA Event Activity", e); throw
		 * new MHVException("Exception Caught in logging HRA Event Activity",
		 * e); } }
		 *
		 * private void auditShareDataWithHRAEvent(Long userProfileId,
		 * ActivityActorTypeEnumeration actor, boolean accept, Boolean
		 * successful, String activityActionType){ try { Activity activity = new
		 * Activity();
		 *
		 * if (accept) {
		 * activity.setAction(ActivityActionTypeEnumeration.CONTINUE.
		 * getDescription()); } else {
		 * activity.setAction(ActivityActionTypeEnumeration.CANCEL.
		 * getDescription()); }
		 * activity.setActivityType(ActivityTypeEnumeration.SHARE.getDescription
		 * ()); activity.setCompletionTime(new Timestamp(new Date().getTime()));
		 * activity.setPerformerType(actor.getDescription());
		 * activity.setUserProfileId(userProfileId);
		 * activity.setStatus(successful); if(activityActionType != null) {
		 * activity.setDetailValue(abbreviateString(activityActionType,255)); }
		 * if(activityActionType != null) {
		 * activity.setDetailValue(abbreviateString(activityActionType,255)); }
		 *
		 * activityRepository.save(activity); } catch (Exception e) {
		 * log.error("Exception caught in auditing auditShareDataWithHRAEvent",
		 * e); } }
		 *
		 * @POST
		 *
		 * @Path("/auditAppointments/{userProfileId}")
		 *
		 * @Consumes(MediaType.APPLICATION_JSON)
		 *
		 * @Override public void auditViewAppointments(String userName, Long
		 * userProfileId) {
		 *
		 * Activity viewAppointments = new Activity();
		 * viewAppointments.setAction(ActivityActionTypeEnumeration.VIEW.
		 * getDescription());
		 * viewAppointments.setActivityType(ActivityActionTypeEnumeration.
		 * APPOINTMENTS.getDescription()); UserProfile userProfile=
		 * userProfileRepository.findOne(userProfileId);
		 * viewAppointments.setUserProfileId(userProfile.getId());
		 * viewAppointments.setStatus(Boolean.TRUE);
		 * viewAppointments.setPerformerType(PerformerTypeEnumeration.valueOf(
		 * "Self").toString()); viewAppointments.setCompletionTime(new Date());
		 * activityRepository.save(viewAppointments); }
		 *
		 * @Override public void auditUpdateEmailProfileEvent(Long subjectId,
		 * String actor, String action) { // TODO Auto-generated method stub
		 *
		 * }
		 *
		 * @Override public void auditNotificationEvent(Long subjectId, Boolean
		 * successful, ActivityActionTypeEnumeration activityActionType, String
		 * activityActionDetail){ Activity activity = new Activity();
		 * activity.setAction(activityActionType.toString());
		 * if(activityActionDetail != null) {
		 * activity.setDetailValue(abbreviateString(activityActionDetail,255));
		 * } activity.setActivityType(ActivityTypeEnumeration.NOTIFICATIONS.
		 * getDescription()); activity.setCompletionTime(new Timestamp(new
		 * Date().getTime()));
		 * activity.setPerformerType(PerformerTypeEnumeration.valueOf("System").
		 * toString()); activity.setUserProfileId(subjectId);
		 * activity.setStatus(successful); activityRepository.save(activity); }
		 *
		 * @Override public void auditSendVAHealthSummaryEvent(Long subjectId,
		 * Boolean successful, ActivityActionTypeEnumeration activityActionType,
		 * String activityActionDetail){ Activity activity = new Activity();
		 * activity.setAction(activityActionType.toString());
		 * if(activityActionDetail != null) {
		 * activity.setDetailValue(abbreviateString(activityActionDetail,255));
		 * } activity.setActivityType(ActivityTypeEnumeration.
		 * SEND_VA_HEALTH_SUMMARY.getDescription());
		 * activity.setCompletionTime(new Timestamp(new Date().getTime()));
		 * activity.setPerformerType(PerformerTypeEnumeration.valueOf("System").
		 * toString()); activity.setUserProfileId(subjectId);
		 * activity.setStatus(successful); activityRepository.save(activity); }
		 *
		 * @Override public void auditMviEvent(Long subjectId, String actor,
		 * Boolean successful, String activityActionType, String activityType,
		 * ResponseUtil response){ try { Activity activity = new Activity();
		 *
		 * activity.setAction(activityActionType);
		 * activity.setActivityType(activityType);
		 * activity.setCompletionTime(new Date());
		 * activity.setPerformerType(actor);
		 * activity.setUserProfileId(subjectId); activity.setStatus(successful);
		 * activity.setDetailValue(null);
		 *
		 * activityRepository.save(activity); } catch (Exception e) { log.
		 * error("Exception caught in auditing auditMviUnauthenticateRequestEvent"
		 * , e); response.setSuccess(false); response.setFailure(true);
		 * response.setFailureMessage(e.getMessage()); }
		 *
		 * }
		 *
		 *
		 * @Override public void auditDSLogonMappingActivity(Long subjectId,
		 * Boolean successful, String actor, String actionType, String
		 * activityDetailValue, ResponseUtil response) {
		 *
		 * try { Activity activity = new Activity();
		 * activity.setAction(actionType); if(activityDetailValue != null) {
		 * activity.setDetailValue(abbreviateString(activityDetailValue,255)); }
		 * activity.setActivityType(ActivityActionTypeEnumeration.LOGIN.
		 * getDescription()); activity.setCompletionTime(new Timestamp(new
		 * Date().getTime())); activity.setPerformerType(actor);
		 * activity.setUserProfileId(subjectId); activity.setStatus(successful);
		 * activityRepository.save(activity); } catch(Exception e){
		 * log.error("Exception caught in auditing auditDSLogonMappingActivity",
		 * e); response.setSuccess(false); response.setFailure(true);
		 * response.setFailureMessage(e.getMessage()); }
		 *
		 * }
		 *
		 *
		 * @POST
		 *
		 * @Path("/loginActivityFailed/{userProfileId}")
		 *
		 * @Consumes(MediaType.APPLICATION_JSON) public void
		 * createFailedLoginActivity(Long userProfileId) throws MHVException{
		 * boolean loginActivity= createLoginActivity(null, userProfileId,
		 * Boolean.FALSE); if(!loginActivity){ String errorMessage =
		 * "Exception Caught Creatring Failed login Activity"; throw new
		 * MHVException(errorMessage); } }
		 *
		 * private static String abbreviateString(String input, int maxLength) {
		 * if (input.length() <= maxLength) return input; else return
		 * input.substring(0, maxLength-2) + ".."; }
		 *
		 *
		 * public void auditViewAppointments(Long userProfileId) {
		 * auditViewAppointments(null,userProfileId); }
		 *
		 * @POST
		 *
		 * @Path("/auditAppointmentsFromCalendar/{userProfileId}")
		 *
		 * @Consumes(MediaType.APPLICATION_JSON) public void
		 * auditViewAppointmentsFromCalendar(Long userProfileId) { Activity
		 * viewAppointments = new Activity();
		 * viewAppointments.setAction(ActivityActionTypeEnumeration.VIEW.
		 * getDescription());
		 * viewAppointments.setActivityType(ActivityActionTypeEnumeration.
		 * APPOINTMENTS.getDescription()); UserProfile userProfile=
		 * userProfileRepository.findOne(userProfileId);
		 * viewAppointments.setUserProfileId(userProfile.getId());
		 * viewAppointments.setStatus(Boolean.TRUE);
		 * viewAppointments.setPerformerType(PerformerTypeEnumeration.valueOf(
		 * "Self").toString()); viewAppointments.setCompletionTime(new Date());
		 * viewAppointments.
		 * setDetailValue("VA Appointment calendar viewing by user");
		 * activityRepository.save(viewAppointments);
		 *
		 * }
		 *
		 * @POST
		 *
		 * @Path("/auditChemistryHematology/{userProfileId}")
		 *
		 * @Consumes(MediaType.APPLICATION_JSON) public void
		 * auditViewChemistryHematology(Long userProfileId) { Activity
		 * viewChemistryHematology = new Activity();
		 * viewChemistryHematology.setAction(ActivityActionTypeEnumeration.VIEW.
		 * getDescription());
		 * viewChemistryHematology.setActivityType(ActivityActionTypeEnumeration
		 * .VIEW_CHEMISTRY_HEMATOLOGY.getDescription()); UserProfile
		 * userProfile= userProfileRepository.findOne(userProfileId);
		 * viewChemistryHematology.setUserProfileId(userProfile.getId());
		 * viewChemistryHematology.setStatus(Boolean.TRUE);
		 * viewChemistryHematology.setPerformerType(PerformerTypeEnumeration.
		 * valueOf("Self").toString());
		 * viewChemistryHematology.setCompletionTime(new Date());
		 * activityRepository.save(viewChemistryHematology); }
		 *
		 *
		 * @POST
		 *
		 * @Path("/auditViewAllergy/{userProfileId}")
		 *
		 * @Consumes(MediaType.APPLICATION_JSON) public void
		 * auditViewAllergy(Long userProfileId) { Activity viewAllergy = new
		 * Activity(); viewAllergy.setAction(ActivityActionTypeEnumeration.VIEW.
		 * getDescription());
		 * viewAllergy.setActivityType(ActivityActionTypeEnumeration.
		 * VIEW_ALLERGY.getDescription()); UserProfile userProfile=
		 * userProfileRepository.findOne(userProfileId);
		 * viewAllergy.setUserProfileId(userProfile.getId());
		 * viewAllergy.setStatus(Boolean.TRUE);
		 * viewAllergy.setPerformerType(PerformerTypeEnumeration.valueOf("Self")
		 * .toString()); viewAllergy.setCompletionTime(new Date());
		 * activityRepository.save(viewAllergy);
		 *
		 * }
		 *
		 * @POST
		 *
		 * @Path("/auditViewWellnessReminders/{userProfileId}")
		 *
		 * @Consumes(MediaType.APPLICATION_JSON) public void
		 * auditViewWellnessReminders(Long userProfileId) { Activity
		 * viewWellnessReminders = new Activity();
		 * viewWellnessReminders.setAction(ActivityActionTypeEnumeration.VIEW.
		 * getDescription());
		 * viewWellnessReminders.setActivityType(ActivityActionTypeEnumeration.
		 * VIEW_WELLNESS_REMINDERS.getDescription()); UserProfile userProfile=
		 * userProfileRepository.findOne(userProfileId);
		 * viewWellnessReminders.setUserProfileId(userProfile.getId());
		 * viewWellnessReminders.setStatus(Boolean.TRUE);
		 * viewWellnessReminders.setPerformerType(PerformerTypeEnumeration.
		 * valueOf("Self").toString());
		 * viewWellnessReminders.setCompletionTime(new Date());
		 * activityRepository.save(viewWellnessReminders);
		 *
		 * }
		 *
		 * @Override public void auditRegistrationEvent(UserProfileDTO
		 * userProfile, ResponseUtil response) { // TODO Auto-generated method
		 * stub
		 *
		 * }
		 *
		 * @Override public void auditAccountDeactivateEauthHashEvent(
		 * UserProfileDTO userProfile, ResponseUtil response) { // TODO
		 * Auto-generated method stub
		 *
		 * }
		 *
		 * @Override public void createAccountActivityLog(ActivityDTO activity,
		 * ResponseUtil response) throws MHVException { // TODO Auto-generated
		 * method stub
		 *
		 * }
		 *
		 * @Override public void auditMviEvent(UserProfileDTO userProfile,
		 * ActivityActorTypeEnumeration actor, Boolean successful,
		 * ActivityActionTypeEnumeration activityActionType,
		 * ActivityTypeEnumeration activityType, ResponseUtil response) { //
		 * TODO Auto-generated method stub
		 *
		 * }
		 *
		 * @Override public void auditMviEvent(UserProfileDTO userProfile,
		 * String actor, Boolean successful, ActivityActionTypeEnumeration
		 * activityActionType, ActivityTypeEnumeration activityType,
		 * ResponseUtil response) { // TODO Auto-generated method stub
		 *
		 * }
		 *
		 * @Override public void auditAccountDeactivateEauthHashEvent(
		 * UserProfileDTO userProfile, ResponseUtil response) { // TODO
		 * Auto-generated method stub
		 *
		 * }
		 *
		 * @Override public void auditMviLoginEvent(UserProfileDTO userProfile,
		 * ActivityActorTypeEnumeration actor, Boolean successful,
		 * ActivityActionTypeEnumeration activityActionType, ResponseUtil
		 * response) { // TODO Auto-generated method stub
		 *
		 * }
		 *
		 * @Override public void auditAccountUpgradeEvent(UserProfileDTO
		 * userProfile, Boolean successful, ActivityActorTypeEnumeration
		 * performer, ResponseUtil response) { // TODO Auto-generated method
		 * stub
		 *
		 * }
		 *
		 * @Override public void
		 * auditMviUnauthenticateRequestEvent(UserProfileDTO userProfile,
		 * ActivityActorTypeEnumeration actor, Boolean successful, ResponseUtil
		 * response) { // TODO Auto-generated method stub
		 *
		 * }
		 *
		 * @Override public void auditMviCorrelateEvent(UserProfileDTO
		 * userProfile, ActivityActorTypeEnumeration actor, Boolean successful,
		 * ResponseUtil response) { // TODO Auto-generated method stub
		 *
		 * }
		 *
		 * @Override public void auditAccountDeactivateEvent(UserProfileDTO
		 * userProfile, ActivityActorTypeEnumeration actor, Boolean successful,
		 * ResponseUtil response) { // TODO Auto-generated method stub
		 *
		 * }
		 *
		 * @Override public void createAccountActivityLog(ActivityDTO activity,
		 * ResponseUtil response) throws MHVException { // TODO Auto-generated
		 * method stub
		 *
		 * } }
		 */

}
