package gov.va.med.mhv.admin.service.impl.mhvusers;

import java.util.StringTokenizer;

import javax.ws.rs.Consumes;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.commons.mail.HtmlEmail;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.stereotype.Service;

import gov.va.med.mhv.admin.dto.EmailDTO;
import gov.va.med.mhv.admin.exception.EmailFailureException;
import gov.va.med.mhv.admin.handler.MviProperties;
import gov.va.med.mhv.admin.service.mhvusers.EmailService;


@Path("/")
@Service("emailService")
@Component
public class EmailServiceImpl implements EmailService {
	
	private static final Log LOG = LogFactory.getLog(EmailServiceImpl.class);
	
	@Autowired
	private MviProperties mviProperties;
	private static final String EMAIL_DELIM_CHAR = ",";
	
	
	@POST
	@Consumes("application/json")
	@Produces("application/json")
	@Path("/sendEmail/{emailFrom}/{emailName}/{emailSubject}/{bodyMessage}/{recipientEmail}")
	public Boolean sendEmail(@PathParam("emailFrom") String emailFrom, @PathParam("emailName") String emailName, @PathParam("emailSubject") String emailSubject, @PathParam("bodyMessage") String bodyMessage, @PathParam("recipientEmail") String recipientEmail) throws EmailFailureException {
		System.out.println("SOP: sendTempEmail service starting point");
		HtmlEmail htmlEmail = new HtmlEmail();

		try {
			htmlEmail.setSubject(emailSubject);
			htmlEmail.setHostName(mviProperties.getSmtpHost());
			htmlEmail.setDebug(mviProperties.getSmtpDebug());
			htmlEmail.setHtmlMsg(bodyMessage);
			
			StringTokenizer toTok = new StringTokenizer(recipientEmail, EMAIL_DELIM_CHAR);
			while(toTok.hasMoreTokens()) {
				htmlEmail.addTo(toTok.nextToken());
			}
			htmlEmail.setFrom(emailFrom);
			htmlEmail.setMsg(bodyMessage);
			htmlEmail.send();
		} catch (Exception e) {
			String errMsg = String.format("AdminPortal: Error sending message: %s", e.getMessage());
			LOG.error(errMsg);
			throw new EmailFailureException(errMsg, e);
		}

		return true;
	}
	 
	@POST
	@Consumes("application/json")
	@Produces("application/json")
	@Path("/sendTempEmail")
	public Boolean sendTempEmail(EmailDTO emailDto) throws EmailFailureException {
		LOG.info("sendEmail service starting point");
		System.out.println("SOP: sendEmail service starting point");
		HtmlEmail htmlEmail = new HtmlEmail();

		try {
			htmlEmail.setSubject(emailDto.getSubject());
			htmlEmail.setHostName(mviProperties.getSmtpHost());
			htmlEmail.setDebug(mviProperties.getSmtpDebug());
			htmlEmail.setHtmlMsg(emailDto.getMessage());
			System.out.println("########################### >> emailDto.getRecipient()" + emailDto.getRecipient());
			StringTokenizer toTok = new StringTokenizer(emailDto.getRecipient(), EMAIL_DELIM_CHAR);
			while(toTok.hasMoreTokens()) {
				htmlEmail.addTo(toTok.nextToken());
			}
			htmlEmail.setFrom(emailDto.getFrom());
			htmlEmail.setMsg(emailDto.getMessage());
			htmlEmail.send();
		} catch (Exception e) {
			String errMsg = String.format("AdminPortal: Error sending message: %s", e.getMessage());
			LOG.error(errMsg);
			throw new EmailFailureException(errMsg, e);
		}

		return true;
	}

}
