package gov.va.med.mhv.admin.service.impl.mhvusers;

import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Date;
import java.util.LinkedHashMap;
import java.util.List;

import javax.ws.rs.Consumes;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.core.MediaType;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.stereotype.Service;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import gov.va.med.mhv.admin.converter.PatientLocalConverter;
import gov.va.med.mhv.admin.converter.UserProfileLocalConverter;
import gov.va.med.mhv.admin.enums.ActivityActionTypeEnumeration;
import gov.va.med.mhv.admin.enums.ActivityActorTypeEnumeration;
import gov.va.med.mhv.admin.enums.ActivityTypeEnumeration;
import gov.va.med.mhv.admin.enums.AuthenticationStatusEnumeration;
import gov.va.med.mhv.admin.enums.PatientCorrelationStatusEnumeration;
import gov.va.med.mhv.admin.exception.MHVRuntimeException;
import gov.va.med.mhv.admin.repository.InPersonAuthenticationRepository;
import gov.va.med.mhv.admin.repository.PatientRepository;
import gov.va.med.mhv.admin.repository.EmployeeRepository;
import gov.va.med.mhv.admin.service.mhvusers.InPersonAuthenticationService;
import gov.va.med.mhv.admin.service.mhvusers.MviIntegrationService;
import gov.va.med.mhv.admin.util.MessagesUtil;
import gov.va.med.mhv.admin.service.mhvusers.ActivityService;
import gov.va.med.mhv.common.api.dto.InPersonAuthenticationDTO;
import gov.va.med.mhv.common.api.dto.PatientDTO;
import gov.va.med.mhv.common.api.dto.UserProfileDTO;
import gov.va.med.mhv.common.api.exception.MHVException;
import gov.va.med.mhv.common.api.util.ResponseUtil;
import gov.va.med.mhv.common.data.converter.InPersonAuthenticationConverter;
import gov.va.med.mhv.common.data.converter.InPersonAuthenticationLightConverter;
import gov.va.med.mhv.common.data.converter.PatientConverter;
import gov.va.med.mhv.common.data.model.InPersonAuthentication;
import gov.va.med.mhv.common.data.model.Patient;
import gov.va.med.mhv.admin.model.Employee;

@Path("/")
@Service("inPersonAuthenticationService")
@Component
public class InPersonAuthenticationServiceImpl implements InPersonAuthenticationService {

	private static final InPersonAuthenticationConverter IPA_CONVERTER = new InPersonAuthenticationConverter();
	private static final InPersonAuthenticationLightConverter IPALIGHT_CONVERTER = new InPersonAuthenticationLightConverter();
	private static final PatientConverter PATIENT_CONVERTER = new PatientConverter();

	private static Logger log = LogManager.getLogger(InPersonAuthenticationServiceImpl.class);

	@Autowired
	private InPersonAuthenticationRepository inpersonAuthenticationRepository;

	@Autowired
	private PatientRepository patientRepository;

	@Autowired
	private EmployeeRepository employeeRepository;

	@Autowired
	InPersonAuthenticationConverter converter;

	@Autowired
	UserProfileLocalConverter userProfileConverter;

	@Autowired
	MviIntegrationService mviService;

	@Autowired
	private ActivityService activityService;

	@Autowired
	PatientLocalConverter patientConverter;
	
	@Autowired
	private MessagesUtil messagesUtil;
	

	@Override
	@GET
	@Path("/isPatientAuthenticated/{patientId}")
	@Produces(MediaType.APPLICATION_JSON)
	public Boolean isPatientAuthenticated(@PathParam("patientId") Long patientId) throws MHVException {
		System.out.println("******************************************* isPatientAuthenticated method" + patientId);
		List<InPersonAuthentication> ipaPatients = null;
		InPersonAuthentication ipaPatient = null;
		Boolean isPatientAUthenticated = false;
		try {
			ipaPatients = inpersonAuthenticationRepository.getAuthenticationForPatient(patientId);

			if (null != ipaPatients && ipaPatients.size() > 0) {
				ipaPatient = ipaPatients.get(0);
			} else {
				throw new MHVException(
						"Error in getting patinet details from getAuthenticationForPatient() " + patientId);
			}

			if (null != ipaPatient.getStatus() && ipaPatient.getStatus()
					.equalsIgnoreCase(AuthenticationStatusEnumeration.AUTHENTICATED.getDescription())) {
				isPatientAUthenticated = true;
			} else {
				isPatientAUthenticated = false;
			}
		} catch (Exception e) {
			log.error("Error in getting patinet details from getAuthenticationForPatient() " + patientId, e);
			throw new MHVRuntimeException(
					"Error in getting patinet details from getAuthenticationForPatient() " + patientId);
		}

		return isPatientAUthenticated;
	}

	@Override
	@GET
	@Path("/getIPAPatientById/{patientId}")
	@Produces(MediaType.APPLICATION_JSON)
	public InPersonAuthenticationDTO getIPAPatientById(@PathParam("patientId") Long patientId) throws MHVException {
		List<InPersonAuthentication> ipaPatients = null;
		InPersonAuthentication ipaPatient = null;
		InPersonAuthenticationDTO inPersonAuthenticationDTO = null;
		try {
			ipaPatients = inpersonAuthenticationRepository.getAuthenticationForPatient(patientId);

			if (null != ipaPatients && ipaPatients.size() > 0) {
				System.out.println(">>>> InPersonAuthenticationServiceImpl - getIPAPatientById - ipaPatients is not null");
				ipaPatient = ipaPatients.get(0);
			}
			if (ipaPatient != null) {
				inPersonAuthenticationDTO = converter.convertDomainInpersonAuthentication(ipaPatient);
				converter.addFacilityInfoDto(ipaPatient, inPersonAuthenticationDTO);
				System.out.println(">>>> InPersonAuthenticationServiceImpl - getIPAPatientById - inPersonAuthenticationDTO - id:"+inPersonAuthenticationDTO.getId());
			}
		} catch (Exception e) {
			log.error("Error in getting patinet details from getAuthenticationForPatient() " + patientId, e);
			throw new MHVRuntimeException(
					"Error in getting patinet details from getAuthenticationForPatient() " + patientId);
		}

		return inPersonAuthenticationDTO;
	}

	@Override
	@POST
	@Path("/forceUnauthenticateUnCorrelateWithAdminUserName/{ipaId}/{adminUserName}")
	@Consumes(MediaType.APPLICATION_JSON)
	@Produces(MediaType.APPLICATION_JSON)
	public InPersonAuthenticationDTO forceUnauthenticateUnCorrelateWithAdminUserName(@PathParam("ipaId") Long ipaId,
			@PathParam("adminUserName") String adminUserName) throws MHVException {
		ResponseUtil<Patient> response = new ResponseUtil();
		InPersonAuthentication ipa = null;
		try {
			ipa = inpersonAuthenticationRepository.findOne(ipaId);
			//AuthenticationStatusEnumeration status = AuthenticationStatusEnumeration.UNAUTHENTICATED;
			ipa.setParticipationFormSigned(Boolean.FALSE);
			ipa.setIdentificationPresented(Boolean.FALSE);
			ipa.setApprovedForRecordsAccess(Boolean.FALSE);
			ipa.setDefermentReason(null);
			ipa.setAuthenticatingFacility(null);
			ipa.setAuthenticationDate(null);

			// Jazz Id: 230989 - adding HL7v3 Version code in MVI Request
			// Header.
			mviService.mviAuthenticateWithVersion(patientConverter.convertDomainPatient(ipa.getPatient()),
					userProfileConverter.convertDomainUserProfile(ipa.getPatient().getUserProfile()), false, false,
					adminUserName);

			ipa.setMviAuthenticationStatus(null);
			ipa = saveIPA(ipa, "Unauthenticated");
			System.out.println(
					"&&&&&&&&&&&&&&&&&&&& InPersonAuthenticationServiceImpl - Unauthenticated &&&&&&&&&&&&&& ");
			mviService.forceUncorrelation(ipa.getPatient().getId(), ipa.getPatient().getUserProfile().getId(),
					adminUserName, response);
			
		} catch (Exception e) {
			log.error("Error in getting patinet details from getAuthenticationForPatient() ");
			throw new MHVRuntimeException("Error in getting patinet details from getAuthenticationForPatient() ");
		}
		
		return converter.convertDomainInpersonAuthentication(ipa);
	}

	@Override
	@GET
	@Path("/getAuthenticationForPatient/{patientId}")
	@Produces(MediaType.APPLICATION_JSON)
	public InPersonAuthenticationDTO getAuthenticationForPatient(@PathParam("patientId") Long patientId)
			throws MHVException {
		InPersonAuthenticationDTO response = null;
		InPersonAuthentication ipa = null;
		try {
			System.out.println(
					">>>> InPersonAuthenticationServiceImpl - getAuthenticationForPatient - patient:" + patientId);
			List<InPersonAuthentication> bos = inpersonAuthenticationRepository.getAuthenticationForPatient(patientId);
			if (!bos.isEmpty()) {
				System.out.println(">>>> InPersonAuthenticationServiceImpl - getAuthenticationForPatient - inside if");
				ipa = bos.get(0);
				System.out.println(">>>> InPersonAuthenticationServiceImpl - getAuthenticationForPatient - inside if - ipaId:"+ ipa.getId());
			} else {
				//TODO: Verify the functionality in the legacy code where there is no IPA record for the patient
				System.out
						.println(">>>> InPersonAuthenticationServiceImpl - getAuthenticationForPatient - inside else");
				InPersonAuthentication ipa1 = new InPersonAuthentication();
				ipa1.setStatus("Unauthenticated");
				ipa1.setVideoViewed(Boolean.FALSE);
				ipa1.setParticipationFormSigned(Boolean.FALSE);
				ipa1.setIdentificationPresented(Boolean.FALSE);
				ipa1.setApprovedForRecordsAccess(Boolean.FALSE);
				Patient patient = new Patient();
				patient.setId(patientId);
				ipa1.setPatient(patient);
				
				 
				ipa = saveIPA(ipa1, "Unauthenticated");
				// enterInfoLog("getAuthenticationForPatient():creating new
				// IPA", "ipa id:",
				// response.getInPersonAuthentication().getId()+"");
			}
			System.out.println(">>>> InPersonAuthenticationServiceImpl - getAuthenticationForPatient -before conversion :"+ ipa.getPatientId()+"::"+ipa.getPatient().getId());
			response = converter.convertDomainInpersonAuthentication(ipa);
			converter.addPatientDto(ipa, response);
			converter.addFacilityInfoDto(ipa, response);
			System.out.println(">>>> InPersonAuthenticationServiceImpl - getAuthenticationForPatient -after conversion :"+response.getPatient().getId());			
		} catch (Exception e) {
			log.error("Error in getting patinet details from getAuthenticationForPatient() " + patientId, e);
			throw new MHVRuntimeException(
					"Error in getting patinet details from getAuthenticationForPatient() " + patientId);
		}

		//return converter.convertDomainInpersonAuthentication(ipa);
		return response;
	}
	
	@Transactional
    private InPersonAuthentication saveIPA(InPersonAuthentication ipa,
            String status)
        {	
			//ipa.setMviAuthenticationStatus(status);
			ipa.setStatus(status);
			System.out.println(
					"&&&&&&&&&&&&&&&&&&&& InPersonAuthenticationServiceImpl - Unauthenticated &&&&&&&&&&&&&& " + status);
            ipa = inpersonAuthenticationRepository.save(ipa);
            return ipa;
        }

	
	@Override
	@POST
	@Path("/deferAuthentication")
	@Consumes(MediaType.APPLICATION_JSON)
	@Produces(MediaType.APPLICATION_JSON)
	@Transactional
	public ResponseUtil<List<InPersonAuthenticationDTO>> deferAuthentication(InPersonAuthenticationDTO ipaDTO)
			throws MHVException {
		ResponseUtil<List<InPersonAuthenticationDTO>> response = new ResponseUtil<List<InPersonAuthenticationDTO>>();
		try {
			System.out.println(">>>> InPersonAuthenticationServiceImpl - deferAuthentication - ipaDTO.getId(): " + ipaDTO.getId());
			InPersonAuthentication ipa = converter.convertInpersonAuthenticationDTO(ipaDTO);
			converter.addPatientProfile(ipaDTO, ipa);
			converter.addFacilityInfo(ipaDTO, ipa);
			
			System.out.println(">>>> InPersonAuthenticationServiceImpl - deferAuthentication - After conversion - ipa.getId(): " + ipa.getId());			

			Boolean formSigned = ipa.getParticipationFormSigned() == null ? Boolean.FALSE : ipa.getParticipationFormSigned();
			Boolean idPresented = ipa.getIdentificationPresented() == null ? Boolean.FALSE : ipa.getIdentificationPresented();
			AuthenticationStatusEnumeration status = null;
			
			System.out.println(">>>> InPersonAuthenticationServiceImpl - deferAuthentication - formSigned: " + formSigned);
			System.out.println(">>>> InPersonAuthenticationServiceImpl - deferAuthentication - idPresented: " + idPresented);
			
			System.out.println(">>>> InPersonAuthenticationServiceImpl - deferAuthentication - ipa.getPatient().getId():" + ipa.getPatient().getId());
			System.out.println(">>>> InPersonAuthenticationServiceImpl - deferAuthentication - ipa.getPatientId():" + ipa.getPatientId());

			System.out.println(">>>> InPersonAuthenticationServiceImpl - deferAuthentication - ipa.getAuthenticatingFacility().getId():" + ipa.getAuthenticatingFacility().getId());
			System.out.println(">>>> InPersonAuthenticationServiceImpl - deferAuthentication - ipa.getAuthenticatingFacilityId():" + ipa.getAuthenticatingFacilityId());				
									
			if (formSigned.booleanValue() && idPresented.booleanValue()) {
				status = AuthenticationStatusEnumeration.PREREQUISITESCOMPLETE;
			} else {
				status = AuthenticationStatusEnumeration.INPROCESS;
			}
			ipa.setRemovalReason(null);
			ipa.setStatus(status.getDescription());		
			
			System.out.println(">>>> InPersonAuthenticationServiceImpl - deferAuthentication - Just before saving - ipa.getId(): " + ipa.getId());
			ipa = inpersonAuthenticationRepository.save(ipa);
			System.out.println(">>>> InPersonAuthenticationServiceImpl - deferAuthentication - Just After saving - ipa.getId(): " + ipa.getId());

			List<InPersonAuthenticationDTO> ipas = new ArrayList<InPersonAuthenticationDTO>();
			ipas.add(converter.convertDomainInpersonAuthentication(ipa));
			response.setPojoObject(ipas);
			response.setFailure(false);
			response.setSuccess(true);
			response.setFailureMessage("");
			response.setSuccessMessage("The selected prerequisite information has been successfully saved.");

			if (null == response.getValidationErrors()) {
				LinkedHashMap<String, String> validationErrors = new LinkedHashMap<String, String>();
				validationErrors.put("deferAuthentication", "There are no Validation Errors.");
				response.setValidationErrors(validationErrors);
			} else {
				response.getValidationErrors().put("deferAuthentication", "There are no Validation Errors.");
			}

			System.out.println(">>>> InPersonAuthenticationServiceImpl - deferAuthentication - set ResponseUtil - 6.");

			if (null == response.getInfoMessages()) {
				LinkedHashMap<String, String> infoMessages = new LinkedHashMap<String, String>();
				infoMessages.put("deferAuthentication","The selected prerequisite information has been successfully saved.");
				response.setInfoMessages(infoMessages);
			} else {
				response.getInfoMessages().put("deferAuthentication", "The selected prerequisite information has been successfully saved.");
			}

			System.out.println(">>>> InPersonAuthenticationServiceImpl - deferAuthentication - returning ResponseUtil<List<InPersonAuthenticationDTO>> object.");
			return response;

		} catch (Exception e) {
			log.error(">>>> Error in InPersonAuthenticationServiceImpl - deferAuthentication - Error during Defer Authentication." + e);
			System.out.println(">>>> Error in InPersonAuthenticationServiceImpl - deferAuthentication - Error during Defer Authentication.");
			throw new MHVRuntimeException(">>>> Error in InPersonAuthenticationServiceImpl - deferAuthentication - Error during Defer Authentication.");
		}
	}
	
	@Override
	@POST
	@Path("/match/{adminUserName}")
	@Consumes(MediaType.APPLICATION_JSON)
	@Produces(MediaType.APPLICATION_JSON)
	@Transactional
	public ResponseUtil<List<InPersonAuthenticationDTO>> match(InPersonAuthenticationDTO ipaDTO, @PathParam("adminUserName") String adminUserName) throws MHVException {
		ResponseUtil<List<InPersonAuthenticationDTO>> response = new ResponseUtil<List<InPersonAuthenticationDTO>>();
		ResponseUtil<PatientDTO> responseRt = new ResponseUtil<PatientDTO>();		
		try {
			System.out.println(">>>> InPersonAuthenticationServiceImpl - match - ipaDTO.getId(): " + ipaDTO.getId());
			InPersonAuthentication ipa = converter.convertInpersonAuthenticationDTO(ipaDTO);
			System.out.println(">>>> InPersonAuthenticationServiceImpl - match - 0a");			
			converter.addPatientProfile(ipaDTO, ipa);
			System.out.println(">>>> InPersonAuthenticationServiceImpl - match - 0b");			
			//converter.addFacilityInfo(ipaDTO, ipa);
			System.out.println(">>>> InPersonAuthenticationServiceImpl - match - 0c");
			
			System.out.println(">>>> InPersonAuthenticationServiceImpl - match - ipaDTO.getPatient().getId(): " + ipaDTO.getPatient().getId());
			System.out.println(">>>> InPersonAuthenticationServiceImpl - match - Calling MVI Service for Match.");
			
			// TODO should change the return to responseUtil<PatientDTO>
			responseRt = mviService.matchPersonInIPAWithVersion(userProfileConverter.convertDomainUserProfile(ipa.getPatient().getUserProfile()), adminUserName);

			System.out.println(">>>> InPersonAuthenticationServiceImpl - match - after the call - 1");	
			
			PatientDTO patientDTO = (PatientDTO) response.getPojoObject();
			Patient patient = patientConverter.convertDTOProfile(patientDTO);
			
			/*
			if(patient != null) {
				//currentIPA.setPatient(patient);
				setSelectedUserProfileAccountStatus(getAccountStatus(patient));
			}
			
			
			// updateSearchResults();
			// determineMatchStatus();
			// determineDisplayButtons();
						
			*/
						
			List<InPersonAuthenticationDTO> ipas = new ArrayList<InPersonAuthenticationDTO>();
			ipas.add(converter.convertDomainInpersonAuthentication(ipa));
			response.setPojoObject(ipas);

			System.out.println(">>>> InPersonAuthenticationServiceImpl - match - after the call - 2");			
			
			if (responseRt.isFailure()) {
				response.setFailure(true);
				response.setSuccess(false);
				response.setFailureMessage(responseRt.getFailureMessage()); 	
				
				ActivityActorTypeEnumeration authenticator = ActivityActorTypeEnumeration.MHV_AUTHENTICATOR;
				if (adminUserName != null && adminUserName.equals("SYSTEM")) {
					authenticator = ActivityActorTypeEnumeration.SYSTEM;
				}
				activityService.auditMviEvent(userProfileConverter.convertDomainUserProfile(ipa.getPatient().getUserProfile()), authenticator,
						false, ActivityActionTypeEnumeration.UPGRADE_TO_ADVANCED, ActivityTypeEnumeration.ACCOUNT_UPGRADE, response);
				
				System.out.println(">>>> InPersonAuthenticationServiceImpl - match - after the call - 3");				
			}
			else {
				response.setSuccess(true);
				response.setFailure(false);
				response.setFailureMessage("");
				response.setSuccessMessage("The Match process for MHV User "+ ipa.getPatient().getUserProfile().getFirstName()+" "+ ipa.getPatient().getUserProfile().getLastName()+" has been successfully completed.");

				ActivityActorTypeEnumeration authenticator = ActivityActorTypeEnumeration.MHV_AUTHENTICATOR;
				if (adminUserName != null && adminUserName.equals("SYSTEM")) {
					authenticator = ActivityActorTypeEnumeration.SYSTEM;
				}
				activityService.auditMviEvent(userProfileConverter.convertDomainUserProfile(ipa.getPatient().getUserProfile()), authenticator,
						true, ActivityActionTypeEnumeration.UPGRADE_TO_ADVANCED, ActivityTypeEnumeration.ACCOUNT_UPGRADE, response);
				
				System.out.println(">>>> InPersonAuthenticationServiceImpl - match - after the call - 3a");				
			}
			
			if (null == responseRt.getValidationErrors()) {
				LinkedHashMap<String, String> validationErrors = new LinkedHashMap<String, String>();
				validationErrors.put("match", "There are no Validation Errors.");
				response.setValidationErrors(validationErrors);
			} else {
				response.setValidationErrors(responseRt.getValidationErrors());
			}
			System.out.println(">>>> InPersonAuthenticationServiceImpl - match - after the call - 4");
			if (null == responseRt.getInfoMessages()) {
				LinkedHashMap<String, String> infoMessages = new LinkedHashMap<String, String>();
				infoMessages.put("match","The Match process for MHV User "+ ipa.getPatient().getUserProfile().getFirstName()+" "+ ipa.getPatient().getUserProfile().getLastName()+" has been successfully completed.");
				response.setInfoMessages(infoMessages);
			} else {
				response.setInfoMessages(responseRt.getInfoMessages());
			}
			System.out.println(">>>> InPersonAuthenticationServiceImpl - match - after the call - 5");

			return response;
		} catch (Exception e) {
			log.error(">>>> Error in InPersonAuthenticationServiceImpl - match - Error during Match Process." + e);
			System.out.println(">>>> Error in InPersonAuthenticationServiceImpl - match - Error during Match Process.");
			throw new MHVRuntimeException(">>>> Error in InPersonAuthenticationServiceImpl - match - Error during Match Process.");
		}
	}	
	

	@Override
	@POST
	@Path("/syncCorrelate/{adminUserName}")
	@Consumes(MediaType.APPLICATION_JSON)
	@Produces(MediaType.APPLICATION_JSON)
	@Transactional
	public ResponseUtil<List<InPersonAuthenticationDTO>> syncCorrelate(InPersonAuthenticationDTO ipaDTO, @PathParam("adminUserName") String adminUserName) throws MHVException {
		ResponseUtil<List<InPersonAuthenticationDTO>> response = new ResponseUtil<List<InPersonAuthenticationDTO>>();
		ResponseUtil<PatientDTO> responseRt = new ResponseUtil<PatientDTO>();		
		try {
			System.out.println(">>>> InPersonAuthenticationServiceImpl - syncCorrelate - ipaDTO.getId(): " + ipaDTO.getId());
			InPersonAuthentication ipa = converter.convertInpersonAuthenticationDTO(ipaDTO);
			System.out.println(">>>> InPersonAuthenticationServiceImpl - syncCorrelate - 0a");			
			converter.addPatientProfile(ipaDTO, ipa);
			System.out.println(">>>> InPersonAuthenticationServiceImpl - syncCorrelate - 0b");			
			//converter.addFacilityInfo(ipaDTO, ipa);
			System.out.println(">>>> InPersonAuthenticationServiceImpl - syncCorrelate - 0c");
			
			System.out.println(">>>> InPersonAuthenticationServiceImpl - syncCorrelate - ipaDTO.getPatient().getId(): " + ipaDTO.getPatient().getId());
			System.out.println(">>>> InPersonAuthenticationServiceImpl - syncCorrelate - Calling MVI Service for syncCorrelate.");
			
			PatientDTO patientDTO = patientConverter.convertDomainPatient(ipa.getPatient());
			
			// TODO should change the return to responseUtil<PatientDTO>
			responseRt = mviService.addPersonForCorrelationForIPAWithVersion(patientDTO, userProfileConverter.convertDomainUserProfile(ipa.getPatient().getUserProfile()), adminUserName);

			System.out.println(">>>> InPersonAuthenticationServiceImpl - syncCorrelate - after the call - 1");	
			
			Patient patient = patientConverter.convertDTOProfile(patientDTO);
						
			List<InPersonAuthenticationDTO> ipas = new ArrayList<InPersonAuthenticationDTO>();
			ipas.add(converter.convertDomainInpersonAuthentication(ipa));
			response.setPojoObject(ipas);

			System.out.println(">>>> InPersonAuthenticationServiceImpl - syncCorrelate - after the call - 2");			
			
			if (responseRt.isFailure()) {
				response.setFailure(true);
				response.setSuccess(false);
				response.setFailureMessage(responseRt.getFailureMessage()); 	
				
				ActivityActorTypeEnumeration authenticator = ActivityActorTypeEnumeration.MHV_AUTHENTICATOR;
				if (adminUserName != null && adminUserName.equals("SYSTEM")) {
					authenticator = ActivityActorTypeEnumeration.SYSTEM;
				}
				activityService.auditMviEvent(userProfileConverter.convertDomainUserProfile(ipa.getPatient().getUserProfile()), authenticator,
						false, ActivityActionTypeEnumeration.UPGRADE_TO_ADVANCED, ActivityTypeEnumeration.ACCOUNT_UPGRADE, response);
				
				System.out.println(">>>> InPersonAuthenticationServiceImpl - syncCorrelate - after the call - 3");				
			}
			else {
				response.setSuccess(true);
				response.setFailure(false);
				response.setFailureMessage("");
				response.setSuccessMessage("Correlation was successful for MHV User "+ ipa.getPatient().getUserProfile().getFirstName()+" "+ ipa.getPatient().getUserProfile().getLastName());

				ActivityActorTypeEnumeration authenticator = ActivityActorTypeEnumeration.MHV_AUTHENTICATOR;
				if (adminUserName != null && adminUserName.equals("SYSTEM")) {
					authenticator = ActivityActorTypeEnumeration.SYSTEM;
				}
				activityService.auditMviEvent(userProfileConverter.convertDomainUserProfile(ipa.getPatient().getUserProfile()), authenticator,
						true, ActivityActionTypeEnumeration.UPGRADE_TO_ADVANCED, ActivityTypeEnumeration.ACCOUNT_UPGRADE, response);
				
				System.out.println(">>>> InPersonAuthenticationServiceImpl - syncCorrelate - after the call - 3a");				
			}
			
			if (null == responseRt.getValidationErrors()) {
				LinkedHashMap<String, String> validationErrors = new LinkedHashMap<String, String>();
				validationErrors.put("syncCorrelate", "There are no Validation Errors.");
				response.setValidationErrors(validationErrors);
			} else {
				response.setValidationErrors(responseRt.getValidationErrors());
			}
			System.out.println(">>>> InPersonAuthenticationServiceImpl - syncCorrelate - after the call - 4");
			if (responseRt.getInfoMessages().size() == 0) {
				LinkedHashMap<String, String> infoMessages = new LinkedHashMap<String, String>();
				infoMessages.put(messagesUtil.getMviIpaCorrelationSuccess(), String.format(messagesUtil.getMviIpaCorrelationSuccess(), ipa.getPatient().getUserProfile().getFirstName()+" "+ ipa.getPatient().getUserProfile().getLastName()));							
				response.setInfoMessages(infoMessages);
				System.out.println(">>>> InPersonAuthenticationServiceImpl - syncCorrelate - after the call - 4a");
			} else {
				response.setInfoMessages(responseRt.getInfoMessages());
				System.out.println(">>>> InPersonAuthenticationServiceImpl - syncCorrelate - after the call - 4b - Count:"+ responseRt.getInfoMessages().size());
			}
			System.out.println(">>>> InPersonAuthenticationServiceImpl - syncCorrelate - after the call - 5");

			return response;
		} catch (Exception e) {
			log.error(">>>> Error in InPersonAuthenticationServiceImpl - syncCorrelate - Error during Correlattion Process." + e);
			System.out.println(">>>> Error in InPersonAuthenticationServiceImpl - syncCorrelate - Error during Correlattion Process.");
			throw new MHVRuntimeException(">>>> Error in InPersonAuthenticationServiceImpl - syncCorrelate - Error during Correlattion Process.");
		}
	}	
	
	
	@Override
	@POST
	@Path("/terminateAuthentication")
	@Consumes(MediaType.APPLICATION_JSON)
	@Produces(MediaType.APPLICATION_JSON)
	@Transactional
	public ResponseUtil<List<InPersonAuthenticationDTO>> terminateAuthentication(InPersonAuthenticationDTO ipaDTO)
			throws MHVException {
		ResponseUtil<List<InPersonAuthenticationDTO>> response = new ResponseUtil<List<InPersonAuthenticationDTO>>();
		try {
			System.out.println(">>>> InPersonAuthenticationServiceImpl - terminateAuthentication - ipaDTO.getId(): " + ipaDTO.getId());
			InPersonAuthentication ipa = converter.convertInpersonAuthenticationDTO(ipaDTO);
			converter.addPatientProfile(ipaDTO, ipa);
			converter.addFacilityInfo(ipaDTO, ipa);
			
			AuthenticationStatusEnumeration status = null;			
			status = AuthenticationStatusEnumeration.UNAUTHENTICATED;
			ipa.setStatus(status.getDescription());
			
	        ipa.setParticipationFormSigned(Boolean.FALSE);
	        ipa.setIdentificationPresented(Boolean.FALSE);
	        ipa.setApprovedForRecordsAccess(Boolean.FALSE);
	        ipa.setDefermentReason(null);
	        ipa.setAuthenticatingFacility(null);
	        ipa.setRemovalReason(null);
	        ipa.setAuthenticationDate(null);
	        ipa.setAuthenticatedBy(null);
	        ipa.setMviAuthenticationStatus(null);		
			
			ipa = inpersonAuthenticationRepository.save(ipa);

			List<InPersonAuthenticationDTO> ipas = new ArrayList<InPersonAuthenticationDTO>();
			ipas.add(converter.convertDomainInpersonAuthentication(ipa));
			response.setPojoObject(ipas);
			response.setFailure(false);
			response.setSuccess(true);
			response.setFailureMessage("");
			response.setSuccessMessage("The Authentication process for MHV User "+ ipa.getPatient().getUserProfile().getFirstName()+" "+ ipa.getPatient().getUserProfile().getMiddleName()+" "+ipa.getPatient().getUserProfile().getLastName()+" has been successfully terminated.");

			if (null == response.getValidationErrors()) {
				LinkedHashMap<String, String> validationErrors = new LinkedHashMap<String, String>();
				validationErrors.put("terminateAuthentication", "There are no Validation Errors.");
				response.setValidationErrors(validationErrors);
			} else {
				response.getValidationErrors().put("terminateAuthentication", "There are no Validation Errors.");
			}

			if (null == response.getInfoMessages()) {
				LinkedHashMap<String, String> infoMessages = new LinkedHashMap<String, String>();
				infoMessages.put("terminateAuthentication","The Authentication process for MHV User "+ ipa.getPatient().getUserProfile().getFirstName()+" "+ ipa.getPatient().getUserProfile().getMiddleName()+" "+ ipa.getPatient().getUserProfile().getLastName()+" has been successfully terminated.");
				response.setInfoMessages(infoMessages);
			} else {
				response.getInfoMessages().put("terminateAuthentication", "The Authentication process for MHV User "+ ipa.getPatient().getUserProfile().getFirstName()+" "+ ipa.getPatient().getUserProfile().getMiddleName()+" "+ ipa.getPatient().getUserProfile().getLastName()+" has been successfully terminated.");
			}

			System.out.println(">>>> InPersonAuthenticationServiceImpl - terminateAuthentication - returning ResponseUtil<List<InPersonAuthenticationDTO>> object.");
			return response;

		} catch (Exception e) {
			log.error(">>>> Error in InPersonAuthenticationServiceImpl - terminateAuthentication - Error during Terminate Authentication." + e);
			System.out.println(">>>> Error in InPersonAuthenticationServiceImpl - terminateAuthentication - Error during Terminate Authentication.");
			throw new MHVRuntimeException(">>>> Error in InPersonAuthenticationServiceImpl - terminateAuthentication - Error during Terminate Authentication.");
		}
	}	

	@Override
	@POST
	@Path("/authenticateWithAdminUserName/{ipaId}/{adminUserName}/{authenticatingFacilityId}")
	@Consumes(MediaType.APPLICATION_JSON)
	@Produces(MediaType.APPLICATION_JSON)
	@Transactional
	public ResponseUtil<List<InPersonAuthenticationDTO>> authenticateWithAdminUserName(@PathParam("ipaId") Long ipaId,
			@PathParam("adminUserName") String adminUserName, @PathParam("delayReason") Long authenticatingFacilityId) throws MHVException {
		try {
			System.out.println(
					">>>> InPersonAuthenticationServiceImpl - authenticateWithAdminUserName - ipaId: " + ipaId);
			InPersonAuthentication ipa = inpersonAuthenticationRepository.findOne(ipaId);
			return mviAuthenticate(ipa, adminUserName, authenticatingFacilityId);
		} catch (Exception e) {
			log.error(">>>> Error in InPersonAuthenticationServiceImpl - authenticateWithAdminUserName - Error during Authentication." + e);
			System.out.println(">>>> Error in InPersonAuthenticationServiceImpl - authenticateWithAdminUserName - Error during Authentication.");
			throw new MHVRuntimeException(">>>> Error in InPersonAuthenticationServiceImpl - authenticateWithAdminUserName - Error during Authentication.");
		}
	}

	private ResponseUtil<List<InPersonAuthenticationDTO>> mviAuthenticate(InPersonAuthentication ipa, String adminUserName, Long authenticatingFacilityId) {
		ResponseUtil<List<InPersonAuthenticationDTO>> response = new ResponseUtil<List<InPersonAuthenticationDTO>>();
		ResponseUtil<PatientDTO> responseRt = new ResponseUtil<PatientDTO>();		
		try {
			System.out.println(">>>> InPersonAuthenticationServiceImpl - mviAuthenticate - ipaId: " + ipa.getId());
			AuthenticationStatusEnumeration status = AuthenticationStatusEnumeration.AUTHENTICATED;
			ipa.setParticipationFormSigned(Boolean.TRUE);
			ipa.setIdentificationPresented(Boolean.TRUE);
			ipa.setApprovedForRecordsAccess(Boolean.TRUE);
			ipa.setDefermentReason(null);
			ipa.setAuthenticatingFacilityId(authenticatingFacilityId);
			ipa.setAuthenticatedBy(getEmployeeName(adminUserName));  // e.g. Hall, Lisa (vhaiswhalll)
			ipa.setAuthenticationDate(new Timestamp(new Date().getTime()));
			ipa.setStatus(status.getDescription());

			System.out.println(">>>> InPersonAuthenticationServiceImpl - mviAuthenticate - Inside else block.. Calling mviService.mviAuthenticateWithVersion");			
			responseRt = mviService.mviAuthenticateWithVersion(patientConverter.convertDomainPatient(ipa.getPatient()),
						userProfileConverter.convertDomainUserProfile(ipa.getPatient().getUserProfile()), true, false, adminUserName);

			// Boolean mviServiceResponse = response.getFailure();
			if (!responseRt.isFailure()) {
				if (log.isDebugEnabled()) {
					log.debug(">>>> InPersonAuthenticationServiceImpl - mviAuthenticate - SUCCESS unauthenticating patient with ICN:"+ ipa.getPatient().getIcn());
				}
				System.out.println(">>>> InPersonAuthenticationServiceImpl - mviAuthenticate - SUCCESS unauthenticating patient with ICN:"+ ipa.getPatient().getIcn());
				ipa.setMviAuthenticationStatus("OK");
				response.setSuccess(true);
				response.setFailure(false);
				response.setSuccessMessage(responseRt.getSuccessMessage());
				ActivityActorTypeEnumeration authenticator = ActivityActorTypeEnumeration.MHV_AUTHENTICATOR;
				if (adminUserName != null && adminUserName.equals("SYSTEM")) {
					authenticator = ActivityActorTypeEnumeration.SYSTEM;
				}
				activityService.auditMviEvent(userProfileConverter.convertDomainUserProfile(ipa.getPatient().getUserProfile()), authenticator,
						true, ActivityActionTypeEnumeration.UPGRADE_TO_PREMIUM, ActivityTypeEnumeration.ACCOUNT_UPGRADE, response);					
								
			} else {
				if (log.isDebugEnabled()) {
					log.debug(">>>> InPersonAuthenticationServiceImpl - mviAuthenticate - ERROR unauthenticating patient with ICN:"+ ipa.getPatient().getIcn());
				}
				System.out.println(">>>> InPersonAuthenticationServiceImpl - mviAuthenticate - ERROR unauthenticating patient with ICN:"+ ipa.getPatient().getIcn());
				response.setFailure(true);
				response.setSuccess(false);
				response.setFailureMessage(responseRt.getFailureMessage());

				ActivityActorTypeEnumeration authenticator = ActivityActorTypeEnumeration.MHV_AUTHENTICATOR;
				if (adminUserName != null && adminUserName.equals("SYSTEM")) {
					authenticator = ActivityActorTypeEnumeration.SYSTEM;
				}
				activityService.auditMviEvent(userProfileConverter.convertDomainUserProfile(ipa.getPatient().getUserProfile()), authenticator,
						false, ActivityActionTypeEnumeration.UPGRADE_TO_PREMIUM, ActivityTypeEnumeration.ACCOUNT_UPGRADE, response);					
				
			}
			ipa = inpersonAuthenticationRepository.save(ipa);
			List<InPersonAuthenticationDTO> ipas = new ArrayList<InPersonAuthenticationDTO>();
			ipas.add(converter.convertDomainInpersonAuthentication(ipa));
			response.setPojoObject(ipas);

			if (null == responseRt.getValidationErrors()) {
				LinkedHashMap<String, String> validationErrors = new LinkedHashMap<String, String>();
				validationErrors.put("mviAuthenticate", "There are no Validation Errors.");
				response.setValidationErrors(validationErrors);
			} else {
				response.setValidationErrors(responseRt.getValidationErrors());
			}

			if (null == responseRt.getInfoMessages()) {
				LinkedHashMap<String, String> infoMessages = new LinkedHashMap<String, String>();
				//infoMessages.put("mviAuthenticate", "Authentication was successful for MHV User "+ ipa.getPatient().getUserProfile().getFirstName()+" "+ ipa.getPatient().getUserProfile().getLastName()+".");
				response.setInfoMessages(infoMessages);
			} else {
				response.setInfoMessages(responseRt.getInfoMessages());
			}

			System.out.println(">>>> InPersonAuthenticationServiceImpl - mviAuthenticate - returning ResponseUtil<List<InPersonAuthenticationDTO>> object.");
			return response;
		} catch (Exception e) {
			log.error(">>>> Error in InPersonAuthenticationServiceImpl - mviAuthenticate - Error during Authentication. "+ e);
			System.out.println(">>>> Error in InPersonAuthenticationServiceImpl - mviAuthenticate - Error during Authentication.");
			throw new MHVRuntimeException(">>>> Error in InPersonAuthenticationServiceImpl - mviAuthenticate - Error during Authentication.");
		}
	}

	@Override
	@POST
	@Path("/unauthenticateWithAdminUserName/{ipaId}/{adminUserName}/{removeAuthenticatingReason}")
	@Consumes(MediaType.APPLICATION_JSON)
	@Produces(MediaType.APPLICATION_JSON)
	@Transactional
	public ResponseUtil<List<InPersonAuthenticationDTO>> unauthenticateWithAdminUserName(@PathParam("ipaId") Long ipaId,
			@PathParam("adminUserName") String adminUserName, @PathParam("removeAuthenticatingReason") String removeAuthenticatingReason) throws MHVException {
		try {
			System.out.println(
					">>>> InPersonAuthenticationServiceImpl - unauthenticateWithAdminUserName - ipaId:" + ipaId + " - removeAuthenticatingReason:"+removeAuthenticatingReason);
			InPersonAuthentication ipa = inpersonAuthenticationRepository.findOne(ipaId);
			return mviUnauthenticate(ipa, adminUserName, removeAuthenticatingReason);
		} catch (Exception e) {
			log.error(">>>> Error in InPersonAuthenticationServiceImpl - unauthenticateWithAdminUserName - Error during Remove Authentication." + e);
			System.out.println(">>>> Error in InPersonAuthenticationServiceImpl - unauthenticateWithAdminUserName - Error during Remove Authentication.");
			throw new MHVRuntimeException(">>>> Error in InPersonAuthenticationServiceImpl - unauthenticateWithAdminUserName - Error during Remove Authentication.");
		}
	}

	private ResponseUtil<List<InPersonAuthenticationDTO>> mviUnauthenticate(InPersonAuthentication ipa, String adminUserName, String removeAuthenticatingReason) {
		ResponseUtil<List<InPersonAuthenticationDTO>> response = new ResponseUtil<List<InPersonAuthenticationDTO>>();
		ResponseUtil<PatientDTO> responseRt = new ResponseUtil<PatientDTO>();
		try {
			System.out.println(">>>> InPersonAuthenticationServiceImpl - mviUnauthenticate - ipaId:" + ipa.getId());
			AuthenticationStatusEnumeration status = AuthenticationStatusEnumeration.UNAUTHENTICATED;
			ipa.setParticipationFormSigned(Boolean.FALSE);
			ipa.setIdentificationPresented(Boolean.FALSE);
			ipa.setApprovedForRecordsAccess(Boolean.FALSE);
			ipa.setRemovalReason(removeAuthenticatingReason);
			ipa.setDefermentReason(null);
			ipa.setAuthenticatingFacility(null);
			ipa.setAuthenticationDate(null);
			ipa.setStatus(status.getDescription());
			System.out.println(">>>> InPersonAuthenticationServiceImpl - mviUnauthenticate - status:" + status.getDescription());
			// JAZZ: Task#19823 - When a IPA user unauthenticates a user that is not matched and/or
			// Correlated, then the system will not send the unauth flags to MVI since the user is not correlated
			Patient patient = ipa.getPatient();
			System.out.println(">>>> InPersonAuthenticationServiceImpl - mviUnauthenticate - patient:" + patient.getId());
			if (patient.getCorrelationStatus() == PatientCorrelationStatusEnumeration.UNCORRELATED.getValue()
					|| patient.getCorrelationStatus() == PatientCorrelationStatusEnumeration.MVIDATAMATCH.getValue()) {
				ipa.setMviAuthenticationStatus("OK");
				System.out.println(">>>> InPersonAuthenticationServiceImpl - mviUnauthenticate - Inside if block");
				patient.setPatientSynchronizations(null);
				patientRepository.save(patient);
			} else {
				// MHV_CodeCR1514 - US12.4 MVI Compliance Implementation - If the MVI flag is turned on call, MVI call instead of MPI call
				ipa.setMviAuthenticationStatus("PENDING_UNAUTH");
				// Jazz Id: 230989 - adding HL7v3 Version code in MVI Request Header.
				// For now skipped calling the patient service. Calling directly MviIntegrationService
				// patientServiceResponse = ServiceFactory.createPatientService().mviUnauthenticateWithAdminUserName(ipa.getPatient(), adminUserName);
				System.out.println(">>>> InPersonAuthenticationServiceImpl - mviUnauthenticate - Inside else block.. Calling mviService.mviAuthenticateWithVersion");
				responseRt = mviService.mviAuthenticateWithVersion(patientConverter.convertDomainPatient(ipa.getPatient()),
						userProfileConverter.convertDomainUserProfile(ipa.getPatient().getUserProfile()), false, false, adminUserName);

				// Boolean mviServiceResponse = response.getFailure();
				if (!responseRt.isFailure()) {
					if (log.isDebugEnabled()) {
						log.debug(">>>> InPersonAuthenticationServiceImpl - mviUnauthenticate - SUCCESS unauthenticating patient with ICN:"+ ipa.getPatient().getIcn());
					}
					System.out.println(">>>> InPersonAuthenticationServiceImpl - mviUnauthenticate - SUCCESS unauthenticating patient with ICN:"+ ipa.getPatient().getIcn());
					// JAZZ: Task#19820 - Remove un-correlation call when user’s account is unauthenticated
					ipa.setMviAuthenticationStatus("OK");
					// MHV_CodeCR1918 - US12.4.x MVI Compliance Implementation - If the patient status is correleated or pending corr/uncorr try to uncorrelate. Only
					response.setSuccess(true);
					response.setFailure(false);
					response.setSuccessMessage(responseRt.getSuccessMessage());

					ActivityActorTypeEnumeration authenticator = ActivityActorTypeEnumeration.MHV_AUTHENTICATOR;
					if (adminUserName != null && adminUserName.equals("SYSTEM")) {
						authenticator = ActivityActorTypeEnumeration.SYSTEM;
					}
					activityService.auditMviEvent(userProfileConverter.convertDomainUserProfile(ipa.getPatient().getUserProfile()), authenticator,
							true, ActivityActionTypeEnumeration.PREMIUM_ACCOUNT_REVOKED, ActivityTypeEnumeration.ACCOUNT_UPGRADE, response);					
					
				} else {
					if (log.isDebugEnabled()) {
						log.debug(">>>> InPersonAuthenticationServiceImpl - mviUnauthenticate - ERROR unauthenticating patient with ICN:"+ ipa.getPatient().getIcn());
					}
					System.out.println(">>>> InPersonAuthenticationServiceImpl - mviUnauthenticate - ERROR unauthenticating patient with ICN:"+ ipa.getPatient().getIcn());
					response.setFailure(true);
					response.setSuccess(false);
					response.setFailureMessage(responseRt.getFailureMessage());

					ActivityActorTypeEnumeration authenticator = ActivityActorTypeEnumeration.MHV_AUTHENTICATOR;
					if (adminUserName != null && adminUserName.equals("SYSTEM")) {
						authenticator = ActivityActorTypeEnumeration.SYSTEM;
					}
					activityService.auditMviEvent(userProfileConverter.convertDomainUserProfile(ipa.getPatient().getUserProfile()), authenticator,
							false, ActivityActionTypeEnumeration.PREMIUM_ACCOUNT_REVOKED, ActivityTypeEnumeration.ACCOUNT_UPGRADE, response);					
				}
			}
			
			ipa = inpersonAuthenticationRepository.save(ipa);
			List<InPersonAuthenticationDTO> ipas = new ArrayList<InPersonAuthenticationDTO>();
			ipas.add(converter.convertDomainInpersonAuthentication(ipa));
			response.setPojoObject(ipas);
			
			if (null == responseRt.getValidationErrors()) {
				LinkedHashMap<String, String> validationErrors = new LinkedHashMap<String, String>();
				validationErrors.put("mviUnauthenticate", "There are no Validation Errors.");
				response.setValidationErrors(validationErrors);
			} else {
				response.setValidationErrors(responseRt.getValidationErrors());
			}

			if (null == responseRt.getInfoMessages()) {
				LinkedHashMap<String, String> infoMessages = new LinkedHashMap<String, String>();
				//infoMessages.put("mviUnauthenticate", "The authentication for MHV User "+ ipa.getPatient().getUserProfile().getFirstName()+" "+ ipa.getPatient().getUserProfile().getLastName()+" has been successfully removed.");
				response.setInfoMessages(infoMessages);
			} else {
				response.setInfoMessages(responseRt.getInfoMessages());
			}

			System.out.println(">>>> InPersonAuthenticationServiceImpl - mviUnauthenticate - returning ResponseUtil<List<InPersonAuthenticationDTO>> object.");
			return response;
		} catch (Exception e) {
			log.error(">>>> Error in InPersonAuthenticationServiceImpl - mviUnauthenticate - Error during Remove Authentication. "+ e);
			System.out.println(">>>> Error in InPersonAuthenticationServiceImpl - mviUnauthenticate - Error during Remove Authentication.");
			throw new MHVRuntimeException(">>>> Error in InPersonAuthenticationServiceImpl - mviUnauthenticate - Error during Remove Authentication.");
		}
	}
	
	private String getEmployeeName(String userName) {
		//MHV_CodeCR1514 - US12.4 MVI Compliance Implementation - Section 3.7 Add Authenticated By

		Employee employee = employeeRepository.findEmployeeByUserName(userName);		

		//Set up the Authenticated by in this format: VA Staff -  Smith, Jane DNS   SMITHJ
		return employee.getLastName() + ", " +
			employee.getFirstName()+ " " + "(" + employee.getUserName() + ")";
	}	

/*	private String getAccountStatus(Patient patient) {
		Boolean isPatientAuthenticated = false;
		if(patient != null){
			isPatientAuthenticated = isPatientAuthenticated(patient.getId());
			if (patient.getMatchedDateTime() == null){
				return AccountStatusEnumeration.BASIC;
			} else if(isPatientAuthenticated != null) {
				if(isPatientAuthenticated && patient.getCorrelationStatus().equals(PatientCorrelationStatusUtils.CORRELATED))
					return AccountStatusEnumeration.PREMIUM;
				else if(isPatientAuthenticated && !patient.getCorrelationStatus().equals(PatientCorrelationStatusUtils.CORRELATED))
					return AccountStatusEnumeration.ADVANCED;
	    		else if(!isPatientAuthenticated && patient.getMatchedDateTime() != null)
	    			return AccountStatusEnumeration.ADVANCED;
			}else{
				return AccountStatusEnumeration.ADVANCED;
			}
		}else{
			return AccountStatusEnumeration.BASIC;
		}
		return AccountStatusEnumeration.ADVANCED;
	}
	*/
}

