package gov.va.med.mhv.admin.service.impl.mhvusers;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import javax.naming.Name;
import javax.naming.NamingException;
import javax.naming.directory.BasicAttribute;
import javax.naming.directory.DirContext;
import javax.naming.directory.ModificationItem;
import javax.naming.ldap.LdapContext;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;
import javax.persistence.TypedQuery;
import javax.ws.rs.Consumes;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.core.MediaType;

import org.apache.commons.lang.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.hibernate.criterion.CriteriaQuery;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DataAccessException;
import org.springframework.ldap.core.LdapTemplate;
import org.springframework.ldap.support.LdapNameBuilder;
import org.springframework.stereotype.Component;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import gov.va.med.mhv.admin.converter.TermsConditionsVerHistConverter;
import gov.va.med.mhv.admin.converter.UserAcceptExtTermsVerConverter;
import gov.va.med.mhv.admin.converter.UserAcceptTermsHistConverter;
import gov.va.med.mhv.admin.converter.UserProfileDeactivationAuditConverter;
import gov.va.med.mhv.admin.converter.UserProfileLocalConverter;
import gov.va.med.mhv.admin.converter.UserProfileSMConverter;
import gov.va.med.mhv.admin.dto.SMSUserViewDTO;
import gov.va.med.mhv.admin.dto.TermsConditionsVerHistDTO;
import gov.va.med.mhv.admin.dto.UserAcceptExtTermsVerDTO;
import gov.va.med.mhv.admin.dto.UserAcceptTermsHistDTO;
import gov.va.med.mhv.admin.dto.UserProfileDeactivationAuditDTO;
import gov.va.med.mhv.admin.enums.ActivityActorTypeEnumeration;
import gov.va.med.mhv.admin.model.EauthHash;
import gov.va.med.mhv.admin.model.SMSUserView;
import gov.va.med.mhv.admin.model.TermsConditionsVerHist;
import gov.va.med.mhv.admin.model.UserAcceptExtTermsVer;
import gov.va.med.mhv.admin.model.UserAcceptTermsHist;
import gov.va.med.mhv.admin.model.UserProfileDeactivationAudit;
import gov.va.med.mhv.admin.repository.EauthHashRepository;
import gov.va.med.mhv.admin.repository.TermsConditionsVerHistRepository;
import gov.va.med.mhv.admin.repository.UserAcceptExtTermsVerRepository;
import gov.va.med.mhv.admin.repository.UserAcceptTermsHistRepository;
import gov.va.med.mhv.admin.repository.UserProfileDeactivationAuditRepository;
import gov.va.med.mhv.admin.repository.UserProfileDeactivationReasonRepository;
import gov.va.med.mhv.admin.repository.UserProfileRepository;
import gov.va.med.mhv.admin.repository.UserProfileSMRepository;
import gov.va.med.mhv.admin.service.mhvusers.ActivityService;
import gov.va.med.mhv.admin.service.mhvusers.UserProfileService;
import gov.va.med.mhv.admin.persist.ldap.LdapEnv;
import gov.va.med.mhv.admin.util.MessagesUtil;
import gov.va.med.mhv.admin.util.Precondition;
import gov.va.med.mhv.common.api.dto.PatientDTO;
import gov.va.med.mhv.common.api.dto.UserProfileDTO;
import gov.va.med.mhv.common.api.dto.UserProfileDeactivationReasonDTO;
import gov.va.med.mhv.common.api.exception.MHVException;
import gov.va.med.mhv.common.api.util.ResponseUtil;
import gov.va.med.mhv.common.data.converter.UserProfileDeactivationReasonConverter;
import gov.va.med.mhv.common.data.model.UserProfile;
import gov.va.med.mhv.common.data.model.UserProfileDeactivationReason;

@Path("/")
@Service("userProfileService")
@Component
public class UserProfileServiceImpl implements UserProfileService {

	private static Logger log = LogManager.getLogger(UserProfileServiceImpl.class);

	@Autowired
	private UserProfileRepository userProfileRepository;

	@Autowired
	private UserProfileSMRepository userProfileSMRepository;

	@Autowired
	private UserProfileDeactivationReasonRepository userProfileDeactivationReasonRepository;

	@Autowired
	private EauthHashRepository eauthHashRepository;

	@Autowired
	private ActivityService activityService;

	@Autowired
	private UserProfileDeactivationAuditRepository userProfileDeactivationAuditRepository;

	@Autowired
	private UserAcceptTermsHistRepository userAcceptTermHistRepository;

	@Autowired
	private TermsConditionsVerHistRepository termsConditionsVerHistRepository;

	@Autowired
	private UserAcceptExtTermsVerRepository userAcceptExtTermsVerRepository;

	@Autowired
	private UserProfileLocalConverter userProfileConverter;

	@Autowired
	private UserProfileSMConverter userProfileSMConverter;
	
	//@PersistenceContext
	//private EntityManager em;
	
	protected EntityManager entityManager;	

	/*
	 * @Autowired private UserProfileValidator validator;
	 */

	@Autowired
	private UserProfileDeactivationAuditConverter userProfileDeactivationAuditConverter;

	@Autowired
	private UserProfileDeactivationReasonConverter userProfileDeactivationReasonConverter;

	@Autowired
	private UserAcceptTermsHistConverter userAcceptTermsHistConverter;

	@Autowired
	private TermsConditionsVerHistConverter termsConditionsVerHistConverter;

	@Autowired
	private UserAcceptExtTermsVerConverter userAcceptExtTermsVerConverter;

	@Autowired
	private MessagesUtil messagesUtil;
	
	@Autowired
	private LdapEnv env;

	@Autowired
	private LdapTemplate ldapTemplate;	

	private String WILDCARD = "%";

	@Override
	@POST
	// @Path("/searchUser/{firstName}/{lastName}/{email}/{ssn}/{userName}/{birthDate}/{mhvId}")
	@Path("/searchUserExtended")
	@Produces(MediaType.APPLICATION_JSON)
	@Consumes(MediaType.APPLICATION_JSON)
	public List<UserProfileDTO> searchUserExtended(UserProfileDTO userProfileDTO) throws MHVException {
		List<UserProfile> userProfiles = null;

		String lowerFirstName = StringUtils.isNotBlank(userProfileDTO.getFirstName())
				? StringUtils.lowerCase(userProfileDTO.getFirstName()) + WILDCARD : WILDCARD;
		String lowerLastName = StringUtils.isNotBlank(userProfileDTO.getLastName())
				? StringUtils.lowerCase(userProfileDTO.getLastName()) + WILDCARD : WILDCARD;
		String lowerEmail = StringUtils.isNotBlank(userProfileDTO.getConfEmail())
				? StringUtils.lowerCase(userProfileDTO.getConfEmail()) + WILDCARD : WILDCARD;
		String lowerMhvId = StringUtils.isNotBlank(userProfileDTO.getMhvId())
				? StringUtils.lowerCase(userProfileDTO.getMhvId()) + WILDCARD : WILDCARD;
		String lowerUserName = StringUtils.isNotBlank(userProfileDTO.getUserName())
				? StringUtils.lowerCase(userProfileDTO.getUserName()) + WILDCARD : WILDCARD;
		String lowerSsn = StringUtils.isNotBlank(userProfileDTO.getSsn())
				? WILDCARD + StringUtils.lowerCase(userProfileDTO.getSsn()) + WILDCARD : WILDCARD;

		Date birthDate = userProfileDTO.getBirthDate();

		String str_date = null;
		Date cal_date = null;
		if (null == birthDate || birthDate.equals("null")) {
			birthDate = null;
		} else {
			str_date = dateToString(birthDate, "dd-MMM-yyyy");
			SimpleDateFormat dateFormat = new SimpleDateFormat("dd-MMM-yyyy");
			try {
				Calendar cal = Calendar.getInstance();
				cal.setTime(dateFormat.parse(str_date));
				cal.add(Calendar.DATE, 1);
				String convertedDate = dateFormat.format(cal.getTime());
				cal_date = stringToDate(convertedDate);
			} catch (Exception e) {
				log.error("Error in getting restrictSipAccess", e);
				throw new RuntimeException("Invalid Date format");
			}

		}

		System.out.println("&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&& lowerFirstName: " + lowerFirstName);
		System.out.println("&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&& lastName: " + lowerLastName);
		System.out.println("&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&& lowerSsn: " + lowerSsn);
		System.out.println("&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&& lowerMhvId: " + lowerMhvId);
		System.out.println("&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&& lowerUserName: " + lowerUserName);
		System.out.println("&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&& birthDate: " + cal_date);
		System.out.println("&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&& lowerEmail: " + lowerEmail);

		// Email is blank
		if (StringUtils.isBlank(userProfileDTO.getConfEmail())) {
			// Birthdate is not null
			if (cal_date != null) {
				userProfiles = this.userProfileRepository.getProfileByNameMhvIdUserNameWithBirthDate(lowerFirstName,
						lowerLastName, lowerMhvId, lowerUserName, lowerSsn, cal_date);
				System.out.println("&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&& FIRST");
			} else {
				userProfiles = this.userProfileRepository.getProfileByNameMhvIdUserName(lowerFirstName, lowerLastName,
						lowerMhvId, lowerUserName, lowerSsn);
				System.out.println("&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&& SECOND");
			}
		} else {
			if (cal_date != null) {
				userProfiles = this.userProfileRepository.getProfileByNameEmailMhvIdUserNameWithBirthDate(
						lowerFirstName, lowerLastName, lowerEmail, lowerMhvId, lowerUserName, lowerSsn, cal_date);
				System.out.println("&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&& THIRD");
			} else {
				userProfiles = this.userProfileRepository.getProfileByNameEmailMhvIdUserName(lowerFirstName,
						lowerLastName, lowerEmail, lowerMhvId, lowerUserName, lowerSsn);
				System.out.println("&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&& FOUR");
			}
		}

		if (userProfiles == null || userProfiles.size() == 0) {
			throw new MHVException("No users were found for the search criteria entered.");
		}

		return userProfileConverter.convertList(userProfiles);

	}

	@Override
	@POST
	@Path("/searchUserIPAExtended")
	@Produces(MediaType.APPLICATION_JSON)
	@Consumes(MediaType.APPLICATION_JSON)
	public List<UserProfileDTO> searchUserIPAExtended(UserProfileDTO userProfileDTO) throws MHVException {
		List<UserProfile> userProfiles = null;

		String lowerFirstName = StringUtils.isNotBlank(userProfileDTO.getFirstName())
				? StringUtils.lowerCase(userProfileDTO.getFirstName()) + WILDCARD : WILDCARD;
		String lowerLastName = StringUtils.isNotBlank(userProfileDTO.getLastName())
				? StringUtils.lowerCase(userProfileDTO.getLastName()) + WILDCARD : WILDCARD;
		String lowerEmail = StringUtils.isNotBlank(userProfileDTO.getConfEmail())
				? StringUtils.lowerCase(userProfileDTO.getConfEmail()) + WILDCARD : WILDCARD;
		String lowerMhvId = StringUtils.isNotBlank(userProfileDTO.getMhvId())
				? StringUtils.lowerCase(userProfileDTO.getMhvId()) + WILDCARD : WILDCARD;
		String lowerUserName = StringUtils.isNotBlank(userProfileDTO.getUserName())
				? StringUtils.lowerCase(userProfileDTO.getUserName()) + WILDCARD : WILDCARD;
		String lowerSsn = StringUtils.isNotBlank(userProfileDTO.getSsn())
				? WILDCARD + StringUtils.lowerCase(userProfileDTO.getSsn()) + WILDCARD : WILDCARD;

		Date birthDate = userProfileDTO.getBirthDate();
		if (null == birthDate || birthDate.equals("null")) {
			birthDate = null;
		}

		System.out.println(">>>> UserProfileServiceImpl - searchUserIPAExtended - lowerFirstName:" + lowerFirstName);
		System.out.println(">>>> UserProfileServiceImpl - searchUserIPAExtended - lowerLastName: " + lowerLastName);
		System.out.println(">>>> UserProfileServiceImpl - searchUserIPAExtended - lowerSsn: " + lowerSsn);
		System.out.println(">>>> UserProfileServiceImpl - searchUserIPAExtended - lowerMhvId: " + lowerMhvId);
		System.out.println(">>>> UserProfileServiceImpl - searchUserIPAExtended - lowerUserName: " + lowerUserName);
		System.out.println(">>>> UserProfileServiceImpl - searchUserIPAExtended - birthDate: " + birthDate);
		System.out.println(">>>> UserProfileServiceImpl - searchUserIPAExtended - lowerEmail: " + lowerEmail);

		// Email is blank
		if (StringUtils.isBlank(userProfileDTO.getConfEmail())) {
			// Birthdate is not null
			if (birthDate != null) {
				userProfiles = this.userProfileRepository.getProfileByNameMhvIdUserNameWithBirthDate(lowerFirstName,
						lowerLastName, lowerMhvId, lowerUserName, lowerSsn, birthDate);
				System.out.println(">>>> UserProfileServiceImpl - searchUserIPAExtended - FIRST");
			} else {
				userProfiles = this.userProfileRepository.getProfileByNameMhvIdUserName(lowerFirstName, lowerLastName,
						lowerMhvId, lowerUserName, lowerSsn);
				System.out.println(">>>> UserProfileServiceImpl - searchUserIPAExtended - SECOND");
			}
		} else {
			if (birthDate != null) {
				userProfiles = this.userProfileRepository.getProfileByNameEmailMhvIdUserNameWithBirthDate(
						lowerFirstName, lowerLastName, lowerEmail, lowerMhvId, lowerUserName, lowerSsn, birthDate);
				System.out.println(">>>> UserProfileServiceImpl - searchUserIPAExtended - THIRD");
			} else {
				userProfiles = this.userProfileRepository.getProfileByNameEmailMhvIdUserName(lowerFirstName,
						lowerLastName, lowerEmail, lowerMhvId, lowerUserName, lowerSsn);
				System.out.println(">>>> UserProfileServiceImpl - searchUserIPAExtended - FOUR");
			}
		}

		// if (userProfiles == null || userProfiles.size() == 0) {
		// throw new MHVException("No users were found for the search criteria
		// entered.");
		// }
		
		/*
	    List<UserProfile> uProfilesTmp = getUserProfiles();
	    if (uProfilesTmp == null || uProfilesTmp.size() == 0) {
	    	System.out.println(">>>> UserProfileServiceImpl - createNamedQuery - TEST:");
	    }
	    
		if (uProfilesTmp.size() > 0) {
			for (Iterator itr = uProfilesTmp.iterator(); itr.hasNext();) {
				UserProfileDTO uprofile = (UserProfileDTO) itr.next();
		    	System.out.println(">>>> UserProfileServiceImpl - createNamedQuery - uprofile_id:"+uprofile.getId());				
			}
		}	    
	    */
		return userProfileConverter.convertList(userProfiles);

	}
	
    @Transactional
    public List<UserProfile> getUserProfiles() throws DataAccessException {
        Query query = getEntityManager().createQuery("select u from UserProfile u where lower(u.userName) = lower('jm1hf8nh3jtd')");
        List<UserProfile> resultList = query.getResultList();
        return resultList;
    }
	

	@Override
	@GET
	@Path("/getSMUser/{userName}")
	@Produces(MediaType.APPLICATION_JSON)
	public SMSUserViewDTO getSMUser(@PathParam("userName") String userName) throws MHVException {
		SMSUserViewDTO sMSUserViewDTO = new SMSUserViewDTO();
		try {
			SMSUserView sMSUserView = userProfileSMRepository.getSMSUserByExternalUserName(userName);
			if (sMSUserView != null) {
				sMSUserViewDTO = userProfileSMConverter.convert(sMSUserView);
			}
		} catch (Exception e) {
			throw new MHVException(e);
		}
		return sMSUserViewDTO;
	}
	
	@Override
	@GET
	@Path("/resetPassword/{userName}/{newPassword}")
	@Produces(MediaType.APPLICATION_JSON)
	public void resetPassword(@PathParam("userName") String userName, @PathParam("newPassword") String newPassword) throws MHVException {
		UserProfileDTO userProfileDTO = new UserProfileDTO();

		try {
			resetPasswordLdap(userName, newPassword);
		} catch(Exception e) {
			//addError(response,UserManagementMessages.USER_DOES_NOT_EXIST, new String[]{userName});
			log.error("Error in resetPassword for userName: "+ userName, " and exception: " + e);
			throw new MHVException(e);			
		}
	}	

	@Override
	@GET
	@Path("/findUserProfileById/{userProfileId}")
	@Produces(MediaType.APPLICATION_JSON)
	public UserProfileDTO findUserProfileById(@PathParam("userProfileId") Long userProfileId) throws MHVException {
		UserProfileDTO userProfileDTO = new UserProfileDTO();
		System.out.println("&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&& userProfileId" + userProfileId);
		try {
			UserProfile userProfile = userProfileRepository.findOne(userProfileId);

			if (userProfile != null) {
				System.out.println("&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&& userProfileId"
						+ userProfile.getPasswordHintAnswer1());
				userProfileDTO = userProfileConverter.convertDomainUserProfile(userProfile);
			} else {
				throw new MHVException("UserProfile Not Found for the userProfileId::" + userProfileId);
			}
		} catch (Exception e) {
			throw new MHVException(e);
		}
		System.out.println("&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&& userProfileId" + userProfileDTO.getConfEmail());
		return userProfileDTO;
	}

	@GET
	@Consumes(MediaType.APPLICATION_JSON)
	@Produces({ MediaType.APPLICATION_JSON, MediaType.TEXT_PLAIN })
	@Path("/findUserProfileDeactivationAuditByUser/{userProfileId}")
	@Override
	public List<UserProfileDeactivationAuditDTO> findUserProfileDeactivationAuditByUser(
			@PathParam("userProfileId") Long userProfileId) throws MHVException {
		List<UserProfileDeactivationAudit> deactivationAudits = null;
		List<UserProfileDeactivationAuditDTO> auditList = null;
		try {
			deactivationAudits = userProfileDeactivationAuditRepository
					.findUserProfileDeactivationAuditByUser(userProfileId);

			if (null != deactivationAudits && deactivationAudits.size() > 0) {
				auditList = userProfileDeactivationAuditConverter.convertList(deactivationAudits);
			}
		} catch (Exception e) {
			log.error("Error in getting UserProfileDeactivationAudit details" + userProfileId, e);
			throw new MHVException("Error in getting UserProfileDeactivationAudit details" + userProfileId);
		}
		return auditList;
	}

	@GET
	@Produces(MediaType.APPLICATION_JSON)
	@Path("/findDeactivationReason/{reasonId}")
	@Override
	public UserProfileDeactivationReasonDTO findDeactivationReason(@PathParam("reasonId") Integer reasonId)
			throws MHVException {
		UserProfileDeactivationReasonDTO userProfileDeactivationReasonDTO = null;

		System.out.println("&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&& reasonId : " + reasonId);
		try {
			UserProfileDeactivationReason reason = userProfileDeactivationReasonRepository.findOne(reasonId);
			System.out.println("&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&& reason != null : " + reasonId);
			userProfileDeactivationReasonDTO = userProfileDeactivationReasonConverter.convert(reason);
		} catch (Exception e) {
			log.error("Error in getting findDeactivationReason", e);
			throw new MHVException("Error in getting findDeactivationReason ", e);
		}
		return userProfileDeactivationReasonDTO;
	}

	@GET
	@Consumes(MediaType.APPLICATION_JSON)
	@Produces(MediaType.APPLICATION_JSON)
	@Path("/userhasDeletedAccount/{userProfileId}")
	@Override
	public Boolean userhasDeletedAccount(@PathParam("userProfileId") Long userProfileId) throws MHVException {
		// List<UserProfileDTO> userProfileDtos = null;
		boolean hasDeletedAccount = false;
		try {
			UserProfile userProfile = userProfileRepository.findOne(userProfileId);
			List<UserProfile> userProfiles = userProfileRepository.getProfileByNameBirthDateSsnIncludeDeleted1(
					userProfile.getFirstName(), userProfile.getLastName().toLowerCase() + "%",
					userProfile.getBirthDate(), userProfile.getSsn());

			Set<String> deletedLastNames = new HashSet<String>();
			for (Iterator iterator = userProfiles.iterator(); iterator.hasNext();) {
				UserProfile uProfile = (UserProfile) iterator.next();
				if (uProfile.getLastName().indexOf("DELETED") > -1) {
					deletedLastNames.add(uProfile.getLastName());
				}
			}

			hasDeletedAccount = deletedLastNames.size() > 1;
		} catch (Exception e) {
			log.error("Error in getting findDeactivationReason", e);
			// response.setFailure(true);
			throw new MHVException("Error in getting findDeactivationReason ", e);
		}
		return hasDeletedAccount;
	}

	@POST
	@Consumes(MediaType.APPLICATION_JSON)
	@Produces(MediaType.APPLICATION_JSON)
	@Path("/restrictSipAccess")
	@Override
	@Transactional
	public UserProfileDTO restrictSipAccess(UserProfileDTO userProfileDto) throws MHVException {

		UserProfile profile = null;

		try {
			// profile = userProfileRepository.findOne(userProfile.getId());
			// TODO:GEETHA
			// profile.setRestrictSipAccess(true);

			/*
			 * if (userProfile.getRestrictSipAccessReason() == null ||
			 * userProfile.getRestrictSipAccessReason().length()==0) { //
			 * addError(response, "restrict.sip.access.description", new String
			 * [] {"reactivateNote"}); throw new
			 * MHVException("Description is a required field.");
			 * //copyMessages(response, userProfile); } else {
			 */
			if (null != userProfileDto) {
				System.out.println("Block/UnBlock Service call 1entity.getRestrictSIPAccess() >>>>>>>>>>>>>>>>>>>>>>>"
						+ userProfileDto.getRestrictSIPAccess());
				System.out.println(
						"Block/UnBlock Service call 2userProfile.getRestrictSIPAccess() >>>>>>>>>>>>>>>>>>>>>>>"
								+ userProfileDto.getRestrictSIPAccess());
				System.out.println(
						"Block/UnBlock Service call 3>>>>>>>>>>>>>>>>>>>>>> entity.getRestrictSIPAccessReason() >"
								+ userProfileDto.getRestrictSIPAccessReason());
				// profile = userProfileConverter.convertDTOProfile(profile,
				// userProfileDto);
				UserProfile newUP = userProfileRepository.findOne(userProfileDto.getId());
				newUP.setRestrictSIPAccess(userProfileDto.getRestrictSIPAccess());
				newUP.setRestrictSIPAccessReason(userProfileDto.getRestrictSIPAccessReason());
				newUP.setRestrictSIPAccesssDate(new Date());
				newUP.setRestrictSIPAccessBy(userProfileDto.getRestrictSIPAccessBy());
				profile = userProfileRepository.save(newUP);
				// profile = doUserProfileRestrictSipSave(profile);
			}
		} catch (Exception e) {
			log.error("Error in getting restrictSipAccess", e);
			throw new MHVException("Error in getting findDeactivationReason ", e);
		}

		return userProfileConverter.convertDomainUserProfile(profile);
	}

	/*
	 * Admin Portal_CodeCR1959 - React/Deact Impl
	 */
	private UserProfile doUserProfileRestrictSipSave(UserProfile entity) {
		UserProfile userProfile = null;
		/*
		 * if (entity != null) { userProfile =
		 * userProfileRepository.findOne(entity.getId()); } else { userProfile =
		 * new UserProfile(); // Since this is a new entity, set the crate date
		 * to now. userProfile.setCreatedDate(new Date()); }
		 * userProfile.setDeathDate(null);
		 */
		// userProfile.validate();
		try {
			if (entity != null) {
				System.out.println("Block/UnBlock Service call entity.getRestrictSIPAccess() >>>>>>>>>>>>>>>>>>>>>>>"
						+ entity.getRestrictSIPAccess());
				System.out
						.println("Block/UnBlock Service call userProfile.getRestrictSIPAccess() >>>>>>>>>>>>>>>>>>>>>>>"
								+ entity.getRestrictSIPAccess());
				System.out.println(
						"Block/UnBlock Service call >>>>>>>>>>>>>>>>>>>>>> entity.getRestrictSIPAccessReason() >"
								+ entity.getRestrictSIPAccessReason());
				UserProfile newUP = userProfileRepository.findOne(entity.getId());
				newUP.setRestrictSIPAccess(entity.getRestrictSIPAccess());
				newUP.setRestrictSIPAccessReason(entity.getRestrictSIPAccessReason());
				newUP.setRestrictSIPAccesssDate(new Date());
				newUP.setRestrictSIPAccessBy(entity.getRestrictSIPAccessBy());
				userProfile = userProfileRepository.save(newUP);
				System.out.println(
						"Block/UnBlock Service call >>>>>>>>>>>>>>>>>>>>>>>" + userProfile.getRestrictSIPAccess());
			}

		} catch (Exception e) {
			e.printStackTrace();
		}

		return userProfile;
	}

	@GET
	@Produces(MediaType.APPLICATION_JSON)
	@Path("/findUserAcceptTermsHistByUserProfileId/{userProfileId}")
	@Override
	public List<UserAcceptTermsHistDTO> findUserAcceptTermsHistByUserProfileId(
			@PathParam("userProfileId") Long userProfileId) throws MHVException {
		List<UserAcceptTermsHistDTO> list;

		try {
			List<UserAcceptTermsHist> userAcceptTermHistList = userAcceptTermHistRepository
					.getUserAcceptTermsHistByUserProfileId(userProfileId);
			list = userAcceptTermsHistConverter.convertList(userAcceptTermHistList);
		} catch (Exception e) {
			throw new MHVException("Unable to check for User Accepted Terms History. Please contact the Help Desk.");
		}

		return list;
	}

	@GET
	@Produces(MediaType.APPLICATION_JSON)
	@Path("/findTermsConditionsVerHistById/{termsConditionsVerHistId}")
	@Override
	public List<TermsConditionsVerHistDTO> findTermsConditionsVerHistById(
			@PathParam("termsConditionsVerHistId") Long termsConditionsVerHistId) throws MHVException {
		List<TermsConditionsVerHistDTO> list;

		try {
			List<TermsConditionsVerHist> termsConditionsVerHisList = termsConditionsVerHistRepository
					.getVersionById(termsConditionsVerHistId);
			list = termsConditionsVerHistConverter.convertList(termsConditionsVerHisList);
		} catch (Exception e) {
			throw new MHVException(
					"Unable to check for Terms and Conditions Version History. Please contact the Help Desk.");
		}

		return list;
	}

	@GET
	@Produces(MediaType.APPLICATION_JSON)
	@Path("/findUserAcceptExtTermsVerByUserProfileId/{userProfileId}")
	@Override
	public List<UserAcceptExtTermsVerDTO> findUserAcceptExtTermsVerByUserProfileId(
			@PathParam("userProfileId") Long userProfileId) throws MHVException {
		List<UserAcceptExtTermsVerDTO> list;

		try {
			List<UserAcceptExtTermsVer> userAcceptExtTermsVerList = userAcceptExtTermsVerRepository
					.getUserAcceptExtTermsVerByUserProfileId(userProfileId);
			list = userAcceptExtTermsVerConverter.convertList(userAcceptExtTermsVerList);
		} catch (Exception e) {
			e.printStackTrace();
			throw new MHVException("Unable to check for External Terms Version History. Please contact the Help Desk.",
					e);
		}

		return list;
	}

	@POST
	@Consumes(MediaType.APPLICATION_JSON)
	@Produces(MediaType.APPLICATION_JSON)
	@Path("/reactivateUserInHelpDesk/{userProfileId}/{reasonId}/{changedBy}/{notes}")
	@Override
	public void reactivateUserInHelpDesk(@PathParam("userProfileId") Long userProfileId,
			@PathParam("reasonId") Integer reasonId, @PathParam("changedBy") String changedBy,
			@PathParam("notes") String notes) throws MHVException {
		ResponseUtil<UserProfile> response = new ResponseUtil<UserProfile>();

		System.out.println(
				"<<<<<<<<<<<UserProfileServiceImpl private reactivateUserInHelpDesk() >>>>>>> : " + userProfileId);

		try {
			UserProfile userProfile = userProfileRepository.findOne(userProfileId);
			System.out.println("<<<<<<<<<<<UserProfileServiceImpl private userProfile >>>>>>> : " + userProfile);
			UserProfileDeactivationReason reason = userProfileDeactivationReasonRepository.findOne(reasonId);
			System.out.println("<<<<<<<<<<<UserProfileServiceImpl private reason >>>>>>> : " + reason.getId());

			// Precondition.assertNotNull("userProfile", userProfile);
			Precondition.assertNotNull("changedBy", changedBy);
			if (reason != null) {
				if (log.isDebugEnabled()) {
					log.debug("Deactivating account '" + userProfile.getUserName() + ", because " + reason.getName()
							+ ". " + StringUtils.defaultString(notes));
					response.setFailure(false);

				}
			} else {
				response.setFailure(true);
				response.getInfoMessages().put("Reason is missing.", "reactivateReason");
				// addError(response,
				// "mvi.helpdesk.devalidate.error.description.missing", new
				// String [] {"reactivateNote"});
				if (log.isDebugEnabled()) {
					log.debug("No deactivation reason for user:" + userProfile.getUserName());

				}
				throw new MHVException(response.getInfoMessages(), response.getValidationErrors(),
						response.getFailureMessage());
				// addError(response,
				// "mvi.helpdesk.devalidate.error.reason.missing", new String []
				// {"reactivateReason"});

			}

			if ((notes == null) || (notes.length() == 0)) {
				response.setFailure(true);
				response.getInfoMessages().put("Description is missing.", "reactivateNote");
				// addError(response,
				// "mvi.helpdesk.devalidate.error.description.missing", new
				// String [] {"reactivateNote"});
				throw new MHVException(response.getInfoMessages(), response.getValidationErrors(),
						response.getFailureMessage());
			}

			if (!response.isFailure()) {
				System.out.println("UserProfileServiceImpl private !response.isFailure()" + !response.isFailure());
				// US12.5 - Make sure to delete "DELETE"
				if (userProfile.getLastName().indexOf("DELETED") > -1) {
					userProfile.setLastName(
							userProfile.getLastName().substring(0, userProfile.getLastName().indexOf("DELETED")));
				}
				userProfile.setLastName(userProfile.getLastName().replaceAll("DECEASED", ""));

				System.out.println(" ((((())))))))))))))))UserProfileServiceImpl private userProfile.getLastName() "
						+ userProfile.getLastName());

				// JAZZ:19556 - Make sure that the correct message display when
				// there is a duplicate user
				if (duplicateCheck(userProfile)) {
					response.getInfoMessages().put(messagesUtil.getMviHelpdeskDevalidateErrorReasonMissing(), String
							.format(messagesUtil.getMviHelpdeskDevalidateErrorReasonMissing(), "deactivateReason"));
					// addError(response, "mvi.helpdesk.revalidate.fail", new
					// Object[] {getFullName(userProfile)});
					activityService.auditAccountReactivateEvent(
							userProfileConverter.convertDomainUserProfile(userProfile),
							ActivityActorTypeEnumeration.HELP_DESK_ADMINISTRATOR, false, response);
					// return response;
				}

				userProfile.setDeactivationReason(null);
				userProfile.setDeathDate(null);
				userProfile = doUserProfileReactivateSave(userProfile, userProfile.getSsn());
				if (!response.isFailure()) {
					// copyMessages(response, userProfile);
					auditReactivationChange(userProfile, reason, changedBy, notes);
					activityService.auditAccountReactivateEvent(
							userProfileConverter.convertDomainUserProfile(userProfile),
							ActivityActorTypeEnumeration.HELP_DESK_ADMINISTRATOR, true, response);
				} else {
					// Note this audit is only for deact/react and goes to
					// user_profile_deact_audit table; different from aal table.
					activityService.auditAccountReactivateEvent(
							userProfileConverter.convertDomainUserProfile(userProfile),
							ActivityActorTypeEnumeration.HELP_DESK_ADMINISTRATOR, false, response);
				}
			} else {
				System.out.println("UserProfileServiceImpl private response.isFailure()" + response.isFailure());
				activityService.auditAccountReactivateEvent(userProfileConverter.convertDomainUserProfile(userProfile),
						ActivityActorTypeEnumeration.HELP_DESK_ADMINISTRATOR, false, response);
			}
		} catch (Exception e) {
			log.error("Error in getting findDeactivationReason" + e);
			// response.setFailure(true);
			throw new MHVException(response.getInfoMessages(), response.getValidationErrors(),
					response.getFailureMessage());
		}
		// return response;
	}

	// Admin Portal_CodeCR1959 - React/Deact Impl
	private void auditReactivationChange(UserProfile userProfile, UserProfileDeactivationReason reason,
			String changedBy, String notes) {
		UserProfileDeactivationAudit audit = new UserProfileDeactivationAudit();
		audit.setAction(1);
		audit.setOplock(1);
		audit.setUserProfile(userProfile);
		audit.setDeactivationReason(reason);
		audit.setChangeDate(new Date());
		audit.setChangedBy(changedBy);
		audit.setNotes(notes);
		userProfileDeactivationAuditRepository.save(audit);
	}

	private boolean duplicateCheck(UserProfile userProfile) {
		String firstName = StringUtils.lowerCase(userProfile.getFirstName());
		String lastName = StringUtils.lowerCase(userProfile.getLastName());

		if (userProfileRepository.getProfileByNameBirthDateSsnExcludeId(firstName, lastName, userProfile.getBirthDate(),
				userProfile.getSsn(), userProfile.getId()).size() > 0) {
			if (log.isDebugEnabled())
				log.debug("MviIntegrationServiceImpl.duplicateCheck(): Duplicate account found for the user: "
						+ userProfile.getUserName());
			return true;
		}
		return false;
	}

	@POST
	@Consumes(MediaType.APPLICATION_JSON)
	@Produces({ MediaType.APPLICATION_JSON })
	@Path("/changeActivationState/{userProfileId}/{reasonId}/{changedBy}/{notes}/{deathDate}/{isActivate}")
	@Override
	public void changeActivationState(@PathParam("userProfileId") Long userProfileId,
			@PathParam("reasonId") Integer reasonId, @PathParam("changedBy") String changedBy,
			@PathParam("notes") String notes, @PathParam("deathDate") String deathDate,
			@PathParam("isActivate") Boolean isActivate) throws MHVException {
		deathDate = null;
		ResponseUtil response = new ResponseUtil();
		UserProfile userProfile = userProfileRepository.findOne(userProfileId);
		UserProfileDeactivationReason reason = userProfileDeactivationReasonRepository.findOne(reasonId);
		Precondition.assertNotNull("userProfile", userProfile);
		String originalLastName = userProfile.getLastName();
		System.out.println("#$@#$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$ changeActivationState in USERPROFILE");
		if (reason != null) {
			System.out.println(
					"#$@#$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$ changeActivationState in USERPROFILE reason != null "
							+ reason.getName());
			List<UserProfile> userProfiles = new ArrayList<UserProfile>();
			String lastNameAppend = "";

			Set<String> deletedLastNames = new HashSet<String>();
			for (Iterator iterator = userProfiles.iterator(); iterator.hasNext();) {
				UserProfile uProfile = (UserProfile) iterator.next();
				if (uProfile.getLastName().indexOf("DELETED") > -1) {
					deletedLastNames.add(uProfile.getLastName());
				}
			}

			if (deletedLastNames.size() == 0) {
				lastNameAppend = "DELETED";
			} else {
				if (!deletedLastNames.contains(userProfile.getLastName() + "DELETEDONE")) {
					lastNameAppend = "DELETEDONE";
				} else if (!deletedLastNames.contains(userProfile.getLastName() + "DELETEDTWO")) {
					lastNameAppend = "DELETEDTWO";
				} else if (!deletedLastNames.contains(userProfile.getLastName() + "DELETEDTWO")) {
					lastNameAppend = "DELETEDTWO";
				} else if (!deletedLastNames.contains(userProfile.getLastName() + "DELETEDTHREE")) {
					lastNameAppend = "DELETEDTHREE";
				} else if (!deletedLastNames.contains(userProfile.getLastName() + "DELETEDFOUR")) {
					lastNameAppend = "DELETEDFOUR";
				} else if (!deletedLastNames.contains(userProfile.getLastName() + "DELETEDFIVE")) {
					lastNameAppend = "DELETEDFIVE";
				} else if (!deletedLastNames.contains(userProfile.getLastName() + "DELETEDSIX")) {
					lastNameAppend = "DELETEDSIX";
				} else if (!deletedLastNames.contains(userProfile.getLastName() + "DELETEDSEVEN")) {
					lastNameAppend = "DELETEDSEVEN";
				} else if (!deletedLastNames.contains(userProfile.getLastName() + "DELETEDEIGHT")) {
					lastNameAppend = "DELETEDEIGHT";
				} else if (!deletedLastNames.contains(userProfile.getLastName() + "DELETEDNINE")) {
					lastNameAppend = "DELETEDNINE";
				} else if (!deletedLastNames.contains(userProfile.getLastName() + "DELETEDTEN")) {
					lastNameAppend = "DELETEDTEN";
				} else if (!deletedLastNames.contains(userProfile.getLastName() + "DELETEDELEVEN")) {
					lastNameAppend = "DELETEDELEVEN";
				} else if (!deletedLastNames.contains(userProfile.getLastName() + "DELETEDTWELVE")) {
					lastNameAppend = "DELETEDTWELVE";
				}
			}
			if (reason.getName().equals("Deceased")) {
				userProfile.setLastName(userProfile.getLastName() + "DECEASED");
			} else {
				// US12.5 - Make sure to add "DELETE"
				userProfile.setLastName(userProfile.getLastName() + lastNameAppend);
			}
			userProfile.setDeactivationReason(reason);
			userProfile.setDeathDate(null);
			// JAZZ: 19292 - Take out matched date, set mpi controlled flag to
			// null during deactivation.
			userProfile.setIsMPIControlled(false);
		} else {
			System.out.println(
					"#$@#$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$ changeActivationState in USERPROFILE reason == null ");
			response.getInfoMessages().put(messagesUtil.getMviHelpdeskDevalidateErrorReasonMissing(),
					String.format(messagesUtil.getMviHelpdeskDevalidateErrorReasonMissing(), "deactivateReason"));
		}

		if ((notes == null) || (notes.length() == 0)) {
			response.getInfoMessages().put(messagesUtil.getMviHelpdeskDevalidateErrorDescriptionMissing(),
					String.format(messagesUtil.getMviHelpdeskDevalidateErrorDescriptionMissing(), "deactivateNote"));
		}
		// Admin Portal_CodeCR1959 - Deact/React Impl
		UserProfile savedUserProfile = doUserProfileActivateSave(userProfile, false, userProfile.getFccx(), response);
		if (isActivate) {
			if (response.isFailure()) {
				userProfile.setLastName(originalLastName);
				activityService.auditAccountDeactivateEvent(userProfileConverter.convertDomainUserProfile(userProfile),
						ActivityActorTypeEnumeration.HELP_DESK_ADMINISTRATOR, false, response);
				log.error(
						"UserProfileServiceImpl().changeActivationState user:" + savedUserProfile.getId() + ":FAILED");
			} else {
				// Note this audit is only for deact/react and goes to
				// user_profile_deact_audit table; different from aal table.
				try {
					auditActivationStateChange(response, userProfile, reason, changedBy, notes, true);
				} catch (Exception e) {
					log.error("Error in storing UserProfileDeactivationAudit details", e);
					response.setFailure(true);
					throw new MHVException("Error in storing UserProfileDeactivationAudit details");
				}

				activityService.auditAccountDeactivateEvent(userProfileConverter.convertDomainUserProfile(userProfile),
						ActivityActorTypeEnumeration.HELP_DESK_ADMINISTRATOR, true, response);
				// JAZZ:Task#20852 - Take out entry from eauthhash during
				// deactivation for the user
				deleteHashByUserProfileId(userProfile, response);
				log.error(
						"UserProfileServiceImpl().changeActivationState user:" + savedUserProfile.getId() + ":SUCCESS");
			}
		} else {
			System.out.println(
					"#$@#$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$ changeActivationState in USERPROFILE AUDIT DEACTIVATE EVENT ");
			userProfile.setLastName(originalLastName);
			activityService.auditAccountDeactivateEvent(userProfileConverter.convertDomainUserProfile(userProfile),
					ActivityActorTypeEnumeration.HELP_DESK_ADMINISTRATOR, true, response);
			try {
				auditActivationStateChange(response, userProfile, reason, changedBy, notes, true);
			} catch (Exception e) {
				log.error("Error in storing UserProfileDeactivationAudit details", e);
				response.setFailure(true);
				throw new MHVException("Error in storing UserProfileDeactivationAudit details");
			}
		}
		return;
	}

	/*
	 * Admin Portal_CodeCR1959 - React/Deact Impl
	 */
	private UserProfile doUserProfileReactivateSave(UserProfile entity, String verifySsn) {
		UserProfile userProfile = null;
		if (entity != null) {
			Long id = entity.getId();
			if (id != null) {
				userProfile = userProfileRepository.getUserProfileById(id);
				/*
				 * if (doAudit) { activityService.auditUpdateProfileEvent(id,
				 * ActivityActorTypeEnumeration.SELF, response); }
				 */
			} else {
				userProfile = new UserProfile();
				// Since this is a new entity, set the crate date to now.
				userProfile.setCreatedDate(new Date());
			}
			userProfile.setDeathDate(null);
			// userProfileDto.validate();
			// CR2161 and CR2162 Kumar,Vyasa started here
			// bo.validatePasswordHintAnswers();
			// CR2161 and CR2162 Kumar,Vyasa ended here
			// bo.validateMatchingSSN(verifySsn);

			// if (! StringUtils.equals(getSsn(), verifySsn)) {
			// if (userProfile.getSsn().equals(verifySsn)) {
			userProfile = userProfileRepository.save(entity);
			System.out.println("AFTER doUserProfileReactivateSave Save method ");
			// } else {
			/*
			 * Iterator iterator =
			 * bo.getMessages().getErrorMessages().iterator();
			 * while(iterator.hasNext()) { StringBuffer str = new
			 * StringBuffer("doUserProfileReactivateSave() failed for user profile id: "
			 * +entity.getId() + " ERROR:"); Message message =
			 * (Message)iterator.next(); str.append(message.getKey()); if
			 * (message.getInserts().size() > 0) {
			 * str.append("; inserts=[").append(message.getInserts()).
			 * append("]"); } if (message.getProperties().size() > 0) {
			 * str.append("; properties=[").append(message.getProperties()).
			 * append("]"); } if(LOG.isErrorEnabled()) { LOG.error(str); } }
			 */
			// }
		}
		return userProfile;
	}

	/*
	 * JAZZ:Task#20852 - Take out entry from eauthhash during deactivation for
	 * the user
	 */
	public void deleteHashByUserProfileId(UserProfile userProfile, ResponseUtil response) {
		List<EauthHash> eauthHashList = eauthHashRepository.getEauthHashByUserProfileId(userProfile.getId());

		for (EauthHash eauthHash : eauthHashList) {
			eauthHashRepository.delete(eauthHash);
			activityService.auditAccountDeactivateEauthHashEvent(
					userProfileConverter.convertDomainUserProfile(userProfile), response);
		}
	}

	private UserProfile doUserProfileActivateSave(UserProfile entity, boolean doAudit, Date fccxDate,
			ResponseUtil response) {

		UserProfileDTO userProfileDto = null;
		System.out.println("#$@#$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$ doUserProfileActivateSave ");
		if (entity != null) {
			System.out
					.println("#$@#$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$ doUserProfileActivateSave : entity.getLastName()"
							+ entity.getLastName());
			System.out.println("#$@#$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$ doUserProfileActivateSave entity != null ");
			Long id = entity.getId();
			if (id != null) {
				System.out
						.println("#$@#$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$ doUserProfileActivateSave id != null " + id);
				userProfileDto = userProfileConverter
						.convertDomainUserProfile(userProfileRepository.getUserProfileById(id));
				if (doAudit) {
					activityService.auditUpdateProfileEvent(id, ActivityActorTypeEnumeration.SELF, response);
				}
			} else {
				System.out.println("#$@#$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$ doUserProfileActivateSave id == null ");
				userProfileDto = new UserProfileDTO();
				// Since this is a new entity, set the crate date to now.
				userProfileDto.setCreatedDate(new Date());
			}

			if (fccxDate != null) {
				entity.setFccx(fccxDate);
			}
			// validator.userProfileUpdateValidations(response, userProfileDto,
			// false);
			// no validation errors
			if (entity.getBirthDate() != null) {
				System.out.println(
						"#$@#$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$ doUserProfileActivateSaveentity.getBirthDate() != null"
								+ entity.getBirthDate());
				SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd");
				String birthDate = sdf.format(entity.getBirthDate());
				if (birthDate.substring(0, 4).equals("1851") && birthDate.substring(4, 6).equals("01")
						&& birthDate.substring(6).equals("01")) {
					entity.setBirthDate(null);
					response.getInfoMessages().put(messagesUtil.getImpreciseBirthDate(),
							String.format(messagesUtil.getImpreciseBirthDate(), "impreciseBirthDate"));
				}
			} else {
				System.out.println(
						"#$@#$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$ doUserProfileActivateSave : entity.getBirthDate() == null");
				response.getInfoMessages().put(messagesUtil.getImpreciseBirthDate(),
						String.format(messagesUtil.getImpreciseBirthDate(), "impreciseBirthDate"));
			}
			// entity = userProfileConverter.convertDTOProfile(entity,
			// userProfileDto);
			// save user profile in USER_PROFILE table
			userProfileRepository.save(entity);
			System.out.println(
					"#$@#$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$ doUserProfileActivateSaveentity: userProfileRepository.save(entity);"
							+ entity.getLastName());
			response.setPojoObject(entity);
		}
		return entity;
	}

	// Admin Portal_CodeCR1959 - React/Deact Impl
	private void auditActivationStateChange(ResponseUtil response, UserProfile userProfile,
			UserProfileDeactivationReason reason, String changedBy, String notes, boolean isActivate)
			throws MHVException {
		UserProfileDeactivationAudit audit = new UserProfileDeactivationAudit();
		audit.setOplock(1);
		if (isActivate) {
			audit.setAction(1);
		} else {
			audit.setAction(0);
		}
		audit.setUserProfile(userProfile);
		audit.setDeactivationReason(reason);
		audit.setChangeDate(new Date());
		audit.setChangedBy(changedBy);
		audit.setNotes(notes);

		// copyMessages(response,
		// ServiceFactory.createEntityMaintenanceService().save(audit));
		try {
			saveAudit(audit, response);
		} catch (Exception e) {
			log.error("Error in storing UserProfileDeactivationAudit details", e);
			response.setFailure(true);
			throw new MHVException("Error in storing UserProfileDeactivationAudit details");
		}

	}

	private UserProfileDeactivationAudit saveAudit(UserProfileDeactivationAudit audit, ResponseUtil response)
			throws MHVException {
		UserProfileDeactivationAudit storedDeactivationAudit = null;
		try {
			storedDeactivationAudit = userProfileDeactivationAuditRepository.save(audit);
		} catch (Exception e) {
			log.error("Error in storing UserProfileDeactivationAudit details", e);
			response.setFailure(true);
			throw new MHVException("Error in storing UserProfileDeactivationAudit details");
		}

		response.setFailure(false);
		response.setSuccess(true);
		response.setPojoObject(storedDeactivationAudit);

		return storedDeactivationAudit;
	}

	/*
	 * JAZZ:Task#20852 - Take out entry from eauthhash during deactivation for
	 * the user
	 */
	@Transactional
	public ResponseUtil deleteEauthHashByUserProfileId(Long userProfileId) {
		ResponseUtil response = new ResponseUtil();
		try {
			List<EauthHash> eauthHashes = eauthHashRepository.getEauthHashByUserProfileId(userProfileId);
			if (null != eauthHashes) {
				for (EauthHash eauth : eauthHashes) {
					eauthHashRepository.delete(eauth);
					activityService.auditAccountDeactivateEauthHashEvent(
							userProfileConverter.convertDomainUserProfile(eauth.getUserProfile()), response);
				}
			}
			response.setSuccess(true);
		} catch (Exception e) {
			log.error("Exception caught in deleting eauth hash", e);
			response.setFailure(true);
			response.setFailureMessage(e.getMessage());
		}

		return response;
	}

	private String getFullName(UserProfile userProfile) {
		String firstName = userProfile.getFirstName();
		String middleName = userProfile.getMiddleName();
		String lastName = userProfile.getLastName();

		StringBuffer buff = new StringBuffer();
		buff.append(firstName);
		if (middleName != null) {
			buff.append(' ');
			buff.append(middleName);
		}
		buff.append(' ');
		buff.append(lastName);

		return (buff.toString());
	}
	
	
	public void resetPasswordLdap(String userName, String newPassword) throws MHVException {

		LdapContext context = (LdapContext)ldapTemplate.getContextSource().getReadWriteContext();

		final ModificationItem[] modifications = new ModificationItem[1];
		modifications[0] = new ModificationItem(DirContext.REPLACE_ATTRIBUTE, new BasicAttribute(env.getUserPasswordAttribute(), newPassword));

		try {
			context.modifyAttributes(buildDn(userName), modifications);
		} catch (Exception e) {
			//throw new MHVUserDoesNotExistException(e);
			throw new MHVException("Error in storing UserProfileDeactivationAudit details");
		}

	}

	private Name buildDn(String uId) {
		return LdapNameBuilder.newInstance()
				.add("ou","People")
				.add("uid", uId )
				.build();
	}	

	public static String dateToString(Date date, String format) {
		String stringDate = null;
		try {
			if (date != null) {
				SimpleDateFormat sdf = new SimpleDateFormat(format);
				stringDate = sdf.format(date);
			}
		} catch (Exception e) {
			throw new RuntimeException("Invalid Date format");
		}
		return stringDate;
	}

	private static Date stringToDate(String dateStr) {
		String stringDate = "";
		Date date = null;
		try {
			if (dateStr != null) {
				SimpleDateFormat formatter = new SimpleDateFormat("dd-MMM-yyyy");
				date = formatter.parse(dateStr);
			}
		} catch (Exception e) {
			throw new RuntimeException("Invalid Date format");
		}
		return date;
	}

    public EntityManager getEntityManager() {
        return entityManager;
    }
    
    @PersistenceContext
    public void setEntityManager(EntityManager entityManager) {
        this.entityManager = entityManager;
    }
	
}