package gov.va.med.mhv.admin.service.impl.vastaff;

import gov.va.med.mhv.admin.dto.EmployeeSearchResult;
import gov.va.med.mhv.admin.ldap.LdapEnvironment;
import gov.va.med.mhv.admin.service.vastaff.EmployeeSearchService;
import gov.va.med.mhv.admin.util.MessagesUtil;
import gov.va.med.mhv.common.api.exception.MHVException;
import gov.va.med.mhv.common.api.util.ResponseUtil;

import java.util.ArrayList;
import java.util.Hashtable;
import java.util.List;
import java.util.LinkedHashMap;

import javax.naming.Context;
import javax.naming.NamingEnumeration;
import javax.naming.NamingException;
import javax.naming.InsufficientResourcesException;
import javax.naming.LimitExceededException;
import javax.naming.directory.Attributes;
import javax.naming.directory.SearchResult;
import javax.naming.ldap.InitialLdapContext;
import javax.naming.ldap.LdapContext;
import javax.naming.directory.SearchControls;
//import javax.naming.ldap.Control;

import javax.ws.rs.GET;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.core.MediaType;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.stereotype.Service;


@Path("/")
@Service("employeeSearchService")
public class EmployeeSearchServiceImpl implements EmployeeSearchService {

	protected static Logger log = LogManager.getLogger(EmployeeSearchServiceImpl.class);
	private static SearchControls SEARCH_CONTROLS = null;


	@Autowired
	private LdapEnvironment ldapEnvironment;

	@Autowired
	private MessagesUtil messagesUtil;

    /**
     * Execute the SearchByUserName service
     * Searches user in AD
     * @return List<EmployeeSearchResult>
     */
	@Override
	@GET
	@Path("/searchByUserName/{userName}")
	@Produces(MediaType.APPLICATION_JSON)
	public List<EmployeeSearchResult> searchByUserName(@PathParam("userName") String userName) throws MHVException {
		List<EmployeeSearchResult> employeeSearchResultList = new ArrayList<EmployeeSearchResult>();
		if (userName != null) {
			userName = userName.replaceAll("\\*", ""); // no user entered
														// wildcard characters
			userName = userName.trim(); // trim spaces
		}

		// create the search string
		String searchFilter = "(&(objectClass=user)(sAMAccountName=" + userName + "))";
		return performSearch(employeeSearchResultList,searchFilter);

	}


    private static final EmployeeSearchResult convertToEmployeeSearchResult(
            Attributes attributes) throws NamingException {
            EmployeeSearchResult employeeSearchResult = new EmployeeSearchResult();

            if (attributes.get("sAMAccountName") != null)
                employeeSearchResult.setUserName((String) attributes.get(
                    "sAMAccountName").get());

            if (attributes.get("givenName") != null)
                employeeSearchResult.setFirstName((String) attributes.get(
                    "givenName").get());

            if (attributes.get("sn") != null)
                employeeSearchResult.setLastName((String) attributes.get("sn")
                    .get());

            if (attributes.get("title") != null)
                employeeSearchResult.setTitle((String) attributes.get("title")
                    .get());

            if (attributes.get("physicalDeliveryOfficeName") != null)
                employeeSearchResult.setOffice((String) attributes.get(
                    "physicalDeliveryOfficeName").get());

            if (attributes.get("department") != null)
                employeeSearchResult.setDepartment((String) attributes.get(
                    "department").get());

            if (attributes.get("company") != null)
                employeeSearchResult.setCompany((String) attributes.get("company")
                    .get());

            if (attributes.get("telephoneNumber") != null)
                employeeSearchResult.setPhone((String) attributes.get(
                    "telephoneNumber").get());

            if (attributes.get("mail") != null)
                employeeSearchResult
                    .setEmail((String) attributes.get("mail").get());

            employeeSearchResult.setFullName(employeeSearchResult.getLastName()
                + ", " + employeeSearchResult.getFirstName());

            return employeeSearchResult;
        }
    
    /**
     * Execute the SearchByName service
     * Searches user in AD
     * @return List<EmployeeSearchResult>
     */
    @Override
    @GET
	@Path("/searchByName/{firstName}/{lastName}")
	@Produces(MediaType.APPLICATION_JSON)
    public List<EmployeeSearchResult> searchByName(@PathParam("firstName")  String firstName, @PathParam("lastName")  String lastName ) throws MHVException {
    	List<EmployeeSearchResult> employeeSearchResultList = new ArrayList<EmployeeSearchResult>();
        if (lastName != null) {
           // lastName = lastName.replaceAll("\\*", ""); // no user entered
                                                        // wildcard characters
            lastName = lastName.trim(); // trim spaces
        }

        if (firstName != null) {
            //firstName = firstName.replaceAll("\\*", ""); // no user entered
                                                            // wildcard
                                                            // characters
            firstName = firstName.trim(); // trim spaces
        } else {
            firstName = "";
        }

        // create the search string, add wildcard to end of criteria per requirements
        String searchFilter = "(&(objectClass=user)(sn=" + lastName
            + "*)(givenName=" + firstName + "*))";
        
		return performSearch(employeeSearchResultList, searchFilter);

	}

	private List<EmployeeSearchResult> performSearch(List<EmployeeSearchResult> employeeSearchResultList, String searchFilter) throws MHVException{
		SEARCH_CONTROLS = new SearchControls();
		SEARCH_CONTROLS.setReturningAttributes(new String[]{"sAMAccountName","cn","givenName","sn","title","physicalDeliveryOfficeName","department","company","telephoneNumber","mail"}); //Specify the attributes to return
		SEARCH_CONTROLS.setSearchScope(SearchControls.SUBTREE_SCOPE);
		Hashtable<String, String> AD_LDAP_ENVIRONMENT = new Hashtable<String, String>();
		AD_LDAP_ENVIRONMENT.put(Context.INITIAL_CONTEXT_FACTORY, ldapEnvironment.getAdIntialCtxFactory());
		AD_LDAP_ENVIRONMENT.put(Context.SECURITY_AUTHENTICATION, ldapEnvironment.getAdSecurityAuthentication());
		AD_LDAP_ENVIRONMENT.put(Context.PROVIDER_URL, ldapEnvironment.getAdUrl());
		AD_LDAP_ENVIRONMENT.put(Context.SECURITY_PRINCIPAL, ldapEnvironment.getAdAdminPrincipal());
		AD_LDAP_ENVIRONMENT.put(Context.SECURITY_CREDENTIALS, ldapEnvironment.getAdAdminPassword());
		
		ResponseUtil response = new ResponseUtil();
		LdapContext ldapContext = null;
		
		try {
			ldapContext = new InitialLdapContext(AD_LDAP_ENVIRONMENT, null);
			// ActiveDirectory Search
			NamingEnumeration searchResults = ldapContext.search(ldapEnvironment.getAdSearchBase(), searchFilter, SEARCH_CONTROLS);
			// no results found
			if (searchResults.hasMoreElements() == false) {
				response.setFailure(true);
				LinkedHashMap<String, String>  validationErrors=response.getValidationErrors();
				validationErrors.put(
						"No VA staff members were found matching your search criteria.", "No VA staff members were found matching your search criteria.");
				response.setValidationErrors(validationErrors);
				throw new MHVException(response.getInfoMessages(), response.getValidationErrors(), response.getFailureMessage());
				
			} else {
				while (searchResults.hasMoreElements()) {
					SearchResult searchResult = (SearchResult) searchResults.next();
					Attributes searchResultAttributes = searchResult.getAttributes();
					if (searchResultAttributes != null) {
						employeeSearchResultList.add(convertToEmployeeSearchResult(searchResultAttributes));
					}
				}
			}
		}catch (InsufficientResourcesException e) {
			log.error("InsufficientResourcesException while trying to connect to Active Directory: ", e);
			response.setFailure(true);
			throw new MHVException("InsufficientResourcesException while trying to connect to Active Directory:");			
		}
		catch (LimitExceededException e) {
			log.error("LimitExceededException while trying to connect to Active Directory: ", e);
			response.setFailure(true);
			throw new MHVException("LimitExceededException while trying to connect to Active Directory: ");
		} catch (NamingException e) {
			log.error("Failed to connect to Active Directory: ", e);
			response.setFailure(true);
			throw new MHVException("Failed to connect to Active Directory:");
		} 
		finally {
			try {
				if (ldapContext != null) {
					ldapContext.close();
				}
			} catch (NamingException e) {
				log.error("Failed to close connection to Active Directory: ", e);
				response.setFailure(true);
				throw new MHVException(messagesUtil.getEmployeeSearchFailed());
			} finally {
				ldapContext = null; // release handle to connection regardless
			}
		}
		return employeeSearchResultList;
	}

}
