package gov.va.med.mhv.admin.util.mvi;

import gov.va.med.mhv.admin.handler.MviProperties;
import gov.va.med.mhv.admin.util.mvi.MviConstants;
import gov.va.med.mhv.mvi.xsd.ActClassControlAct;
import gov.va.med.mhv.mvi.xsd.CD;
import gov.va.med.mhv.mvi.xsd.COCTMT090100UV01AssignedPerson;
import gov.va.med.mhv.mvi.xsd.COCTMT090100UV01Person;
import gov.va.med.mhv.mvi.xsd.CS;
import gov.va.med.mhv.mvi.xsd.CommunicationFunctionType;
import gov.va.med.mhv.mvi.xsd.EN;
import gov.va.med.mhv.mvi.xsd.EntityClassDevice;
import gov.va.med.mhv.mvi.xsd.II;
import gov.va.med.mhv.mvi.xsd.INT;
import gov.va.med.mhv.mvi.xsd.MCCIMT000100UV01Device;
import gov.va.med.mhv.mvi.xsd.MCCIMT000100UV01Receiver;
import gov.va.med.mhv.mvi.xsd.MCCIMT000100UV01Sender;
import gov.va.med.mhv.mvi.xsd.PRPAIN201305UV02Document;
import gov.va.med.mhv.mvi.xsd.PRPAIN201305UV02Document.PRPAIN201305UV02;
import gov.va.med.mhv.mvi.xsd.PRPAIN201305UV02QUQIMT021001UV01ControlActProcess;
import gov.va.med.mhv.mvi.xsd.PRPAMT201306UV02ParameterList;
import gov.va.med.mhv.mvi.xsd.PRPAMT201306UV02QueryByParameter;
import gov.va.med.mhv.mvi.xsd.QUQIMT021001UV01DataEnterer;
import gov.va.med.mhv.mvi.xsd.TS;
import gov.va.med.mhv.mvi.xsd.XActMoodIntentEvent;

import java.math.BigInteger;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.apache.xmlbeans.XmlOptions;
import org.springframework.stereotype.Component;

@Component
public class CreateMvi1305SearchRequest {

	private final Logger log = LogManager.getLogger(CreateMvi1305SearchRequest.class);

	@SuppressWarnings("rawtypes")
	public Object createPatientSearchXMLRequest(String icn, MviProperties properties, String versionInfo) {

		//Jazz Id: 230989 - adding HL7v3 Version code in MVI Request Header.
		String dataEntererId = "";
		String given = "";
		String family = "";
		
		if(versionInfo != null) {
			String[] versionProps = versionInfo.split(",");
			for (String prop : versionProps) {
				if(prop.startsWith("dataEntererId")) {
					dataEntererId = prop.split("=")[1];
				} else if(prop.startsWith("given")) {
					given = prop.split("=")[1];
				} else if(prop.startsWith("family")) {
					family = prop.split("=")[1];
				}
			}
		}
		
		PRPAIN201305UV02Document root1305Doc = PRPAIN201305UV02Document.Factory.newInstance();
		PRPAIN201305UV02 root1305Req = root1305Doc.addNewPRPAIN201305UV02();
		
		root1305Req.setITSVersion(MviConstants.MVI_XML_REQ_ITS_VERSION);
		/*
		<urn:id root="2.16.840.1.113883.3.933" extension="MCID-12345"/>
			<urn:creationTime value="20070428150301 "/>
			<urn:interactionId root="2.16.840.1.113883.1.6"
		 */
		II rootId = root1305Req.addNewId();
		rootId.setRoot(properties.getMviRootId());

		String mcidExtension = MviUtil.getMCIDExtension();
		long mcidUniqValue = MviUtil.getUniqueId();

		rootId.setExtension(mcidExtension + String.valueOf(mcidUniqValue));

		TS creationTime = root1305Req.addNewCreationTime();
		creationTime.setValue(MviUtil.getCreationTime());

		II interactionId = root1305Req.addNewInteractionId();
		interactionId.setRoot(properties.getMviInteractionId());
		interactionId.setExtension(MviConstants.MVI_INTERACTION_SEARCH_TYPE);
		/*
		    <urn:processingCode code="T "/>
		    <urn:processingModeCode code="R "/>
		    <urn:acceptAckCode code="AL "/>
		 */
		CS processingCode = root1305Req.addNewProcessingCode();
		if (properties.getMviEnvType().equalsIgnoreCase("prod"))
			processingCode.setCode(MviConstants.MVI_PROCESSING_PROD_CODE);
		else
			processingCode.setCode(MviConstants.MVI_PROCESSING_NONPROD_CODE);
		CS processingModeCode = root1305Req.addNewProcessingModeCode();
		processingModeCode.setCode(MviConstants.MVI_PROCESSING_MODE_CODE);

		CS acceptAckCode = root1305Req.addNewAcceptAckCode();
		acceptAckCode.setCode(MviConstants.MVI_ACCEPT_ACK_CODE);
		/*
		<urn:receiver typeCode="RCV">
		    <urn:device classCode="DEV" determinerCode="INSTANCE">
		        <urn:id root="2.16.840.1.113883.4.349 "/>
		        <urn:telecom value="https://example.org/PatientFeed"/>
		    </urn:device>
		</urn:receiver>
		 */
		MCCIMT000100UV01Receiver receiver = root1305Req.addNewReceiver();
		receiver.setTypeCode(CommunicationFunctionType.RCV);
		MCCIMT000100UV01Device device = receiver.addNewDevice();
		device.setClassCode(EntityClassDevice.DEV);
		device.setDeterminerCode(MviConstants.MVI_DETERMINER_CODE);
		II deviceId = device.addNewId();
		deviceId.setRoot(properties.getVaOidValue());
		// TEL deviceTelecom = device.addNewTelecom();
		// deviceTelecom.setValue("http://servicelocation/PDQuery");

		/*
		<urn:sender typeCode="SND">
		    <urn:device classCode="DEV" determinerCode="INSTANCE">
		        <urn:id extension="200MH " root="2.16.840.1.113883.3.933"/>
		    </urn:device>
		</urn:sender>
		 */
		MCCIMT000100UV01Sender sender = root1305Req.addNewSender();
		sender.setTypeCode(CommunicationFunctionType.SND);
		MCCIMT000100UV01Device senderDevice = sender.addNewDevice();
		senderDevice.setClassCode(EntityClassDevice.DEV);
		senderDevice.setDeterminerCode(MviConstants.MVI_DETERMINER_CODE);
		II senderId = senderDevice.addNewId();
		senderId.setExtension(properties.getMviReceivingFacilityId());
		senderId.setRoot(properties.getMviRootId());

		/*
		<urn:controlActProcess classCode="CACT" moodCode="EVN">
		<urn:code codeSystem="2.16.840.1.113883.1.6" code="PRPA_TE201305UV02"/>
		<urn:queryByParameter>

		 */
		PRPAIN201305UV02QUQIMT021001UV01ControlActProcess controlActProcess = root1305Req.addNewControlActProcess();
		root1305Req.setControlActProcess(controlActProcess);
		controlActProcess.setClassCode(ActClassControlAct.CACT);
		controlActProcess.setMoodCode(XActMoodIntentEvent.EVN);
		
		//Jazz Id: 230989 - adding HL7v3 Version code in MVI Request Header.
		if(versionInfo != null) {
			QUQIMT021001UV01DataEnterer dataEnterer = controlActProcess.addNewDataEnterer();
			dataEnterer.setContextControlCode("AP");
			dataEnterer.setTypeCode("ENT");
			COCTMT090100UV01AssignedPerson assignedPersons = dataEnterer.addNewAssignedPerson();
			assignedPersons.setClassCode("ASSIGNED");
			
			II assignedPersonDataEntererId = assignedPersons.addNewId();
			
			if(dataEntererId != null && dataEntererId.length() > 0) {
				assignedPersonDataEntererId.setExtension(dataEntererId);
			}

			assignedPersonDataEntererId.setRoot(properties.getMviRootId());
			
			COCTMT090100UV01Person assignedPerson = assignedPersons.addNewAssignedPerson();
			assignedPerson.setDeterminerCode("INSTANCE");
			assignedPerson.setClassCode("PSN");
			EN en = assignedPerson.addNewName();
			if(given != null && given.length() > 0) {
				en.addGiven(given);
			}
			
			if(family != null && family.length() > 0) {
				en.addFamily(family);
			}
		}
		
		
		CD controlCode = controlActProcess.addNewCode();
		controlCode.setCodeSystem(properties.getMviInteractionId());
		controlCode.setCode(MviConstants.MVI_SEARCH_CONTROL_CODE);

		/*
		<urn:queryId root="2.16.840.1.113883.3.933" extension="18204"/>
		<urn:statusCode code="new"/>
		<urn:initialQuantity value="1"/>
		 */
		PRPAMT201306UV02QueryByParameter controlQueryParameter = controlActProcess.addNewQueryByParameter();
		II controlQueryId = controlQueryParameter.addNewQueryId();
		controlQueryId.setRoot(properties.getMviRootId());
		controlQueryId.setExtension(String.valueOf(mcidUniqValue));

		CS controlStatusCode = controlQueryParameter.addNewStatusCode();
		controlStatusCode.setCode(MviConstants.MVI_NEW_STATUS_CODE);
		INT initialQuantity = controlQueryParameter.addNewInitialQuantity();
		initialQuantity.setValue(new BigInteger("1"));
		CS modifyCode = controlQueryParameter.addNewModifyCode();
		modifyCode.setCode("MVI.COMP1.AMS");
		PRPAMT201306UV02ParameterList parameterList = controlQueryParameter.addNewParameterList();
		
		CS versionCode = root1305Req.addNewVersionCode();
		versionCode.setCode("4.0");
        root1305Req.setVersionCode(versionCode);

        II id = parameterList.addNewId();
               
    	// String ICN="1012581128V549975";
		String identfierExtensionFinal =
			icn.concat(MviConstants.CARET_SYMBOL + properties.getMviNationalIdentifier());
		// String identfierExtensionFinal = icn.concat(MviConstants.CARET_SYMBOL + properties.getMviNationalIdentifier());
		// patientIdentifierValue.setRoot(properties.getVaOidValue());
//		if (properties.getMviEnvType().equalsIgnoreCase("prod"))
//			id.setRoot(properties.getMviProdDodrootId());
//		else
//			id.setRoot(properties.getMviNonprodDodrootId());
		
		id.setRoot(properties.getMviRootId());
		id.setExtension(identfierExtensionFinal);
		//patientIdentifierValue.setExtension(identfierExtensionFinal);
		//patientIdentifier.setSemanticsText(MviConstants.MVI_PATIENT_ID);
		
		// Printing the Request XML For fine output
		XmlOptions options = new XmlOptions();
		options.put(XmlOptions.LOAD_LINE_NUMBERS);
		options.setSavePrettyPrint();
		options.setSavePrettyPrintIndent(4);
		if (log.isDebugEnabled()) {
			log.debug("MVI Patient Search Request 1305Doc \n" + root1305Doc.xmlText(options));
		}

		// TEST the validity of an XML
		List validationErrors = new ArrayList();
		options.setErrorListener(validationErrors);

		boolean valid = root1305Doc.validate(options);
		if (valid) {
			if (log.isDebugEnabled()) {
				log.debug("Its valid 1305 Search Request xml");
			}
		} else {
			// CR1964 - Make sure that the code doesn't go into the infinite loop.
			Iterator itr = validationErrors.iterator();
			if (log.isDebugEnabled()) {
				while (itr.hasNext()) {
					log.error("Not a valid 1305 Search xml : " + itr.next().toString());
				}
			}
		}
		return root1305Doc.xmlText();
	}	
	
}
