package gov.va.med.mhv.admin.web.controller;

import java.sql.Timestamp;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.el.ValueExpression;
import javax.faces.application.FacesMessage;
import javax.faces.bean.ManagedBean;
import javax.faces.bean.ManagedProperty;
import javax.faces.bean.ViewScoped;
import javax.faces.context.FacesContext;
import javax.faces.event.ComponentSystemEvent;
import javax.portlet.PortletRequest;
import javax.portlet.PortletSession;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.primefaces.component.datatable.DataTable;
import org.primefaces.event.data.SortEvent;
import org.springframework.beans.factory.annotation.Autowired;

import com.fasterxml.jackson.databind.ObjectMapper;

import gov.va.med.mhv.admin.dto.ActivityDTO;
import gov.va.med.mhv.admin.enums.ActivityActorTypeEnumeration;
import gov.va.med.mhv.admin.service.mhvusers.AccountActivityLogService;
import gov.va.med.mhv.admin.web.formbean.UserProfileFormBean;
import gov.va.med.mhv.admin.web.util.WebUtility;
import gov.va.med.mhv.common.api.dto.UserProfileDTO;
import gov.va.med.mhv.common.api.exception.MHVException;

@ManagedBean(name = "accountActivityController")
@ViewScoped
public class AccountActivityController {

	@SuppressWarnings("unused")
	private static final long serialVersionUID = -1947472689045056321L;
	private static final Logger LOG = LogManager.getLogger(AccountActivityController.class);

	private ActivityDTO selectedActivity = new ActivityDTO();
	private List<ActivityDTO> activities;
	private String firstName;
	private String lastName;
	private String tillDate;
	private String minimumDate;
	private String maximumDate;
	private static final String MMDDYYYY_FORMAT = "MM/dd/yyyy";
	private static final String USERPROFILE_DTO_KEY = "LIFERAY_SHARED_userprofiledto";
	protected int rowsPerPage;
	protected ValueExpression sortColumn;
	protected String sortBy;

	@Autowired
	private ObjectMapper mapper;

	@ManagedProperty(value = "#{userProfileInfo}")
	UserProfileFormBean userProfileInfo;

	@ManagedProperty(value = "#{accountActivityProxy}")
	private AccountActivityLogService accountActivityProxy;

	// Initialization method
	public void init(ComponentSystemEvent event) {
		FacesContext context = FacesContext.getCurrentInstance();
		System.out.println("TTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTT REACHED HERE 1");

		setCalendarDisplayDates();
		userProfileInfo.setActivitySummaryTable(
				(DataTable) FacesContext.getCurrentInstance().getViewRoot().findComponent("accountActivity:actvtList"));
		if (!context.isPostback()) {
			setRowsPerPage(10);
			Calendar cal = Calendar.getInstance();
			userProfileInfo.getSearchActivityCriteria().setToDate(cal.getTime());
			cal.add(Calendar.YEAR, -3);
			userProfileInfo.getSearchActivityCriteria().setFromDate(cal.getTime());
			tillDate = WebUtility.dateToString(cal.getTime(), "MMM dd, yyyy");
			userProfileInfo.getSearchActivityCriteria()
					.setPerformerType(ActivityActorTypeEnumeration.EVERYONE.getDescription());
			// PortletRequest request = (PortletRequest)
			// FacesContext.getCurrentInstance().getExternalContext()
			// .getRequest();
			// User user = null;
			// UserProfileDTO userProfile = null;
			if (!FacesContext.getCurrentInstance().isPostback()) {
				setRowsPerPage(10);
			}
		} else {
			if (sortColumn != null && sortBy != null) {
				userProfileInfo.getActivitySummaryTable().setValueExpression("sortBy", sortColumn);
				userProfileInfo.getActivitySummaryTable().setSortOrder(sortBy);
			}
		}
	}

	protected UserProfileDTO getUserProfileDTOFromSession() throws MHVException {
		UserProfileDTO userProfile;
		PortletSession session = null;
		try {
			PortletRequest request = (PortletRequest) FacesContext.getCurrentInstance().getExternalContext()
					.getRequest();
			session = request.getPortletSession();

			String patientStr = (String) session.getAttribute(USERPROFILE_DTO_KEY, PortletSession.APPLICATION_SCOPE);
			userProfile = mapper.readValue(patientStr, UserProfileDTO.class);

		} catch (Exception ex) {
			throw new MHVException("Unable to get UserProfileDTO from session: " + ex);
		}
		return userProfile;
	}

	public String accountSummaryPrint() {
		// resetMessages();
		userProfileInfo.setActivitySummaryTable(
				(DataTable) FacesContext.getCurrentInstance().getViewRoot().findComponent("accountActivity:actvtList"));
		return "account-summary-print?faces-redirect=true";
	}

	private void setCalendarDisplayDates() {
		Calendar calendar = Calendar.getInstance();
		int year = calendar.get(Calendar.YEAR);
		calendar.set(year, 11, 31, 0, 0, 0);
		maximumDate = WebUtility.dateToString(calendar.getTime(), MMDDYYYY_FORMAT);
		calendar.set(year - 3, 0, 01, 0, 0, 0);
		minimumDate = WebUtility.dateToString(calendar.getTime(), MMDDYYYY_FORMAT);
	}

	// Display Activity button call coming from User Details page
	public String displayActivity() {
		setCalendarDisplayDates();
		Calendar cal = Calendar.getInstance();
		userProfileInfo.getSearchActivityCriteria().setToDate(cal.getTime());
		cal.add(Calendar.YEAR, -3);
		userProfileInfo.getSearchActivityCriteria().setFromDate(cal.getTime());
		tillDate = WebUtility.dateToString(cal.getTime(), "MMM dd, yyyy");
		userProfileInfo.getSearchActivityCriteria()
				.setPerformerType(ActivityActorTypeEnumeration.EVERYONE.getDescription());
        
		//String toDate = dateToString(userProfileInfo.getSearchActivityCriteria().getToDate(), "dd/MM/yyyy HH:mm:ss");
		//String fromDate = dateToString(userProfileInfo.getSearchActivityCriteria().getFromDate(), "dd/MM/yyyy HH:mm:ss");
		
		//String toDate = (new Timestamp(userProfileInfo.getSearchActivityCriteria().getToDate()).toString();
		//String fromDate = new Timestamp(userProfileInfo.getSearchActivityCriteria().getFromDate().toString();
		Timestamp toDate = new Timestamp(userProfileInfo.getSearchActivityCriteria().getToDate().getTime());
		Timestamp fromDate = new Timestamp(userProfileInfo.getSearchActivityCriteria().getFromDate().getTime());

		String userName = FacesContext.getCurrentInstance().getExternalContext().getRequestParameterMap()
				.get("userName");

		if (userName == null) {
			userName = userProfileInfo.getSelectedProfile().getUserName();
		}
		System.out.println("%%%%%%%%%%%%%%%%%%%%%%%%% userName " + userProfileInfo.getSelectedProfile().getUserName());
		// searchActivityCriteria.setUserName(userName);
		System.out.println("TTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTT USERNAME: " + userName);
		System.out.println("TTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTT FROMDATE: " + fromDate);
		System.out.println("TTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTT TODATE: " + toDate);
		System.out.println("TTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTT PERFORMERTYPE: "
				+ userProfileInfo.getSearchActivityCriteria().getPerformerType());
		setRowsPerPage(10);
		userProfileInfo.setActivities(null);
		userProfileInfo.setActivities(findActivitiesBySearchTimestamp(userName, fromDate, toDate,
				userProfileInfo.getSearchActivityCriteria().getPerformerType()));
		if (userProfileInfo.getActivities() != null && userProfileInfo.getActivities().size() > 0) {

		}
		return "account-activity-summary?faces-redirect=true";
	}

	// Back to Summary
	public String returnToSummary() {

		userProfileInfo.getSearchActivityCriteria()
				.setPerformerType(ActivityActorTypeEnumeration.EVERYONE.getDescription());

		//String toDate = dateToString(userProfileInfo.getSearchActivityCriteria().getToDate(), "dd/MM/yyyy HH:mm:ss");
		//String fromDate = dateToString(userProfileInfo.getSearchActivityCriteria().getFromDate(),
		//		"dd/MM/yyyy HH:mm:ss");
		Timestamp toDate = new Timestamp(userProfileInfo.getSearchActivityCriteria().getToDate().getTime());
		Timestamp fromDate = new Timestamp(userProfileInfo.getSearchActivityCriteria().getFromDate().getTime());		

		String userName = FacesContext.getCurrentInstance().getExternalContext().getRequestParameterMap()
				.get("userName");

		if (userName == null) {

			userName = userProfileInfo.getSelectedProfile().getUserName();
		}
		// searchActivityCriteria.setUserName(userName);
		userProfileInfo.setActivities(null);
		userProfileInfo.setActivities(findActivitiesBySearchTimestamp(userName, fromDate, toDate,
				userProfileInfo.getSearchActivityCriteria().getPerformerType()));
		if (userProfileInfo.getActivities() != null && userProfileInfo.getActivities().size() > 0) {

		}
		return "account-activity-summary?faces-redirect=true";
	}

	// Search
	public String searchByCriteria() {

		//String toDate = dateToString(userProfileInfo.getSearchActivityCriteria().getToDate(), "dd/MM/yyyy HH:mm:ss");
		//String fromDate = dateToString(userProfileInfo.getSearchActivityCriteria().getFromDate(),
		//		"dd/MM/yyyy HH:mm:ss");

		Timestamp toDate = new Timestamp(userProfileInfo.getSearchActivityCriteria().getToDate().getTime());
		Timestamp fromDate = new Timestamp(userProfileInfo.getSearchActivityCriteria().getFromDate().getTime());		
		
		String userName = FacesContext.getCurrentInstance().getExternalContext().getRequestParameterMap()
				.get("userName");

		if (userName == null) {
			userName = userProfileInfo.getSelectedProfile().getUserName();
		}
		System.out.println("%%%%%%%%%%%%%%%%%%%%%%%%% userName " + userProfileInfo.getSelectedProfile().getUserName());
		// searchActivityCriteria.setUserName(userName);
		System.out.println("TTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTT USERNAME" + userName);
		System.out.println("TTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTT FROMDATE"
				+ userProfileInfo.getSearchActivityCriteria().getFromDate());
		System.out.println(
				"TTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTT TODATE" + userProfileInfo.getSearchActivityCriteria().getToDate());
		System.out.println("TTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTT PERFORMERTYPE: "
				+ userProfileInfo.getSearchActivityCriteria().getPerformerType());

		setRowsPerPage(10);

		userProfileInfo.setActivities(null);
		userProfileInfo.setActivities(findActivitiesBySearchTimestamp(userName, fromDate, toDate,
				userProfileInfo.getSearchActivityCriteria().getPerformerType()));

		if (userProfileInfo.getActivities() != null && userProfileInfo.getActivities().size() > 0) {

			for (ActivityDTO a : userProfileInfo.getActivities()) {
				System.out.println("%%%%%%%%%%%%%%%%%%%%%%%%% REACHED HERE 0" + a.getActivityType());
			}

		}
		// System.out.println("TTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTT REACHED HERE 4"
		// + userProfileInfo.getActivities().size());
		return "account-activity-summary?faces-redirect=true";
	}

	private List<ActivityDTO> findActivitiesBySearchTimestamp(String userName, Timestamp fromDate, Timestamp toDate,
			String performerType) {
		List<ActivityDTO> dtoList = null;
		try {
			dtoList = accountActivityProxy.getAccountActvityBySearch(userName, fromDate, toDate, performerType);
			updateSearchDatesWithTime();
			if (dtoList != null && dtoList.size() > 0) {
				System.out.println("TTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTT dtoList.size()" + dtoList.size());
			}

			int correctedCount = correctSortOrder(dtoList);
			if (LOG.isDebugEnabled()) {
				LOG.debug("Corrected " + correctedCount + " records sort order.");
			}

		} catch (MHVException e) {
			LOG.error("Error in findActivities", e);
			// super.processErrorMessages(e);
		} catch (Exception e) {
			LOG.error("Error in Find Activities:", e);
			FacesContext.getCurrentInstance().addMessage(null, new FacesMessage(FacesMessage.SEVERITY_ERROR,
					"Error Processing request", "Error initializing account activity data view."));
		}

		return dtoList;
	}
	
	/*
	private List<ActivityDTO> findActivitiesBySearch(String userName, Date fromDate, Date toDate,
			String performerType) {
		List<ActivityDTO> dtoList = null;
		try {
			dtoList = accountActivityProxy.getAccountActvityBySearch(userName, fromDate, toDate, performerType);
			updateSearchDatesWithTime();
			if (dtoList != null && dtoList.size() > 0) {
				System.out.println("TTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTT dtoList.size()" + dtoList.size());
			}

			int correctedCount = correctSortOrder(dtoList);
			if (LOG.isDebugEnabled()) {
				LOG.debug("Corrected " + correctedCount + " records sort order.");
			}

		} catch (MHVException e) {
			LOG.error("Error in findActivities", e);
			// super.processErrorMessages(e);
		} catch (Exception e) {
			LOG.error("Error in Find Activities:", e);
			FacesContext.getCurrentInstance().addMessage(null, new FacesMessage(FacesMessage.SEVERITY_ERROR,
					"Error Processing request", "Error initializing account activity data view."));
		}

		return dtoList;
	}
	*/	

	public AccountActivityLogService getAccountActivityProxy() {
		return accountActivityProxy;
	}

	public void setAccountActivityProxy(AccountActivityLogService accountActivityProxy) {
		this.accountActivityProxy = accountActivityProxy;
	}

	private static String dateToString(Date date, String format) {
		String stringDate = "";
		try {
			if (date != null) {
				SimpleDateFormat sdf = new SimpleDateFormat(format);
				stringDate = sdf.format(date);
			}
		} catch (Exception e) {
			throw new RuntimeException("Invalid Date format: " + e);
		}
		return stringDate;
	}
	
	private static String dateToTimestamp(Timestamp tstamp, String format) {
		String stringTimestamp = "";
		try {
			if (tstamp != null) {
				SimpleDateFormat sdf = new SimpleDateFormat(format);
				stringTimestamp = sdf.format(tstamp);
			}
		} catch (Exception e) {
			throw new RuntimeException("Invalid TimeStamp format: " + e);
		}
		return stringTimestamp;
	}	

	public String showDetail(ActivityDTO activityDTO) {
		userProfileInfo.setSelectedActivity(activityDTO);
		return "account-activity-detail?faces-redirect=true";
	}

	public void onSort(SortEvent event) {
		userProfileInfo.getActivitySummaryTable().setSortBy(event.getSortColumn().getValueExpression("sortBy"));
		userProfileInfo.getActivitySummaryTable().setSortOrder(event.isAscending() ? "ascending" : "descending");
	}

	private int correctSortOrder(List<ActivityDTO> dtoList) {
		int count = 0;

		// ADD SEQUENCIAL MILLISECOND IF TWO RECORDS HAVE EXACT TIMES TO SECONDS
		Map<Date, List<Long>> dateMap = new HashMap<Date, List<Long>>();

		// ADD UNSORTED ID'S
		for (ActivityDTO a : dtoList) {
			Date d = a.getCompletionTime();
			if (!dateMap.containsKey(d)) {
				dateMap.put(d, new ArrayList<Long>());
			}
			dateMap.get(d).add(a.getActivityId());
		}

		// REMOVE THE DATE KEY IF THERE IS ONLY ONE IN THE SAME MINUTE
		List<Date> dates = new ArrayList<Date>();
		for (Date d : dateMap.keySet()) {
			dates.add(d);
		}

		for (Date d : dates) {
			if (dateMap.get(d).size() > 1) {
				Collections.sort(dateMap.get(d));
			} else {
				dateMap.remove(d);
			}
		}

		// IF THERE ARE DUPLICATE RECORDS BASED ON DATE/TIME DOWN TO THE SECOND
		// THEN ASSIGN MILLISECOND IN ORDER BASED ON SEQ ID
		Calendar c = Calendar.getInstance();
		for (ActivityDTO a : dtoList) {
			if (dateMap.containsKey(a.getCompletionTime())) {
				int idx = dateMap.get(a.getCompletionTime()).indexOf(a.getActivityId());
				c.setTime(a.getCompletionTime());
				c.set(Calendar.MILLISECOND, idx);
				a.setCompletionTime(c.getTime());
				count++;
			}
		}

		return count;
	}

	private void updateSearchDatesWithTime() {
		Date to_Date = userProfileInfo.getSearchActivityCriteria().getToDate();
		if (to_Date != null) {
			Calendar calendar = Calendar.getInstance();
			calendar.setTime(to_Date);
			int year = calendar.get(Calendar.YEAR);
			int month = calendar.get(Calendar.MONTH);
			int date = calendar.get(Calendar.DATE);
			calendar.set(year, month, date, 23, 59, 59);
			userProfileInfo.getSearchActivityCriteria().setToDate(calendar.getTime());
		}
	}

	public int getRowsPerPage() {
		return rowsPerPage;
	}

	public void setRowsPerPage(int rowsPerPage) {
		this.rowsPerPage = rowsPerPage;
	}

	public void setActivities(List<ActivityDTO> activities) {
		this.activities = activities;
	}

	public ValueExpression getSortColumn() {
		return sortColumn;
	}

	public void setSortColumn(ValueExpression sortColumn) {
		this.sortColumn = sortColumn;
	}

	public String getSortBy() {
		return sortBy;
	}

	public void setSortBy(String sortBy) {
		this.sortBy = sortBy;
	}

	public UserProfileFormBean getUserProfileInfo() {
		return userProfileInfo;
	}

	public void setUserProfileInfo(UserProfileFormBean userProfileInfo) {
		this.userProfileInfo = userProfileInfo;
	}

	public ActivityDTO getSelectedActivity() {
		return selectedActivity;
	}

	public void setSelectedActivity(ActivityDTO selectedActivity) {
		this.selectedActivity = selectedActivity;
	}

	public String getFirstName() {
		return firstName;
	}

	public void setFirstName(String firstName) {
		this.firstName = firstName;
	}

	public String getLastName() {
		return lastName;
	}

	public void setLastName(String lastName) {
		this.lastName = lastName;
	}

	public String getTillDate() {
		return tillDate;
	}

	public void setTillDate(String tillDate) {
		this.tillDate = tillDate;
	}

	public String getMinimumDate() {
		return minimumDate;
	}

	public void setMinimumDate(String minimumDate) {
		this.minimumDate = minimumDate;
	}

	public String getMaximumDate() {
		return maximumDate;
	}

	public void setMaximumDate(String maximumDate) {
		this.maximumDate = maximumDate;
	}

	public int getPages() {
		return this.userProfileInfo.getActivitySummaryTable().getPageCount();
	}

	public List<ActivityDTO> getActivities() {
		return activities;
	}

}
