package gov.va.med.mhv.admin.web.controller;

import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Map;


import javax.faces.application.FacesMessage;
import javax.faces.bean.ManagedBean;
import javax.faces.bean.ManagedProperty;
import javax.faces.bean.RequestScoped;
import javax.faces.context.FacesContext;
import javax.faces.event.ComponentSystemEvent;
import javax.ws.rs.BadRequestException;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.BeansException;

import gov.va.med.mhv.admin.dto.EmployeeDTO;
import gov.va.med.mhv.admin.dto.EmployeeOrgRoleDTO;
import gov.va.med.mhv.admin.dto.EmployeeOrgRoleHistoryDTO;
import gov.va.med.mhv.admin.dto.EmployeeSearchResult;
import gov.va.med.mhv.admin.service.vastaff.AdminUserService;
import gov.va.med.mhv.admin.service.vastaff.EmployeeRoleManagementService;
import gov.va.med.mhv.admin.service.vastaff.EmployeeSearchService;
import gov.va.med.mhv.admin.web.formbean.CommentsFormBean;
import gov.va.med.mhv.admin.web.formbean.ManagedStaffMember;
import gov.va.med.mhv.admin.web.formbean.SearchResultsFormBean;
import gov.va.med.mhv.admin.web.util.MessageUtil;
import gov.va.med.mhv.admin.web.util.SortUtil;
import gov.va.med.mhv.admin.web.util.SpringContext;
import gov.va.med.mhv.common.api.exception.MHVException;

@ManagedBean(name = "staffController")
@RequestScoped
public class ManageVAStaffController {

	private static Logger log = LogManager.getLogger(ManageVAStaffController.class);

	@ManagedProperty(value = "#{staffSearchResults}")
	SearchResultsFormBean searchResults;

	@ManagedProperty(value = "#{commentsBean}")
	CommentsFormBean commentsBean;
	
	@ManagedProperty(value = "#{adminUserServiceProxy}")
	private AdminUserService adminUserServiceProxy;

	@ManagedProperty(value = "#{employeeRoleManagementServiceProxy}")
	private EmployeeRoleManagementService employeeRoleManagementServiceProxy;

	@ManagedProperty(value = "#{employeeSearchServiceProxy}")
	private EmployeeSearchService employeeSearchServiceProxy;
	

	public String manageStaffMember(EmployeeOrgRoleDTO employeeOrg) {
		return this.manageStaffMember(employeeOrg.getEmployee().getUserName());
	}

	public String manageStaffMember(EmployeeSearchResult employeeResult) {
		return this.manageStaffMember(employeeResult.getUserName());
	}

	public String manageStaffMember(String userName) {
		String result = "manage-staff-roles";
		/*
		 * if (FacesContext.getCurrentInstance().getMessages() != null) {
		 * FacesContext.getCurrentInstance().getMessages().remove(); }
		 */

		try {
			if (userName != null) {
				ManagedStaffMember staffMember = new ManagedStaffMember();

				// get employee from Active Directory (i.e. Existence Check)
				try {
					List<EmployeeSearchResult> employeeSearchResults = employeeSearchServiceProxy
							.searchByUserName(userName);
					if (employeeSearchResults != null && employeeSearchResults.size() > 0) {
						// OK, the user exists and is permitted
						EmployeeSearchResult ldapEmployeeSearchResult = employeeSearchResults.get(0);
						if (ldapEmployeeSearchResult != null) {
							staffMember.setLdapEmployee(ldapEmployeeSearchResult);
						}
					}
				} catch (MHVException ex) {
					staffMember.setLdapEmployee(null);
					FacesContext.getCurrentInstance().addMessage("No search results.",
							new FacesMessage(FacesMessage.SEVERITY_INFO,
									"No VA staff members were found matching your search criteria.",
									MessageUtil.NO_WRAPPER));
					log.error("FAILURE: No VA staff members were found matching your search criteria: " + ex);
				}

				// get employee data from local database
				EmployeeDTO ldapEmployee = null;
				try {
					ldapEmployee = employeeRoleManagementServiceProxy.getEmployeeByUserName(userName);
					staffMember.setLdapEmployeeToManage(ldapEmployee);
				} catch (MHVException e) {
					log.error("FAILURE: Not able to connect to getEmployeeByUserName API method: " + e);
				}

				searchResults.setManaged(staffMember);

				// also load the comments (by loading the employee object which
				// contains the comments)
				EmployeeDTO emp = getEmployee(userName);
				if (emp == null && ldapEmployee != null) {
					emp = new EmployeeDTO();
					emp.setUserName(ldapEmployee.getUserName());
					emp.setLastName(ldapEmployee.getLastName());
					emp.setFirstName(ldapEmployee.getFirstName());
				}

				if (null != emp) {
					// sort organizations by default sort order
					new SortUtil().imposeDefaultSortOrderManageStaffMember(emp.getEmployeeOrgRoles());
				}

				staffMember.setManagedEmployee(emp);

				// Load the role history
				getRoleHistory();

			}
		} catch (BeansException e) {
			FacesContext.getCurrentInstance().addMessage("Framework Error.", new FacesMessage("Unexpected error."));
			log.error(
					"Error in employeeRoleManagementServiceProxy.getEmployeeOrgRolesAssignedToEmployeeByName() - Searching Proxy 'employeeRoleManagementServiceProxy'",
					e);
		} catch (BadRequestException e) {
			FacesContext.getCurrentInstance().addMessage("Service Error.", new FacesMessage("Service error."));
			log.error(
					"Error in employeeRoleManagementServiceProxy.getEmployeeOrgRolesAssignedToEmployeeByName() - Searching Proxy 'employeeRoleManagementServiceProxy'",
					e);
		}

		// Set the page to Assign Roles tab/view when the page gets called
		searchResults.setActiveIndexValue(0);

		return result;
	}

	// Get Role History
	public List<EmployeeOrgRoleHistoryDTO> getRoleHistory() {
		List<EmployeeOrgRoleHistoryDTO> roleHistoriesForEmployee = searchResults.getManaged()
				.getRoleHistoriesForEmployee();

		if (roleHistoriesForEmployee == null || roleHistoriesForEmployee.size() == 0) {
			try {
				if (searchResults.getManaged().getManagedEmployee() != null) {
					Long employeeId = searchResults.getManaged().getManagedEmployee().getEmployeeId();
					if (employeeId != null) {
						roleHistoriesForEmployee = employeeRoleManagementServiceProxy
								.getRoleHistoriesForEmployee(employeeId);
						new SortUtil().imposeDefaultSortOrderRoleHistory(roleHistoriesForEmployee);
						searchResults.getManaged().setRoleHistoriesForEmployee(roleHistoriesForEmployee);
					}
				}
			} catch (MHVException e) {
				FacesContext.getCurrentInstance().addMessage(null,
						new FacesMessage(FacesMessage.SEVERITY_ERROR, e.getFailureMessage(), null));
				log.error(
						"Error in employeeRoleManagementServiceProxy.getEmployeeOrgRolesAssignedToEmployeeByName() - Searching Proxy 'employeeRoleManagementServiceProxy'",
						e);
			}
		}
		// Set the page to Roles History tab/view when the page gets called
		searchResults.setActiveIndexValue(1);
		return roleHistoriesForEmployee;
	}

	/**
	 * This method looks up the EmployeeDTO object and remembers it for the user
	 * under management. It places it in the searchResults bean that is session
	 * memory for all management for a single user.
	 * 
	 * @param userName
	 * @return
	 */
	public EmployeeDTO getEmployee(String userName) {
		EmployeeDTO employee = null;
		try {
			employee = adminUserServiceProxy.findEmployeeByUserName(userName);
		} catch (BeansException e) {
			FacesContext.getCurrentInstance().addMessage("Application Error.", new FacesMessage("Unexpected error."));
			log.error("Error in adminUserService.getEmployee()", e);
			throw e;
		} catch (MHVException e) {
			FacesContext.getCurrentInstance().addMessage(null,
					new FacesMessage(FacesMessage.SEVERITY_ERROR, e.getFailureMessage(), null));
		}
		return employee;
	}

	public void fetchComments(ComponentSystemEvent event) {
		try {
			if (searchResults.getManaged() != null && searchResults.getManaged().getManagedEmployee() != null) {
				commentsBean.setComments(searchResults.getManaged().getManagedEmployee().getComments());
			}
		} catch (Exception e) {
			log.error("Error in adminUserService.saveComment()", e);
			FacesContext.getCurrentInstance().addMessage(null,
					new FacesMessage(FacesMessage.SEVERITY_ERROR, "Error initializing Comments tab.", null));
		}
	}

	// Save Comments
	public String saveComment() {
		String result = "manage-staff-roles";
		try {
			String comment = commentsBean.getComments();

			// first populate from database field
			String userName = null;
			if (searchResults.getManaged() != null && searchResults.getManaged().getManagedEmployee() != null) {
				userName = searchResults.getManaged().getManagedEmployee().getUserName();
			}

			// if database field not present, populate from LDAP
			if (userName == null) {
				userName = searchResults.getManaged().getLdapEmployee().getUserName();
			}

			// now save
			EmployeeDTO employeeDTO = adminUserServiceProxy.saveEmployee(userName, comment);
			if (searchResults.getManaged() != null) {
				new SortUtil().imposeDefaultSortOrderManageStaffMember(employeeDTO.getEmployeeOrgRoles());
				searchResults.getManaged().setManagedEmployee(employeeDTO);
			}

			searchResults.getManaged().setManagedEmployee(employeeDTO); // update
																		// the
																		// attached
																		// Employee
			FacesContext.getCurrentInstance().addMessage(null,
					new FacesMessage(FacesMessage.SEVERITY_INFO, "The employee has been successfully edited.", null));

			// Set the page to Comments tab/view after successful save to the
			// database
			searchResults.setActiveIndexValue(2);

		} catch (BeansException e) {
			FacesContext.getCurrentInstance().addMessage("Application Error.", new FacesMessage("Unexpected error."));
			log.error("Error in adminUserService.saveComment()", e);
		} catch (MHVException e) {
			FacesContext.getCurrentInstance().addMessage(null,
					new FacesMessage(FacesMessage.SEVERITY_ERROR, e.getFailureMessage(), null));
			log.error("Error in adminUserService.saveComment()", e);
		} catch (Exception e) {
			FacesContext.getCurrentInstance().addMessage(null,
					new FacesMessage(FacesMessage.SEVERITY_ERROR, "Error saving comment.", null));
			log.error("Error in adminUserService.saveComment()", e);
		}

		return result;
	}

	public CommentsFormBean getCommentsBean() {
		return commentsBean;
	}

	public void setCommentsBean(CommentsFormBean commentsBean) {
		this.commentsBean = commentsBean;
	}

	public AdminUserService getAdminUserServiceProxy() {
		return adminUserServiceProxy;
	}

	public void setAdminUserServiceProxy(AdminUserService adminUserServiceProxy) {
		this.adminUserServiceProxy = adminUserServiceProxy;
	}

	public SearchResultsFormBean getSearchResults() {
		return searchResults;
	}

	public void setSearchResults(SearchResultsFormBean searchResults) {
		this.searchResults = searchResults;
	}

	public EmployeeRoleManagementService getEmployeeRoleManagementServiceProxy() {
		return employeeRoleManagementServiceProxy;
	}

	public void setEmployeeRoleManagementServiceProxy(
			EmployeeRoleManagementService employeeRoleManagementServiceProxy) {
		this.employeeRoleManagementServiceProxy = employeeRoleManagementServiceProxy;
	}

	public EmployeeSearchService getEmployeeSearchServiceProxy() {
		return employeeSearchServiceProxy;
	}

	public void setEmployeeSearchServiceProxy(EmployeeSearchService employeeSearchServiceProxy) {
		this.employeeSearchServiceProxy = employeeSearchServiceProxy;
	}
	

}
