package gov.va.med.mhv.admin.web.formbean;

import gov.va.med.mhv.admin.dto.EmployeeDTO;
import gov.va.med.mhv.admin.dto.EmployeeOrgRoleDTO;
import gov.va.med.mhv.admin.dto.EmployeeSearchResult;
import gov.va.med.mhv.admin.dto.OrgDTO;
import gov.va.med.mhv.admin.dto.RoleDTO;
import gov.va.med.mhv.admin.service.vastaff.EmployeeRoleManagementService;
import gov.va.med.mhv.admin.web.controller.ManageVAStaffController;
import gov.va.med.mhv.admin.web.util.MessageProvider;
import gov.va.med.mhv.admin.web.util.SessionUtil;
import gov.va.med.mhv.admin.web.util.SpringContext;
import gov.va.med.mhv.common.api.exception.MHVException;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.TreeSet;
import java.util.Map;

import javax.faces.application.FacesMessage;
import javax.faces.bean.ManagedBean;
import javax.faces.bean.ManagedProperty;
import javax.faces.bean.ViewScoped;
import javax.faces.context.FacesContext;
//import javax.faces.model.SelectItem;
import javax.faces.event.ValueChangeEvent;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.BeansException;

@ManagedBean(name="assignNewRoleForm")
@ViewScoped
public class AssignNewRoleFormBean implements Serializable{
	
	private static final Long NO_SELECTION = -1L;
	private static final String NO_SELECTION_STRING = "no_selection";

	private static final long serialVersionUID = -4812667309686055817L;

	private static Logger log = LogManager.getLogger(AssignNewRoleFormBean.class);
	
	List<OrgDTO> organizations = null;
	Set<String> organizationLevels = null;
	List<OrgDTO> filteredOrganizations = null;
	List<RoleDTO> roles = null;
	
	String selectedOrganizationLevel;
	Long selectedOrganization;
	Long selectedRole;

	
	@ManagedProperty(value="#{staffSearchResults}")
	SearchResultsFormBean searchResults;
	
	@ManagedProperty(value="#{sessionUtil}")
	SessionUtil sessionUtil;
	
	public AssignNewRoleFormBean() {
	}

	
	private RoleDTO getRoleForID(Long id) {
		RoleDTO result = null;
		if (roles != null && id != null) {
			for (RoleDTO o : roles) {
				if (id.equals(o.getRoleId())) {
					result = o;
					break;
				}
			}
		}
		return result;
	}
	
	
	private OrgDTO getOrgForID(Long id) {
		OrgDTO result = null;
		if (organizations != null && id != null) {
			for (OrgDTO o : organizations) {
				if (id.equals(o.getOrgId())) {
					result = o;
					break;
				}
			}
		}
		return result;
	}


	public String activateRole(EmployeeOrgRoleDTO role) {
		
		if (!role.getActive()) {
			try {
					role.setActive(Boolean.TRUE);
					EmployeeRoleManagementService service = (EmployeeRoleManagementService) SpringContext.getApplicationContext().getBean("employeeRoleManagementServiceProxy");
					Long authenticatedEmployeeId = sessionUtil.getCurrentUser().getEmployeeId();
					service.updateOrgRoleStatus(authenticatedEmployeeId, role.getOrgRoleId(), role.getActive());
					
					// lookup, format, and return a success message
					EmployeeDTO emp = searchResults.getManaged().getManagedEmployee();
					String formattedSuccessMessage = new MessageProvider().getMessage("roleEdittedMessage");
					String selectedOrgStr = role.getOrganization().getName();
					String selectedRoleStr = role.getRole().getName();
					formattedSuccessMessage = String.format(formattedSuccessMessage, new Object[] {selectedOrgStr, selectedRoleStr, emp.getFirstName(), emp.getLastName()});
					FacesContext.getCurrentInstance().addMessage(null, new FacesMessage(FacesMessage.SEVERITY_INFO, formattedSuccessMessage, null));
					reloadRoleHistories();
			} catch (BeansException e) {
				role.setActive(Boolean.FALSE);
				FacesContext.getCurrentInstance().addMessage("Application Error.", new FacesMessage("Unexpected error."));
				log.error("Error in manageVAStaffController.activateRole", e);
			} catch (MHVException e) {
				role.setActive(Boolean.FALSE);
				FacesContext.getCurrentInstance().addMessage(null, new FacesMessage(FacesMessage.SEVERITY_ERROR, e.getFailureMessage(), null));
				log.error("Error in manageVAStaffController.activateRole", e);
			}
		}
		else {
			FacesContext.getCurrentInstance().addMessage(null, new FacesMessage("Attempting to activate an already-active role."));
		}
		return "";
	}
	
	public String deactivateRole(EmployeeOrgRoleDTO role) {
		if (role.getActive()) {
			try {
				
				role.setActive(Boolean.FALSE);
				EmployeeRoleManagementService service = (EmployeeRoleManagementService) SpringContext.getApplicationContext().getBean("employeeRoleManagementServiceProxy");
				Long authenticatedEmployeeId = sessionUtil.getCurrentUser().getEmployeeId();
				service.updateOrgRoleStatus(authenticatedEmployeeId, role.getOrgRoleId(), role.getActive());
				
				// lookup, format, and return a success message
				EmployeeDTO emp = searchResults.getManaged().getManagedEmployee();
				String formattedSuccessMessage = new MessageProvider().getMessage("roleEdittedMessage");
				String selectedOrgStr = role.getOrganization().getName();
				String selectedRoleStr = role.getRole().getName();
				formattedSuccessMessage = String.format(formattedSuccessMessage, new Object[] {selectedOrgStr, selectedRoleStr, emp.getFirstName(), emp.getLastName()});
				FacesContext.getCurrentInstance().addMessage(null, new FacesMessage(FacesMessage.SEVERITY_INFO, formattedSuccessMessage, null));
				reloadRoleHistories();
			} catch (BeansException e) {
				role.setActive(Boolean.TRUE);
				FacesContext.getCurrentInstance().addMessage("Application Error.", new FacesMessage("Unexpected error."));
				log.error("Error in manageVAStaffController.activateRole", e);
			} catch (MHVException e) {
				role.setActive(Boolean.TRUE);
				FacesContext.getCurrentInstance().addMessage(null, new FacesMessage(FacesMessage.SEVERITY_ERROR, e.getFailureMessage(), null));
				log.error("Error in manageVAStaffController.activateRole", e);
			}
		}
		else {
			FacesContext.getCurrentInstance().addMessage(null, new FacesMessage("Attempting to deactive an already-deactivated role."));
		}
		
		return "";
	}
	
	public String getSelectedOrganizationLevel() {
		return selectedOrganizationLevel;
	}

	public void setSelectedOrganizationLevel(String selectedOrganizationLevel) {
		this.selectedOrganizationLevel = selectedOrganizationLevel;
	}

	public Long getSelectedOrganization() {
		return selectedOrganization;
	}

	public void setSelectedOrganization(Long selectedOrganization) {
		this.selectedOrganization = selectedOrganization;
	}

	public Long getSelectedRole() {
		return selectedRole;
	}

	public void setSelectedRole(Long selectedRole) {
		this.selectedRole = selectedRole;
	}

	public void setOrganizationLevels(Set<String> organizationLevels) {
		this.organizationLevels = organizationLevels;
	}

	public void setFilteredOrganizations(List<OrgDTO> filteredOrganizations) {
		this.filteredOrganizations = filteredOrganizations;
	}

	public void setRoles(List<RoleDTO> roles) {
		this.roles = roles;
	}

	public Set<String> getOrganizationLevels() {
		if (organizationLevels == null || organizationLevels.isEmpty()) {
			List<OrgDTO> allOrgs = getOrganizations();
			organizationLevels = new TreeSet<String>();
			for (OrgDTO org : allOrgs) {
				organizationLevels.add(org.getTypeOfOrganization());
			}
		}
		return organizationLevels;
	}
	
	public List<OrgDTO> getFilteredOrganizations() {
		if (filteredOrganizations == null || filteredOrganizations.isEmpty()) {
			filteredOrganizations = new ArrayList<OrgDTO>();
	
			if (selectedOrganizationLevel != null && !NO_SELECTION_STRING.equals(selectedOrganizationLevel)) {
				for (OrgDTO org : getOrganizations()) {
					String typeOfOrganization = org.getTypeOfOrganization();
					if (typeOfOrganization != null && typeOfOrganization.equals(selectedOrganizationLevel)) {
						filteredOrganizations.add(org);
					}
				}
			}
		}
		return filteredOrganizations;
	}
	
	
	
	private List<OrgDTO> getOrganizations() {
		if (organizations == null || organizations.isEmpty()) {
			try {
				EmployeeRoleManagementService employeeRoleService = (EmployeeRoleManagementService) SpringContext.getApplicationContext().getBean("employeeRoleManagementServiceProxy");
				EmployeeDTO currentUser = sessionUtil.getCurrentUser();
				if (currentUser != null) {
					Long employeeId = currentUser.getEmployeeId();
					organizations = employeeRoleService.getOrganizationsEmployeeCanManage(employeeId);
				}
			} catch (BeansException e) {
				FacesContext.getCurrentInstance().addMessage("Application Error.", new FacesMessage("Failure loading Organization list."));
				log.error("Error loading Organization list.", e);
			} catch (MHVException e) {
				FacesContext.getCurrentInstance().addMessage(null, new FacesMessage(FacesMessage.SEVERITY_ERROR, "Unable to load organization List: " + e.getFailureMessage(), null));
				log.error("Error loading Organization list.", e);
			}
		}
		if (organizations == null) {
			organizations = new ArrayList<OrgDTO>(0);
		}
		return organizations;
	}
	
	
	public List<RoleDTO> getRoles() {
		if (roles == null) {
			if (selectedOrganization != null && !NO_SELECTION.equals(selectedOrganization)) {
				try {
					EmployeeRoleManagementService employeeRoleService = (EmployeeRoleManagementService) SpringContext.getApplicationContext().getBean("employeeRoleManagementServiceProxy");
					Long empId = sessionUtil.getCurrentUser().getEmployeeId();
					roles = employeeRoleService.getRolesAtOrganizationEmployeeCanManage(empId, selectedOrganization);
				} catch (BeansException e) {
					FacesContext.getCurrentInstance().addMessage("Application Error.", new FacesMessage("Failure loading Role list."));
					log.error("Error loading Organization list.", e);
				} catch (MHVException e) {
					FacesContext.getCurrentInstance().addMessage(null, new FacesMessage(FacesMessage.SEVERITY_ERROR, "Unable to load Role List: " + e.getFailureMessage(), null));
					log.error("Error loading Organization list.", e);
				}
			}
			else {
				FacesContext.getCurrentInstance().addMessage("Please make an Organization selection.", new FacesMessage("Please make an Organization selection."));
			}
		}
		return roles;
	}
	
	public void organizationLevelSelected() { 
		//coordinate selected org, orgtype, role with these select items.
		
		this.selectedOrganization = null;
		this.selectedRole = null;
		
		this.filteredOrganizations = null;
		this.roles = null;
		
	}
	
	public void organizationChanged(ValueChangeEvent e){
		//assign new value to localeCode
		this.selectedOrganization = (Long) e.getNewValue();
	}

	public void roleChanged(ValueChangeEvent e){
		this.selectedRole = (Long) e.getNewValue();
	}

	
	public void organizationSelected() {
		this.selectedRole = null;
		this.roles = null;
	}
	
	public void roleSelected() {
	}
	
	/**
	 * Saves new Role for Managed Employee
	 * @return
	 */
	public String save() {
		String result = "";
		try {
			if (selectedOrganizationLevel == null || NO_SELECTION_STRING.equals(selectedOrganizationLevel)
				|| selectedOrganization == null || NO_SELECTION.equals(selectedOrganization)
				|| selectedRole == null || NO_SELECTION.equals(selectedRole)) {
				FacesContext.getCurrentInstance().addMessage(null, new FacesMessage("Please select an Organization Level, an Organization and a Role before saving a new Role."));
			}
			else {
					EmployeeRoleManagementService employeeRoleService = (EmployeeRoleManagementService) SpringContext.getApplicationContext().getBean("employeeRoleManagementServiceProxy");
					EmployeeDTO authenticatedUser = sessionUtil.getCurrentUser();
					
					// establish userName from LDAP
					EmployeeSearchResult ldapEmployee = searchResults.getManaged().getLdapEmployee();
					String managedUserName = null;
					if (ldapEmployee != null) {
						managedUserName = ldapEmployee.getUserName();
					}
					
					// if userName not in LDAP, establish from EVault Database
					EmployeeDTO managedEmployee = searchResults.getManaged().getManagedEmployee();
					if (managedUserName == null && managedEmployee != null) {
						managedUserName = managedEmployee.getUserName();
					}
					
					EmployeeDTO emp = managedEmployee;
					OrgDTO orgForID = getOrgForID(this.selectedOrganization);
					RoleDTO roleForID = getRoleForID(this.selectedRole);
					EmployeeOrgRoleDTO eorDTO = employeeRoleService.assignEmployeeOrganizationRole(authenticatedUser.getEmployeeId(), managedUserName, orgForID.getOrgId(), roleForID.getRoleId());
					List<EmployeeOrgRoleDTO> employeeOrgRoles = emp.getEmployeeOrgRoles();
					if (employeeOrgRoles == null) {
						employeeOrgRoles = new ArrayList<EmployeeOrgRoleDTO>();
						employeeOrgRoles.add(eorDTO);
					}
					else {
						employeeOrgRoles.add(eorDTO);
					}
					
					
					
					// lookup, format, and return a success message
					String formattedSuccessMessage = new MessageProvider().getMessage("roleAddedMessage");
					String selectedOrgStr = this.getOrgForID(this.selectedOrganization).getName();
					String selectedRoleStr = this.getRoleForID(this.selectedRole).getName();
					formattedSuccessMessage = String.format(formattedSuccessMessage, new Object[] {emp.getFirstName(), emp.getLastName(), selectedOrgStr, selectedRoleStr});
					FacesContext.getCurrentInstance().addMessage(null, new FacesMessage(FacesMessage.SEVERITY_INFO, formattedSuccessMessage, null));
			}
			
			// refresh the list of roles
			reloadManagedUser();
			
			// refresh the historical roles
			reloadRoleHistories();
			
			// Set the page to Assign Roles tab/view when the page gets called
			searchResults.setActiveIndexValue(0);
			//Clear values once user saved a new Role
			setSelectedRole(new Long(-1));

			setSelectedOrganization(new Long(-1));

			setSelectedOrganizationLevel("no_selection");
			//stay on current page
			result = ""; 
			
		} catch (BeansException e) {
			FacesContext.getCurrentInstance().addMessage(null, new FacesMessage(FacesMessage.SEVERITY_ERROR, "Failure saving Role Assignment.", null));
			log.error("Error saving role.", e);
			
		} catch (MHVException e) {
			
				processErrorMessages(e);
				System.out.println("********************************************NOTHING.equals(e.getFailureMessage())");
				//FacesContext.getCurrentInstance().addMessage(null, new FacesMessage(FacesMessage.SEVERITY_ERROR, e.getFailureMessage(), null));
				log.error("Error saving role.", e);
		}
		 catch (Exception e) {
			FacesContext.getCurrentInstance().addMessage(null, new FacesMessage(FacesMessage.SEVERITY_ERROR, "Error saving Role Assignment.", null));
			log.error("Error saving role.", e);
		}
		
		return result;		
	}
	
	
	
	public String reset() {
		this.selectedOrganizationLevel = null;
		this.selectedOrganization = null;
		this.selectedRole = null;
		
		this.filteredOrganizations = null;
		this.roles = null;
		
		return "";
	}
	

	public SearchResultsFormBean getSearchResults() {
		return searchResults;
	}


	public void setSearchResults(SearchResultsFormBean searchResults) {
		this.searchResults = searchResults;
	}


	public SessionUtil getSessionUtil() {
		return sessionUtil;
	}


	public void setSessionUtil(SessionUtil sessionUtil) {
		this.sessionUtil = sessionUtil;
	}
	
	public String reloadManagedUser() {
		FacesContext context = FacesContext.getCurrentInstance();
		ManageVAStaffController controller = (ManageVAStaffController)context.getApplication()
				.evaluateExpressionGet(context, "#{staffController}", ManageVAStaffController.class);
		return controller.manageStaffMember(searchResults.getManaged().getManagedEmployee().getUserName());
	}
	
	public String reloadRoleHistories() {
		String result = null;
		FacesContext context = FacesContext.getCurrentInstance();
		ManageVAStaffController controller = (ManageVAStaffController)context.getApplication()
				.evaluateExpressionGet(context, "#{staffController}", ManageVAStaffController.class);
		searchResults.getManaged().setRoleHistoriesForEmployee(null);
		controller.getRoleHistory();
		return result;
	}
	
	public void processErrorMessages(MHVException exp) {
		// response has errors
		if (null != exp.getErrorCode()) {
			Map<String, String> validationErrors = exp.getValidationErrorMessages();
			validationErrors.put(
					"The employee is already in this role at this organization.", "The employee is already in this role at this organization.");
			if (log.isDebugEnabled()) {
				int size = null != validationErrors ? validationErrors.size() : 0;
				log.debug("validationErrors.size() : " + size);
			}
			
			// check if there are any validation errors in response object
			if (null != validationErrors && validationErrors.size() > 0) {	
				System.out.println("********************************************null != validationErrors ");
				for (Map.Entry<String, String> err : validationErrors.entrySet()) {
					if (log.isDebugEnabled()) {
						if (null != err) {
							log.debug("entry.getKey() : " + err.getKey());
							log.debug("entry.getValue() : " + err.getValue());
						}
					}
					System.out.println("********************************************err.getValue() " + err.getValue());
					FacesContext.getCurrentInstance().addMessage(null, new FacesMessage(FacesMessage.SEVERITY_ERROR, err.getValue(), err.getValue()));
				}
			}else {
				System.out.println("********************************************null == validationErrors" );
			}

			// check for non-validation errors in response object
			if (null != exp.getFailureMessage() && exp.getFailureMessage().length() > 0) {
				if (log.isDebugEnabled()) {
					log.debug("FailureMessage : " + exp.getFailureMessage());
				}

				//FacesContext.getCurrentInstance().addMessage(null, new FacesMessage(FacesMessage.SEVERITY_ERROR, ERR_PRCS_RQST, ERR_PRCS_RQST));
			}
		}
	}
	
}
