package gov.va.med.mhv.admin.authentication;

import javax.security.auth.callback.Callback;
import javax.security.auth.callback.NameCallback;
import javax.security.auth.callback.CallbackHandler;
import javax.security.auth.callback.UnsupportedCallbackException;

/**
 * CallbackHandler for username.
 *
 * This will extract the username from the token and use the NameCallBack to
 * allow the configured authenticator's to build the Subject
 */
class MHVAdminCallbackHandlerImpl implements CallbackHandler {
	private String userName;

	/**
	 * Create instance of the callbackhandler with the username.
	 *
	 * @param user
	 *            A String containing the name of the user from the identity
	 *            assertion token
	 */
	MHVAdminCallbackHandlerImpl(String user) {
		userName = user;
	}

	/**
	 * Used by the authenticators' login modules to get the user name that the
	 * identity asserter extracted from the identity assertion token. This name
	 * can only be retrieved via a NameCallback.
	 *
	 * @param callbacks
	 *            An array of Callback objects indicating what data the login
	 *            module is trying to extract from this callback handler. It
	 *            must only contain NameCallbacks.
	 *
	 * @exception UnsupportedCallbackException
	 *                thrown if any of the callbacks aren't NameCallbacks.
	 *
	 * @see CallbackHandler
	 */
	public void handle(Callback[] callbacks)
			throws UnsupportedCallbackException {
		// loop over the callbacks
		for (int i = 0; i < callbacks.length; i++) {
			Callback callback = callbacks[i];
			if (!(callback instanceof NameCallback)) {
				throw new UnsupportedCallbackException(callback,
						"Invalid Callback type");
			}

			NameCallback nameCallback = (NameCallback) callback;
			nameCallback.setName(userName);
		}
	}
}
