package gov.va.med.mhv.bluebutton.web.bean;

import gov.va.med.mhv.integration.phr.transfer.FacilityExtractStatus;
import gov.va.med.mhv.integration.phr.transfer.PatientExtractStatus;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.List;

import javax.faces.bean.ManagedBean;

import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

@ManagedBean
@Component
@Scope("session")
public class PhrStatusBean implements Serializable {
	private static final long serialVersionUID = -4455323209252190285L;

	private PatientExtractStatus status;

	private Boolean isRefreshCompleteForImaging = false;
	private Boolean isRefreshComplete = false;
	private Boolean isRefreshNeeded = true;
	private Boolean isProgressDomainsExist = false;
	
	public boolean getIsRefreshCompleteForImaging() {
		return isRefreshCompleteForImaging;
	}

	public void setIsRefreshCompleteForImaging(Boolean isRefreshCompleteForImaging) {
		this.isRefreshCompleteForImaging = isRefreshCompleteForImaging;
	}

	public boolean getIsRefreshComplete() {
		return isRefreshComplete;
	}

	public void setIsRefreshComplete(Boolean isRefreshComplete) {
		this.isRefreshComplete = isRefreshComplete;
	}

	public boolean getIsRefreshNeeded() {
		return isRefreshNeeded;
	}

	public void setIsRefreshNeeded(Boolean isRefreshNeeded) {
		this.isRefreshNeeded = isRefreshNeeded;
	}

	public boolean getIsProgressDomainsExist() {
		return isProgressDomainsExist;
	}

	public void setIsProgressDomainsExist(Boolean isProgressDomainsExist) {
		this.isProgressDomainsExist = isProgressDomainsExist;
	}
	
	public boolean getIsPatientExtractStatusLoaded() {
		return !(status == null);
	}

	public PatientExtractStatus getPatientExtractStatus() {
		return status;
	}

	public void setPatientExtractStatus(PatientExtractStatus status) {
		this.status = status;
		
		//Compile and set the statuses once
		this.setIsRefreshComplete(getRefreshComplete());
		this.setIsRefreshCompleteForImaging(getRefreshComplete("ImagingStudy"));
	}

	/**
	 * Check if inProgress and was it updated today?
	 * 
	 * @return
	 */
	private boolean getRefreshComplete() {
		assert(status != null);
		
//		debugStatus();
		
		for (FacilityExtractStatus facilityExtractStatus : status.getFacilityExtractStatusList() ) {
			//Processing Refresh Extrapolation of domain (1-M)
			isProgressDomainsExist = facilityExtractStatus.getLoadStatus()==null;
			if( isProgressDomainsExist ) {
				break;
			}
			
			isProgressDomainsExist = facilityExtractStatus.getLoadStatus().equals("NEW");
			if( isProgressDomainsExist ) {
				break;
			}
		}
		
		return !isProgressDomainsExist;
	}

	/**
	 * Check if inProgress and was it updated today?
	 * 
	 * If the load status is null (means in progress)
	 * Or 'NEW' it is in progress
	 * 
	 * @return
	 */
	private boolean getRefreshComplete(String extract) {
		assert(status != null);
		boolean isComplete = true;
//		debugStatus();
		for (FacilityExtractStatus facilityExtractStatus : status.getFacilityExtractStatusList()) {
			if( facilityExtractStatus.getExtract().equals(extract) ) {
				if( facilityExtractStatus.getLoadStatus() == null ) {
					isComplete = false;
					break;
				}
				else if( facilityExtractStatus.getLoadStatus() != null && !facilityExtractStatus.getLoadStatus().equals("NEW") ) {
					isComplete = false;
					break;
				}
			}
		}
		return isComplete;
	}
	
	public Date getLastModifiedForBBMI() {
		return getLastModifiedForExtract("ImagingStudy");
	}
	
	private Date getLastModifiedForExtract(String extract) {
		Date response = new Date();
		if (status != null && status.getFacilityExtractStatusList() != null) {
			for (FacilityExtractStatus s : status.getFacilityExtractStatusList()) {
				if (s.getExtract().trim().equalsIgnoreCase(extract)) {
					if (s.getLastModified() != null) {
						//response = CommonUtility.dateToStringTimeZone( s.getLastModified(), "MM/dd/yyyy") + " at " + CommonUtility.dateToStringTimeZone(s.getLastModified(), "HH:mm");
						response = s.getLastModified();
						break;
					}
				}
			}
		}
		return response;
	}
	
	/**
	 * 	Appointments
		Allergy
		ChemistryHematology
		WellnessReminders
		DodMilitaryService
		VPR
		ImagingStudy
	 * @return
	 */
	
	public List<String> getInProgressDomains() {
		assert(status != null);
		
		List<String> domains = new ArrayList<String>();
		
		List<String> inProgressExtracts = new ArrayList<String>();
		if(status != null) {
//			debugStatus();
			for( FacilityExtractStatus fes: status.getFacilityExtractStatusList())  {
				if( fes.getLoadStatus()==null || fes.getLoadStatus().equals("NEW") ) {
					inProgressExtracts.add(fes.getExtract());
				}
			}
		}
		
		if(inProgressExtracts.contains("Allergy")) {
			domains.add("VA Allergies");
		}
		
		if(inProgressExtracts.contains("Appointments")) {
			domains.add("VA Appointments");
		}
		
		if(inProgressExtracts.contains("WellnessReminders")) {
			domains.add("VA Wellness Reminders");
		}
		
		if(inProgressExtracts.contains("VPR") && inProgressExtracts.contains("ChemistryHematology")) {
			domains.add("VA Laboratory Results: Chemistry/Hematology/Microbiology");
		} else if ( !inProgressExtracts.contains("VPR") && inProgressExtracts.contains("ChemistryHematology") ) {
			domains.add("VA Laboratory Results: Chemistry/Hematology");
		} else if(inProgressExtracts.contains("VPR") && !inProgressExtracts.contains("ChemistryHematology")) {
			domains.add("VA Laboratory Results: Microbiology");
		}
		
		if(inProgressExtracts.contains("ImagingStudy")) {
			domains.add("VA Medical Imaging and Reports");
		}
		
		if(inProgressExtracts.contains("VPR")) {
			domains.add("VA Admissions and Discharges");
			domains.add("VA Demographics");
			domains.add("VA Electrocardiogram (EKG)");
			domains.add("VA Immunizations");
			domains.add("VA Pathology Reports: Surgical Pathology/Cytology/Electron Microscopy");
			domains.add("VA Problem List");
			domains.add("VA Notes");
			domains.add("VA Radiology");
			domains.add("VA Vitals and Readings");
		}
		
		Collections.sort(domains);

		return domains;
	}
	
	public List<String> getImagingDomain() {
		List<String> domains = new ArrayList<String>();
		domains.add("VA Medical Imaging and Reports");
		return domains;
	}
	
//	private void debugStatus() {
//		for (FacilityExtractStatus f : status.getFacilityExtractStatusList() ) {
//			System.out.println("FacilityExtractStatus: extr=" + f.getExtract() + ",stat=" + f.getLoadStatus() + ",req=" + f.getLastRequested() + ",com=" + f.getLastCompleted());
//		}
//	}
	
	
}
