package gov.va.med.mhv.getcare.common.dto;

import gov.va.med.mhv.getcare.enums.CountryEnumeration;
import gov.va.med.mhv.getcare.enums.FacilityTypeEnumeration;
import gov.va.med.mhv.getcare.enums.StateEnumeration;

import java.io.Serializable;

import com.fasterxml.jackson.annotation.JsonIgnore;

public class TreatmentFacilityDTO implements Serializable {
	private static final long serialVersionUID = 5086343418258783161L;
	private static final String regex = "[-()\\sx]";

	private Long userProfileId;
	private Long treatmentFacilityId;
	private String addressCity;
	private String addressCountry;
	private String addressPostalCode;
	private String addressProvince;
	private String addressState;
	private String addressStreet1;
	private String addressStreet2;
	private String comments;
	private String contactInfoFax;
	private String contactInfoWorkPhone;
	private String contactInfoWorkPhoneExt;
	private String facilityName;
	private String facilityType;
	private Boolean homeFacility;

	public TreatmentFacilityDTO() {
	}

	public Long getUserProfileId() {
		return userProfileId;
	}

	public void setUserProfileId(Long userProfileId) {
		this.userProfileId = userProfileId;
	}

	public Long getTreatmentFacilityId() {
		return treatmentFacilityId;
	}

	public void setTreatmentFacilityId(Long treatmentFacilityId) {
		this.treatmentFacilityId = treatmentFacilityId;
	}

	public String getAddressCity() {
		return addressCity;
	}

	public void setAddressCity(String addressCity) {
		this.addressCity = addressCity;
	}

	public String getAddressCountry() {
		return addressCountry;
	}

	public void setAddressCountry(String addressCountry) {
		this.addressCountry = addressCountry;
	}

	public String getAddressPostalCode() {
		return addressPostalCode;
	}

	public void setAddressPostalCode(String addressPostalCode) {
		this.addressPostalCode = addressPostalCode;
	}

	public String getAddressProvince() {
		return addressProvince;
	}

	public void setAddressProvince(String addressProvince) {
		this.addressProvince = addressProvince;
	}

	public String getAddressState() {
		return addressState;
	}

	public void setAddressState(String addressState) {
		this.addressState = addressState;
	}

	public String getAddressStreet1() {
		return addressStreet1;
	}

	public void setAddressStreet1(String addressStreet1) {
		this.addressStreet1 = addressStreet1;
	}

	public String getAddressStreet2() {
		return addressStreet2;
	}

	public void setAddressStreet2(String addressStreet2) {
		this.addressStreet2 = addressStreet2;
	}

	public String getComments() {
		return comments;
	}

	public void setComments(String comments) {
		this.comments = comments;
	}

	public String getContactInfoFax() {
		return contactInfoFax;
	}

	public void setContactInfoFax(String contactInfoFax) {
		this.contactInfoFax = contactInfoFax;
	}

	public String getContactInfoWorkPhone() {
		return contactInfoWorkPhone;
	}

	public void setContactInfoWorkPhone(String contactInfoWorkPhone) {
		this.contactInfoWorkPhone = contactInfoWorkPhone;
	}

	public String getContactInfoWorkPhoneExt() {
		return contactInfoWorkPhoneExt;
	}

	public void setContactInfoWorkPhoneExt(String contactInfoWorkPhoneExt) {
		this.contactInfoWorkPhoneExt = contactInfoWorkPhoneExt;
	}

	public String getFacilityName() {
		return facilityName;
	}

	public void setFacilityName(String facilityName) {
		this.facilityName = facilityName;
	}

	public String getFacilityType() {
		return facilityType;
	}

	public void setFacilityType(String facilityType) {
		this.facilityType = facilityType;
	}

	public Boolean getHomeFacility() {
		return homeFacility;
	}

	public void setHomeFacility(Boolean homeFacility) {
		this.homeFacility = homeFacility;
	}

	@JsonIgnore
	public CountryEnumeration[] getCountries() {
		return CountryEnumeration.values();
	}
	
	@JsonIgnore
	public StateEnumeration[] getStates() {
		return StateEnumeration.values();
	}

	@Override
	@JsonIgnore
	public String toString() {
		return "TreatmentFacilityDTO [userProfileId=" + userProfileId + ", treatmentFacilityId=" + treatmentFacilityId
				+ ", addressCity=" + addressCity + ", addressCountry=" + addressCountry + ", addressPostalCode="
				+ addressPostalCode + ", addressProvince=" + addressProvince + ", addressState=" + addressState
				+ ", addressStreet1=" + addressStreet1 + ", addressStreet2=" + addressStreet2 + ", comments=" + comments
				+ ", contactInfoFax=" + contactInfoFax + ", contactInfoWorkPhone=" + contactInfoWorkPhone
				+ ", contactInfoWorkPhoneExt=" + contactInfoWorkPhoneExt + ", facilityName=" + facilityName
				+ ", facilityType=" + facilityType + ", homeFacility=" + homeFacility + "]";
	}

	@JsonIgnore
	public String getDisplayFacilityType() {
		String facilityType = null;
		if (getFacilityType() != null) {
			facilityType = FacilityTypeEnumeration.valueOf(getFacilityType()).getDescription();
		}
		return facilityType;
	}

	@JsonIgnore
	public String getDisplayVAHome() {
		return getHomeFacility() ? "Yes" : "No";
	}

	@JsonIgnore
	public String getAddress() {
		StringBuffer sb = new StringBuffer();
		boolean first = true;
		if (getAddressStreet1() != null) {
			sb.append(getAddressStreet1());
			first = false;
		}
		if (getAddressStreet2() != null) {
			if (!first) {
				sb.append(" ");
				first = false;
			}
			sb.append(getAddressStreet2());
		}
		if (getAddressCity() != null) {
			if (!first) {
				sb.append(" ");
				first = false;
			}
			sb.append(getAddressCity());
		}
		if (getAddressState() != null) {
			if (!first) {
				sb.append(",");
				sb.append(" ");
				first = false;
			}
			sb.append(getAddressState());
		}
		if (getAddressPostalCode() != null) {
			if (!first) {
				sb.append(" ");
				first = false;
			}
			sb.append(getAddressPostalCode());
		}
		return sb.toString();
	}

	@JsonIgnore
	public String getPhone() {
		StringBuffer sb = new StringBuffer();
		if (getContactInfoWorkPhone() != null) {
			sb.append(getContactInfoWorkPhone());
		}
		if (getContactInfoWorkPhoneExt() != null) {
			sb.append(" ");
			sb.append("x");
			sb.append(getContactInfoWorkPhoneExt());
		}
		return sb.toString();
	}

	@JsonIgnore
	public String getPhoneNumbersOnly() {
		String inputString = getPhone();
		String outputString = null;
		if (inputString != null) {
			outputString = inputString.replaceAll(regex, "");
		}
		return outputString;
	}

}
