package gov.va.med.mhv.getcare.batch.processor;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.batch.core.BatchStatus;
import org.springframework.batch.core.Job;
import org.springframework.batch.core.JobExecution;
import org.springframework.batch.core.JobExecutionListener;
import org.springframework.batch.core.JobParameters;
import org.springframework.batch.core.JobParametersBuilder;
import org.springframework.batch.core.JobParametersInvalidException;
import org.springframework.batch.core.configuration.JobLocator;
import org.springframework.batch.core.launch.JobLauncher;
import org.springframework.batch.core.launch.NoSuchJobException;
import org.springframework.batch.core.repository.JobExecutionAlreadyRunningException;
import org.springframework.batch.core.repository.JobInstanceAlreadyCompleteException;
import org.springframework.batch.core.repository.JobRestartException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

@Component("appointmentsListener")
public class AppointmentsExecutionListener implements JobExecutionListener  {
	
	private static final String EXEC_TIME = "executionTime";
	
	@Autowired
	private JobLauncher jobLauncher;
	
	@Autowired
	private JobLocator jobLocator;
	
	private static final Logger LOG = LogManager.getLogger(AppointmentsExecutionListener.class);

	@Override
	public void afterJob(JobExecution jobExecution) {
		
		
		if(jobExecution.getStatus() == BatchStatus.FAILED){
			String jobName=jobExecution.getJobInstance().getJobName();
			JobParametersBuilder builder = new JobParametersBuilder();
			JobParameters jobParameters = builder.addLong(EXEC_TIME, System.currentTimeMillis()).toJobParameters();
			try {
				Job job = jobLocator.getJob(jobName);
				this.jobLauncher.run(job, jobParameters);
			} catch (JobExecutionAlreadyRunningException e) {
				LOG.error(e);
			} catch (JobRestartException e) {
				LOG.error(e);
			} catch (JobInstanceAlreadyCompleteException e) {
				LOG.error(e);
			} catch (JobParametersInvalidException e) {
				LOG.error(e);
			}catch (NoSuchJobException e) {
				LOG.error(e);
			}
		 }
	}
	

	@Override
	public void beforeJob(JobExecution arg0) {
		// TODO Auto-generated method stub
	}
}
