/**
 * 
 */
package gov.va.med.mhv.liferay.saml.aal;

import java.io.IOException;
import java.io.OutputStream;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.google.gson.internal.bind.DateTypeAdapter;
import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;
import com.liferay.portal.kernel.util.PropsUtil;

/**
 * @author vhaislegberb
 *
 */
public class AccountActivityClient {
	
	private static Log LOGGER = LogFactoryUtil.getLog(AccountActivityClient.class);
	
	private static final String PROP_KEY_API_ENDPOINT = "mhvapi.endpoint.uri";
	
	private String apiEndpoint = PropsUtil.get(PROP_KEY_API_ENDPOINT);

	public void createAccountActivity(Long userProfileId, String actor, boolean status, 
			String activityType, String action, String detailValue, String apiToken) {

		GsonBuilder builder = new GsonBuilder();
		Gson accountActivity = builder.create();
		
		Map<String, Object> accountActivityMap = new HashMap<>();
		accountActivityMap.put("userProfileId", userProfileId);
		accountActivityMap.put("activityType", activityType);
		accountActivityMap.put("action", action);
		accountActivityMap.put("completionTime", new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSSZ").format(new Date()));
		accountActivityMap.put("performerType", actor);
		accountActivityMap.put("status", status);
		accountActivityMap.put("detailValue", detailValue);

		String accountActivityJson = accountActivity.toJson(accountActivityMap);
		
		if(LOGGER.isDebugEnabled()) {
			LOGGER.debug("Account Activity JSON: \n" + accountActivityJson);
			LOGGER.debug(this.apiEndpoint + "/usermgmt/activity");
			LOGGER.debug("HTTP Request Method: POST");
			LOGGER.debug("HTTP Content-Type: application/json");
			LOGGER.debug("MHV API Token: " + apiToken);
		}
		
		HttpURLConnection conn = null;

		try {
			URL url = new URL(this.apiEndpoint + "/usermgmt/activity");
			conn = (HttpURLConnection) url.openConnection();
			conn.setDoOutput(true);
			conn.setRequestMethod("POST");
			conn.setRequestProperty("Content-Type", "application/json");
			conn.setRequestProperty("Token", apiToken);
			
			OutputStream os = conn.getOutputStream();
			os.write(accountActivityJson.getBytes());
			os.flush();

			if (conn.getResponseCode() != HttpURLConnection.HTTP_OK && conn.getResponseCode() != HttpURLConnection.HTTP_NO_CONTENT) {
				LOGGER.error(String.format("Error creating account activity log entry. HTTP Response code: '%d'", conn.getResponseCode()));
				throw new RuntimeException("Account Activity Log Creation Failed : HTTP error code : " + conn.getResponseCode());
			}

		} catch (MalformedURLException e) {

			LOGGER.error("The URL provided for the activity service is not valid.", e);

		} catch (IOException e) {

			LOGGER.error("An I/O Exception occured when creating the account activity log.", e);

		} finally {
			if(conn != null) {
				conn.disconnect();
			}
		}

	}
}
