package gov.va.med.mhv.rxrefill.batch.processor;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.batch.item.ItemProcessor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import gov.va.med.mhv.common.api.exception.MHVException;
import gov.va.med.mhv.rxrefill.data.model.UserProfileEmailInfo;
import gov.va.med.mhv.rxrefill.service.PharmacyRefillWebService;

@Component("emailTrackingProcessor")
public class EmailTrackingProcessor implements ItemProcessor<UserProfileEmailInfo, UserProfileEmailInfo> {

	private static final Logger logger = LogManager.getLogger(EmailTrackingProcessor.class);
	
	@Autowired
	@Qualifier("rxRefillWebService")
	private PharmacyRefillWebService pharmacyRefillWebService;
	
	@Value("${rxTrackingNotifications.from.email}")
	private String fromEmail;
	
	@Value("${rxTrackingNotifications.from.emailName}")
	private String fromEmailName;
	
	@Value("${rxTrackingNotifications.hostName}")
	private String hostName;
	
	
	@Override
	public UserProfileEmailInfo process(UserProfileEmailInfo userprofileEmail) throws Exception {
  	    sendRxTrackingEmailNotificationForUser(userprofileEmail);
		return userprofileEmail;
	}
	
	private void sendRxTrackingEmailNotificationForUser(UserProfileEmailInfo userprofileEmail) {
	   try {
		    pharmacyRefillWebService.sendRxTrackingEmailNotifications(fromEmail,fromEmailName,userprofileEmail.getUserprofileId(), userprofileEmail.getFirstName(), userprofileEmail.getContactInfoEmail(),hostName);
	   } catch (MHVException e) {
		   logger.error("Rx Tracking Email Notification Error for the User::"+ userprofileEmail.getUserprofileId(),e);
	   }
	}
}