package gov.va.med.mhv.rxrefill.batch.processor;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.text.SimpleDateFormat;
import java.util.Date;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.batch.core.StepContribution;
import org.springframework.batch.core.scope.context.ChunkContext;
import org.springframework.batch.core.step.tasklet.Tasklet;
import org.springframework.batch.repeat.RepeatStatus;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.dao.DataAccessException;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.ResultSetExtractor;
import org.springframework.stereotype.Component;

import gov.va.med.mhv.common.api.exception.MHVException;
import gov.va.med.mhv.rxrefill.batch.enums.UserEmailNotifTypeCode;
import gov.va.med.mhv.rxrefill.service.PharmacyRefillWebService;

@Component("rxEmailNotifJobStatus")
public class RxTrackingEmailNotifJobStautsTasklet implements Tasklet {
	
	private static final Logger logger = LogManager.getLogger(RxTrackingEmailNotifJobStautsTasklet.class);
	
	@Autowired
	@Qualifier("rxRefillWebService")
	private PharmacyRefillWebService pharmacyRefillWebService;
	
	@Value("${rxTrackingNotifications.from.email}")
	private String fromEmail;
	
	@Value("${rxTrackingNotifications.from.emailName}")
	private String fromEmailName;
	
	private static final String EMAIL_CONTENT_TYPE = "text/html";
	private static final String BREAK = "<br/>";
	private static final String EMAIL_SUBJECT_CHIO = "Rx Notification Nightly Background Job";
	
	@Value("${rxTrackingNotifications.CHIO.to.email}")
	private String emailAddress;
	
	@Value("${MHV_ENV_PROPERTY}")
	private String environment;
	
	@Autowired
	protected JdbcTemplate evaultJdbcTemplate;
	
	private static final String RXTRACK_NOTIF_QUERY="select count(*) from BATCH_JOB_EXECUTION a,BATCH_JOB_INSTANCE b where" + 
	                                         " a.job_instance_id = b.job_instance_id and b.job_name= 'rxEmailTrackingJob' and " +
	                                         " to_char(a.start_time,'YYYYMMDD')= to_char(sysdate,'YYYYMMDD') ";

	
	@Override
	public RepeatStatus execute(StepContribution contribution, ChunkContext chunkContext) throws Exception {
	    findEntryForRxTrackingForCurrentDate(UserEmailNotifTypeCode.RXTRACKING);
		return RepeatStatus.FINISHED;
	}

	private void findEntryForRxTrackingForCurrentDate(UserEmailNotifTypeCode userEmailNotifyCode) throws MHVException {
		try{
			int jobrunCount=evaultJdbcTemplate.query(RXTRACK_NOTIF_QUERY, new ResultSetExtractor<Integer>() {
				public Integer extractData(ResultSet rs) throws SQLException, DataAccessException {
					if(rs.next()) {
						return rs.getInt(1);
					}
					return 0;
				}
			});
			System.out.println("Rx Tracking Nofification job run count::"+ jobrunCount);
			logger.debug("Rx Tracking Nofification job run count::"+ jobrunCount);
			if(jobrunCount == 0){
				pharmacyRefillWebService.sendRxTrackingEmailJobStatusNotification(fromEmail, fromEmailName, environment+EMAIL_SUBJECT_CHIO, buildEmailBodyForCHIO(), emailAddress, EMAIL_CONTENT_TYPE);
			}
		}catch(Exception e){
			logger.error("Error in Email Notification Job::",e);
		}
	}
	
	 public static String buildEmailBodyForCHIO() {
	        StringBuffer emailBody = new StringBuffer();
	        SimpleDateFormat dateFormat = new SimpleDateFormat ("MM/dd/yyyy");
	        Date today = new Date();
	        emailBody.append(BREAK);
	        emailBody.append("The nightly background job did not run as scheduled on "+dateFormat.format(today)+".");
	        emailBody.append(BREAK);
	        emailBody.append(BREAK);
	        emailBody.append(BREAK);
	        emailBody.append("<strong>Please do not reply directly to this email as it is an automated message from the My HealtheVet System.</strong>");
	        emailBody.append(BREAK);
	        emailBody.append(BREAK);
	        return emailBody.toString();
	    }
}
