package gov.va.med.mhv.rxrefill.batch.processor;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.batch.item.ItemProcessor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Component;

import gov.va.med.mhv.common.api.exception.MHVException;
import gov.va.med.mhv.rxrefill.data.model.UserProfileForFacilityUpdate;
import gov.va.med.mhv.rxrefill.service.PharmacyRefillWebService;

@Component("rxTrackingFacilityUpdateProcessor")
public class RxTrackingFacilityUpdateProcessor implements ItemProcessor<UserProfileForFacilityUpdate, UserProfileForFacilityUpdate> {

	private static final Logger logger = LogManager.getLogger(RxTrackingFacilityUpdateProcessor.class);

	@Autowired
	@Qualifier("rxRefillWebService")
	private PharmacyRefillWebService pharmacyRefillWebService;
	
	@Override
	public UserProfileForFacilityUpdate process(UserProfileForFacilityUpdate userProfileForFacilityUpdate) throws Exception {
  	    doRxTrackingFacilityUpdateForUser(userProfileForFacilityUpdate);
		return userProfileForFacilityUpdate;
	}
	
	private void doRxTrackingFacilityUpdateForUser(UserProfileForFacilityUpdate userProfileForFacilityUpdate) {
	   try {
		    pharmacyRefillWebService.updateTreatmentFacilitesForExistingPatient(userProfileForFacilityUpdate.getUserprofileId());
	   } catch (MHVException e) {
		   logger.error("Rx Tracking Facility Update Error for the User::"+ userProfileForFacilityUpdate.getUserprofileId(),e);
	   }
	}
}