package gov.va.med.domain.persistence.util;

import gov.va.med.mhv.rxrefill.data.model.Function;
import gov.va.med.mhv.rxrefill.data.repository.FunctionRepository;
import gov.va.med.mhv.rxrefill.exception.MHVRuntimeException;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * @author Slava Uchitel
 * @version $Id: FunctionLookup.java,v 1.4 2005/08/09 13:45:25 ed.brundage Exp $
 * @since MHV 2.0 <br>04/27/2005
 */
@Component
public class FunctionLookup {

	private static Logger log = LogManager.getLogger(FunctionLookup.class);

	private HashMap<String, Function> mapById = new HashMap<String, Function>();

	@Autowired
	private FunctionRepository functionRepository;

	public void load() {
		synchronized(mapById) {
			mapById = new HashMap<String, Function>();

			List<gov.va.med.mhv.rxrefill.data.model.Function> functionList = null;
			try {
				functionList = functionRepository.findAll();
			} catch(Exception e) {
				log.error("Error in fetching values from FUNCTIONS table", e);
				throw new MHVRuntimeException("Error in fetching values from FUNCTIONS table", e);
			}

			if (null != functionList && functionList.size() > 0) {
				for(Function function : functionList) {
					mapById.put(function.getFunctionName(), function);
				}
			}
		}
	}

	public Function getById(String functionName) {
		return (Function) mapById.get(functionName);
	}

	public Collection<Function> getAll() {
		return mapById.values();
	}

	public List<Function> getByCategoryId(String categoryId) {
		ArrayList<Function> result = new ArrayList<Function>();
		Iterator<Function> iterator = getAll().iterator();
		while(iterator.hasNext()) {
			Function function = iterator.next();
			if(((String)function.getCategoryType().getId()).equals(categoryId))
				result.add(function);
		}
		return result;
	}

	public Long getIdByFunctionName(String functionName) {
		Long result = null;
		Iterator<Function> iterator = getAll().iterator();
		while (iterator.hasNext()) {
			Function function = iterator.next();
			if(function.getFunctionName().equals(functionName))
				if (function.getId() != null &&
				function.getId() instanceof Long) {
					result = (Long) function.getId();
					break;
				}
		}
		return result;
	}

}
