package gov.va.med.mhv.usermgmt.batch.listener;

import java.util.List;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.batch.core.BatchStatus;
import org.springframework.batch.core.JobExecution;
import org.springframework.batch.core.JobExecutionListener;
import org.springframework.stereotype.Component;

@Component
public class MviDowngradeJobListener implements JobExecutionListener {
	private static final Logger logger = LogManager.getLogger(MviDowngradeJobListener.class);

	@Override
	public void afterJob(JobExecution jobExecution) {
		logger.info(String.format("*** Finished job %s: %s", jobExecution.getJobInstance().getJobName(),
				jobExecution.toString()));

		if (jobExecution.getStatus() == BatchStatus.FAILED) {
			logger.error("*** MVI Downgrade Batch Job Failed: " + jobExecution.toString());

			if (logger.isWarnEnabled()) {
				List<Throwable> errors = jobExecution.getAllFailureExceptions();
				if (errors != null) {
					int cnt = 0;
					for (Throwable t : errors) {
						
						if (++cnt > 20) {
							logger.warn(t + "\\n...truncated");
						} else {
							logger.warn(t);
						}
					}
				}
			}
		}
	}

	@Override
	public void beforeJob(JobExecution jobExecution) {
		logger.info(String.format("*** Starting job %s: %s", jobExecution.getJobInstance().getJobName(),
				jobExecution.toString()));
	}

}
