package gov.va.med.mhv.usermgmt.batch.processor;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.List;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.batch.core.StepExecution;
import org.springframework.batch.core.annotation.AfterStep;
import org.springframework.batch.item.ItemProcessor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import gov.va.med.mhv.common.api.util.ResponseUtil;
import gov.va.med.mhv.common.data.model.InPersonAuthentication;
import gov.va.med.mhv.common.data.model.Patient;
import gov.va.med.mhv.common.data.model.UserProfile;
import gov.va.med.mhv.usermgmt.data.repository.UserProfileRepository;
import gov.va.med.mhv.usermgmt.service.InPersonAuthenticationService;
import gov.va.med.mhv.usermgmt.service.MviIntegrationService;
import gov.va.med.mhv.usermgmt.service.handler.MviProperties;

@Component
public class AccountDeactivationFixProcessor implements ItemProcessor<Patient, Patient> {
	private static final Logger logger = LogManager.getLogger(AccountDeactivationFixProcessor.class);

	private static final String STR_V = "V";
	private static final String STR_A = "A";
	private List<String> accountList = new ArrayList<String>();

	@Autowired
	private MviIntegrationService mviIntegrationService;

	@Autowired
	private InPersonAuthenticationService inPersonAuthenticationService;

	@Autowired
	private UserProfileRepository userProfileRepository;

	@Autowired
	private MviProperties mviProperties;

	@Override
	public Patient process(Patient patient) throws Exception {

		@SuppressWarnings("rawtypes")
		ResponseUtil response = new ResponseUtil();
		boolean processUpdate = false;
		String firstName = patient.getUserProfile().getFirstName();
		String lastName = patient.getUserProfile().getLastName();
		lastName = lastName.replace("DELETED", "");
		patient.getUserProfile().setLastName(lastName);
		String gender = patient.getUserProfile().getGender();
		Date birthDate = patient.getUserProfile().getBirthDate();
		InPersonAuthentication ipaInitial = new InPersonAuthentication();
		InPersonAuthentication ipaResult = new InPersonAuthentication();
		List<gov.va.med.mhv.common.data.model.UserProfile> userProfiles = userProfileRepository
				.getProfileByNameBirthDate(firstName, lastName, gender, birthDate);
		if (userProfiles.size() < 2) {
			processUpdate = true;
		}
		if (userProfiles.size() > 1) {
			for (UserProfile u : userProfiles) {
				Date createdDate = u.getCreatedDate();
				Date newAccountCreatedDate = getNewAccountCreatedDate(u);
				if (createdDate.before(newAccountCreatedDate)) {
					processUpdate = false;
					patient.getUserProfile().setLastName(lastName + "DELETED");
					if (!u.getId().toString().equals(patient.getUserProfile().getId().toString())) {
						accountList.add(patient.getUserProfileId().toString()
								+ " New Account created no updates done. New Account id: " + u.getId().toString());
					}
				}
			}
		}
		if (processUpdate) {
			String icn = patient.getIcn();
			icn = icn.replace(STR_A, STR_V);
			patient.setIcn(icn);
			try {
				ResponseUtil responseIPA = new ResponseUtil();
				inPersonAuthenticationService.getIPAPatientById(patient.getId(), responseIPA);
				if (response.isFailure()) {
					logger.debug("*********InPersonAuthentication Response isFailure() for id: cas"
							+ patient.getUserProfileId());
					accountList.add(patient.getUserProfile().getId().toString()
							+ " Error in InPersonAuthenticationService.  Nothing updated. "
							+ response.getFailureMessage());
					patient.getUserProfile().setLastName(lastName + "DELETED");
				} else {
					ipaInitial = (InPersonAuthentication) responseIPA.getPojoObject();
					String mviAuth = ipaInitial.getMviAuthenticationStatus();
					mviIntegrationService.matchPersonInIPA(patient.getUserProfile(), response, mviProperties, null);
					if (response.isSuccess()) {
						Patient mviPatient = (Patient) response.getPojoObject();
						String mviIcn = mviPatient.getIcn();
						if (!mviIcn.equals(patient.getIcn())) {
							patient.getUserProfile().setLastName(lastName + "DELETED");
							accountList.add(patient.getUserProfile().getId().toString()
									+ " MVI icn does not match MHV icn.  Nothing updated.");
							// if correlation status is 2
						} else if (mviPatient.getCorrelationStatus().intValue() == 2) {
							patient.getUserProfile().setDeactivationAdmin("");
							patient.getUserProfile().setDeactivationCode(null);
							patient.getUserProfile().setDeactivationDateTime(null);
							patient.getUserProfile()
									.setDeactivationDescriptionText("Reactivated by system due to error");
							patient.getUserProfile().setAccountDeactivationWarning(null);
							patient.getUserProfile().setDeactivationReason(null);
							patient.setIcn(mviPatient.getIcn());
							inPersonAuthenticationService.getIPAPatientById(patient.getId(), responseIPA);
							ipaResult = (InPersonAuthentication) responseIPA.getPojoObject();
							ipaResult.setMviAuthenticationStatus(mviAuth);
							inPersonAuthenticationService.saveIPA(ipaResult);
							ResponseUtil responseIPASave = new ResponseUtil();
							if (responseIPASave.isFailure()) {
								accountList.add(patient.getUserProfile().getId().toString()
										+ " Error in InPersonAuthenticationService Save.  Nothing updated. "
										+ response.getFailureMessage());
								patient.getUserProfile().setLastName(lastName + "DELETED");
							}
							accountList.add(patient.getUserProfile().getId().toString() + " Updated existing account.");
							// if correlation status anything other than 2 do
							// not process and put in error log
						} else {
							accountList.add(patient.getUserProfile().getId().toString()
									+ " Error in matchPersonInIPA.  Nothing updated. correlation status: "
									+ mviPatient.getCorrelationStatus());
							patient.getUserProfile().setLastName(lastName + "DELETED");
						}
					} else {
						// if error occurs than denote it and get put in log
						// file
						accountList.add(patient.getUserProfile().getId().toString()
								+ " Error in matchPersonInIPA.  Nothing updated. " + response.getFailureMessage());
						patient.getUserProfile().setLastName(lastName + "DELETED");
					}
				}
			} catch (Exception e) {
				accountList.add(patient.getUserProfile().getId().toString()
						+ " Error in AccountDeactivationFixProcessor try block.  Nothing updated. "
						+ response.getFailureMessage());
				patient.getUserProfile().setLastName(lastName + "DELETED");
				logger.error(e);
				return patient;
			}
		}
		return patient;
	}

	@AfterStep
	public void createAuditReport(StepExecution stepExecution) {
		if (accountList != null) {
			if (accountList.size() > 0) {
				BufferedWriter out = null;
				try {
					String fileDate = getFileDate();
					String fileName = "Account-Deactivation-Fix-" + fileDate + ".txt";
					// append to file
					FileWriter fstream = new FileWriter(fileName, true); 
					out = new BufferedWriter(fstream);
					for (String id : accountList) {
						out.write(id + "\r\n");
					}
				} catch (IOException e) {
					logger.error("Error in createAuditReport: " + e);
				} finally {
					if (out != null) {
						try {
							out.close();
							accountList.clear();
						} catch (IOException e) {
							logger.error(e);
						}
					}
				}
			}
		}
	}

	public Date getNewAccountCreatedDate(UserProfile u) {
		Date date1 = new Date(2017, Calendar.MARCH, 25);
		Calendar date = new GregorianCalendar();
		date.setTime(date1);
		Date marchDate = date.getTime();
		return marchDate;
	}

	public String getFileDate() {
		DateFormat df = new SimpleDateFormat("yyyy-MM-dd-hh-mm-ss");
		return df.format(new Date());
	}
}