package gov.va.med.mhv.usermgmt.batch.processor;

import java.util.ArrayList;
import java.util.List;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.batch.core.StepExecution;
import org.springframework.batch.core.annotation.AfterStep;
import org.springframework.batch.item.ItemProcessor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import com.itextpdf.text.DocumentException;

import gov.va.med.mhv.common.api.util.ResponseUtil;
import gov.va.med.mhv.common.data.model.Patient;
import gov.va.med.mhv.common.data.model.UserProfile;
import gov.va.med.mhv.usermgmt.common.enums.ActivityActorTypeEnumeration;
import gov.va.med.mhv.usermgmt.report.PdfCronJobReportCreator;
import gov.va.med.mhv.usermgmt.service.MviIntegrationService;
import gov.va.med.mhv.usermgmt.service.PatientService;
import gov.va.med.mhv.usermgmt.util.DetermineMatchCounter;

@Component
public class AccountSyncRecordsMhvIcnMissingProcessor implements ItemProcessor<Patient, Patient> {
	private static final Logger logger = LogManager.getLogger(AccountSyncRecordsMhvIcnMissingProcessor.class);

	@Value("${mhv.icn.missing}")
	private String mhvIcnMissingBatchReport;

	@Autowired
	protected MviIntegrationService mviIntegrationService;

	@Autowired
	private PatientService patientService;

	@Autowired
	protected DetermineMatchCounter determineMatchCounter;

	@Autowired
	private PdfCronJobReportCreator pdfCronJobReportCreator;

	private List<Patient> mhvIcnMissingList = new ArrayList<Patient>();

	@Override
	public Patient process(Patient patient) throws Exception {
		ResponseUtil responseUtil = new ResponseUtil();
		Patient mviPatientByUserProfile = null;
		boolean found = false;

		try {
			if (mhvIcnMissingList.size() >= 0) {
				for (Patient patientFromList : mhvIcnMissingList) {
					if (patientFromList.getIcn().equals(patient.getIcn())) {
						found = true;
					}
				}
			}
			if (!found) {
				mhvIcnMissingList.add(patient);
				mviPatientByUserProfile = searchForPatientInMVIByUserProfile(patient, responseUtil);
				int matchCounter = determineMatchCounter.determineMatchCounter(patient, mviPatientByUserProfile);
				if (matchCounter >= 4) {
					updateMHVWithMVIPatient(mviPatientByUserProfile, responseUtil);
				}
			}
		} catch (Exception e) {
			logger.error(e);
		}

		return patient;
	}

	@AfterStep
	public void createPdfReport(StepExecution stepExecution) {

		try {
			if (mhvIcnMissingList != null) {
				if (mhvIcnMissingList.size() > 0) {
					pdfCronJobReportCreator.createPdfReport(mhvIcnMissingList, mhvIcnMissingBatchReport,
							stepExecution.getJobExecution().getStartTime(), mhvIcnMissingList.size(),
							mhvIcnMissingList.size());
				} else {
					pdfCronJobReportCreator.createPdfReport(mhvIcnMissingList, mhvIcnMissingBatchReport,
							stepExecution.getJobExecution().getStartTime(), mhvIcnMissingList.size(), 0);
				}
			}
		} catch (Exception e) {
			logger.error("Error writing pdf file ", e);
		}

		mhvIcnMissingList.clear();
	}

	private Patient searchForPatientInMVIByUserProfile(Patient patient, ResponseUtil responseUtil) {
		UserProfile userProfile = patient.getUserProfile();

		mviIntegrationService.searchPersonInMVI(userProfile, false, responseUtil, null);
		return (Patient) responseUtil.getPojoObject();
	}

	private void updateMHVWithMVIPatient(Patient patient, ResponseUtil response) {

		patientService.updatePatientRegistryInformation(patient, response);

		mviIntegrationService.addPersonForCorrelationForIPA(patient, patient.getUserProfile(),
				ActivityActorTypeEnumeration.SYSTEM.toString(), response, null);
	}
}