package gov.va.med.mhv.usermgmt.batch.processor;

import java.util.ArrayList;
import java.util.List;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.batch.core.StepExecution;
import org.springframework.batch.core.annotation.AfterStep;
import org.springframework.batch.item.ItemProcessor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import com.itextpdf.text.DocumentException;

import gov.va.med.mhv.common.api.dto.FacilityInfoDTO;
import gov.va.med.mhv.common.api.dto.InPersonAuthenticationDTOLight;
import gov.va.med.mhv.common.api.exception.MHVException;
import gov.va.med.mhv.common.api.util.ResponseUtil;
import gov.va.med.mhv.common.data.converter.FacilityInfoConveter;
import gov.va.med.mhv.common.data.model.FacilityInfo;
import gov.va.med.mhv.common.data.model.InPersonAuthentication;
import gov.va.med.mhv.common.data.model.Patient;
import gov.va.med.mhv.common.data.model.UserProfile;
import gov.va.med.mhv.usermgmt.report.PdfCronJobReportCreator;
import gov.va.med.mhv.usermgmt.service.FacilityInfoService;
import gov.va.med.mhv.usermgmt.service.InPersonAuthenticationService;
import gov.va.med.mhv.usermgmt.service.MviIntegrationService;
import gov.va.med.mhv.usermgmt.service.PatientService;
import gov.va.med.mhv.usermgmt.service.handler.MviProperties;
import gov.va.med.mhv.usermgmt.util.DetermineMatchCounter;

@Component
public class AccountSyncRecordsNotMatchedToMVIProcessor implements ItemProcessor<Patient, Patient> {
	private static final Logger logger = LogManager.getLogger(AccountSyncRecordsNotMatchedToMVIProcessor.class);

	@Value("${records.not.matched.to.mvi}")
	private String recordNotMatchedToMVIBatchReport;

	@Value("${mhv.icn.missing.two}")
	private String icnMissingBatchReport;

	@Autowired
	private InPersonAuthenticationService inPersonAuthenticationService;

	@Autowired
	protected MviIntegrationService mviIntegrationService;

	@Autowired
	private PatientService patientService;

	@Autowired
	private MviProperties mviProperties;

	@Autowired
	protected DetermineMatchCounter determineMatchCounter;

	@Autowired
	private PdfCronJobReportCreator pdfCronJobReportCreator;

	@Autowired
	private FacilityInfoService facilityInfoService;

	private List<Patient> recordNotMatchedToMVIList = new ArrayList<Patient>();
	private List<Patient> icnMissingList = new ArrayList<Patient>();
	private int totalCounter = 0;

	@Override
	public Patient process(Patient patient) throws Exception {
		ResponseUtil responseUtil = new ResponseUtil();
		Patient mviPatientByICN = null;
		Patient mviPatientByUserProfile = null;
		boolean foundNot = false;

		try {
			searchForPatientInMVIByICN(patient.getIcn(), responseUtil);
			// it is assumed ICN's match because search is done using icn
			mviPatientByICN = (Patient) responseUtil.getPojoObject();
			if (mviPatientByICN == null) {
				logger.debug("*******AccountSyncRecordsNotMatchedtoMVI mviPatientByICN is null");
				mviPatientByUserProfile = searchForPatientInMVIByUserProfile(patient, responseUtil);
				if (mviPatientByUserProfile == null) {
					logger.debug("*******AccountSyncRecordsNotMatchedtoMVI mviPatientByUserProfile is null");
					if (recordNotMatchedToMVIList.size() >= 0) {
						for (Patient patientFromList : recordNotMatchedToMVIList) {
							if (patientFromList.getIcn().equals(patient.getIcn())) {
								foundNot = true;
							}
						}
					}
				}
				if (!foundNot) {
					logger.debug("*******AccountSyncRecordsNotMatchedtoMVI FoundNot for patient: " + patient.getIcn());
					recordNotMatchedToMVIList.add(patient);
					totalCounter++;
					return patient;
				}
			} else {
				if (mviPatientByICN.getIcn() == null) {
					addToError(patient);
					totalCounter++;
					return null;
				}
				int matchCounter = determineMatchCounter.determineMatchCounter(patient, mviPatientByICN);
				if (matchCounter >= 4) {
					updateMHVWithMVIPatient(patient, responseUtil);
				} else {
					if (recordNotMatchedToMVIList.size() >= 0) {
						for (Patient patientFromList : recordNotMatchedToMVIList) {
							if (!patientFromList.getIcn().equals(patient.getIcn())) {
								recordNotMatchedToMVIList.add(patient);
								totalCounter++;
								return patient;
							}
						}
					}
				}
			}
		} catch (Exception e) {
			logger.error(e);
		}
		return null;
	}

	@AfterStep
	public void createPdfReport(StepExecution stepExecution) {
		try {
			if (recordNotMatchedToMVIList != null) {
				if (recordNotMatchedToMVIList.size() > 0) {
					pdfCronJobReportCreator.createPdfReport(recordNotMatchedToMVIList, recordNotMatchedToMVIBatchReport,
							stepExecution.getJobExecution().getStartTime(), totalCounter,
							recordNotMatchedToMVIList.size());
				} else {
					pdfCronJobReportCreator.createPdfReport(recordNotMatchedToMVIList, recordNotMatchedToMVIBatchReport,
							stepExecution.getJobExecution().getStartTime(), totalCounter, 0);
				}
			}
			if (icnMissingList != null) {
				if (icnMissingList.size() > 0) {
					pdfCronJobReportCreator.createPdfReport(icnMissingList, icnMissingBatchReport,
							stepExecution.getJobExecution().getStartTime(), totalCounter, icnMissingList.size());
				} else {
					pdfCronJobReportCreator.createPdfReport(icnMissingList, icnMissingBatchReport,
							stepExecution.getJobExecution().getStartTime(), totalCounter, 0);
				}
			}
		} catch (Exception e) {
			logger.error("Error writing pdf file ", e);
		}
		recordNotMatchedToMVIList.clear();
		icnMissingList.clear();
	}

	private void addToError(Patient patient) {
		boolean foundIcn = false;

		if (icnMissingList.size() > 0) {
			for (gov.va.med.mhv.common.data.model.Patient patientFromList : icnMissingList) {
				if (patientFromList.getIcn().equals(patient.getIcn())) {
					foundIcn = true;
					break;
				}
			}
		}

		if (!foundIcn) {
			icnMissingList.add(patient);
			totalCounter++;
		}
	}

	private Patient searchForPatientInMVIByUserProfile(Patient patient, ResponseUtil responseUtil) {
		UserProfile userProfile = patient.getUserProfile();
		mviIntegrationService.searchPersonInMVI(userProfile, false, responseUtil, null);
		return (Patient) responseUtil.getPojoObject();
	}

	private Patient searchForPatientInMVIByICN(String icn, ResponseUtil responseUtil) {
		mviIntegrationService.searchPersonInMVIByICNAccountSync(icn, responseUtil);
		return (Patient) responseUtil.getPojoObject();
	}

	private void updateMHVWithMVIPatient(Patient patient, ResponseUtil response) {
		patientService.updatePatientRegistryInformation(patient, response);
		mviIntegrationService.addPersonForCorrelationForIPA(patient, patient.getUserProfile(), "SYSTEM", response,
				null);
		mviIntegrationService.matchPersonInIPA(patient.getUserProfile(), response, mviProperties, null);
		inPersonAuthenticationService.getIPAPatientById(patient.getId(), response);
		if (response.isSuccess()) {
			InPersonAuthentication ipa = (InPersonAuthentication) response.getPojoObject();
			InPersonAuthenticationDTOLight ipaLight = new InPersonAuthenticationDTOLight();
			ipaLight.setVideoViewed(ipa.getVideoViewed());
			ipaLight.setIdentificationPresented(ipa.getIdentificationPresented());
			ipaLight.setParticipationFormSigned(ipa.getParticipationFormSigned());
			ipaLight.setApprovedForRecordsAccess(ipa.getApprovedForRecordsAccess());
			FacilityInfoDTO fidto = new FacilityInfoDTO();
			try {
				fidto = getAustinMHVFacilityInfo();
			} catch (MHVException e) {
				logger.error(e);
			}
			ipaLight.setAuthenticatingFacilityDTO(fidto);
			ipaLight.setDefermentReason(ipa.getDefermentReason());
			ipaLight.setSignedFormOnlineDateTime(ipa.getSignedFormOnlineDateTime());
			ipaLight.setAuthenticatedBy(ipa.getAuthenticatedBy());
			ipaLight.setRemovalReason(null);
			ipaLight.setPatientId(patient.getUserProfileId());
			ipaLight.setParticipationFormSigned(true);
			ipaLight.setIdentificationPresented(true);
			ipaLight.setOplock(0);
			ipaLight.setId(ipa.getId());
			inPersonAuthenticationService.authenticateFull(ipaLight, response);
		}
	}

	public FacilityInfoDTO getAustinMHVFacilityInfo() throws MHVException {
		List<FacilityInfo> facInfoList = facilityInfoService.queryByStationNumber("200MH");
		if (facInfoList != null && facInfoList.size() > 0) {
			FacilityInfo fi = facInfoList.get(0);
			return new FacilityInfoConveter().convertDomainFacilityInfo(fi);
		}
		return null;
	}
}