package gov.va.med.mhv.usermgmt.batch.processor;

import java.util.ArrayList;
import java.util.List;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.batch.core.StepExecution;
import org.springframework.batch.core.annotation.AfterStep;
import org.springframework.batch.item.ItemProcessor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import gov.va.med.mhv.common.data.model.Patient;
import gov.va.med.mhv.usermgmt.report.PdfCronJobReportCreator;

@Component
public class AccountSyncRequiredFieldsNotPresentProcessor implements ItemProcessor<Patient, Patient> {
	private static final Logger logger = LogManager.getLogger(AccountSyncRequiredFieldsNotPresentProcessor.class);

	@Value("${required.fields.not.present}")
	private String requiredFieldsNotPresentReport;

	@Autowired
	private PdfCronJobReportCreator pdfCronJobReportCreator;

	private List<Patient> requiredFieldsNotPresentList = new ArrayList<Patient>();

	@Override
	public Patient process(Patient patient) throws Exception {
		boolean found = false;

		try {

			// only add unique ICNs
			for (Patient patientFromList : requiredFieldsNotPresentList) {
				if (patientFromList.getIcn().equals(patient.getIcn())) {
					found = true;
					break;
				}
			}
			if (!found) {
				requiredFieldsNotPresentList.add(patient);
			}
		} catch (Exception e) {
			logger.error(e);
		}

		return patient;
	}

	@AfterStep
	public void afterStepExecution(StepExecution stepExecution) {

		try {
			pdfCronJobReportCreator.createPdfReport(requiredFieldsNotPresentList, requiredFieldsNotPresentReport,
					stepExecution.getJobExecution().getStartTime(), stepExecution.getReadCount(),
					requiredFieldsNotPresentList.size());
		} catch (Exception e) {
			logger.error("Error writing pdf file ", e);
		}

		requiredFieldsNotPresentList.clear();
	}
}