package gov.va.med.mhv.usermgmt.batch.processor;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.List;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.batch.core.StepExecution;
import org.springframework.batch.core.annotation.AfterStep;
import org.springframework.batch.item.ItemProcessor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import gov.va.med.mhv.common.api.util.ResponseUtil;
import gov.va.med.mhv.common.data.model.InPersonAuthentication;
import gov.va.med.mhv.common.data.model.Patient;
import gov.va.med.mhv.usermgmt.service.InPersonAuthenticationService;
import gov.va.med.mhv.usermgmt.service.MviIntegrationService;

@Component("patientUnauthenticationProcessor")
public class PatientUnAuthenticationProcessor implements ItemProcessor<Patient, Patient> {
	private static Logger logger = LogManager.getLogger(PatientUnAuthenticationProcessor.class);

	@Autowired
	protected MviIntegrationService mviIntegrationService;

	@Autowired
	private InPersonAuthenticationService inPersonAuthenticationService;

	private List<String> accountList = new ArrayList<String>();

	@Override
	public Patient process(Patient patient) throws Exception {
		ResponseUtil responseUtil = new ResponseUtil();

		try {

			if (logger.isDebugEnabled()) {
				logger.debug(String.format(
						"Processing MVI Unauthentication Uncorrelate for Batch for Patient: %d with Account Creation Date: %s, Last Login Date: %s",
						patient.getId(), patient.getUserProfile().getCreatedDate(),
						patient.getUserProfile().getLastLogin()));
			}

			inPersonAuthenticationService.getIPAPatientById(patient.getId(), responseUtil);

			if (responseUtil.isSuccess()) {
				InPersonAuthentication ipa = (InPersonAuthentication) responseUtil.getPojoObject();
				inPersonAuthenticationService.unauthenticateWithAdminUserName(ipa,
						patient.getUserProfile().getUserName(), responseUtil);
				if (responseUtil.isFailure()) {
					accountList.add(patient.getUserProfile().getId().toString()
							+ " Error in MVI UnauthenticateUncorrelateForBatch method.  Nothing updated. "
							+ responseUtil.getFailureMessage());
				} else {
					accountList.add(patient.getUserProfile().getId().toString()
							+ " UnauthenticationUncorrelateForBatch performed on account.");
				}
			} else {
				accountList.add(patient.getUserProfile().getId().toString()
						+ " Error in IPA getIPAByuserProfileId method.  Nothing updated. "
						+ responseUtil.getFailureMessage());
			}
		} catch (Exception e) {
			logger.error("Error in process: " + e);
		}

		return patient;
	}

	@AfterStep
	public void createAuditReport(StepExecution stepExecution) {
		if (accountList != null) {
			if (accountList.size() > 0) {
				BufferedWriter out = null;
				try {
					DateFormat df = new SimpleDateFormat("yyyy-MM-dd_hh-mm-ss");
					String fileDate = df.format(stepExecution.getJobExecution().getStartTime());
					String fileName = "Account-Unauthentication-" + fileDate + ".txt";
					// append to file
					FileWriter fstream = new FileWriter(fileName, true);
					out = new BufferedWriter(fstream);
					for (String id : accountList) {
						out.write(id + "\r\n");
					}
				} catch (Exception e) {
					logger.error("Error creating report: " + e);
				} finally {
					if (out != null) {
						try {
							out.close();
						} catch (IOException e) {
							logger.error("Error closing file " + e);
						}
					}
					accountList.clear();
				}
			}
		}
	}

}