package gov.va.med.mhv.usermgmt.batch.writer;

import java.util.List;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.batch.core.ExitStatus;
import org.springframework.batch.core.StepExecution;
import org.springframework.batch.core.StepExecutionListener;
import org.springframework.batch.item.ItemWriter;

import com.itextpdf.text.DocumentException;

import gov.va.med.mhv.common.data.model.Patient;
import gov.va.med.mhv.usermgmt.report.MviPdfReport;

public class AccountSyncRecordsNotMatchedToMviItemWriter implements ItemWriter<Patient>, StepExecutionListener {
	private static final Logger logger = LogManager.getLogger(AccountSyncRecordsNotMatchedToMviItemWriter.class);

	private MviPdfReport recordsNotMatchedMviReport = new MviPdfReport();
	private String recordsNotMatchedMviReportName;
	private String recordsNotMatchedMviReportTitle;

	@SuppressWarnings("unchecked")
	@Override
	public void write(List<? extends Patient> patients) throws Exception {

		if (recordsNotMatchedMviReport != null) {
			logger.debug("document=" + recordsNotMatchedMviReport.getDocument());
			recordsNotMatchedMviReport.write((List<Patient>) patients);
		}
	}

	@Override
	public ExitStatus afterStep(StepExecution stepExecution) {
		try {
			
			recordsNotMatchedMviReport.close(stepExecution.getReadCount());

		} catch (DocumentException e) {
			logger.error("Error closing pdf report ", e);
		}

		return stepExecution.getExitStatus();
	}

	@Override
	public void beforeStep(StepExecution stepExecution) {
		try {
			recordsNotMatchedMviReport.create(recordsNotMatchedMviReportTitle, recordsNotMatchedMviReportName, stepExecution.getJobExecution().getStartTime());
		} catch (DocumentException e) {
			logger.error("Error creating pdf report ", e);
		}
	}

	public String getRecordsNotMatchedMviReportName() {
		return recordsNotMatchedMviReportName;
	}
	public void setRecordsNotMatchedMviReportName(String recordsNotMatchedMviReportName) {
		this.recordsNotMatchedMviReportName = recordsNotMatchedMviReportName;
	}
	public String getRecordsNotMatchedMviReportTitle() {
		return recordsNotMatchedMviReportTitle;
	}
	public void setRecordsNotMatchedMviReportTitle(String recordsNotMatchedMviReportTitle) {
		this.recordsNotMatchedMviReportTitle = recordsNotMatchedMviReportTitle;
	}
}