package gov.va.med.mhv.usermgmt.batch.writer;

import java.util.List;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.batch.core.ExitStatus;
import org.springframework.batch.core.StepExecution;
import org.springframework.batch.core.StepExecutionListener;
import org.springframework.batch.item.ItemWriter;

import com.itextpdf.text.DocumentException;

import gov.va.med.mhv.common.data.model.Patient;
import gov.va.med.mhv.usermgmt.report.MviPdfReport;

public class AccountSyncRequiredFieldsNotPresentPdfItemWriter implements ItemWriter<Patient>, StepExecutionListener  {
	private static final Logger logger = LogManager.getLogger(AccountSyncRequiredFieldsNotPresentPdfItemWriter.class);
	
	private MviPdfReport recordsNotMatchedMviReport = new MviPdfReport();
	private String recordsNotMatchedMviReportName;
	private String recordsNotMatchedMviReportTitle;
	
	@SuppressWarnings("unchecked")
	@Override
	public void write(List<? extends Patient> patients) throws Exception {

		recordsNotMatchedMviReport.write((List<Patient>) patients);
	}

	public void beforeStep(StepExecution stepExecution) {

		try {
			recordsNotMatchedMviReport.create(recordsNotMatchedMviReportTitle, recordsNotMatchedMviReportName, stepExecution.getJobExecution().getStartTime());
		} catch (DocumentException e) {
			logger.error("Error creating recordsNotMatchedMviReport pdf report ", e);
		}
	}

	public ExitStatus afterStep(StepExecution stepExecution) {

		try {
			recordsNotMatchedMviReport.close(stepExecution.getReadCount());
		} catch (DocumentException e) {
			logger.error("Error closing recordsNotMatchedMviReport pdf report ", e);
		}

		return stepExecution.getExitStatus();
	}

	public String getRecordsNotMatchedMviReportName() {
		return recordsNotMatchedMviReportName;
	}
	public void setRecordsNotMatchedMviReportName(String recordsNotMatchedMviReportName) {
		this.recordsNotMatchedMviReportName = recordsNotMatchedMviReportName;
	}
	public String getRecordsNotMatchedMviReportTitle() {
		return recordsNotMatchedMviReportTitle;
	}
	public void setRecordsNotMatchedMviReportTitle(String recordsNotMatchedMviReportTitle) {
		this.recordsNotMatchedMviReportTitle = recordsNotMatchedMviReportTitle;
	}
}
