package gov.va.med.mhv.usermgmt.report;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.attribute.PosixFilePermission;
import java.sql.Timestamp;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.stereotype.Component;

import com.itextpdf.text.BaseColor;
import com.itextpdf.text.Chunk;
import com.itextpdf.text.Document;
import com.itextpdf.text.DocumentException;
import com.itextpdf.text.Element;
import com.itextpdf.text.Font;
import com.itextpdf.text.PageSize;
import com.itextpdf.text.Phrase;
import com.itextpdf.text.Rectangle;
import com.itextpdf.text.pdf.PdfPCell;
import com.itextpdf.text.pdf.PdfPTable;
import com.itextpdf.text.pdf.PdfWriter;

import gov.va.med.mhv.common.data.model.Patient;

@Component
public class PdfCronJobReportCreator {
	private static final Logger logger = LogManager.getLogger(PdfCronJobReportCreator.class);

	private static Font subFont = new Font(Font.FontFamily.TIMES_ROMAN, 9, Font.NORMAL);
	private static Font tableFont = new Font(Font.FontFamily.TIMES_ROMAN, 9, Font.BOLD);
	private static Font titleFont = new Font(Font.FontFamily.TIMES_ROMAN, 11, Font.BOLD);
	
	public void createPdfReport(List<Patient> patients, String fileName, Date fileDate, long totalNumberProcessed, int totalReviewed) throws DocumentException {
    	SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
    	String veteranRel = "Veteran";
    	String patientRel = "Patient";
    	String healthCareProviderRel = "Health Care Provider";
    	String otherRel = "Other";
    	String recordsProcessed= "Total MHV Records Processed";
    	String exceptionReported = "Total Exceptions Reported Here";
    	
    	SimpleDateFormat sdfDate = new SimpleDateFormat("yyyy-MM-dd_HH-mm-ss");
    	fileName = fileName + sdfDate.format(fileDate) + ".pdf";
    	Document document = new Document(PageSize.LETTER.rotate());
    	
        try {
        	File file = new File(fileName);
        	if(file.exists()) {
        		file.delete();
        		file = new File(fileName);
        		file.createNewFile();
        		Set<PosixFilePermission> perms = new HashSet<>();
        		perms.add(PosixFilePermission.OWNER_READ);
        		perms.add(PosixFilePermission.OWNER_WRITE);
        		Files.setPosixFilePermissions(file.toPath(), perms);
        	}
        	FileOutputStream fop = new FileOutputStream(file);
			PdfWriter.getInstance(document, fop);
		} catch (FileNotFoundException e) {
			logger.error(e);
		} catch (IOException e) {
			logger.error(e);
		}
		document.open();
	    document.addTitle(fileName);
		document.addSubject(fileName);
		document.addAuthor("MyHealtheVet");
		document.addCreator("MyHealtheVet");
		document.addCreationDate();

		PdfPTable table1 = new PdfPTable(1);
		table1.setHorizontalAlignment(Element.ALIGN_LEFT);
		table1.getDefaultCell().setBackgroundColor(BaseColor.WHITE);
		table1.getDefaultCell().setBorder(Rectangle.NO_BORDER);
		table1.getDefaultCell().setNoWrap(true);
		PdfPCell table1cell1 = new PdfPCell(new Phrase(new Chunk("Title:        " + fileName, titleFont)));
		table1cell1.setHorizontalAlignment(Element.ALIGN_LEFT);
		table1cell1.setBorder(Rectangle.NO_BORDER);
		table1cell1.setBorderColor(BaseColor.WHITE);
		table1cell1.setColspan(1);
		table1.addCell(table1cell1);
		document.add(table1);

		PdfPTable table2 = new PdfPTable(1);
		table2.setHorizontalAlignment(Element.ALIGN_LEFT);
		table2.getDefaultCell().setBackgroundColor(BaseColor.WHITE);
		table2.getDefaultCell().setBorder(Rectangle.NO_BORDER);
		table2.getDefaultCell().setNoWrap(true);
		SimpleDateFormat sdfRunDate = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss z");
		PdfPCell table2cell1 = new PdfPCell(new Phrase(new Chunk("Run Date: " + sdfRunDate.format(fileDate), titleFont)));
		table2cell1.setHorizontalAlignment(Element.ALIGN_LEFT);
		table2cell1.setBorder(Rectangle.NO_BORDER);
		table2cell1.setBorderColor(BaseColor.WHITE);
		table2cell1.setColspan(1);
		table2.addCell(table2cell1);
		document.add(table2);

		PdfPTable table3 = new PdfPTable(34);
		table3.setHorizontalAlignment(Element.ALIGN_LEFT);
		table3.getDefaultCell().setHorizontalAlignment(Element.ALIGN_LEFT);
		table3.getDefaultCell().setBorder(Rectangle.NO_BORDER);
		table3.getDefaultCell().setBackgroundColor(BaseColor.WHITE);
		table3.getDefaultCell().setNoWrap(false);

		PdfPCell table3cell1 = new PdfPCell(new Phrase(new Chunk("Unique Profile Id", tableFont)));
		table3cell1.setBorderColor(BaseColor.GRAY);
		table3cell1.setBorderWidth(1);
		table3cell1.setBorderWidthLeft(1);
		table3cell1.setBorderWidthTop(1);
		table3cell1.setBorderWidthBottom(1);
		table3cell1.setBorderWidthRight(1);
		table3cell1.setHorizontalAlignment(Element.ALIGN_CENTER);
		table3cell1.setVerticalAlignment(Element.ALIGN_CENTER);
		table3cell1.setNoWrap(false);
		table3cell1.setColspan(3);
		table3.addCell(table3cell1);

		PdfPCell table3cell2 = new PdfPCell(new Phrase(new Chunk("Relationship to the VA", tableFont)));
		table3cell2.setBorderColor(BaseColor.GRAY);
		table3cell2.setBorderWidth(1);
		table3cell2.setBorderWidthLeft(1);
		table3cell2.setBorderWidthTop(1);
		table3cell2.setBorderWidthBottom(1);
		table3cell2.setBorderWidthRight(1);
		table3cell2.setHorizontalAlignment(Element.ALIGN_CENTER);
		table3cell2.setVerticalAlignment(Element.ALIGN_CENTER);
		table3cell2.setNoWrap(false);
		table3cell2.setColspan(3);
		table3.addCell(table3cell2);

		PdfPCell table3cell3 = new PdfPCell(new Phrase(new Chunk("ICN", tableFont)));
		table3cell3.setBorderColor(BaseColor.GRAY);
		table3cell3.setBorderWidth(1);
		table3cell3.setBorderWidthLeft(1);
		table3cell3.setBorderWidthTop(1);
		table3cell3.setBorderWidthBottom(1);
		table3cell3.setBorderWidthRight(1);
		table3cell3.setHorizontalAlignment(Element.ALIGN_CENTER);
		table3cell3.setVerticalAlignment(Element.ALIGN_CENTER);
		table3cell3.setNoWrap(false);
		table3cell3.setColspan(5);
		table3.addCell(table3cell3);

		PdfPCell table3cell4 = new PdfPCell(new Phrase(new Chunk("First Name", tableFont)));
		table3cell4.setBorderColor(BaseColor.GRAY);
		table3cell4.setBorderWidth(1);
		table3cell4.setBorderWidthLeft(1);
		table3cell4.setBorderWidthTop(1);
		table3cell4.setBorderWidthBottom(1);
		table3cell4.setBorderWidthRight(1);
		table3cell4.setHorizontalAlignment(Element.ALIGN_CENTER);
		table3cell4.setVerticalAlignment(Element.ALIGN_CENTER);
		table3cell4.setNoWrap(false);
		table3cell4.setColspan(2);
		table3.addCell(table3cell4);

		PdfPCell table3cell5 = new PdfPCell(new Phrase(new Chunk("Middle Name", tableFont)));
		table3cell5.setBorderColor(BaseColor.GRAY);
		table3cell5.setBorderWidth(1);
		table3cell5.setBorderWidthLeft(1);
		table3cell5.setBorderWidthTop(1);
		table3cell5.setBorderWidthBottom(1);
		table3cell5.setBorderWidthRight(1);
		table3cell5.setHorizontalAlignment(Element.ALIGN_CENTER);
		table3cell5.setVerticalAlignment(Element.ALIGN_CENTER);
		table3cell5.setNoWrap(false);
		table3cell5.setColspan(2);
		table3.addCell(table3cell5);

		PdfPCell table3cell6 = new PdfPCell(new Phrase(new Chunk("Last Name", tableFont)));
		table3cell6.setBorderColor(BaseColor.GRAY);
		table3cell6.setBorderWidth(1);
		table3cell6.setBorderWidthLeft(1);
		table3cell6.setBorderWidthTop(1);
		table3cell6.setBorderWidthBottom(1);
		table3cell6.setBorderWidthRight(1);
		table3cell6.setHorizontalAlignment(Element.ALIGN_CENTER);
		table3cell6.setVerticalAlignment(Element.ALIGN_CENTER);
		table3cell6.setNoWrap(false);
		table3cell6.setColspan(2);
		table3.addCell(table3cell6);

		PdfPCell table3cell7 = new PdfPCell(new Phrase(new Chunk("Gender", tableFont)));
		table3cell7.setBorderColor(BaseColor.GRAY);
		table3cell7.setBorderWidth(1);
		table3cell7.setBorderWidthLeft(1);
		table3cell7.setBorderWidthTop(1);
		table3cell7.setBorderWidthBottom(1);
		table3cell7.setBorderWidthRight(1);
		table3cell7.setHorizontalAlignment(Element.ALIGN_CENTER);
		table3cell7.setVerticalAlignment(Element.ALIGN_CENTER);
		table3cell7.setNoWrap(false);
		table3cell7.setColspan(2);
		table3.addCell(table3cell7);

		PdfPCell table3cell8 = new PdfPCell(new Phrase(new Chunk("Date of Birth", tableFont)));
		table3cell8.setBorderColor(BaseColor.GRAY);
		table3cell8.setBorderWidth(1);
		table3cell8.setBorderWidthLeft(1);
		table3cell8.setBorderWidthTop(1);
		table3cell8.setBorderWidthBottom(1);
		table3cell8.setBorderWidthRight(1);
		table3cell8.setHorizontalAlignment(Element.ALIGN_CENTER);
		table3cell8.setVerticalAlignment(Element.ALIGN_CENTER);
		table3cell8.setNoWrap(false);
		table3cell8.setColspan(3);
		table3.addCell(table3cell8);

		PdfPCell table3cell9 = new PdfPCell(new Phrase(new Chunk("SSN", tableFont)));
		table3cell9.setBorderColor(BaseColor.GRAY);
		table3cell9.setBorderWidth(1);
		table3cell9.setBorderWidthLeft(1);
		table3cell9.setBorderWidthTop(1);
		table3cell9.setBorderWidthBottom(1);
		table3cell9.setBorderWidthRight(1);
		table3cell9.setHorizontalAlignment(Element.ALIGN_CENTER);
		table3cell9.setVerticalAlignment(Element.ALIGN_CENTER);
		table3cell9.setNoWrap(false);
		table3cell9.setColspan(3);
		table3.addCell(table3cell9);

		PdfPCell table3cell10 = new PdfPCell(new Phrase(new Chunk("Correlation Date", tableFont)));
		table3cell10.setBorderColor(BaseColor.GRAY);
		table3cell10.setBorderWidth(1);
		table3cell10.setBorderWidthLeft(1);
		table3cell10.setBorderWidthTop(1);
		table3cell10.setBorderWidthBottom(1);
		table3cell10.setBorderWidthRight(1);
		table3cell10.setHorizontalAlignment(Element.ALIGN_CENTER);
		table3cell10.setVerticalAlignment(Element.ALIGN_CENTER);
		table3cell10.setNoWrap(false);
		table3cell10.setColspan(3);
		table3.addCell(table3cell10);

		PdfPCell table3cell11 = new PdfPCell(new Phrase(new Chunk("Match Date", tableFont)));
		table3cell11.setBorderColor(BaseColor.GRAY);
		table3cell11.setBorderWidth(1);
		table3cell11.setBorderWidthLeft(1);
		table3cell11.setBorderWidthTop(1);
		table3cell11.setBorderWidthBottom(1);
		table3cell11.setBorderWidthRight(1);
		table3cell11.setHorizontalAlignment(Element.ALIGN_CENTER);
		table3cell11.setVerticalAlignment(Element.ALIGN_CENTER);
		table3cell11.setNoWrap(false);
		table3cell11.setColspan(3);
		table3.addCell(table3cell11);

		PdfPCell table3cell12 = new PdfPCell(new Phrase(new Chunk("Account Created Date", tableFont)));
		table3cell12.setBorderColor(BaseColor.GRAY);
		table3cell12.setBorderWidth(1);
		table3cell12.setBorderWidthLeft(1);
		table3cell12.setBorderWidthTop(1);
		table3cell12.setBorderWidthBottom(1);
		table3cell12.setBorderWidthRight(1);
		table3cell12.setHorizontalAlignment(Element.ALIGN_CENTER);
		table3cell12.setVerticalAlignment(Element.ALIGN_CENTER);
		table3cell12.setNoWrap(false);
		table3cell12.setColspan(3);
		table3.addCell(table3cell12);
		document.add(table3);

		if(patients.size() > 0) {
			for(Patient patient : patients) {
				PdfPTable table4 = new PdfPTable(34);
				table4.setHorizontalAlignment(Element.ALIGN_LEFT);
				table4.getDefaultCell().setHorizontalAlignment(Element.ALIGN_LEFT);
				table4.getDefaultCell().setBorder(Rectangle.NO_BORDER);
				table4.getDefaultCell().setBackgroundColor(BaseColor.WHITE);
				table4.getDefaultCell().setNoWrap(false);
				String uniqueId = patient.getUserProfile().getId() != null ? String.valueOf(patient.getUserProfile().getId()) : "";
				PdfPCell table4cell1 = new PdfPCell(new Phrase(new Chunk(uniqueId, subFont)));
				table4cell1.setBorderColor(BaseColor.GRAY);
				table4cell1.setBorderWidth(1);
				table4cell1.setBorderWidthLeft(1);
				table4cell1.setBorderWidthTop(1);
				table4cell1.setBorderWidthBottom(1);
				table4cell1.setBorderWidthRight(1);
				table4cell1.setHorizontalAlignment(Element.ALIGN_CENTER);
				table4cell1.setVerticalAlignment(Element.ALIGN_CENTER);
				table4cell1.setNoWrap(true);
				table4cell1.setColspan(3);
				table4.addCell(table4cell1);

		    	String relationship = "";
		    	String relationshipSeparator = ", ";
		    	if(patient.getUserProfile().getIsVeteran() != null) {
		    		if(patient.getUserProfile().getIsVeteran()) {
		    			relationship += veteranRel;
		    		}
		    	}
		    	if(patient.getUserProfile().getIsPatient() != null) {
		    		if(patient.getUserProfile().getIsPatient()) {
		    			if(relationship.length() > 1) {
		    				relationship += relationshipSeparator + patientRel;
		    			} else {
		    				relationship += patientRel;
		    			}
		    		}
		    	}
		    	if(patient.getUserProfile().getIsHealthCareProvider() != null) {
		    		if(patient.getUserProfile().getIsHealthCareProvider()) {
		    			if(relationship.length() > 1) {
		    				relationship += relationshipSeparator + healthCareProviderRel;
		    			} else {
		    				relationship += patientRel;
		    			}
		    		}
		    	}
		    	if(patient.getUserProfile().getIsOther() != null) {
		    		if(patient.getUserProfile().getIsOther()) {
		    			if(relationship.length() > 1) {
		    				relationship += relationshipSeparator + otherRel;
		    			} else {
		    				relationship += patientRel;
		    			}
		    		}
		    	}
		    	PdfPCell table4cell2 = new PdfPCell(new Phrase(new Chunk(relationship, subFont)));
				table4cell2.setBorderColor(BaseColor.GRAY);
				table4cell2.setBorderWidth(1);
				table4cell2.setBorderWidthLeft(1);
				table4cell2.setBorderWidthTop(1);
				table4cell2.setBorderWidthBottom(1);
				table4cell2.setBorderWidthRight(1);
				table4cell2.setHorizontalAlignment(Element.ALIGN_CENTER);
				table4cell2.setVerticalAlignment(Element.ALIGN_CENTER);
				table4cell2.setNoWrap(false);
				table4cell2.setColspan(3);
				table4.addCell(table4cell2);

				String icn = patient.getIcn() != null ? patient.getIcn() : "";
				PdfPCell table4cell3 = new PdfPCell(new Phrase(new Chunk(icn, subFont)));
				table4cell3.setBorderColor(BaseColor.GRAY);
				table4cell3.setBorderWidth(1);
				table4cell3.setBorderWidthLeft(1);
				table4cell3.setBorderWidthTop(1);
				table4cell3.setBorderWidthBottom(1);
				table4cell3.setBorderWidthRight(1);
				table4cell3.setHorizontalAlignment(Element.ALIGN_CENTER);
				table4cell3.setVerticalAlignment(Element.ALIGN_CENTER);
				table4cell3.setNoWrap(true);
				table4cell3.setColspan(5);
				table4.addCell(table4cell3);

				String firstName = patient.getUserProfile().getFirstName() != null ? patient.getUserProfile().getFirstName() : "";
				PdfPCell table4cell4 = new PdfPCell(new Phrase(new Chunk(firstName, subFont)));
				table4cell4.setBorderColor(BaseColor.GRAY);
				table4cell4.setBorderWidth(1);
				table4cell4.setBorderWidthLeft(1);
				table4cell4.setBorderWidthTop(1);
				table4cell4.setBorderWidthBottom(1);
				table4cell4.setBorderWidthRight(1);
				table4cell4.setHorizontalAlignment(Element.ALIGN_CENTER);
				table4cell4.setVerticalAlignment(Element.ALIGN_CENTER);
				table4cell4.setNoWrap(false);
				table4cell4.setColspan(2);
				table4.addCell(table4cell4);

				String middleName = patient.getUserProfile().getMiddleName() != null ? patient.getUserProfile().getMiddleName() : "";
				PdfPCell table4cell5 = new PdfPCell(new Phrase(new Chunk(middleName, subFont)));
				table4cell5.setBorderColor(BaseColor.GRAY);
				table4cell5.setBorderWidth(1);
				table4cell5.setBorderWidthLeft(1);
				table4cell5.setBorderWidthTop(1);
				table4cell5.setBorderWidthBottom(1);
				table4cell5.setBorderWidthRight(1);
				table4cell5.setHorizontalAlignment(Element.ALIGN_CENTER);
				table4cell5.setVerticalAlignment(Element.ALIGN_CENTER);
				table4cell5.setNoWrap(false);
				table4cell5.setColspan(2);
				table4.addCell(table4cell5);

				String lastName = patient.getUserProfile().getLastName() != null ? patient.getUserProfile().getLastName() : "";
				PdfPCell table4cell6 = new PdfPCell(new Phrase(new Chunk(lastName, subFont)));
				table4cell6.setBorderColor(BaseColor.GRAY);
				table4cell6.setBorderWidth(1);
				table4cell6.setBorderWidthLeft(1);
				table4cell6.setBorderWidthTop(1);
				table4cell6.setBorderWidthBottom(1);
				table4cell6.setBorderWidthRight(1);
				table4cell6.setHorizontalAlignment(Element.ALIGN_CENTER);
				table4cell6.setVerticalAlignment(Element.ALIGN_CENTER);
				table4cell6.setNoWrap(false);
				table4cell6.setColspan(2);
				table4.addCell(table4cell6);

				String gender = patient.getUserProfile().getGender() != null ? patient.getUserProfile().getGender() : "";
				PdfPCell table4cell7 = new PdfPCell(new Phrase(new Chunk(gender, subFont)));
				table4cell7.setBorderColor(BaseColor.GRAY);
				table4cell7.setBorderWidth(1);
				table4cell7.setBorderWidthLeft(1);
				table4cell7.setBorderWidthTop(1);
				table4cell7.setBorderWidthBottom(1);
				table4cell7.setBorderWidthRight(1);
				table4cell7.setHorizontalAlignment(Element.ALIGN_CENTER);
				table4cell7.setVerticalAlignment(Element.ALIGN_CENTER);
				table4cell7.setNoWrap(true);
				table4cell7.setColspan(2);
				table4.addCell(table4cell7);

				String birthDate = "";
				String[] birthDateValues = new String[3];
				if(patient.getUserProfile().getBirthDate() != null) {
					birthDate = sdf.format(patient.getUserProfile().getBirthDate());
					birthDateValues = birthDate.split("-");
					birthDate = birthDateValues[2] + "/" + birthDateValues[1] + "/" + birthDateValues[0];
				}
				PdfPCell table4cell8 = new PdfPCell(new Phrase(new Chunk(birthDate, subFont)));
				table4cell8.setBorderColor(BaseColor.GRAY);
				table4cell8.setBorderWidth(1);
				table4cell8.setBorderWidthLeft(1);
				table4cell8.setBorderWidthTop(1);
				table4cell8.setBorderWidthBottom(1);
				table4cell8.setBorderWidthRight(1);
				table4cell8.setHorizontalAlignment(Element.ALIGN_CENTER);
				table4cell8.setVerticalAlignment(Element.ALIGN_CENTER);
				table4cell8.setNoWrap(true);
				table4cell8.setColspan(3);
				table4.addCell(table4cell8);

				String ssn = patient.getUserProfile().getSsn() != null ? patient.getUserProfile().getSsn() : "";
				PdfPCell table4cell9 = new PdfPCell(new Phrase(new Chunk(ssn, subFont)));
				table4cell9.setBorderColor(BaseColor.GRAY);
				table4cell9.setBorderWidth(1);
				table4cell9.setBorderWidthLeft(1);
				table4cell9.setBorderWidthTop(1);
				table4cell9.setBorderWidthBottom(1);
				table4cell9.setBorderWidthRight(1);
				table4cell9.setHorizontalAlignment(Element.ALIGN_CENTER);
				table4cell9.setVerticalAlignment(Element.ALIGN_CENTER);
				table4cell9.setNoWrap(true);
				table4cell9.setColspan(3);
				table4.addCell(table4cell9);

				String correlatedDate = "";
				String[] correlatedDateValues = new String[3];
				if(patient.getCorrelatedDateTime() != null) {
					Timestamp stamp = (Timestamp) patient.getCorrelatedDateTime();
					correlatedDate = sdf.format(new Date(stamp.getTime()));
					correlatedDateValues = correlatedDate.split("-");
					correlatedDate = correlatedDateValues[2] + "/" + correlatedDateValues[1] + "/" + correlatedDateValues[0];
				}
				PdfPCell table4cell10 = new PdfPCell(new Phrase(new Chunk(correlatedDate, subFont)));
				table4cell10.setBorderColor(BaseColor.GRAY);
				table4cell10.setBorderWidth(1);
				table4cell10.setBorderWidthLeft(1);
				table4cell10.setBorderWidthTop(1);
				table4cell10.setBorderWidthBottom(1);
				table4cell10.setBorderWidthRight(1);
				table4cell10.setHorizontalAlignment(Element.ALIGN_CENTER);
				table4cell10.setVerticalAlignment(Element.ALIGN_CENTER);
				table4cell10.setNoWrap(true);
				table4cell10.setColspan(3);
				table4.addCell(table4cell10);

				String matchedDate = "";
				String[] matchedDateValues = new String[3];
				if(patient.getMatchedDateTime() != null) {
					Timestamp stamp = (Timestamp) patient.getMatchedDateTime();
					matchedDate = sdf.format(new Date(stamp.getTime()));
					matchedDateValues = matchedDate.split("-");
					matchedDate = matchedDateValues[2] + "/" + matchedDateValues[1] + "/" + matchedDateValues[0];
				}
				PdfPCell table4cell11 = new PdfPCell(new Phrase(new Chunk(matchedDate, subFont)));
				table4cell11.setBorderColor(BaseColor.GRAY);
				table4cell11.setBorderWidth(1);
				table4cell11.setBorderWidthLeft(1);
				table4cell11.setBorderWidthTop(1);
				table4cell11.setBorderWidthBottom(1);
				table4cell11.setBorderWidthRight(1);
				table4cell11.setHorizontalAlignment(Element.ALIGN_CENTER);
				table4cell11.setVerticalAlignment(Element.ALIGN_CENTER);
				table4cell11.setNoWrap(true);
				table4cell11.setColspan(3);
				table4.addCell(table4cell11);

				String createdDate = "";
				String[] createdDateValues = new String[3];
				if(patient.getUserProfile().getCreatedDate() != null) {
					Timestamp stamp = (Timestamp) patient.getUserProfile().getCreatedDate();
					createdDate = sdf.format(new Date(stamp.getTime()));
					createdDateValues = createdDate.split("-");
					createdDate = createdDateValues[2] + "/" + createdDateValues[1] + "/" + createdDateValues[0];
				}
				PdfPCell table4cell12 = new PdfPCell(new Phrase(new Chunk(createdDate, subFont)));
				table4cell12.setBorderColor(BaseColor.GRAY);
				table4cell12.setBorderWidth(1);
				table4cell12.setBorderWidthLeft(1);
				table4cell12.setBorderWidthTop(1);
				table4cell12.setBorderWidthBottom(1);
				table4cell12.setBorderWidthRight(1);
				table4cell12.setHorizontalAlignment(Element.ALIGN_CENTER);
				table4cell12.setVerticalAlignment(Element.ALIGN_CENTER);
				table4cell12.setNoWrap(true);
				table4cell12.setColspan(3);
				table4.addCell(table4cell12);
				document.add(table4);
			}
		}
		PdfPTable table5 = new PdfPTable(1);
		table5.setHorizontalAlignment(Element.ALIGN_LEFT);
		table5.getDefaultCell().setHorizontalAlignment(Element.ALIGN_LEFT);
		table5.getDefaultCell().setBorder(Rectangle.NO_BORDER);
		table5.getDefaultCell().setBackgroundColor(BaseColor.WHITE);
		table5.getDefaultCell().setNoWrap(false);

		PdfPCell table5cell1 = new PdfPCell(new Phrase(new Chunk(recordsProcessed + "   " + String.valueOf(totalNumberProcessed), subFont)));
		table5cell1.setBorderColor(BaseColor.WHITE);
		table5cell1.setBorder(Rectangle.NO_BORDER);
		table5cell1.setHorizontalAlignment(Element.ALIGN_LEFT);
		table5cell1.setVerticalAlignment(Element.ALIGN_LEFT);
		table5cell1.setNoWrap(true);
		table5cell1.setColspan(1);
		table5.addCell(table5cell1);
		document.add(table5);

		PdfPTable table6 = new PdfPTable(1);
		table6.setHorizontalAlignment(Element.ALIGN_LEFT);
		table6.getDefaultCell().setHorizontalAlignment(Element.ALIGN_LEFT);
		table6.getDefaultCell().setBorder(Rectangle.NO_BORDER);
		table6.getDefaultCell().setBackgroundColor(BaseColor.WHITE);
		table6.getDefaultCell().setNoWrap(false);

		PdfPCell table6cell1 = new PdfPCell(new Phrase(new Chunk(exceptionReported + "  " + String.valueOf(totalReviewed), subFont)));
		table6cell1.setBorderColor(BaseColor.WHITE);
		table6cell1.setBorder(Rectangle.NO_BORDER);
		table6cell1.setHorizontalAlignment(Element.ALIGN_LEFT);
		table6cell1.setVerticalAlignment(Element.ALIGN_LEFT);
		table6cell1.setNoWrap(true);
		table6cell1.setColspan(1);
		table6.addCell(table6cell1);
		document.add(table6);
		document.close();
    }
}
