package gov.va.med.mhv.usermgmt.essapi.service.impl;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

import javax.ws.rs.core.Context;
import javax.ws.rs.core.HttpHeaders;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.UriInfo;

import org.apache.commons.lang3.StringUtils;
import org.apache.cxf.jaxrs.ext.MessageContext;
import org.apache.cxf.jaxrs.impl.ResponseBuilderImpl;

import gov.va.med.mhv.common.api.enumeration.CountryEnum;
import gov.va.med.mhv.common.api.enumeration.ErrorEnum;
import gov.va.med.mhv.common.api.enumeration.StateEnum;
import gov.va.med.mhv.common.api.exception.MHVException;
import gov.va.med.mhv.common.api.transfer.ClientApplication;
import gov.va.med.mhv.common.api.util.ClientApplicationHelper;
import gov.va.med.mhv.common.api.util.MHVApiUtility;
import gov.va.med.mhv.common.api.util.MHVEssApiUtility;
import gov.va.med.mhv.usermgmt.essapi.service.CommonEnumAPIService;

public class CommonEnumAPIServiceImpl implements CommonEnumAPIService{
	
	@Context
	MessageContext mc;
	
	ClientApplication clientApplication = null;
	
	@Override
	public Response getStatesUS() {
		//Run Authentications and Authorizations
		checkRegUrlAuth();

		ResponseBuilderImpl builder = new ResponseBuilderImpl();
		Map<String, String> treeMap = new TreeMap<String, String>(StateEnum.STATES_BY_DETAIL);
		return builder.entity(treeMap).status(200).build();
	}

	@Override
	public Response getCountries() {

		//Run Authentications and Authorizations
		checkRegUrlAuth();

		ResponseBuilderImpl builder = new ResponseBuilderImpl();
		Map<String, String> treeMap = new TreeMap<String, String>(CountryEnum.COUNTRY_BY_DETAIL);
		return builder.entity(treeMap).status(200).build();
	}
	
	private void checkRegUrlAuth() {

		try {
			clientApplication = getclientApplicationUsingHeaderValue(  mc.getHttpHeaders(), mc.getUriInfo() );
		} catch (MHVException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		//Check Application Authentication
		if(clientApplication == null)
			MHVEssApiUtility.throwException(ErrorEnum.APPLICATION_AUTHENTICATION_FAILED_101);
		//Check Application Authentication First
		if( !checkClientSourcePermissions( clientApplication, mc.getHttpHeaders(), mc.getUriInfo() ) ) {
			MHVEssApiUtility.throwException(ErrorEnum.APPLICATION_AUTHENTICATION_FAILED_101);
		}		
		//Check Application Authorization Second
		if( !checkClientAuthorizationPermissions( clientApplication, mc.getHttpHeaders(), mc.getUriInfo() ) ) {
			MHVEssApiUtility.throwException(ErrorEnum.APPLICATION_AUTHORIZATION_FAILED_102);
		}		
	}
	
	private boolean checkClientSourcePermissions( ClientApplication client, HttpHeaders httpHeaders, UriInfo uriInfo ) {
		String path = uriInfo.getRequestUri().getPath();
		if (client == null)
			return false;
		for( String permission: client.getPermissions()) {
			if(path.contains(permission)) {
				return true;
			}
		}
		return false;
	}

	private boolean checkClientAuthorizationPermissions( ClientApplication client, HttpHeaders httpHeaders, UriInfo uriInfo ) {
		String path = uriInfo.getRequestUri().getPath();
		for( String permission: client.getAuthorizations()) {
			if(path.contains(permission)) {
				return true;
			}
		}
		return false;
	}

	private ClientApplication getclientApplicationUsingHeaderValue( HttpHeaders httpHeaders, UriInfo uriInfo ) throws MHVException{
		String appToken = MHVApiUtility.getValueOfHeaderElement(httpHeaders, "appToken");
		ClientApplication source = null;

		if (StringUtils.isEmpty(appToken)) {
			throw new MHVException(ErrorEnum.MISSING_APPLICATION_TOKEN_132);
		} else {
			try {
				source = ClientApplicationHelper.findClientApplication(appToken);
			} catch (Exception e) {
				throw new MHVException(ErrorEnum.APPLICATION_AUTHENTICATION_FAILED_101);
			}
		}

		return source;
	}

}
