package gov.va.med.mhv.usermgmt.essapi.service.impl;

import java.sql.Timestamp;
import java.util.Date;
import java.util.HashMap;
import java.util.List;

import javax.ws.rs.Consumes;
import javax.ws.rs.Produces;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.HttpHeaders;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.UriInfo;

import org.apache.commons.lang3.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.cxf.jaxrs.ext.MessageContext;
import org.apache.cxf.jaxrs.impl.ResponseBuilderImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;

import gov.va.med.mhv.common.api.dto.FacilityInfoDTO;
import gov.va.med.mhv.common.api.enumeration.ErrorEnum;
import gov.va.med.mhv.common.api.exception.MHVException;
import gov.va.med.mhv.common.api.transfer.ClientApplication;
import gov.va.med.mhv.common.api.util.ClientApplicationHelper;
import gov.va.med.mhv.common.api.util.MHVApiUtility;
import gov.va.med.mhv.common.api.util.MHVEssApiUtility;
import gov.va.med.mhv.common.api.util.ResponseUtil;
import gov.va.med.mhv.common.data.converter.FacilityInfoConveter;
import gov.va.med.mhv.common.data.model.FacilityInfo;
import gov.va.med.mhv.common.data.model.InPersonAuthentication;
import gov.va.med.mhv.common.data.model.Patient;
import gov.va.med.mhv.common.data.model.PatientRegistryChange;
import gov.va.med.mhv.common.data.model.UserProfile;
import gov.va.med.mhv.mvi.xsd.II;
import gov.va.med.mhv.usermgmt.common.enums.ActivityActionTypeEnumeration;
import gov.va.med.mhv.usermgmt.common.enums.ActivityActorTypeEnumeration;
import gov.va.med.mhv.usermgmt.common.enums.ActivityTypeEnumeration;
import gov.va.med.mhv.usermgmt.common.enums.AuthenticationStatusEnumeration;
import gov.va.med.mhv.usermgmt.common.enums.PatientCorrelationStatusEnumeration;
import gov.va.med.mhv.usermgmt.common.enums.UserAcceptExtTermsEvents;
import gov.va.med.mhv.usermgmt.data.model.TermsConditionsVerHistory;
import gov.va.med.mhv.usermgmt.data.model.UserAcceptExtTermsVer;
import gov.va.med.mhv.usermgmt.data.model.UserAcceptTermsHistory;
import gov.va.med.mhv.usermgmt.data.repository.InPersonAuthenticationRepository;
import gov.va.med.mhv.usermgmt.data.repository.PatientRepository;
import gov.va.med.mhv.usermgmt.data.repository.TermsConditionsVerHistoryRepository;
import gov.va.med.mhv.usermgmt.data.repository.UserAcceptExtTermsVerRepository;
import gov.va.med.mhv.usermgmt.data.repository.UserAcceptTermsHistoryRepository;
import gov.va.med.mhv.usermgmt.data.repository.UserProfileRepository;
import gov.va.med.mhv.usermgmt.essapi.service.RegistrationAPIService;
import gov.va.med.mhv.usermgmt.essapi.service.util.RandomUserNameGenerator;
import gov.va.med.mhv.usermgmt.essapi.transfer.AcctUpgradeTO;
import gov.va.med.mhv.usermgmt.essapi.transfer.RegisterResTO;
import gov.va.med.mhv.usermgmt.essapi.transfer.RegisterTO;
import gov.va.med.mhv.usermgmt.essapi.transfer.UserProfileAPIConverter;
import gov.va.med.mhv.usermgmt.service.AccountActivityLogService;
import gov.va.med.mhv.usermgmt.service.AccountValidatorService;
import gov.va.med.mhv.usermgmt.service.InPersonAuthenticationService;
import gov.va.med.mhv.usermgmt.service.MviAsyncIntegrationService;
import gov.va.med.mhv.usermgmt.service.MviIntegrationService;
import gov.va.med.mhv.usermgmt.service.handler.MviProperties;
import gov.va.med.mhv.usermgmt.util.activity.ActivityHelper;
import gov.va.med.mhv.usermgmt.util.mvi.MviConstants;
import gov.va.med.mhv.usermgmt.util.mvi.MviUtil;

@Produces({"application/xml","application/json","application/octet-stream"})
@Consumes({"application/xml","application/json"})
public class RegistrationAPIServiceImpl implements RegistrationAPIService{

	@Context
	MessageContext mc;

	//Service List

	@Autowired
	private MviIntegrationService mviIntegrationService;

	@Autowired
	private MviAsyncIntegrationService mviAsyncIntegrationService;

	@Autowired
	private AccountValidatorService accountValidatorService;

	@Autowired
	private InPersonAuthenticationService ipaService;

	@Autowired
	@Qualifier("activityServiceLocal")
	private AccountActivityLogService activityLogService;

	// Repository List

	@Autowired
	private UserProfileRepository userProfileRepository;

	@Autowired
	private PatientRepository patientRepository;

	@Autowired
	private InPersonAuthenticationRepository inpersonAuthenticationRepository;

	@Autowired
	TermsConditionsVerHistoryRepository termsConditionsVerHistoryRepository;

	@Autowired
	private UserAcceptTermsHistoryRepository usrAcceptTermHistRepository;

	@Autowired
	private UserAcceptExtTermsVerRepository userAcceptExtTermsVerRepository;

	@Autowired
	private UserProfileAPIConverter userProfileApiConverter;

	@Autowired
	private MviProperties mviProperties;

	private static final Log log = LogFactory.getLog(RegistrationAPIServiceImpl.class);

	ClientApplication clientApplication = null;
	String cspId = null;

	/* (non-Javadoc)
	 * Service to execute Alternate Registration
	 */
	@Override
	public Response registerEss(RegisterTO registerTO) {

		//clean up Call
		//cleanUpDevData();

		boolean isPreExistingAccount = false;
	//	boolean isOnlyCorrRequired = false;
		boolean isSearchbyPII = false;
		UserProfile userProfileDomain = null;
		Patient existingPatient = null;

		//Run Authentications and Authorizations
		checkRegUrlAuth();

		ResponseUtil<Object> response = new ResponseUtil<Object>();

		cspId = registerTO.getSignInPartners();
		if(cspId == null || cspId.isEmpty())
			MHVEssApiUtility.throwException(ErrorEnum.USER_REGISTRATION_INVALID_INPUT);

		// Step 1:  Lookup in MVI thru ICN
		ResponseUtil<Object> searchByICNResponse = new ResponseUtil<Object>();
		checkSumICN(registerTO.getIcn());
		HashMap<String, Object> patientInfoMap = mviIntegrationService.search1305PersonInMVIByICN(registerTO.getIcn().trim(), searchByICNResponse);

		if(searchByICNResponse.isFailure()) {
			MHVEssApiUtility.throwExceptionOnErrors(ErrorEnum.USER_REGISTRATION_MVI_ISSUE, searchByICNResponse);
		}

		if(patientInfoMap.get(MviConstants.DFN) != null) {
			//200MH exists against the ICN
			log.error("ICN Exists in MVI with 200MH" + registerTO.getIcn());
			MHVEssApiUtility.throwException(ErrorEnum.USER_REGISTRATION_EXIST_CORR);
		}


		// Step 2: Lookup in eVault..2a. Search by ICN and 2b. Search by FN, LN, SSN, DOB
		List<Patient> patList = patientRepository.getPatientByIcn(registerTO.getIcn().trim());
		if(patList.isEmpty()) {
			List<UserProfile> userProfileList =  userProfileRepository .getActiveProfileByBirthDateSSN(
					(Date)patientInfoMap.get(MviConstants.BIRTH_DAY),
					(String)patientInfoMap.get(MviConstants.SSN),
					(String)patientInfoMap.get(MviConstants.FIRST_NAME),
					(String)patientInfoMap.get(MviConstants.LAST_NAME)
					);
			if (userProfileList.size() > 1) {
				//Multiple Users Exist... Cry out LOUD
				// Jazz Defect 446105- Fortify Errors for "Privacy Violation"
				//log.error("Multiple Users Exist in MHV with SSN:" + (String)patientInfoMap.get(MviConstants.SSN) +
				//		" AND BIRTHDATE: "+ (Date)patientInfoMap.get(MviConstants.BIRTH_DAY));
				log.error("Multiple Users Exist in MHV with SSN - First_Name:" + (String)patientInfoMap.get(MviConstants.FIRST_NAME));
				MHVEssApiUtility.throwException(ErrorEnum.USER_REGISTRATION_FAILED_MULTI_USER_EXISTS);
			}
			else if (userProfileList.size() == 1) {
				userProfileDomain = userProfileList.get(0);
				isPreExistingAccount = true;
				isSearchbyPII = true;
			}
		}else {
			userProfileDomain = patList.get(0).getUserProfile();
			isPreExistingAccount = true;
			//Check Profile SSN with MVI Response just to be double sure
			if(!userProfileDomain.getSsn().equals(patientInfoMap.get(MviConstants.SSN))) {
				MHVEssApiUtility.throwException(ErrorEnum.USER_REGISTRATION_ICNSSN_MISMATCH);
			}
		}

		if(isPreExistingAccount) {
			//Check whether account is restricted
			if (userProfileDomain.getRestrictSIPAccess()) {
				MHVEssApiUtility.throwException(ErrorEnum.USER_REGISTRATION_RESTRICT_SIP);
			}

			//Account Exists in MHV
			List<Patient> patientList = patientRepository.getPatientByUserProfileId(userProfileDomain.getId());
			existingPatient = patientList.get(0);

			//Check Correlation Status..
			if(existingPatient.getCorrelationStatus().equals(PatientCorrelationStatusEnumeration.CORRELATED.getValue())) {
				// Jazz Defect 446105- Fortify Errors for "Privacy Violation"
				//log.error("User is in CORRELATED statue in MHV with" + " ICN:" + registerTO.getIcn() + " SSN:" + (String)patientInfoMap.get(MviConstants.SSN) +
				//		" AND BIRTHDATE: "+ (Date)patientInfoMap.get(MviConstants.BIRTH_DAY) +" isSearchbyPII :" + isSearchbyPII);
				log.error("User is in CORRELATED statue in MHV with" + " ICN:" + registerTO.getIcn() +" isSearchbyPII :" + isSearchbyPII);
				MHVEssApiUtility.throwException(ErrorEnum.USER_REGISTRATION_EXIST_CORR_MHV);
			}

			//Update ICN if required
			if (isSearchbyPII && !existingPatient.getIcn().equalsIgnoreCase(registerTO.getIcn())) {

				//Update Patient Registry Change based on ICN
				PatientRegistryChange changeRecord = new PatientRegistryChange();
				changeRecord.setRecordedOnDate(new Timestamp(System.currentTimeMillis()));
				changeRecord.setOldIcn(existingPatient.getIcn());
				existingPatient.addPatientRegistryChange(changeRecord);
				existingPatient.setIcn(registerTO.getIcn());
				existingPatient = patientRepository.saveAndFlush(existingPatient);
			}

			//TODO: update evault First Name, Last Name with MVI response??

		}
		else { //New Account Creation is Required
			userProfileDomain = userProfileApiConverter.convertDomainProfile(new UserProfile(), registerTO, patientInfoMap);
			userProfileDomain = setRestrictSignParams(userProfileDomain, registerTO);
			userProfileDomain.setIsAlternate(true);
			getReCreateDistUserName(patientInfoMap, userProfileDomain);

			//Check whether user is patient OR Veteran..Overwrite if required
			II[] searchPatientIdArray = (II[]) patientInfoMap.get(MviConstants.SEARCH_PATIENT_IDARRAY);
			List<String> stationNumberArray = MviUtil.pullStationNumbersfrom1306Mvi(searchPatientIdArray);
			if (MviUtil.checkTreatmentFacilitiesExist(stationNumberArray)) {
				userProfileDomain.setIsPatient(true);
			} else {
				userProfileDomain.setIsPatient(false);
			}

			userProfileDomain = userProfileRepository.saveAndFlush(userProfileDomain);

			//Set HMV T&C for user
			TermsConditionsVerHistory latestTerms = getLatestTermsAndCond();
			UserAcceptTermsHistory usrAcceptTermsHist = new UserAcceptTermsHistory();
			usrAcceptTermsHist.setUserUpdatedTime(new Date());
			usrAcceptTermsHist.setAccepted(Boolean.TRUE);
			usrAcceptTermsHist.setUserProfileId(userProfileDomain.getId());
			usrAcceptTermsHist.setTermsCondVerHistId(latestTerms.getId());
			usrAcceptTermHistRepository.save(usrAcceptTermsHist);


			//Set EXT T&C for user
			UserAcceptExtTermsVer userAcceptExtTermsVer = new UserAcceptExtTermsVer();
			userAcceptExtTermsVer.setExtTermsCondEvent(UserAcceptExtTermsEvents.EXT_ACCOUNT_REGISTRATION_TC.getDescription());
			userAcceptExtTermsVer.setExtTermsCondVer(registerTO.getTermsVersion());
			userAcceptExtTermsVer.setAcceptedDate(MHVApiUtility.convertRFCPatterntoDate(registerTO.getTermsAcceptedDate()));
			userAcceptExtTermsVer.setUserProfileId(userProfileDomain.getId());
			userAcceptExtTermsVer.setExtTermsCondVerNotes(MHVEssApiUtility.makeActivityDetailValue(clientApplication.getName()));
			userAcceptExtTermsVerRepository.save(userAcceptExtTermsVer);


			Patient newPatient = new Patient();
			//why??
			newPatient.setOplock(1);
			newPatient.setCorrelationStatus(PatientCorrelationStatusEnumeration.MVIDATAMATCH.getValue());
			newPatient.setMatchedDateTime(new Date());
			newPatient.setUserProfile(userProfileDomain);
			newPatient.setIcn((String)patientInfoMap.get(MviConstants.ICN));


			existingPatient = patientRepository.saveAndFlush(newPatient);
			//Activity For registration and MVI match
			accountActivityReg(userProfileDomain.getId(), true, ActivityTypeEnumeration.REGISTRATION, ActivityActionTypeEnumeration.ACCOUNT_CREATED);
			//Activity For T&C Accept
			accountActivityReg(userProfileDomain.getId(), true, ActivityTypeEnumeration.TERMS_AND_CONDITIONS, ActivityActionTypeEnumeration.ACCEPT);

		}

		//Facility Update now or later ? - Only for New comers
		//Facility Update
		if(!isPreExistingAccount) {
			mviIntegrationService.updateTreatmentFacilitiesForExistingPatients(existingPatient, response);
			existingPatient = (Patient)response.getPojoObject();
		}

		// Step 3a: Existing/New account trying to CORRELATE
		ResponseUtil<Object> corrResponse = new ResponseUtil<Object>();
		// Activity For Patient Corr Request
		accountActivityReg(userProfileDomain.getId(), true, ActivityTypeEnumeration.ACCOUNT_UPGRADE, ActivityActionTypeEnumeration.ACCOUNT_CORRELATION_REQUESTED);

		mviIntegrationService.addPersonForCorrelationAPI(existingPatient, userProfileDomain, corrResponse, null);
		boolean correlationSuccess = true;
		if(corrResponse.isFailure()) {
			//	MHVEssApiUtility.throwExceptionOnErrors(ErrorEnum.USER_REGISTRATION_MVI_ISSUE, searchByICNResponse);
			correlationSuccess = false;
		}

		if(!isPreExistingAccount) {
			if(existingPatient.getCorrelationStatus().equals(PatientCorrelationStatusEnumeration.CORRELATED.getValue())) {
				accountActivityReg(userProfileDomain.getId(), true, ActivityTypeEnumeration.ACCOUNT_UPGRADE, ActivityActionTypeEnumeration.ACCOUNT_CONNECTED);
			}
		}
		else {
			if(existingPatient.getCorrelationStatus().equals(PatientCorrelationStatusEnumeration.CORRELATED.getValue())) {
				accountActivityReg(userProfileDomain.getId(), true, ActivityTypeEnumeration.ACCOUNT_UPGRADE, ActivityActionTypeEnumeration.UPGRADE_TO_ADVANCED);
			}
		}

		//Facility Update?
		existingPatient = (Patient )corrResponse.getPojoObject();
		mviIntegrationService.updateTreatmentFacilitiesForExistingPatients(existingPatient, response);
		existingPatient = (Patient)response.getPojoObject();

		//Create Response Object
		RegisterResTO res = new RegisterResTO(userProfileDomain.getId(),
				correlationSuccess?"MVI Correlation Completed": "MVI Correlation Queued","Successful");

		// if the flow comes this far. then its a good.
		ResponseBuilderImpl builder = new ResponseBuilderImpl();
		return builder.entity(res).status(200).build();

	}

	/* (non-Javadoc)
	 * Service to convert the account to Premium
	 */
	@Override
	public Response makePremium(AcctUpgradeTO upgradeTO) {
		//Run Authentications and Authorizations
		checkRegUrlAuth();

		//Get Patient info from session
		List<Patient> patientList = patientRepository.getPatientByUserProfileId(upgradeTO.getUserId());
		if (patientList.size() == 0) {
			MHVEssApiUtility.throwException(ErrorEnum.DATA_INTEGRITY_ERROR_117);
		}
		Patient patient = patientList.get(0);

		ResponseUtil<Object> ipaResponse = new ResponseUtil<Object>();

		try {
			ipaService.getIPAPatientById(patient.getId(), ipaResponse);
			InPersonAuthentication ipa = (InPersonAuthentication) ipaResponse.getPojoObject();
			FacilityInfoDTO mhvFacilityDTO = accountValidatorService.getAustinMHVFacilityInfo();
			FacilityInfoConveter facInfoConverter = new FacilityInfoConveter();
			FacilityInfo mhvFacility = facInfoConverter.convertDTOFacilityInfo(mhvFacilityDTO);

			//Check MVI Auth Status..
			if(ipa.getMviAuthenticationStatus() != null && (ipa.getMviAuthenticationStatus().equals("OK") || ipa.getMviAuthenticationStatus().equals("PENDING_AUTH"))) {
				MHVEssApiUtility.throwException(ErrorEnum.USER_MVI_ACCT_ALREADY_UPGRADED);
			}

			//MVI Call for Authentication
			ResponseUtil<Object> mviAuthResponse = new ResponseUtil<Object>();
			mviIntegrationService.mviAuthenticate(patient, patient.getUserProfile(), true, false, mviAuthResponse, null);

			//if fails then it will be picked by the cronjob..
			if(!mviAuthResponse.isFailure()) {
				ipa.setMviAuthenticationStatus("OK");
			}
			else {
				ipa.setMviAuthenticationStatus("PENDING_AUTH");
			}

			//Set EXT T&C for user
			UserAcceptExtTermsVer userAcceptExtTermsVer = new UserAcceptExtTermsVer();
			userAcceptExtTermsVer.setExtTermsCondEvent(UserAcceptExtTermsEvents.EXT_ACCOUNT_UPGRADE_TC.getDescription());
			userAcceptExtTermsVer.setExtTermsCondVer(upgradeTO.getTermsVersion());
			userAcceptExtTermsVer.setAcceptedDate(MHVApiUtility.convertRFCPatterntoDate(upgradeTO.getFormSignedDateTime()));
			userAcceptExtTermsVer.setUserProfileId( patient.getUserProfile().getId());
			userAcceptExtTermsVer.setExtTermsCondVerNotes(MHVEssApiUtility.makeActivityDetailValue(clientApplication.getName()));
			userAcceptExtTermsVerRepository.save(userAcceptExtTermsVer);

			//Set Required params in IPA
			ipa.setAuthenticatedBy("System");
			// Set Austin Facility
			ipa.setAuthenticatingFacility(mhvFacility);
			// Boolean values..
			ipa.setParticipationFormSigned(Boolean.TRUE);
			ipa.setIdentificationPresented(Boolean.TRUE);
			ipa.setVideoViewed(true);
			ipa.setDefermentReason(null);

			// Set Dates
			if(upgradeTO.getFormSignedDateTime() != null) {
				ipa.setSignedFormOnlineDateTime(MHVApiUtility.convertRFCPatterntoDate(
						upgradeTO.getFormSignedDateTime()));
			}
			if(upgradeTO.getFormUpgradeOnlineDate() != null) {
				ipa.setStartUpgradeOnlineDateTime(MHVApiUtility.convertRFCPatterntoDate(
						upgradeTO.getFormUpgradeOnlineDate()));
			}
			if(upgradeTO.getFormUpgradeManualDate() != null) {
				ipa.setStartUpgradeManualDateTime(MHVApiUtility.convertRFCPatterntoDate(
						upgradeTO.getFormUpgradeManualDate()));
			}
			ipa.setAuthenticationDate(new Date());

			//Set status
			ipa.setStatus(AuthenticationStatusEnumeration.AUTHENTICATED.getDescription());

			//Save the state
			inpersonAuthenticationRepository.save(ipa);
			
			//Create Activity Log
			accountActivityReg(patient.getUserProfile().getId(), true, ActivityTypeEnumeration.ACCOUNT_UPGRADE, ActivityActionTypeEnumeration.UPGRADE_TO_PREMIUM);

		} catch (MHVException e) {
			log.error("Exception Occured" + e);
			MHVEssApiUtility.throwException(ErrorEnum.USER_ACCTUPGRADE_FAILED);
		}

		return Response.ok("{\"status\":\"success\" }").build();

	}

	private void getReCreateDistUserName(HashMap<String, Object> patientInfoMap,
			UserProfile userProfileDomain) {
		//UserProfile uf = userProfileRepository.findByUserName(userProfileDomain.getUserName());
		for(int i=0; i<10; i++) {

			String distUserName = RandomUserNameGenerator.userNameGenerate( (String)patientInfoMap.get(MviConstants.FIRST_NAME),
					(String)patientInfoMap.get(MviConstants.LAST_NAME),
					(Date)patientInfoMap.get(MviConstants.BIRTH_DAY)
					);
			UserProfile uf = userProfileRepository.findByUserName(distUserName);
			if(uf == null) {
				userProfileDomain.setUserName(distUserName);
				return;
			}

		}
		log.error("UserName creation try exhausted");
		MHVEssApiUtility.throwException(ErrorEnum.USER_REGISTRATION_FAILED);
		return;

	}

	private void checkRegUrlAuth() {

		try {
			clientApplication = getclientApplicationUsingHeaderValue(  mc.getHttpHeaders(), mc.getUriInfo() );
		} catch (MHVException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		//Check Application Authentication
		if(clientApplication == null)
			MHVEssApiUtility.throwException(ErrorEnum.APPLICATION_AUTHENTICATION_FAILED_101);
		//Check Application Authentication First
		if( !checkClientSourcePermissions( clientApplication, mc.getHttpHeaders(), mc.getUriInfo() ) ) {
			MHVEssApiUtility.throwException(ErrorEnum.APPLICATION_AUTHENTICATION_FAILED_101);
		}
		//Check Application Authorization Second
		if( !checkClientAuthorizationPermissions( clientApplication, mc.getHttpHeaders(), mc.getUriInfo() ) ) {
			MHVEssApiUtility.throwException(ErrorEnum.APPLICATION_AUTHORIZATION_FAILED_102);
		}
	}

	private void checkSumICN(String icn) {
		boolean checkIcn = false;
		if(icn != null)
			checkIcn = 	icn.matches("^\\d{10}[V]{1}\\d{6}");

		if(!checkIcn) {
			MHVEssApiUtility.throwExceptionOnErrors(ErrorEnum.USER_REGISTRATION_INVALID_INPUT, new MHVException("Invalid ICN"));
		}

	}

	private TermsConditionsVerHistory getLatestTermsAndCond(){

		List<TermsConditionsVerHistory> termsList = termsConditionsVerHistoryRepository.getVersionByUpdateTime();

		return termsList.get(0);

	}



	private void accountActivityReg(long id, boolean status, ActivityTypeEnumeration activityTypeEnum, ActivityActionTypeEnumeration actionTypeEnum ) {
		try {
			activityLogService.createAccountActivityLog(
					ActivityHelper.createActivityDTO(
							id,
							ActivityActorTypeEnumeration.SYSTEM,
							status,
							activityTypeEnum,
							actionTypeEnum,
							MHVEssApiUtility.makeActivityDetailValue(cspId != null ? clientApplication.getName() + ":"+ cspId : clientApplication.getName() )
							)
					);
		} catch (MHVException e) {
			log.error("Exception occured in Activity Track", e);
			MHVEssApiUtility.throwExceptionOnErrors(ErrorEnum.UNKNOWN_99, e);
		}
	}

	private boolean checkClientSourcePermissions( ClientApplication client, HttpHeaders httpHeaders, UriInfo uriInfo ) {
		String path = uriInfo.getRequestUri().getPath();
		if (client == null)
			return false;
		for( String permission: client.getPermissions()) {
			if(path.contains(permission)) {
				return true;
			}
		}
		return false;
	}

	private boolean checkClientAuthorizationPermissions( ClientApplication client, HttpHeaders httpHeaders, UriInfo uriInfo ) {
		String path = uriInfo.getRequestUri().getPath();
		for( String permission: client.getAuthorizations()) {
			if(path.contains(permission)) {
				return true;
			}
		}
		return false;
	}

	private ClientApplication getclientApplicationUsingHeaderValue( HttpHeaders httpHeaders, UriInfo uriInfo ) throws MHVException{
		String appToken = MHVApiUtility.getValueOfHeaderElement(httpHeaders, "appToken");
		ClientApplication source = null;

		if (StringUtils.isEmpty(appToken)) {
			throw new MHVException(ErrorEnum.MISSING_APPLICATION_TOKEN_132);
		} else {
			try {
				source = ClientApplicationHelper.findClientApplication(appToken);
			} catch (Exception e) {
				throw new MHVException(ErrorEnum.APPLICATION_AUTHENTICATION_FAILED_101);
			}
		}

		return source;
	}

	private UserProfile setRestrictSignParams(UserProfile userProfileDomain, RegisterTO registerTO) {

		userProfileDomain.setRestrictSIPAccess(false);
		//CSID value should be coming from API input params
		userProfileDomain.setSignInPartners(registerTO.getSignInPartners());

		return userProfileDomain;
	}

	private void cleanUpDevData() {
		//1393824 
		boolean cleanIt = false;
		if(cleanIt) {
			UserProfile ufJunk = userProfileRepository.getUserProfileById(13998212l);
			Patient patJunk = patientRepository.getPatientByUserProfileId(ufJunk.getId()).get(0);
			ResponseUtil<Object> remCorr = new ResponseUtil<Object>();
			mviIntegrationService.removeCorrelationInMVI(patJunk, ufJunk, null, remCorr);
			MHVEssApiUtility.throwException(ErrorEnum.DATA_INTEGRITY_ERROR_117);
		}

	}

}
