package gov.va.med.mhv.usermgmt.main.service.impl;

import java.util.List;
import java.util.StringTokenizer;

import org.apache.commons.mail.HtmlEmail;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import gov.va.med.mhv.usermgmt.common.dto.EmailDTO;
import gov.va.med.mhv.usermgmt.service.EmailFailureException;
import gov.va.med.mhv.usermgmt.service.EmailService;

/**
 * @author vhaislegberb
 *
 */
@Component
public class CommonsHtmlEmailService implements EmailService {

	private static final Logger logger = LogManager.getLogger(CommonsHtmlEmailService.class);

	private static final String EMAIL_DELIM_CHAR = ",";

	@Value("${email.service.smtp.host}")
	private String smtpHost;

	@Value("${email.service.smtp.debug}")
	private Boolean smtpDebug;

	@Value("${email.service.message.bcc}")
	private String bccEmailAddresses;

	@Override
	public boolean send(EmailDTO emailDto) throws EmailFailureException {
		System.out.println(">>>> CommonsHtmlEmailService - send(EmailDTO) - Inside:");
		return send(emailDto.getFrom(), emailDto.getFromName(), emailDto.getSubject(), emailDto.getMessage(),
				emailDto.getRecipient(), null, bccEmailAddresses, emailDto.getMessageContentType());
	}

	@Override
	public boolean send(String from, String fromName, String subject, String message, String recipient,
			String messageContentType) throws EmailFailureException {

		return send(from, fromName, subject, message, recipient, null, bccEmailAddresses, messageContentType);
	}

	@Override
	public boolean send(String from, String fromName, String subject, String message, List<String> recipients,
			String messageContentType) throws EmailFailureException {

		for (String emailTo : recipients) {
			send(from, fromName, subject, message, emailTo, null, bccEmailAddresses, messageContentType);
		}
		return true;
	}

	@Override
	public boolean send(String from, String fromName, String subject, String message, List<String> recipients,
			String cc, String bcc, String messageContentType) throws EmailFailureException {

		for (String emailTo : recipients) {
			send(from, fromName, subject, message, emailTo, cc, bcc, messageContentType);
		}
		return true;
	}

	/**
	 * Note: messageContentType is ignored since it's always sent as HTML email
	 * also function always returns true, change return type to void
	 */
	@Override
	public boolean send(String from, String fromName, String subject, String message, String recipient,
			String cc, String bcc, String messageContentType) throws EmailFailureException {

		HtmlEmail email = new HtmlEmail();
		System.out.println(">>>> CommonsHtmlEmailService - send(.. Message) - Inside - message:"+message);
		try {
			email.setSubject(subject);
			email.setHostName(smtpHost);
			email.setDebug(smtpDebug);
			email.setHtmlMsg(message);

			StringTokenizer toTok = new StringTokenizer(recipient, EMAIL_DELIM_CHAR);
			while (toTok.hasMoreTokens()) {
				email.addTo(toTok.nextToken());
			}

			email.setFrom(from, fromName);

			if (cc != null && cc.length() > 0) {
				StringTokenizer ccTok = new StringTokenizer(cc, EMAIL_DELIM_CHAR);
				while (ccTok.hasMoreTokens()) {
					email.addCc(ccTok.nextToken());
				}
			}

			if (bcc != null && bcc.length() > 0) {
				StringTokenizer bccTok = new StringTokenizer(bcc, EMAIL_DELIM_CHAR);
				while (bccTok.hasMoreTokens()) {
					email.addBcc(bccTok.nextToken());
				}
			}

			//email.setMsg(message);
			email.send();

		} catch (Exception e) {
			logger.error(e);
			throw new EmailFailureException("Error sending email message", e);
		}

		return true;
	}

	// these setters are problematic since this is a singleton, so changing these would affect all consumers
	public void setBccEmailAddresses(String bccEmailAddress) {
		this.bccEmailAddresses = bccEmailAddress;
	}

	public void setSmtpHost(String smtpHost) {
		this.smtpHost = smtpHost;
	}

	public void setSmtpDebug(Boolean smtpDebug) {
		this.smtpDebug = smtpDebug;
	}
}
