package gov.va.med.mhv.usermgmt.main.service.impl;

import java.util.Collection;
import java.util.Date;
import java.util.List;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import gov.va.med.mhv.common.api.dto.FacilityInfoDTO;
import gov.va.med.mhv.common.api.dto.InPersonAuthenticationDTO;
import gov.va.med.mhv.common.api.dto.InPersonAuthenticationDTOLight;
import gov.va.med.mhv.common.api.dto.PatientDTO;
import gov.va.med.mhv.common.api.exception.MHVException;
import gov.va.med.mhv.common.api.util.ResponseUtil;
import gov.va.med.mhv.common.data.converter.InPersonAuthenticationConverter;
import gov.va.med.mhv.common.data.converter.InPersonAuthenticationLightConverter;
import gov.va.med.mhv.common.data.converter.PatientConverter;
import gov.va.med.mhv.common.data.model.InPersonAuthentication;
import gov.va.med.mhv.common.data.model.InPersonAuthenticationLight;
import gov.va.med.mhv.common.data.model.Patient;
import gov.va.med.mhv.usermgmt.common.dto.DirectMessageDTO;
import gov.va.med.mhv.usermgmt.common.enums.AuthenticationStatusEnumeration;
import gov.va.med.mhv.usermgmt.data.model.DirectMessage;
import gov.va.med.mhv.usermgmt.data.repository.InPersonAuthenticationLightRepository;
import gov.va.med.mhv.usermgmt.data.repository.InPersonAuthenticationRepository;
import gov.va.med.mhv.usermgmt.exception.MHVRuntimeException;
import gov.va.med.mhv.usermgmt.main.service.adapter.impl.MpiProperties;
import gov.va.med.mhv.usermgmt.service.InPersonAuthenticationService;
import gov.va.med.mhv.usermgmt.service.MviIntegrationService;
import gov.va.med.mhv.usermgmt.service.PatientService;
import gov.va.med.mhv.usermgmt.service.handler.MviProperties;

@Component
public class InPersonAuthenticationServiceImpl implements InPersonAuthenticationService {
	
	private static final InPersonAuthenticationConverter IPA_CONVERTER = new InPersonAuthenticationConverter();
	private static final InPersonAuthenticationLightConverter IPALIGHT_CONVERTER = new InPersonAuthenticationLightConverter();

	private static Logger log = LogManager.getLogger(InPersonAuthenticationServiceImpl.class);

	@Autowired
	private InPersonAuthenticationRepository inpersonAuthenticationRepository;
	
	@Autowired
	private InPersonAuthenticationLightRepository inpersonAuthenticationLightRepository;

	@Autowired
	PatientService patientService;
	
	@Autowired
	InPersonAuthenticationConverter converter;
	
	@Autowired
	MviIntegrationService mviService;
	
	@Autowired
	PatientConverter patientConverter;
	
	@Autowired
	private MpiProperties mpiProperties;
	
	@Autowired
	private MviProperties mviProperties;
	
	@Override
	@Transactional
	public void getIPAPatientById(Long patientId, ResponseUtil response) {
		List<InPersonAuthentication> ipaPatients = null;
		InPersonAuthentication ipaPatient = null;
		try {
			ipaPatients = inpersonAuthenticationRepository.getAuthenticationForPatient(patientId);

			if (null != ipaPatients && ipaPatients.size() > 0) {
				ipaPatient = ipaPatients.get(0);
			}
			response.setFailure(false);
			response.setSuccess(true);
			response.setPojoObject(ipaPatient);
		} catch (Exception e) {
			log.error("Error in getting patinet details from getAuthenticationForPatient() " + patientId, e);
			response.setFailureMessage("Error in getting patinet details from getAuthenticationForPatient() " + patientId);
			response.setFailure(true);
			response.setSuccess(false);
			throw new MHVRuntimeException("Error in getting patinet details from getAuthenticationForPatient() " + patientId);
		}
		return;
	}

	@Override
	public void getIPAByuserProfileid(Long userProfileid, ResponseUtil response) {
		// TODO Auto-generated method stub
		return;
	}

	@Override
	public void getIPAedPatientForUser(String userName, ResponseUtil response) {
		// TODO Auto-generated method stub
		return;
	}

	
	
	@Override
	public void authenticateOnline(InPersonAuthenticationDTOLight ipaLight, ResponseUtil response) {
		log.debug("called authenticateOnline method in InPersonAuthenticationServiceImpl");
		InPersonAuthenticationDTO ipaFull = null;
		// 1. get patient
		try {
			ResponseUtil patientResponse = patientService.getPatientByUserProfileId(ipaLight.getPatientId());
			PatientDTO patient = (null != patientResponse.getPojoObject()) ? (PatientDTO) patientResponse.getPojoObject() : null;
			// 2. then get IPAs
			if(patient.getIpas() != null && patient.getIpas().size() > 0) {
				ipaFull = patient.getIpas().iterator().next();
			}
		} catch (Exception e) {
			String message = String.format("An unexpected error occurred while looking up patient record by user profile ID. %s", ipaLight.getPatientId());
			log.error(message, e);
		}
		
		// 3. then setup IPADTO from IPADTOLight
		ipaFull.setVideoViewed(ipaLight.getVideoViewed());
		ipaFull.setIdentificationPresented(ipaLight.getIdentificationPresented());
		ipaFull.setParticipationFormSigned(ipaLight.getParticipationFormSigned());
        ipaFull.setApprovedForRecordsAccess(ipaLight.getApprovedForRecordsAccess());
        ipaFull.setAuthenticatingFacilityDTO(ipaLight.getAuthenticatingFacilityDTO());
        ipaFull.setDefermentReason(ipaLight.getDefermentReason());
        ipaFull.setSignedFormOnlineDateTime(ipaLight.getSignedFormOnlineDateTime());
        ipaFull.setAuthenticatedBy(ipaLight.getAuthenticatedBy());				
		log.debug("ipa.getParticipationFormSigned()"+ipaLight.getParticipationFormSigned());
		String status = AuthenticationStatusEnumeration.AUTHENTICATED.getDescription();
        ipaFull.setRemovalReason(null);
        ipaFull.setStatus(status);
        saveInPersonAuthentication(ipaFull);
        //TODO: Set Audit Entry
//        Auditor.auditIPAInProcessEvent(ipa,
//                ActivityActorTypeEnumeration.MHV_AUTHENTICATOR);
		
	}
	
	@Override
	public void authenticateFull(InPersonAuthenticationDTOLight ipaLight, ResponseUtil<InPersonAuthenticationDTOLight> response) {
		log.debug("called authenticateFull method in InPersonAuthenticationServiceImpl");
		InPersonAuthenticationDTO ipaFull = null;
		// 1. get patient
		PatientDTO patientDTO = null;
		try {
			//We are actually passing in the userProfileId in the ipaLight.patientId field and it must be replaced below with the actual value before saving
			ResponseUtil patientResponse = patientService.getPatientByUserProfileId(ipaLight.getPatientId());
			patientDTO = (null != patientResponse.getPojoObject()) ? (PatientDTO) patientResponse.getPojoObject() : null;
			// 2. then get IPAs
			if(patientDTO.getIpas() != null ) {
				log.debug("patientDTO.getIpas() not null");
				if(patientDTO.getIpas().size() > 0){
					log.debug("patientDTO.getIpas().size() > 0");
					ipaFull = patientDTO.getIpas().iterator().next();
				}else{
					log.debug("patientDTO.getIpas().size() = 0");
				}
			}else{
				log.debug("patientDTO.getIpas() null");
			}
		} catch (Exception e) {
			String message = String.format("An unexpected error occurred while looking up patient record by user profile ID. %s", ipaLight.getPatientId());
			log.error(message, e);
		}
		
		
		Patient patient = patientConverter.convertDTOProfile(patientDTO);
		log.debug("********** InPersonAuthenticationServiceImpl.authenticate() calling mviService.mviAuthenticate()");
		mviService.mviAuthenticate(patient, patient.getUserProfile(), true, false, response, null);
		
        
		log.debug("********** InPersonAuthenticationServiceImpl.authenticate() called mviService.mviAuthenticate(): response:"+response.isSuccess());
		if (response.isSuccess()) {
			ipaLight.setMviAuthenticationStatus("OK");
			if(log.isInfoEnabled()){
				log.debug("********** InPersonAuthetnicationService().mviAuthenticate ipaFull.setMviAuthenticationStatus:OK");
				
			}
        } else {
        	ipaLight.setMviAuthenticationStatus("PENDING_AUTH");
			if(log.isDebugEnabled()){
				log.debug("********** InPersonAuthetnicationService().mviAuthenticate ipa.setMviAuthenticationStatus:PENDING_AUTH");
			}
        }

		if(log.isDebugEnabled()){
			log.debug("********** InPersonAuthetnicationService().mviAuthenticate ipa.setMviAuthenticationStatus:" + ipaLight.getMviAuthenticationStatus());
		}

		 Boolean formSigned = ipaLight.getParticipationFormSigned() == null
            ? Boolean.FALSE : ipaLight.getParticipationFormSigned();
        Boolean idPresented = ipaLight.getIdentificationPresented() == null
            ? Boolean.FALSE : ipaLight.getIdentificationPresented();
        String status = "";
        if (formSigned.booleanValue() && idPresented.booleanValue()) {
            status = AuthenticationStatusEnumeration.AUTHENTICATED.getDescription();
        } else {
            status = AuthenticationStatusEnumeration.INPROCESS.getDescription();
        }
        ipaLight.setStatus(status);
        InPersonAuthentication ip = new InPersonAuthentication();
        
        log.debug("xxxxxxxxx actual patient id:" + patient.getId());
        ipaLight.setPatientId(patient.getId());
        saveInPersonAuthenticationLight(ipaLight);
                
        response.setFailure(false);
		response.setSuccess(true);
		response.setPojoObject(ipaLight);
	}
	
	@Override
	public void deferAuthentication(InPersonAuthenticationDTO ipa, ResponseUtil response) {
			log.debug("ipa.getParticipationFormSigned()"+ipa.getParticipationFormSigned());
	        Boolean formSigned = ipa.getParticipationFormSigned() == null
	            ? Boolean.FALSE : ipa.getParticipationFormSigned();
	        Boolean idPresented = ipa.getIdentificationPresented() == null
	            ? Boolean.FALSE : ipa.getIdentificationPresented();
	        String status = "";
	        if (formSigned.booleanValue() && idPresented.booleanValue()) {
	            status = AuthenticationStatusEnumeration.PREREQUISITESCOMPLETE.getDescription();
	        } else {
	            status = AuthenticationStatusEnumeration.INPROCESS.getDescription();
	        }
	        ipa.setRemovalReason(null);
	        ipa.setStatus(status);
	        saveInPersonAuthentication(ipa);
	        
	}
	
	@Override
	public void deferAuthenticationLight(InPersonAuthenticationDTOLight ipa, ResponseUtil response) {
		log.debug("ipa.getParticipationFormSigned()"+ipa.getParticipationFormSigned());
        Boolean formSigned = ipa.getParticipationFormSigned() == null
            ? Boolean.FALSE : ipa.getParticipationFormSigned();
        Boolean idPresented = ipa.getIdentificationPresented() == null
            ? Boolean.FALSE : ipa.getIdentificationPresented();
        String status = "";
        if (formSigned.booleanValue() && idPresented.booleanValue()) {
            status = AuthenticationStatusEnumeration.PREREQUISITESCOMPLETE.getDescription();
        } else {
            status = AuthenticationStatusEnumeration.INPROCESS.getDescription();
        }
        ipa.setRemovalReason(null);
        ipa.setStatus(status);
        saveInPersonAuthenticationLight(ipa);
        //TODO: Set Audit Entry
//        Auditor.auditIPAInProcessEvent(ipa,
//                ActivityActorTypeEnumeration.MHV_AUTHENTICATOR);
		
	}

	@Override
	public void terminateAuthentication(InPersonAuthenticationDTO ipa, ResponseUtil response) {
		// TODO Auto-generated method stub
		return;
	}

	@Override
	public void authenticateMany(Collection patientIds, ResponseUtil response) {
		// TODO Auto-generated method stub
		return;
	}

	@Override
	public void finalizeAuthentication(InPersonAuthenticationDTO ipa, String error, ResponseUtil response) {
		// TODO Auto-generated method stub
		return;
	}

	@Override
	public void getAuthenticationForPatient(Long patientId, ResponseUtil response) {
		// TODO Auto-generated method stub
		return;
	}

	@Override
	public void finalizeUnauthentication(InPersonAuthenticationDTO ipa, String error, ResponseUtil response) {
		// TODO Auto-generated method stub
		return;
	}
	
	@Override
	public InPersonAuthenticationLight saveInPersonAuthenticationLight(InPersonAuthenticationDTOLight ipa) {
		InPersonAuthenticationLight ip = null;
		try {
			if(log.isDebugEnabled()) {
				if(ipa!=null && ipa.getStartUpgradeManualDateTime()!=null)
					log.debug("!!!!!!!!!ipa.getStartUpgradeManualDateTime():"+ipa.getStartUpgradeManualDateTime().toString());
			}
			
			ip = IPALIGHT_CONVERTER.convertInpersonAuthenticationDTOLight(ipa);
			
			ip = this.inpersonAuthenticationLightRepository.save(ip);
		} catch (Exception e) {
			log.error("Error in storing InPersonAuthentication details", e);
			throw new MHVRuntimeException("Error in storing InPersonAuthentication details");
		}
		return ip;

	}
	
	@Override
	public InPersonAuthentication saveInPersonAuthentication(InPersonAuthenticationDTO ipa) {
		InPersonAuthentication ip = null;
		try {
			ip = IPA_CONVERTER.convertInpersonAuthenticationDTO(ipa);
			ip = this.inpersonAuthenticationRepository.save(ip);
		} catch (Exception e) {
			log.error("Error in storing InPersonAuthentication details", e);
			throw new MHVRuntimeException("Error in storing InPersonAuthentication details");
		}
		return ip;

	}

	@Override
	public InPersonAuthentication saveIPA(InPersonAuthentication ipa) {
		InPersonAuthentication storedObject = null;
		try {
			storedObject = inpersonAuthenticationRepository.save(ipa);
		} catch (Exception e) {
			log.error("Error in storing InPersonAuthentication details", e);
			throw new MHVRuntimeException("Error in storing InPersonAuthentication details");
		}
		return storedObject;
	}


	@Override
	public InPersonAuthenticationLight saveIPALight(InPersonAuthenticationLight ipa) {
		log.debug("*********************** InPersonAuthenticationServiceImpl saveIPALight");
		InPersonAuthenticationLight storedObject = null;
		try {
			storedObject = inpersonAuthenticationLightRepository.save(ipa);
		} catch (Exception e) {
			log.error("Error in storing InPersonAuthentication details", e);
			throw new MHVRuntimeException("Error in storing InPersonAuthentication details");
		}
		return storedObject;
	}
	
	@Override
	public void forceUnauthenticateUnCorrelate(InPersonAuthenticationDTO ipa, ResponseUtil response) {
		// TODO Auto-generated method stub
		return;
	}

	@Override
	public void isPatientAuthenticated(Long patientId, ResponseUtil<Boolean> response) {

		List<InPersonAuthentication> ipaPatients = null;
		InPersonAuthentication ipaPatient = null;
		try {
			ipaPatients = inpersonAuthenticationRepository.getAuthenticationForPatient(patientId);

			if (null != ipaPatients && ipaPatients.size() > 0) {
				ipaPatient = ipaPatients.get(0);
			} else {
				response.setFailure(true);
				response.setFailureMessage("ipaPatient is null");
				return;
			}
			
			if (null != ipaPatient.getStatus() 
					&& ipaPatient.getStatus().equalsIgnoreCase(AuthenticationStatusEnumeration.AUTHENTICATED.getDescription())) {
				response.setPojoObject(Boolean.TRUE);
			} else {
				response.setPojoObject(Boolean.FALSE);
			}
		} catch (Exception e) {
			log.error("Error in getting patinet details from getAuthenticationForPatient() " + patientId, e);
			throw new MHVRuntimeException("Error in getting patinet details from getAuthenticationForPatient() " + patientId);
		}

		response.setFailure(false);
		response.setSuccess(true);
		
		return;
	}

	

	

	@Override
	public void authenticate(InPersonAuthenticationDTOLight ipa, ResponseUtil response) {
		// TODO Auto-generated method stub
		
		
	}
	
	public void unauthenticateWithAdminUserName(InPersonAuthentication ipa, String adminUserName, ResponseUtil responseUtil) {
		if(mviProperties.isMviEnabled()) {
			mviUnauthenticate(ipa, adminUserName, responseUtil);
		}

        boolean isEnabled = mpiProperties.isUnlinkEnabled();
        boolean isCorrelated = false;
        if(ipa.getPatient().getCorrelationStatus() == 2) {
        	isCorrelated = true;
        } 

        if (isEnabled
            && !(isCorrelated || isPendingSynchronization(ipa.getPatient())))
        {
            log.error("Cannot Unauthenticate Patient");
			responseUtil.setFailureMessage("Cannot Unauthenticate Patient " + ipa.getPatientId());
			responseUtil.setSuccess(false);
			responseUtil.setFailure(true);
			return;
        }
        String status = (isEnabled && isCorrelated) ? "PendingRemoval" : "Unauthenticated";
        ipa.setStatus(status);
        ipa.setParticipationFormSigned(Boolean.FALSE);
        ipa.setIdentificationPresented(Boolean.FALSE);
        ipa.setApprovedForRecordsAccess(Boolean.FALSE);
        ipa.setDefermentReason(null);
        ipa.setAuthenticatingFacility(null);
        ipa.setAuthenticationDate(null);
        ipa = inpersonAuthenticationRepository.save(ipa);

        responseUtil.setPojoObject(ipa);
        return;
	}
	
	public void mviUnauthenticate(InPersonAuthentication ipa, String adminUserName, ResponseUtil responseUtil) {
        ipa.setParticipationFormSigned(Boolean.FALSE);
        ipa.setIdentificationPresented(Boolean.FALSE);
        ipa.setApprovedForRecordsAccess(Boolean.FALSE);
        ipa.setDefermentReason(null);
        ipa.setAuthenticatingFacility(null);
        ipa.setAuthenticationDate(null);
        ipa.setStatus("Unauthenticated");
		//JAZZ: Task#19823 - When a IPA user unauthenticates a user that is not matched and/or
        //Correlated, then the system will not send the unauth flags to MVI since the user is not correlated
    	Patient patient = ipa.getPatient();
    	if(patient.getCorrelationStatus() == 0 ||  //UNCORRELATED
    	   patient.getCorrelationStatus() == 7) {  //MVIDATAMATCH
		   ipa.setMviAuthenticationStatus("OK");
	       patient.setPatientSynchronizations(null);
	       patientService.savePatient(patient);
    	} else {
			ipa.setMviAuthenticationStatus("PENDING_UNAUTH");
			//MHV_CodeCR1514 - US12.4 MVI Compliance Implementation - If the MVI flag is turned on call, MVI call instead of MPI call
			mviService.mviAuthenticate(patient, patient.getUserProfile(), false, false, responseUtil, null);
            
			if (responseUtil.isSuccess()) {
                if (log.isDebugEnabled()) {
                    log.debug("InPersonAuthenticationServiceImpl.mviUnauthenticate():SUCCESS unauthenticating patient with ICN:" + ipa.getPatient().getIcn());
                }
    			ipa.setMviAuthenticationStatus("OK");
                //try to uncorrelate. Only
            } else {
                if (log.isDebugEnabled()) {
                    log.debug("InPersonAuthenticationServiceImpl.mviUnauthenticate():ERROR unauthenticating patient with ICN:" + ipa.getPatient().getIcn());
                }
            }
    	}
        ipa = inpersonAuthenticationRepository.save(ipa);
        responseUtil.setPojoObject(ipa);
        return;
    }
	
	private boolean isPendingSynchronization(Patient patient) {
        return (patient != null)
            && (patient.getPatientSynchronizations() != null)
            && (!patient.getPatientSynchronizations().isEmpty());
    }
	
	private InPersonAuthentication copyProperties(InPersonAuthentication ipa, InPersonAuthentication copyTo) {
        copyTo.setAuthenticatingFacility(ipa.getAuthenticatingFacility());
        copyTo.setAuthenticationDate(ipa.getAuthenticationDate());
        copyTo.setStatus(ipa.getStatus());
        copyTo.setApprovedForRecordsAccess(ipa.getApprovedForRecordsAccess());
        copyTo.setIdentificationPresented(ipa.getIdentificationPresented());
        copyTo.setParticipationFormSigned(ipa.getParticipationFormSigned());
        copyTo.setDefermentReason(ipa.getDefermentReason());
        return copyTo;
    }

}
