package gov.va.med.mhv.usermgmt.main.service.impl;

import gov.va.med.mhv.usermgmt.data.model.Trace;
import gov.va.med.mhv.usermgmt.service.MoveInboundProcessor;
import gov.va.med.mhv.usermgmt.service.PatientService;
import gov.va.med.mhv.usermgmt.main.registry.UnknownMhvPatientException;
import gov.va.med.mhv.usermgmt.main.registry.PatientChangeMessage;
import gov.va.med.mhv.usermgmt.main.registry.Result;
import gov.va.med.mhv.usermgmt.common.enums.StatusType;
import gov.va.med.mhv.usermgmt.main.registry.Facility;
import gov.va.med.mhv.usermgmt.main.registry.Patient;
import gov.va.med.mhv.usermgmt.main.registry.Status;
import gov.va.med.mhv.common.api.util.ResponseUtil;
import gov.va.med.mhv.common.data.converter.PatientConverter;

import java.io.ByteArrayInputStream;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

@Component
public class MoveInboundProcessorImpl extends MoveInboundProcessor {
	
	private static Logger log = LogManager.getLogger(MoveInboundProcessorImpl.class);

	@Autowired
	private PatientService patientService;
	
	@Autowired
	private PatientConverter patientConverter;
	
	//private String mhvStationNumber = "200MH";
	//private String mhvStationNumber = "200MHS";
	private String mhvStationNumber = "200MHI";
	
	

    public String getMhvStationNumber() {
        return mhvStationNumber;
    }

    public void setMhvStationNumber(String mhvStationNumber) {
        this.mhvStationNumber = mhvStationNumber;
    }

	public Result processTrace(Trace trace) throws Exception {
		log.info("In MoveInboundProcessor.processTrace method" );
		Result result = constructOkResult(trace);
		PatientChangeMessage message = (PatientChangeMessage) deserialize(trace.getTraceData());
		gov.va.med.mhv.usermgmt.main.registry.Patient msgFromPatient =
            message.getFromPatient();
		gov.va.med.mhv.usermgmt.main.registry.Patient msgToPatient =
            message.getToPatient();
		
		
		
		ResponseUtil<gov.va.med.mhv.common.data.model.Patient> response = new ResponseUtil<gov.va.med.mhv.common.data.model.Patient>();
		patientService.getPatientByIcn(msgFromPatient.getIcn(), response);
		gov.va.med.mhv.common.api.dto.PatientDTO mhvFromPatient = null;
		if(response.isSuccess()) {
			if(response.getPojoObject() != null) {
				mhvFromPatient = patientConverter.convertDomainPatient(response.getPojoObject());
			}
		}
		gov.va.med.mhv.common.api.dto.PatientDTO mhvToPatient = null;
		if(msgToPatient!=null){
			log.info("MoveInboundProcessor msgToPatient is NOT null)");
		
			patientService.getPatientByIcn(msgToPatient.getIcn(), response);
			
			if(response.isSuccess()) {
				if(response.getPojoObject() != null) {
					mhvToPatient = patientConverter.convertDomainPatient(response.getPojoObject());
				}
			}
		}else{
			log.info("MoveInboundProcessor msgToPatient is null)");
			mhvToPatient = null;
		}

		/*From supplemental spec:
		 10.5.7	Broadcast from MPI: Move Patient Information (ADT-A43)
		 MPI sends this message to move an existing correlation of a
         treating facility from one ICN to another ICN.
         It is sent to correct situations where a patient record at a
         treating facility was mistakenly correlated to the wrong ICN,
         resulting in a single ICN being shared by two different patients.
		 MHV will remove the VistA-based contents of the user�s eVAult
         originating from the treating facility that is being removed from
         the ICN, and remove the treating facility from the MHV user�s
         treatment facility list.
         In addition, MHV will trigger the display of an informational
         message the next time the user logs into the MHV portal saying an
         administrative adjustment has occurred.
		 In the unlikely event that MHV is the treating facility being
         removed from the ICN, then the MPI Data Quality Team would have
         determined that the MHV user was authenticated to the wrong ICN,
         and the following actions are performed for the MHV User associated
         with the ICN:
			1.	Remove ICN and list of treatment facilities
			2.	Remove the VistA-based contents of the user�s eVAult.
			3.	Update user�s authentication status to Not Authenticated.
			4.	Store the Reason for Removal of Authentication for the
                user as �MPI Data Quality action.�
			5.	Trigger the display of an informational message the next
                time the user logs into the MHV portal indicating
                authentication has been removed from the user�s account.
		 */
		if ((mhvFromPatient == null) && (mhvToPatient == null)) {
			throw new UnknownMhvPatientException(createUnknownMhvPatientsMessage(
				msgFromPatient.getIcn(), msgToPatient.getIcn()));
		}
        if (mhvFromPatient == null) {
			trace.getEvent().setUserId(mhvToPatient.getUserProfileId());
			// if toPatient is known - no action
		} else {
			trace.getEvent().setUserId(mhvFromPatient.getUserProfileId());
			//if fromPatient is known
//			String removedStationNumber = msgFromPatient.getFacilities().
//                iterator().next().getStationNumber();
//			if (removedStationNumber == getMhvStationNumber()) {
//				//uncorrelate
//				Status mhvStatus = getPatientIdentityServiceProxy().
//                    removeCorrelation(mhvFromPatient);
//				result.setStatus(RegistryInterfaceUtils.toStatus(mhvStatus));
//			} else {
//				ArrayList<Facility> newFacilities = removeFacility(
//                    mhvFromPatient, removedStationNumber);
//				Status mhvStatus = getPatientIdentityServiceProxy().
//                    updateFacilities(mhvFromPatient, newFacilities);
//				result.setStatus(RegistryInterfaceUtils.toStatus(mhvStatus));
//			}
		}

        //JAZZ # 128617: MVI A43 Move message processing.
        String reason = "A43 possible ICN Mismatch due to patient having same ICN as another patient - (Do Not Reactivate)";
        String deactivationNote = "A43 Possible ICN Mismatch (Do Not Reactivate)";
        //Status status = patientService.deactivatePatient(mhvFromPatient, null, reason, deactivationNote);
        Status status = null;

    	//String successMessage = "";
        String successMessage = ". The MHV account with ICN " + msgFromPatient.getIcn()+" was not Deactivated successfully";
        //remove this after deactiavetPatient above is working.
        if(false){

    	if(status.getStatusType().equals(StatusType.Ok)) {
    		successMessage = ". The MHV account with ICN " + msgFromPatient.getIcn()+" was Deactivated successfully";
    	} else {
    		if(status.getStatusDescription()!=null && status.getStatusDescription().equals("Patient Not Found")) {
    			successMessage = ". The MHV account with ICN " + msgFromPatient.getIcn()+" was not found";
    		} else {
    			successMessage = ". The MHV account with ICN " + msgFromPatient.getIcn()+" was not Deactivated successfully";
    		}
    	}
        }

        //JAZZ: Task 20182 - ADT-A43 Move message � Stop sending successful acknowledgement back to MVI
        Status finalStatus = new Status();
        finalStatus.setStatusType(StatusType.Fail);
        Date now = new Date();
        SimpleDateFormat format = new SimpleDateFormat("MM/dd/yyyy HH:mm:ss z");
        String date = format.format(now);
        finalStatus.setStatusDescription("Failed ADT A43 MOVE Message at " + date + successMessage + ". Please contact MHV Help Desk");
        result.setStatus(finalStatus);
        trace.getEvent().setStatus("success");
        return result;
	}

    @SuppressWarnings("unchecked")
    private ArrayList<Facility> removeFacility(Patient patient,
        String stationNumberToRemove)
    {
        ArrayList<Facility> facilities = new ArrayList(patient.getFacilities());
        if (facilities != null) {
            for (Facility facility: facilities) {
                if (facility.getStationNumber().equals(stationNumberToRemove)) {
                    facilities.remove(facility);
                    break;
                }
            }
        }
        return facilities;
    }
    
    private Object deserialize(String str) {
		if (str == null) {
			return null;
		}
		byte[] barr = str.getBytes();
		ByteArrayInputStream bais = new ByteArrayInputStream(barr);
		java.beans.XMLDecoder dec = new java.beans.XMLDecoder(bais);
		Object result = dec.readObject();
		return result;
	}
}
