package gov.va.med.mhv.usermgmt.web.controller;

import java.io.IOException;
import java.net.HttpURLConnection;
import java.net.URL;

import javax.faces.bean.ManagedBean;
import javax.faces.context.FacesContext;
import javax.faces.event.ComponentSystemEvent;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import gov.va.med.mhv.usermgmt.common.enums.ActivityActionTypeEnumeration;
import gov.va.med.mhv.usermgmt.common.enums.ActivityTypeEnumeration;
import gov.va.med.mhv.usermgmt.service.AccountValidatorService;
import gov.va.med.mhv.usermgmt.util.activity.ActivityHelper;

@ManagedBean
@Component
@Scope("session")
public class MonthlyBillingStatementController extends AbstractController{
	
	@Autowired
	public AccountValidatorService accountValidatorService;
	
	private static final Logger log = LogManager.getLogger(MonthlyBillingStatementController.class);
	private static final long serialVersionUID = 8063357896611116540L;
	
	@Value("${vafii.patient.billing.stmt}")
	private  String vafiiURL;
	
	@Value("${veterans.billing.stmt}")
	private  String billingURL;
	
	
	private boolean serverStatus=false;
	private  boolean vafiiStatus=false;
	private  boolean billinStmtStatus=false;
	
	public void init(ComponentSystemEvent event){
		if(!FacesContext.getCurrentInstance().isPostback()){
			vafiiStatus=true;
			billinStmtStatus=true;
		}
	}
	
	public void navigateToPatientStmt(){
		 vafiiStatus=false;
		 billinStmtStatus=false;
		 serverStatus=false;
		try {
	        if(!urlValidStatus(vafiiURL)){
	        	vafiiStatus=false;
	        }else{
	        	vafiiStatus=true;
	        }
		} catch (Exception e) {
			log.error("Error Accessing Vafii::URL::"+ vafiiURL+"::" + e);
			vafiiStatus=false;
		}
		if(!vafiiStatus){
			return;
		}
		try {
			if(!urlValidStatus(billingURL)){
				billinStmtStatus=false;
	        }else{
	        	billinStmtStatus=true;
	        }
		} catch (Exception e) {
			log.error("Error Accessing Patient Statement::URL::"+billingURL+"::" + e);
			billinStmtStatus=false;
		}
		
		if(!billinStmtStatus){
			return;
		}
		
		log.debug("Vafii Status::"+vafiiStatus);
		log.debug("Billing Status::"+billinStmtStatus);
		
		if(vafiiStatus && billinStmtStatus){
			serverStatus=true;
			log.debug("Ds Logon User::"+ isDsLogonUser());
			log.debug("Account Type::"+ isPremiumAccountType() );
			if(isDsLogonUser()==true && isPremiumAccountType()==true){
			    createAudit(getUserProfileId());
		    }
		}
	}
	
	private boolean urlValidStatus(String url) throws  IOException{
		HttpURLConnection connection = (HttpURLConnection) new URL(url).openConnection();
        connection.setRequestMethod("HEAD");
        int responseCode = connection.getResponseCode();
        log.debug("URL::" + url + "Response Code::"+ responseCode);
        return (responseCode==200 ? true:false);
	}
	
	
	
	private void createAudit(long userProfileId) {
		try {
			log.debug("Creating AAL Entry for ::"+ userProfileId);
			
			accountValidatorService.createAccountActivityLog(
				ActivityHelper.createActivityDTOForSelf(userProfileId, 
						true, 
						ActivityTypeEnumeration.REQUEST_BILLING_STATEMENT, 
						ActivityActionTypeEnumeration.VETERAN_BILLING_STATEMENT, ""));
		} catch (Exception e) {
    			log.error("Error writing AAL entry for Montly Billing statement::",e);
    	}
	}
	
	public void resetVafiiErrmsg(){
		vafiiStatus=true;
	}

	public  String getVafiiurl() {
		return vafiiURL;
	}


	public boolean isServerStatus() {
		return serverStatus;
	}

	public void setServerStatus(boolean serverStatus) {
		this.serverStatus = serverStatus;
	}

	public String getBillingURL() {
		return billingURL;
	}

	public boolean isVafiiStatus() {
		return vafiiStatus;
	}

	public void setVafiiStatus(boolean vafiiStatus) {
		this.vafiiStatus = vafiiStatus;
	}

	public boolean isBillinStmtStatus() {
		return billinStmtStatus;
	}

	public void setBillinStmtStatus(boolean billinStmtStatus) {
		this.billinStmtStatus = billinStmtStatus;
	}
	
}
