package gov.va.med.mhv.sm.web.actions;

import gov.va.med.mhv.foundation.service.response.CollectionServiceResponse;
import gov.va.med.mhv.foundation.service.response.ServiceResponse;
import gov.va.med.mhv.foundation.util.LogSanitizer;
import gov.va.med.mhv.sm.ad.ADQueryResult;
import gov.va.med.mhv.sm.dao.properties.ManageStaffMemberProperties;
import gov.va.med.mhv.sm.dao.UserDao;
import gov.va.med.mhv.sm.dto.LocationDssUnitsDTO;
import gov.va.med.mhv.sm.dto.UserInfoDTO;
import gov.va.med.mhv.sm.model.Administrator;
import gov.va.med.mhv.sm.model.ClinicalUserType;
import gov.va.med.mhv.sm.model.Clinician;
import gov.va.med.mhv.sm.model.Facility;
import gov.va.med.mhv.sm.service.UserManagementService;
import gov.va.med.mhv.sm.util.UserUtils;
import gov.va.med.mhv.sm.util.WorkloadUtil;
import gov.va.med.mhv.sm.wsclient.adminqueriessvc.Provider;
import gov.va.med.mhv.sm.wsclient.adminqueriessvc.User;
import gov.va.med.mhv.sm.wsclient.adminqueriessvc.UserNew;
import gov.va.med.mhv.sm.wsclient.tiusvc.DSSUnit;

import java.io.File;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.TreeSet;

import org.apache.commons.lang.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.commons.validator.EmailValidator;
import org.apache.struts2.ServletActionContext;
import org.owasp.esapi.ESAPI;
import org.springframework.web.context.WebApplicationContext;
import org.springframework.web.context.support.WebApplicationContextUtils;

import com.opensymphony.xwork2.Preparable;

public class ManageStaff extends BaseSMAdminAction implements Preparable {

	@SuppressWarnings("unused")
	private static final Log log = LogFactory.getLog(ManageStaff.class);

	private static final String CANCEL = "cancel";
	private static final String FAILED = "failed";
	private static final String SELECT_STAFF = "selectStaff";
	private static final String SEARCH_RESULTS = "searchResults";
	private static final String QUERY_RESULTS_IR = "queryResults_IR";
	private static final String BULK_ACTIVATION = "bulkActivation";
	private static final String PATCH29_NATIONAL ="NATIONAL";
	private static final String PATCH29_FIELD ="FIELD";
	private static final String ALL_DSS_UNITS = "*";
	private static final String DATA_DELIMITER = "|";
	private UserManagementService userManagementService;
	private ManageStaffMemberProperties manageStaffMemberProperties;
	private UserDao userDao;
	private Collection<Facility> visns;
	private Collection<Facility> facilities;
	private Long visnId;
	private Long facilityId;
	private String selectedVisnName;
	private String selectedFacilityName;
	private List<Clinician> clinicians;
	private List<Clinician> selectedUsers;
	private List<ClinicalUserType> staffMemberTypes;
	//private HttpServletRequest request;
	private String searchCrit;
	private String emailAD;
	private String firstName;
	private String lastName;
	private String middleName;
	private String duz;
	private String phone;
	private String title;
	private String email;
	private String suffix;
	private String prefix;
	private String activeFlag;
	private String providerKey;
	private String providerClass;
	private String personClass;
	private String[] dssUnits;
	private String pcmmFlag;
	private String requiresCoSigner;
	private String networkId;
	private Long memberTypeId;
	private String memberType;
	private String firstNameAD;
	private String lastNameAD;
	private String phoneAD;
	private String titleAD;
	private String networkIdAD;
	private String selectAction;
	// File upload info for bulk activation
	private File upload;
    private String uploadContentType;
    private String uploadFileName;
    private int added; // number of bulk load staff members activated.
    private static String FILE_SIZE_LIMIT_ERROR = "The file you are trying to upload exceeds the 2MB limit. " +
    										 "Please reduce the file size and try again.";
    private static int FILE_SIZE_LIMIT_2MB = 2 * 1024 * 1024;
	public ManageStaff() {
		super();
	}


	public void prepare() throws Exception {
		super.prepare();
		WebApplicationContext ctx = WebApplicationContextUtils
				.getWebApplicationContext(ServletActionContext
						.getServletContext());
		userManagementService = (UserManagementService) ctx.getBean("userManagementService");
		manageStaffMemberProperties = (ManageStaffMemberProperties)ctx.getBean("manageStaffMemberProperties");
		userDao = (UserDao) ctx.getBean("userDao");
	}


	public String manageStaffMembers() {

		clearSessionAttr();

		return SUCCESS;
	}

	public String selectEntryMethod() {

		if(appliedCancel()){ return CANCEL;	}
		clearSessionAttr();
		facilities = new TreeSet<Facility>();

		if((selectAction != null) && (selectAction.equalsIgnoreCase("Inactivate a Staff Member"))){
			setSessionAttribute("activationType","inactivate");
			return SELECT_STAFF;
		}
		if((selectAction != null) && (selectAction.equalsIgnoreCase("Reactivate a Staff Member"))){
			setSessionAttribute("activationType","reactivate");
			return SELECT_STAFF;
		}
		return SUCCESS;
	}


	public String selectSearchCrit() {

		if(appliedCancel()){ return CANCEL;	}
		clearSessionAttr();
		facilities = new TreeSet<Facility>();
		if((selectAction != null) && ((selectAction.trim()).equalsIgnoreCase("Bulk Activation from CSV File"))){
			return BULK_ACTIVATION;
		}

		return SUCCESS;
	}


	public String selectVisn(){

		getStaffSessionAttr();

		if(visnId == -1L){
			addActionError("Please select a VISN.");
			return SELECT_STAFF;
		}

		for(Facility v: getAdministeredVisns()){
			if(v.getId().equals(visnId)){
				selectedVisnName = v.getName();
				facilities = getAdministeredFacilitiesInVisn(v);
			}
		}

		if(StringUtils.isBlank(selectedVisnName)){
			addActionError("Please select a VISN.");
			return SELECT_STAFF;
		}

		setSessionAttribute("facilities",facilities);
		setSessionAttribute("selectedVisnName",selectedVisnName);

		return SELECT_STAFF;

	}

	public String selectFacility(){

		getStaffSessionAttr();

		// if the visn hasn't been selected first
		if(StringUtils.isBlank(selectedVisnName)){
			addActionError("Please select a VISN.");
			return SELECT_STAFF;
		}

		// make sure a facility has been chosen
		if(facilityId == -1){
			addActionError("Please select a Facility.");
			return SELECT_STAFF;
		}

		String stationNumber = null;

		if(facilities != null){
			for(Facility f: facilities){
				if(f.getId().equals(facilityId)){
					selectedFacilityName = f.getName();
					stationNumber = f.getStationNumber();
				}
			}
		}

		setSessionAttribute("selectedFacilityName",selectedFacilityName);
		setSessionAttribute("stationNumber",stationNumber);

		return SELECT_STAFF;
	}

	private CollectionServiceResponse<UserNew> getPatch29VistAProvider(String lastName,String firstName,String stationNumber,String duz){
		CollectionServiceResponse<UserNew> collectionServiceResponse=null;
		if(null!=duz){
			collectionServiceResponse = getAdminService().getVistAProvider(duz,stationNumber);
			if(log.isInfoEnabled()){
				if(collectionServiceResponse.getCollection()!=null){
					log.info(LogSanitizer.cleanText("ManageStaff=>getPatch29VistAProvider=>DUZLookup->DUZ"+duz+"station#"+stationNumber+"Response size=>"+collectionServiceResponse.getCollection().size()));	
				}
			}
		}else{
			collectionServiceResponse = getAdminService().getVistAProvider(lastName, firstName, stationNumber);	
			if(log.isInfoEnabled()){
				if(collectionServiceResponse.getCollection()!=null){
					log.info(LogSanitizer.cleanText("ManageStaff=>getPatch29VistAProvider=>UserLookup->LastName="+lastName+"FirstName="+firstName+"stationNumber"+stationNumber+"Response size=>"+collectionServiceResponse.getCollection().size()));	
				}
			}
		}
		return collectionServiceResponse;
	}
	
	private CollectionServiceResponse<User> getVistAProvider(String lastName,String firstName,String stationNumber,String duz){
		CollectionServiceResponse<User> collectionServiceResponse=null;
		if(null!=duz){
			collectionServiceResponse = getAdminService().findVistaClinician(duz,stationNumber);
			if(log.isInfoEnabled()){
				if(collectionServiceResponse.getCollection()!=null){
					log.info(LogSanitizer.cleanText("ManageStaff=>getVistAProvider=>DUZLookup->DUZ"+duz+"station#"+stationNumber+"Response size=>"+collectionServiceResponse.getCollection().size()));	
				}
			}
		}else{
			collectionServiceResponse = getAdminService().findVistaClinician(lastName, firstName, stationNumber);
			if(log.isInfoEnabled()){
				if(collectionServiceResponse.getCollection()!=null){
					log.info(LogSanitizer.cleanText("ManageStaff=>getVistAProvider=>UserLookup->LastName="+lastName+"FirstName="+firstName+"station#"+stationNumber+"Response size=>"+collectionServiceResponse.getCollection().size()));	
				}
			}
		}
		return collectionServiceResponse;
	}

	
	private Collection<UserInfoDTO> getVistAStaffMembers(String firstName,String lastName,String stationNumber,String duz){
		//if(null!=manageStaffMemberProperties.getVistAPatch29Status() && manageStaffMemberProperties.getVistAPatch29Status().equalsIgnoreCase(PATCH29_NATIONAL)){
		if(getVistAPatch29Status(stationNumber)){
			CollectionServiceResponse<UserNew> response = getPatch29VistAProvider(lastName,firstName,stationNumber,duz);
			handleErrorMessages(response.getMessages());
			return createUsersInfoList(response,stationNumber);
		}else{
			CollectionServiceResponse<User> response = getVistAProvider(lastName, firstName, stationNumber,duz);
			handleErrorMessages(response.getMessages());
			return createUsersInfoList(response,stationNumber);
		}
	}
	
	private boolean getVistAPatch29Status(String stationNo){
		if(null!=manageStaffMemberProperties.getVistAPatch29Status() && manageStaffMemberProperties.getVistAPatch29Status().equalsIgnoreCase(PATCH29_NATIONAL)){
			return true;
		}else if(null!=manageStaffMemberProperties.getVistAPatch29Status() && manageStaffMemberProperties.getVistAPatch29Status().equalsIgnoreCase(PATCH29_FIELD)){
			if(WorkloadUtil.isCurrentUserIsFieldTester(stationNo, manageStaffMemberProperties.getVistAPatch29TestingSites())){
				return true;
			}else{
				return false;
			}
		}
		return false;
	}
	
	private Collection<UserInfoDTO> createUsersInfoList(CollectionServiceResponse response,String stationNumber){
		List<UserInfoDTO> userInfoDtoList = new ArrayList<UserInfoDTO>();
		if(response.getCollection()!=null && response.getCollection().size()!=0 && ((List)response.getCollection()).get(0) instanceof UserNew){
			List<UserNew> staffMembers = (List<UserNew>)response.getCollection();
			if(log.isInfoEnabled()){
				log.info("ManageStaff=>100 Ensemble Response Instance of UserNew for Station:"+stationNumber+" Size: "+(staffMembers!=null?staffMembers.size():"0"));
			}
			for(UserNew userObj:staffMembers){
				UserInfoDTO userInfoDto= new UserInfoDTO();
				userInfoDto.setFirstName(userObj.getFirstName()!=null?userObj.getFirstName():"");
				userInfoDto.setLastName(userObj.getLastName()!=null?userObj.getLastName():"");
				userInfoDto.setMiddleName(userObj.getMiddleName()!=null?userObj.getMiddleName():"");
				userInfoDto.setSuffix(userObj.getSuffix()!=null?userObj.getSuffix():"");
				userInfoDto.setPrefix(userObj.getPrefix()!=null?userObj.getPrefix():"");
				userInfoDto.setTitle(userObj.getTitle()!=null?userObj.getTitle():"");
				userInfoDto.setWorkPhone(userObj.getPhone()!=null?userObj.getPhone():"");
				userInfoDto.setDuz(userObj.getIEN()!=null?userObj.getIEN():"");
				userInfoDto.setActiveFlag((userObj.getActiveFlag()!=null && userObj.getActiveFlag().equalsIgnoreCase("1"))?"ACTIVE":"INACTIVE");
				userInfoDto.setProviderKey((userObj.getProviderIndicator()!=null && userObj.getProviderIndicator().equalsIgnoreCase("PROVIDER"))?"TRUE":"FALSE");
				userInfoDto.setProviderClass(userObj.getProvCls()!=null?userObj.getProvCls():"");
				userInfoDto.setVistAPersonClass(userObj.getPersCls());
				userInfoDto.setRequiresCoSigner(userObj.getReqCosig()!=null?userObj.getReqCosig():"");
				userInfoDtoList.add(userInfoDto);
			}
		}
		else if(response.getCollection()!=null && response.getCollection().size()!=0 && ((List)response.getCollection()).get(0) instanceof User){
			List<User> staffMembers = (List<User>)response.getCollection();
			if(log.isInfoEnabled()){
				log.info("ManageStaff=>200 Ensemble Response Instance of User for Station:"+stationNumber+" Size: "+(staffMembers!=null?staffMembers.size():"0"));
			}
			for(User userObj:staffMembers){
				UserInfoDTO userInfoDto= new UserInfoDTO();
				userInfoDto.setFirstName(userObj.getFirstName()!=null?userObj.getFirstName():"");
				userInfoDto.setLastName(userObj.getLastName()!=null?userObj.getLastName():"");
				userInfoDto.setMiddleName("");
				userInfoDto.setSuffix("");
				userInfoDto.setPrefix("");
				userInfoDto.setTitle(userObj.getTitle()!=null?userObj.getTitle():"");
				userInfoDto.setWorkPhone(userObj.getPhone()!=null?userObj.getPhone():"");
				userInfoDto.setDuz(userObj.getIEN()!=null?userObj.getIEN():"");
				userInfoDto.setProviderKey(userObj.getProviderIndicator()!=null?"TRUE":"FALSE");
				userInfoDto.setProviderClass("");
				userInfoDto.setRequiresCoSigner("");
				userInfoDtoList.add(userInfoDto);
			}
		}
		return userInfoDtoList;
	}
	
	
	public String searchStaffMembers() {
		if(appliedCancel()){ return CANCEL;	}

		getStaffSessionAttr();
		if(StringUtils.isBlank(selectedVisnName)){
			addActionError("Please select a VISN.");
			return SELECT_STAFF;
		}

		if(StringUtils.isBlank(selectedFacilityName)){
			addActionError("Please select a Facility.");
			return SELECT_STAFF;
		}
		//List<UserNew> staffMembers= new ArrayList<UserNew>();
		List<UserInfoDTO> staffMembers = new ArrayList<UserInfoDTO>();
		clinicians = new ArrayList<Clinician>();
		Clinician clin = null;
		String stationNumber = null;
		facilities = new TreeSet<Facility>();

		getStaffSessionAttr();
		String activationType = null;

		if(((String)getSessionAttribute("activationType")) != null){
			activationType = (String)getSessionAttribute("activationType");
		}
		if(((String)getSessionAttribute("stationNumber")) != null){
			stationNumber = (String)getSessionAttribute("stationNumber");
		}

		if((selectAction != null) && (selectAction.equalsIgnoreCase("Search"))){

			if(searchCrit == null){
				addActionError("Please select a radio button.");
				return SELECT_STAFF;
			}
			if(stationNumber == null){
				
				if(StringUtils.isBlank(selectedVisnName)){
					addActionError("Please select a VISN.");
					return SELECT_STAFF;
				}else{
					addActionError("Please select a facility.");
					return SELECT_STAFF;
				}
			}	
			
			try{
				if(searchCrit.equalsIgnoreCase("0")){
					
					if(lastName.trim().equals("") && firstName.trim().equals("")){
						addActionError("Please enter the First Name and Last Name and select the 'Search by Name' radio button.");
						return SELECT_STAFF;
					}
					
					if(lastName.trim().equals("")){
						addActionError("Please enter the Last Name and select the 'Search by Name' radio button.");
						return SELECT_STAFF;
					}
					
					if(firstName.trim().equals("")){
						addActionError("Please enter the First Name and select the 'Search by Name' radio button.");
						return SELECT_STAFF;
					}
					
					if(firstName.trim().length() < 2 || lastName.trim().length() < 2){
						addActionError("Please enter at least first 2 characters of First Name and Last Name.");
						return SELECT_STAFF;
					}
					
					if((activationType != null) && (activationType.equalsIgnoreCase("inactivate"))){
						clinicians = (List<Clinician>) userManagementService.searchForClinicians(firstName, lastName, stationNumber, true).getCollection();
					}else if((activationType != null) && (activationType.equalsIgnoreCase("reactivate"))){
						clinicians = (List<Clinician>) userManagementService.searchForClinicians(firstName, lastName, stationNumber, false).getCollection();
					}else{
						staffMembers = (List)getVistAStaffMembers(firstName,lastName,stationNumber,null);
						List userManagementStaffMembers = (List<Clinician>) userManagementService.searchForClinicians(firstName, lastName, stationNumber).getCollection();

						if(staffMembers == null){
							addActionError("A search of the VistA database did not produce a match for the information given.");
							return SELECT_STAFF;
						}
						staffMembers = filterStaffMembers(staffMembers,userManagementStaffMembers);
					}
				}else{
					if((duz == null) || ((duz.trim()).equals(""))){
					addActionError("Please enter DUZ and select 'Search by DUZ' radio button.");
					return SELECT_STAFF;
					}
					

					if(duz !=null) duz=duz.trim();
					if((activationType != null) && (activationType.equalsIgnoreCase("inactivate"))){
						ServiceResponse<Clinician> response = userManagementService.getClinicianForStationAndDuz(stationNumber, duz, true);
						clin = response.getPayload();
					}else if((activationType != null) && (activationType.equalsIgnoreCase("reactivate"))){
						ServiceResponse<Clinician> response = userManagementService.getClinicianForStationAndDuz(stationNumber, duz, false);
						clin = response.getPayload();

					}else{
						staffMembers = (List)getVistAStaffMembers(null,null,stationNumber,duz);
						Clinician clinician =  userDao.findAuthClinicianByStationAndDuz(stationNumber,duz);
						ArrayList<Clinician> userManagementStaffMembers = new ArrayList<Clinician>();
						if(clinician !=null)
							userManagementStaffMembers.add(clinician);

						if(staffMembers == null){
							addActionError("A search of the VistA database did not produce a match for the information given.");
						}
						if(userManagementStaffMembers.size()!=0)
							staffMembers = filterStaffMembers(staffMembers,userManagementStaffMembers);
					}

					if(clin!=null){
						clinicians.add(clin);
					}
				}
			}catch(Exception e){
				if(log.isErrorEnabled()){
					log.error("searchStaffMembers()=>Exception occured while search Staff Members"+e.getMessage());
					e.printStackTrace();
				}
				addActionError("An unexpected error has occurred. Please contact the Help Desk, if the problem persists.");
			}
			setSessionAttribute("staffMembers",staffMembers);
			setSessionAttribute("clinicians",clinicians);


			if((activationType != null) && ((activationType.equalsIgnoreCase("inactivate")) || (activationType.equalsIgnoreCase("reactivate")))){
				return QUERY_RESULTS_IR;
			}

			return SUCCESS;

		}

		try{
			String cf = getParameter("changeFacility");
			if(!StringUtils.isBlank(cf) && cf.equalsIgnoreCase("yes")){
				selectedFacilityName = null;
			}
		}catch(Exception e){
			if(log.isErrorEnabled()){
				log.error("Exception occurred in changeFacility");
			}
		}

		return SELECT_STAFF;
	}

	/**
	 * filteredStffMembers
	 * @param vistaStaffMembers
	 * @param userManagementStaffMembers
	 * @return List<User>
	 */
	private List<UserInfoDTO> filterStaffMembers(List<UserInfoDTO> vistaStaffMembers,List<Clinician> userManagementStaffMembers){

			
		ArrayList<UserInfoDTO> filteredStaffMembers = new ArrayList<UserInfoDTO>();
		ArrayList<String> activeClinicians = new ArrayList<String>();
		for(Clinician clinician:userManagementStaffMembers){
			if(clinician.getDuz()!=null){
				activeClinicians.add(clinician.getDuz());
			}
		}
		for(UserInfoDTO user:vistaStaffMembers){
			if(!activeClinicians.contains(user.getDuz())){
				filteredStaffMembers.add(user);
			}
		}
		return filteredStaffMembers;
	}

	public String associateNetworkId(){

		if(appliedCancel()){ return CANCEL;	}
		UserInfoDTO selectedVistaUser = null;
		try{
			duz = (getParameter("ien") == null)?"":getParameter("ien");
			if(duz == null){
				addActionError("Could not find IEN for the selected user.");
			}
			//List<UserNew> searchedUsers = (List<UserNew>)getSessionAttribute("staffMembers");
			List<UserInfoDTO> searchedUsers = (List<UserInfoDTO>)getSessionAttribute("staffMembers");
			
			if(searchedUsers != null){
				for(UserInfoDTO userInfoDto:searchedUsers){
					if(duz.equalsIgnoreCase(userInfoDto.getDuz())){
						selectedVistaUser = userInfoDto;
					}
				}
			}
			
			if(getVistAPatch29Status((String)getSessionAttribute("stationNumber"))){
					ServiceResponse<Provider> pcmmResponse  = getAdminService().getPcmmProvider((String)getSessionAttribute("stationNumber"), duz);
					if(log.isInfoEnabled()){
						log.info("associateNetworkId=>100 pcmmResponse==>"+pcmmResponse);
						log.info("associateNetworkId=>101 pcmmResponse==>"+pcmmResponse.getPayload());
					}
					if(pcmmResponse!=null && pcmmResponse.getPayload()!=null && pcmmResponse.getPayload().getIEN()!=null){
						selectedVistaUser.setPcmmFlag("YES");
					}else{
						selectedVistaUser.setPcmmFlag("NO");
					}
					
					LocationDssUnitsDTO dssUnitsDTO = getAdminService().getDSSUnitsByClinicAndStation((String)getSessionAttribute("stationNumber"), ALL_DSS_UNITS, duz, ALL_DSS_UNITS);
					if(dssUnitsDTO!=null && dssUnitsDTO.getError()==null){
						List<DSSUnit> dssUnits = dssUnitsDTO.getDssUnits();
						StringBuilder strBuilder=new StringBuilder();
						int count=1;
						for(DSSUnit dss:dssUnits){
							strBuilder.append(dss.getDSSName());
							if(count!=dssUnits.size()) strBuilder.append("|");
							count++;
						}
						selectedVistaUser.setDssUnits(strBuilder.toString());
					}
			}	
			setSessionAttribute("selectedVistaUser",selectedVistaUser);
			setSessionAttribute("duz",duz);
			getUserInformation();
		}catch(Exception exp1){
			if(log.isErrorEnabled()){
				log.error("associateNetworkId()=>Exception occurred while getting selected user details."+exp1);
			}
		}

		return SUCCESS;
	}

	
	public void getUserInformation(){
		UserInfoDTO selectedVistAUser = (UserInfoDTO)getSessionAttribute("selectedVistaUser");
		if(selectedVistAUser!=null){
			firstName = selectedVistAUser.getFirstName();
			lastName = selectedVistAUser.getLastName(); 
			middleName = selectedVistAUser.getMiddleName();
			title = selectedVistAUser.getTitle();
			phone = selectedVistAUser.getWorkPhone();
			duz = selectedVistAUser.getDuz();
			suffix = selectedVistAUser.getSuffix() ;
			prefix = selectedVistAUser.getPrefix();
			activeFlag="";
			if(selectedVistAUser.getActiveFlag()!=null){
				activeFlag = selectedVistAUser.getActiveFlag().equals("0")?"ACTIVE":"INACTIVE";
			}
			providerKey = selectedVistAUser.getProviderKey();
			providerClass = selectedVistAUser.getProviderClass();
			try{
				if(selectedVistAUser.getVistAPersonClass()!=null && selectedVistAUser.getPersonClass()==null){
					if(selectedVistAUser.getVistAPersonClass()[0]!=null){
						String personClassification = UserUtils.getProviderPersonClassification(selectedVistAUser.getVistAPersonClass()[0]);
						if(personClassification!=null)
							selectedVistAUser.setPersonClass(UserUtils.formatPersonClass(personClassification));
					}
				}
				personClass = selectedVistAUser.getPersonClass();
			}
			catch(Exception exp2){
				if(log.isErrorEnabled()){
					log.error("getUserInformation=>An Error Occured while extracting Person Class "+exp2);
				}
			}
			pcmmFlag ="";
			if(selectedVistAUser.getPcmmFlag()!=null){
				pcmmFlag = selectedVistAUser.getPcmmFlag();	
			}
			requiresCoSigner = selectedVistAUser.getRequiresCoSigner();
			if(selectedVistAUser.getDssUnits()!=null && selectedVistAUser.getDssUnits().length()!=0){
				dssUnits = StringUtils.split(selectedVistAUser.getDssUnits(),"|");
			}
			setSessionAttribute("selectedVistaUser",selectedVistAUser);
		}
	}
	

	public String verifyWithADResults(){
		getUserInformation();
		if(appliedCancel()){ return CANCEL;	}

		if(isSearchResults()) 
		{	
			return SEARCH_RESULTS; 
		}
		
		getStaffSessionAttr();

		if(isNewSearch())
		{ 
			return SELECT_STAFF;
		}

		if((networkId == null) || ((networkId.trim()).equals(""))){
			addActionError("Please enter the Network ID.");
			return FAILED;
		}

		String stationNumber = null;
		if(((String)getSessionAttribute("stationNumber")) != null){
			stationNumber = (String)getSessionAttribute("stationNumber");
		}

		if (stationNumber == null) {
			addActionError("Please select a facility.");
			return SELECT_STAFF;
		}

		Clinician existingClinician = userDao.findClinicianByStationAndUsername(stationNumber, networkId);
		if (existingClinician != null) {
			addActionError("Network ID " +
						   networkId +
						   " is already in use at this station by " +
						   (existingClinician.isActive() ? "active" : "inactive") +
						   " staff member " +
						   existingClinician.getName());
			return FAILED;
		}

		ADQueryResult staffADResults = null;
		firstNameAD = "";
		lastNameAD = "";
		phoneAD = "";
		titleAD = "";
		networkIdAD = "";
		emailAD = "";
		try{
			
			ServiceResponse<ADQueryResult> response = getAdminService().findADAccount(networkId);
			staffADResults = response.getPayload();
			if(staffADResults == null){
				if(log.isInfoEnabled()){
					log.info(LogSanitizer.cleanText("verifyWithADResults()=>AD Results: Null for networkId"+networkId));
				}
				addActionError("An Email Address could not be found in Outlook using the VistA search results.");
				return FAILED;
			}else{
				firstNameAD = (staffADResults.getFirstName() == null)?"":staffADResults.getFirstName();
				lastNameAD = (staffADResults.getLastName() == null)?"":staffADResults.getLastName();
				phoneAD = (staffADResults.getTelephoneNumber() == null)?"":staffADResults.getTelephoneNumber();
				titleAD = (staffADResults.getTitle() == null)?"":staffADResults.getTitle();
				emailAD = (staffADResults.getEmail() == null)?"":staffADResults.getEmail();
				setSessionAttribute("email",emailAD);
				networkIdAD = networkId;
				this.setEmail(emailAD);
				staffADResults.setUserName(networkId);
			}
		}catch(Exception e){
			if(log.isErrorEnabled()){
				log.error("verifyWithADResults()=>"+e.getMessage() );
				e.printStackTrace();
			}
			addActionError("Error occurred while accessing the active directory.");
		}

		setSessionAttribute("staffADResults",staffADResults);

		return SUCCESS;
	}

	public String assignStaffMemberType(){

 
		if(appliedCancel()){ return CANCEL;	}
		getUserInformation();
		getStaffSessionAttr();

		if(isNewSearch()){ return SELECT_STAFF;	}
		duz = (String)getSessionAttribute("duz");
		staffMemberTypes = (List<ClinicalUserType>)getAdminService().getClinicalUserTypes().getCollection();
		return SUCCESS;
	}

	public String activateStaffMember(){

		if(appliedCancel()){ return CANCEL;	}

		getStaffSessionAttr();

		if(isNewSearch()){ return SELECT_STAFF;	}

		ClinicalUserType clinicalUserType = null;
		boolean userTypeSelected = false;
		duz = (String)getSessionAttribute("duz");
		String stationNumber = (String)getSessionAttribute("stationNumber");
		UserInfoDTO selectedVistaUser = (UserInfoDTO)getSessionAttribute("selectedVistaUser");
		ADQueryResult adr = (ADQueryResult)getSessionAttribute("staffADResults");

		staffMemberTypes = (List<ClinicalUserType>)getAdminService().getClinicalUserTypes().getCollection();
		if(staffMemberTypes != null){
			for(ClinicalUserType cu: staffMemberTypes){
				if((cu.getId()).equals(memberTypeId)){
					clinicalUserType = cu;
					memberType = cu.getName();
					userTypeSelected = true;
				}
			}
		}
		if(!userTypeSelected){
			addActionError("Please select a user type.");
			getUserInformation();
			setSessionAttribute("email",getEmail());
			return FAILED;
		}

		if(getEmail()==null || getEmail().equals("")){
			addActionError("Email Address cannot be blank.");
			return FAILED;
		}
		else
		{
			EmailValidator emailValidator = EmailValidator.getInstance();
			if(!emailValidator.isValid(email)){
				addActionError(ESAPI.encoder().encodeForHTML(email) + " is not a Valid Email Address.");
				setEmail(getEmail());
				setSessionAttribute("email",getEmail());
				return FAILED;
			}
			else
			{
				adr.setEmail(getEmail());
				setSessionAttribute("email",getEmail());
			}
		}

		Administrator admin = (Administrator)getSessionAttribute(CURRENT_USER);
		Clinician clin = null;

		try{
			ServiceResponse<Clinician> response = getAdminService().activateNewProvider(stationNumber, selectedVistaUser, adr,clinicalUserType, admin);
			clin = response.getPayload();
		}catch(Exception exp3){
			if(log.isErrorEnabled()){
				log.error("activateStaffMember()=>Exception occured while active staff member"+exp3);
			}
		}

		if (clin == null){
			addActionError("An unexpected error has occurred. Please contact the Help Desk, if the problem persists.");
			return FAILED;
		}
		getUserInformation();
		return SUCCESS;
	}

	public String activateAnotherStaffMember(){

		firstName=null;
		lastName=null;
		duz=null;
		if((selectAction!=null) && ((selectAction.trim()).equalsIgnoreCase("Done"))){
			clearSessionAttr();
			return CANCEL;
		}
		getStaffSessionAttr();

		return SELECT_STAFF;
	}


	public String showMemberDetails(){

		if(appliedCancel()){ return CANCEL;	}
		Clinician clin = null;
		try{
			duz = (getParameter("duz") == null)?"":getParameter("duz");
			if(duz == null){
				addActionError("Could not find DUZ for the selected user.");
			}
			List<Clinician> searchedUsers = (List<Clinician>)getSessionAttribute("clinicians");
			if(searchedUsers != null){
				for(Clinician u:searchedUsers){
					if(duz.equalsIgnoreCase(u.getDuz())){
						clin = u;
					}
				}
			}
			if(clin != null){
	 			firstName = (clin.getFirstName() == null)?"":clin.getFirstName();
	 			lastName = (clin.getLastName() == null)?"":clin.getLastName();
	 			networkId = (clin.getUsername() == null)?"":clin.getUsername(); 
	 			middleName = (clin.getMiddleName() == null)?"":clin.getMiddleName(); 
	 			title = (clin.getTitle() == null)?"":clin.getTitle();
	 			phone =(clin.getPhone() == null)?"":clin.getPhone(); 
	 			suffix = (clin.getSuffix() == null)?"":clin.getSuffix();
	 			prefix =(clin.getPrefix() == null)?"":clin.getPrefix();
	 			if(clin.getProviderVistAActiveFlag()!=null) activeFlag = clin.getProviderVistAActiveFlag().equals("0")?"ACTIVE":"INACTIVE";
	 			requiresCoSigner = clin.getRequiresCosigner();
	 			if(clin.getProviderPcmmFlag()!=null) pcmmFlag = clin.getProviderPcmmFlag().equals("1")?"YES":"NO";
	 			personClass = clin.getPersonClass()!=null?clin.getPersonClass():"";
	 			if(clin.getProvider()!=null) providerKey = clin.getProvider().equalsIgnoreCase("1")?"TRUE":"FALSE";
	 			providerClass = clin.getProviderClass();
	 			if(clin.getDmmsUnits()!=null) dssUnits = StringUtils.split(clin.getDmmsUnits(),DATA_DELIMITER);
				ClinicalUserType cu = (ClinicalUserType)clin.getClinicalUserType();
				if(cu != null){
					memberType = (cu.getName() == null)?"":cu.getName();
				}
			}
			setSessionAttribute("selectedUser",clin);
			setSessionAttribute("duz",duz);
		}catch(Exception e){
			if(log.isErrorEnabled()){
				log.error("showMemberDetails()=>Exception occurred while getting selected user details.");
				e.printStackTrace();
			}
		}

		return SUCCESS;

	}


	public String inactivateReactivateMember(){

		if(appliedCancel()){ return CANCEL;	}

		if(isSearchResults()) 
		{	
			return SEARCH_RESULTS; 
		}
		
		getStaffSessionAttr();

		if(isNewSearch()){ return SELECT_STAFF;	}
		String activationType = (String)getSessionAttribute("activationType");
		try {
			Clinician c = (Clinician)getSessionAttribute("selectedUser");
			if(c != null){
				if((activationType != null) && (activationType.equalsIgnoreCase("inactivate"))){
					c = (Clinician) userManagementService.inactivateReactivateClinician(c, false).getPayload();
				}else if((activationType != null) && (activationType.equalsIgnoreCase("reactivate"))){
					c = (Clinician) userManagementService.inactivateReactivateClinician(c, true).getPayload();
				}
			}
		} catch (Exception e) {
			log.info("Exception occurred while inactivating the selected user.");
			addActionError("Error occurred while inactivating the selected user.");
		}

		return SUCCESS;

	}

	public String inactivationReactivationComplete(){

		clearSessionAttr();
		return CANCEL;

	}

	public String bulkActivation() {
		if(appliedCancel()){ return CANCEL;	}
		clearSessionAttr();
		List<User> staffMembers= new ArrayList<User>();
		boolean activationSuccess = true;
		try{

			log.info(LogSanitizer.cleanText("file name: " + uploadFileName));
			uploadFileName = (uploadFileName == null)?"":uploadFileName;
			if(uploadFileName.equals("")){
				if(!checkFileSizeMessage()){
					addActionError("Please select a file");
				}
				return FAILED;
			}
			log.info(LogSanitizer.cleanText("Content type: " + getUploadContentType()));
			Object rawContent = getUpload().toURL().getContent();

			InputStream is = (InputStream)rawContent;
			Reader reader = new InputStreamReader(is);
			StringBuilder content = new StringBuilder();
			int read = 0;
			while ( read > -1 ) {
				char[] buffer = new char[1024];
				read = reader.read(buffer);
				content.append(buffer);
				if(FILE_SIZE_LIMIT_2MB<content.length()) {
					addActionError(FILE_SIZE_LIMIT_ERROR);
					return FAILED;
				}
			}
			if(log.isInfoEnabled()){
				log.info(LogSanitizer.cleanText("Contents:\n" + content.toString().trim()));
			}
			String fileContent = content.toString().trim();
			if(fileContent.length() == 0){
				addActionError("The bulk addition of user records failed" +
						" because of the following records. " +
						"Correct these records and repeat bulk addition:");

				addActionError("Line 1-incorrectly formatted.");
				addActionError("No Secure Messaging Staff Members have been activated.");
				return FAILED;
			}
			String[] rows = fileContent.split("\r\n");
			//Validate the Bulk Record File
			if(validateBulkRecordFile(rows)){
				addActionError("No Secure Messaging Staff Members have been activated.");
				return FAILED;
			}else{
				String recordType;
				String firstName;
				String lastName;
				String stationNumber;
				added=0;
				int lineNumber = 0;
				for (int i=0; i<rows.length; i++){
					log.info(LogSanitizer.cleanText("Line" + i + ":" + rows[i]));
					String rowData = rows[i];
					String[] fields = rowData.split(",");
					recordType  = fields[0].trim();
					firstName  = fields[1].trim();
					lastName  = fields[2].trim();
					duz  = fields[3].trim();
					stationNumber  = fields[4].trim();
					networkId  = fields[5].trim();
					Long clinicalUserTypeId  = Long.valueOf(fields[6].trim());
					lineNumber = i + 1;
					log.info(LogSanitizer.cleanText("firstName= "+firstName));
					log.info(LogSanitizer.cleanText("lastName= "+lastName));
					log.info(LogSanitizer.cleanText("duz:" + duz.toString()));
					log.info(LogSanitizer.cleanText("stationNumber:" + stationNumber.toString()));
					log.info(LogSanitizer.cleanText("clinicalUserTypeId:" + clinicalUserTypeId.toString()));
					log.info(LogSanitizer.cleanText("networkId:" + networkId.toString()));
					boolean isAdded = activateBulkStaffMember(lineNumber, duz,stationNumber, clinicalUserTypeId, networkId, firstName, lastName);
					if(isAdded){
						added++;
					}
					activationSuccess = activationSuccess && isAdded;
				}
			}
		}catch (java.io.FileNotFoundException e) 	{
			addActionError("The bulk addition of user records failed" +
					" because of the following records. " +
					"Correct these records and repeat bulk addition:");

			addActionError("Line 1-incorrectly formatted.");
			addActionError("No Secure Messaging Staff Members have been activated.");
			log.error(e);
			return FAILED;

		} catch(Exception e) {
			log.error(e);
			return FAILED;
		}

		if(activationSuccess){
			return SUCCESS;
	    }else{
	    	return FAILED;
	    }
	}

	/**
	 * There are two limits for file upload size struts.multipart.maxSize and maximumSize in the action
	 * interceptor. The problem is for bug :5391 providing custom message is not easy as the
	 * error message is hardcoded in MultiPartRequest. In the following code the exception is
	 * trapped and replaced with the message as stated in specification.
	 */

	private boolean checkFileSizeMessage() {
		boolean hasSizeError = false;
		Collection<?> tmp = getActionErrors();
		if(tmp!= null){
			Collection<String> errors = new ArrayList<String>();
			for (Object o : tmp) {
				if(o != null){
				     if (o.toString().contains("the request was rejected because its size")) {
				       hasSizeError = true;
				       errors.add(FILE_SIZE_LIMIT_ERROR);
				     } else {
				       errors.add(o.toString());
				     }
				}
			}

			setActionErrors(errors);
		}
		return hasSizeError;
	}

	/*
	 * Validate the Bulk Record File, it is a Comma Separated Values (CSV) file where each row is expected to
	 * have a certain number and type of fields. Text fields are enclosed in quotes.
	 * BR3902 The Bulk Record Schema
	 * Retruns true if file is not formating is wrong.
	 * C,FNAME,LNAME,DUZ,STATION_NO,EXTERNAL_USERNAME,CLINICAL_USER_TYPE_ID
       C,JOHN,DOE,12345,994,      JDOE,1
	 */
	private boolean validateBulkRecordFile(String[] rows){
		//keep track of duplicates in current file users
		Map currentFileUsers = new HashMap();

		boolean fileFailed = false;
		for (int i=0; i<rows.length; i++){

			log.info(LogSanitizer.cleanText("Line" + i + ":" + rows[i]));
			boolean failed = false;
			int lineNumber = i + 1;
			String rowData = rows[i];
			String[] fields = rowData.split(",");
			//check if this row has all fields
			if(fields.length == 7){
				String lrecordType  = fields[0].trim();
				//check if Record Type is not null and maximum length is 1
				if(lrecordType== null || lrecordType.length() >1){
					failed = true;
				}
				//check first name
				String lfirstName  = fields[1].trim();
				if(lfirstName == null || lfirstName.length() >50){
					failed = true;
				}
				//check last name
				String llastName  = fields[2].trim();
				if(llastName == null || llastName.length() >50){
					failed = true;
				}
				//check DUZ
				String lduz  = fields[3].trim();
				if(lduz == null || lduz.length() >20){
					failed = true;
				}
				//check STATION_NO
				String lstationNumber  = fields[4].trim();
				if(lstationNumber == null || lstationNumber.length() >10){
					failed = true;
				}
				//check EXTERNAL_USER_NAME
				String lnetworkId  = fields[5].trim();
				if(lnetworkId == null || lnetworkId.length() >50){
					failed = true;
				}
				//check CLINICAL_USER_TYPE_ID
				try{
					Long lclinicalUserTypeId  = Long.valueOf(fields[6].trim());
					if(lclinicalUserTypeId == null){
						failed = true;
					}
				}catch(java.lang.NumberFormatException ex){
					//the clinical user type if is not a number.
					failed = true;
				}

				//if failed because of format
				if(failed){
					if(!fileFailed){
						fileFailed = true;
						addActionError("The bulk addition of user records failed" +
								" because of the following records. " +
								"Correct these records and repeat bulk addition:");
					}
					addActionError("Line " + lineNumber + "-incorrectly formatted.");
				}else{
					String tmpStationNumber = (String)currentFileUsers.get(lnetworkId);

					if(tmpStationNumber!=null){
						if(tmpStationNumber.equals(lstationNumber)){
							if(!fileFailed){
								fileFailed = true;
								addActionError("The bulk addition of user records failed" +
										" because of the following records. " +
										"Correct these records and repeat bulk addition:");
							}
							addActionError("Line " + lineNumber + "- Duplicate Network ID " +lnetworkId +
										   " is already listed in this file at this station by" +
										   " staff member " +
										   ESAPI.encoder().encodeForHTML(lfirstName)+","+ESAPI.encoder().encodeForHTML(llastName));

						}


					}else{
						currentFileUsers.put(lnetworkId,lstationNumber);
					}


					//ensure that you can never activate new staff members that have the same external_user_name and station_no
					Clinician existingClinician = userDao.findClinicianByStationAndUsername(lstationNumber, lnetworkId);
					if (existingClinician != null) {
						if(!fileFailed){
							fileFailed = true;
							addActionError("The bulk addition of user records failed" +
									" because of the following records. " +
									"Correct these records and repeat bulk addition:");
						}
						addActionError("Line "  + lineNumber + "- Network ID " +lnetworkId +
									   " is already in use at this station by " +
									   (existingClinician.isActive() ? "active" : "inactive") +
									   " staff member " +
									   existingClinician.getName());

					}
				}
			}else{
				if(!fileFailed){
					fileFailed = true;
					addActionError("The bulk addition of user records failed" +
							" because of the following records. " +
							"Correct these records and repeat bulk addition:");
				}
				addActionError("Line " + lineNumber + "-incorrectly formatted.");
			}
		}
		return fileFailed;

	}

	public boolean activateBulkStaffMember(int lineNumber, String duz,String stationNumber, Long clinicalUserTypeId, String networkId, String firstName, String lastName){
		boolean successfullyActivated = true;

		Clinician clin = null;
		Administrator admin = (Administrator)getSessionAttribute(CURRENT_USER);
		log.info("Admin isNational???........."+admin.isNational());

		boolean permission = false;

		//If logged-in user is National Admin, they have rights to manage the any facility user.
		if(admin.isNational()){
			permission = true;
		}

		// If logged-in user is VISN Admin, they have rights to manage any facilities under the visn.
		if(admin.getVisns() !=null) {
			log.info("admin.getVisins()......"+admin.getVisns());
			for (Facility visn: admin.getVisns())
			{
				Collection<Facility> facilities = getFacilityService().getFacilitiesForVisn(visn).getCollection();
				for(Facility f1 : facilities){
					log.info("f1.getId()...."+f1.getId());
					log.info("f1.getName...."+f1.getName());
					log.info("f1.getStationNumber()......."+f1.getStationNumber());
					if(f1.getStationNumber().equalsIgnoreCase(stationNumber)){
						permission = true;
						break;
					}
				}
			}
		}

		// If logged-in user is Facility Admin, they have rights to manage only the associated user facilities.
		for(Facility f:admin.getFacilities()) {
			if( f.getStationNumber().equalsIgnoreCase(stationNumber) ) {
				permission = true;
				break;
			}
		}

		log.info("permission........."+permission);


		if( !permission ) {
			addActionError("Line " + lineNumber + ": The staff member's facility doesn't match a facility you can manage.");
			successfullyActivated = false;
		}

		User selectedVistaUser = null;
		List<User> staffMembers= new ArrayList<User>();
		CollectionServiceResponse<User> response = getAdminService().findVistaClinician(duz, stationNumber);
		handleErrorMessages(response.getMessages());
		staffMembers = (List<User>)response.getCollection();
		if(staffMembers == null){
			addActionError("Line " + lineNumber + ": A search of the VistA database did not produce a match for the information given.");
			successfullyActivated = false;
		}else{
			for(User u:staffMembers){
					if(duz.equalsIgnoreCase(u.getIEN())){
					selectedVistaUser = u;
				}
			}
		}
		if(selectedVistaUser == null){
			addActionError("Line " + lineNumber + ": A search of the VistA database did not produce a match for the information given.");
			successfullyActivated = false;
		}

		ClinicalUserType clinicalUserType = null;
		staffMemberTypes = (List<ClinicalUserType>)getAdminService().getClinicalUserTypes().getCollection();
		if(staffMemberTypes != null){
			for(ClinicalUserType cu: staffMemberTypes){
				if((cu.getId()).equals(clinicalUserTypeId)){
					clinicalUserType = cu;
				}
			}
		}
		if(clinicalUserType == null){
			log.info("clinicalUserType: Null");
			addActionError("Line " + lineNumber + ": A search for clinical user type could not produce a match");
			successfullyActivated = false;
		}

		ADQueryResult staffADResults = null;
		ServiceResponse<ADQueryResult> findADAccountResponse = getAdminService().findADAccount(networkId);
		staffADResults = findADAccountResponse.getPayload();
		if(staffADResults == null){
			log.info("AD Results: Null");
			addActionError("Line " + lineNumber + ": An Email Address could not be found in Outlook using the VistA search results.");
			successfullyActivated = false;
		}else{
			staffADResults.setUserName(networkId);
		}

		if(null!=staffADResults && null!=staffADResults.getFirstName() && !staffADResults.getFirstName().equalsIgnoreCase(firstName) ) {

			log.info(LogSanitizer.cleanText("AD firstname didn't match bulk record: " + staffADResults.getFirstName() + ", " + firstName));
			addActionError("Line " + lineNumber + ": " + (lastName==null?"null":ESAPI.encoder().encodeForHTML(lastName)) + ", " + ESAPI.encoder().encodeForHTML(firstName) + " is not recognized as an Outlook email account.");
			successfullyActivated = false;
		}

		if(null!=staffADResults && !staffADResults.getLastName().equalsIgnoreCase(lastName) ) {
			log.info(LogSanitizer.cleanText("AD lastname didn't match bulk record: " + staffADResults.getLastName() + ", " + lastName));
			addActionError("Line " + lineNumber + ": " + ESAPI.encoder().encodeForHTML(lastName) + ", " + (firstName==null?"null":ESAPI.encoder().encodeForHTML(firstName)) + " is not recognized as an Outlook email account.");
			successfullyActivated = false;
		}

		if(staffADResults.getEmail()!=null && !staffADResults.getEmail().equals("")){
			EmailValidator emailValidator = EmailValidator.getInstance();
			if(!emailValidator.isValid(staffADResults.getEmail())){
				addActionError("Line " + lineNumber +  ": Outlook Email Address is invalid.");
				successfullyActivated = false;
			}
		}
		else
		{
			addActionError("Line "+ lineNumber + ": Outlook Email Address can't be blank.");
			successfullyActivated = false;
		}

		try {
			if( true == successfullyActivated ) {
				ServiceResponse<Clinician> response1 = getAdminService().createClinicianAccount(stationNumber, selectedVistaUser, staffADResults, clinicalUserType, admin);
				clin = response1.getPayload();
			}
		} catch(Exception e){
			successfullyActivated = false;
			log.error( e.getMessage() );
		}

		if (clin == null){
			addActionError("Line " + lineNumber + ": Error occurred while activating the account.");
			successfullyActivated = false;
		}

		return successfullyActivated;
	}

	public String bulkActivationComplete() {

		clearSessionAttr();
		if((selectAction!=null) && ((selectAction.trim()).equalsIgnoreCase("Done"))){
			return CANCEL;
		}

		return SUCCESS;
	}


	public void getStaffSessionAttr(){

		if(((Collection<Facility>)getSessionAttribute("facilities")) != null){
			facilities = (Collection<Facility>)getSessionAttribute("facilities");
		}else{
			facilities = new ArrayList<Facility>();
		}
		if(((String)getSessionAttribute("selectedVisnName")) != null){
			selectedVisnName = (String)getSessionAttribute("selectedVisnName");
		}
		if(((String)getSessionAttribute("selectedFacilityName")) != null){
			selectedFacilityName = (String)getSessionAttribute("selectedFacilityName");
		}

	}

	public void clearSessionAttr(){
		removeSessionAttribute("facilities");
		removeSessionAttribute("selectedVisnName");
		removeSessionAttribute("selectedFacilityName");
		removeSessionAttribute("stationNumber");
		removeSessionAttribute("staffMembers");
		removeSessionAttribute("activationType");
		removeSessionAttribute("selectedVistaUser");
	}

	public boolean appliedCancel() {

		if((selectAction!=null) && ((selectAction.trim()).equalsIgnoreCase("Cancel")||(selectAction.trim().equalsIgnoreCase("Exit")))){
			clearSessionAttr();
			return true;
		}
		return false;

	}

	public boolean isNewSearch() {

		if((selectAction!=null) && ((selectAction.trim()).equalsIgnoreCase("New Search"))){
			firstName=null;
			lastName=null;
			duz=null;
			removeSessionAttribute("selectedVistaUser");
			return true;
		}
		return false;

	}
	
	public boolean isSearchResults(){
		if((selectAction!=null) && ((selectAction.trim()).equalsIgnoreCase("Back"))){
			return true;
		}
		return false;
	}


	public void setVisns(Collection<Facility> visns) {
			this.visns = visns;
	}

	public Collection<Facility> getVisns() {
			return visns;
	}

	public void setFacilities(Collection<Facility> facilities) {
			this.facilities = facilities;
	}

	public Collection<Facility> getFacilities() {
			return this.facilities;
	}

	public void setSelectedVisnName(String selectedVisnName) {
			this.selectedVisnName = selectedVisnName;
	}

	public String getSelectedVisnName() {
			return selectedVisnName;
	}

	public void setSelectedFacilityName(String selectedFacilityName) {
			this.selectedFacilityName = selectedFacilityName;
	}

	public String getSelectedFacilityName() {
			return selectedFacilityName;
	}

	public void setVisnId(Long visnId) {
			this.visnId = visnId;
	}

	public Long getVisnId() {
			return this.visnId;
	}

	public void setFacilityId(Long facilityId) {
			this.facilityId = facilityId;
	}

	public Long getFacilityId() {
			return this.facilityId;
	}

	public void setClinicians(List<Clinician> clinicians) {
			this.clinicians = clinicians;
	}

	public List<Clinician> getClinicians() {
			return clinicians;
	}

	public void setSelectAction(String selectAction) {
		this.selectAction = selectAction;
	}

	public String getSelectAction() {
		return selectAction;
	}

	public void setSelectedUsers(List<Clinician> selectedUsers) {
			this.selectedUsers = selectedUsers;
	}

	public List<Clinician> getSelectedUsers() {
			return selectedUsers;
	}

	public void setSearchCrit(String searchCrit) {
			this.searchCrit = searchCrit;
	}

	public String getSearchCrit() {
			return searchCrit;
	}

	public void setDuz(String duz) {
			this.duz = duz;
	}

	public String getDuz() {
			return duz;
	}

	public void setFirstName(String firstName) {
			this.firstName = firstName;
	}

	public String getFirstName() {
			return firstName;
	}

	public void setLastName(String lastName) {
			this.lastName = lastName;
	}

	public String getLastName() {
			return lastName;
	}

	public void setPhone(String phone) {
			this.phone = phone;
	}

	public String getPhone() {
			return phone;
	}

	public void setTitle(String title) {
			this.title = title;
	}

	public String getTitle() {
			return title;
	}

	public void setNetworkId(String networkId) {
			this.networkId = networkId;
	}

	public String getNetworkId() {
			return networkId;
	}

	public void setStaffMemberTypes(List<ClinicalUserType> staffMemberTypes) {
			this.staffMemberTypes = staffMemberTypes;
	}

	public List<ClinicalUserType> getStaffMemberTypes() {
			return staffMemberTypes;
	}

	public void setMemberTypeId(Long memberTypeId) {
			this.memberTypeId = memberTypeId;
	}

	public Long getMemberTypeId() {
			return memberTypeId;
	}

	public void setMemberType(String memberType) {
			this.memberType = memberType;
	}

	public String getMemberType() {
			return memberType;
	}

	public void setFirstNameAD(String firstNameAD) {
			this.firstNameAD = firstNameAD;
	}

	public String getFirstNameAD() {
			return firstNameAD;
	}

	public void setLastNameAD(String lastNameAD) {
			this.lastNameAD = lastNameAD;
	}

	public String getLastNameAD() {
			return lastNameAD;
	}

	public void setPhoneAD(String phoneAD) {
			this.phoneAD = phoneAD;
	}

	public String getPhoneAD() {
			return phoneAD;
	}


	public void setTitleAD(String titleAD) {
			this.titleAD = titleAD;
	}

	public String getTitleAD() {
			return titleAD;
	}

	public void setNetworkIdAD(String networkIdAD) {
			this.networkIdAD = networkIdAD;
	}

	public String getNetworkIdAD() {
			return networkIdAD;
	}

	public String getUploadContentType() {
		return uploadContentType;
	}

	public void setUploadContentType(String contentType) {
		this.uploadContentType = contentType;
	}

	public File getUpload() {
		return upload;
	}

	public void setUpload(File file) {
		this.upload = file;
	}

	public String getUploadFileName() {
		return uploadFileName;
	}

	public void setUploadFileName(String filename) {
		this.uploadFileName = filename;
	}

	public String getEmailAD() {
		return emailAD;
	}

	public void setEmailAD(String emailAD) {
		this.emailAD = emailAD;
	}

	public String getEmail() {
		return email;
	}

	public void setEmail(String email) {
		this.email = email;
	}

	public UserDao getUserDao() {
		return userDao;
	}

	public void setUserDao(UserDao userDao) {
		this.userDao = userDao;
	}


	public int getAdded() {
		return added;
	}


	public void setAdded(int added) {
		this.added = added;
	}

	public String getMiddleName() {
		return middleName;
	}


	public void setMiddleName(String middleName) {
		this.middleName = middleName;
	}


	public String getSuffix() {
		return suffix;
	}


	public void setSuffix(String suffix) {
		this.suffix = suffix;
	}


	public String getPrefix() {
		return prefix;
	}


	public void setPrefix(String prefix) {
		this.prefix = prefix;
	}


	public String getActiveFlag() {
		return activeFlag;
	}


	public void setActiveFlag(String activeFlag) {
		this.activeFlag = activeFlag;
	}


	public String getProviderKey() {
		return providerKey;
	}


	public void setProviderKey(String providerKey) {
		this.providerKey = providerKey;
	}


	public String getProviderClass() {
		return providerClass;
	}


	public void setProviderClass(String providerClass) {
		this.providerClass = providerClass;
	}


	public String getPersonClass() {
		return personClass;
	}


	public void setPersonClass(String personClass) {
		this.personClass = personClass;
	}


	public String getPcmmFlag() {
		return pcmmFlag;
	}


	public void setPcmmFlag(String pcmmFlag) {
		this.pcmmFlag = pcmmFlag;
	}


	public String getRequiresCoSigner() {
		return requiresCoSigner;
	}


	public void setRequiresCoSigner(String requiresCoSigner) {
		this.requiresCoSigner = requiresCoSigner;
	}

	public String[] getDssUnits() {
		return dssUnits;
	}


	public void setDssUnits(String[] dssUnits) {
		this.dssUnits = dssUnits;
	}
	

}
