package gov.va.med.mhv.sm.api.clinician.impl;

import gov.va.med.mhv.foundation.service.response.CollectionServiceResponse;
import gov.va.med.mhv.foundation.service.response.ServiceResponse;
import gov.va.med.mhv.sm.api.clinician.ClinicianAPIService;
import gov.va.med.mhv.sm.api.common.SessionAPIService;
import gov.va.med.mhv.sm.api.enumeration.ErrorEnum;
import gov.va.med.mhv.sm.api.transfer.ClinicianSearchFormTO;
import gov.va.med.mhv.sm.api.transfer.ClinicianUserTO;
import gov.va.med.mhv.sm.api.transfer.Session;
import gov.va.med.mhv.sm.api.transfer.UserTO;
import gov.va.med.mhv.sm.api.transfer.UsersTO;
import gov.va.med.mhv.sm.api.util.PropertiesHelper;
import gov.va.med.mhv.sm.api.util.SMApiUtility;
import gov.va.med.mhv.sm.model.Clinician;
import gov.va.med.mhv.sm.service.AuthenticationService;
import gov.va.med.mhv.sm.service.UserManagementService;
import gov.va.med.mhv.sm.wsclient.queriessvc.User;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import javax.annotation.Resource;

import org.apache.cxf.jaxrs.ext.MessageContext;

public class ClinicianAPIServiceImpl implements ClinicianAPIService {
	
	private SessionAPIService sessionApiService;
	private UserManagementService userManagementService;
	private AuthenticationService authenticationService;

	
	@Resource
	MessageContext mc;

	private static int MAX_PAGE_SIZE = 250;
	
	static {
		if( PropertiesHelper.getProperties() != null && PropertiesHelper.getProperties().containsKey("sm.api.maxpagesize")) {
			String expStr = PropertiesHelper.getProperties().getProperty("sm.api.maxpagesize");
			try {
				int expInt = Integer.parseInt(expStr);
				if( expInt > 0 ) {
					MAX_PAGE_SIZE = expInt;
				}
			} catch (NumberFormatException e) {
				//do nothing
			}
		}
	}
	
	@Override
	public UsersTO searchUsers(ClinicianSearchFormTO form, Long pageNumber, Long pageSize) {
		Session session = sessionApiService.checkSession(mc);
		Clinician c = (Clinician) session.getClinician();

		//Offset the Page Number to 0 base.  (So -1)
		--pageNumber;
		
		if( pageSize > MAX_PAGE_SIZE ) {
			SMApiUtility.throwException(ErrorEnum.PAGE_SIZE_EXCEEDS_LIMIT_133);
		}
		
		//VALIDATION/MINIMUM FIELDS
		if( form.getFirstName().length() == 0 && form.getLastName().length() == 0) {
			SMApiUtility.throwException(ErrorEnum.PARAMETERS_CANNOT_BE_EMPTY_139);
		}
		
		UsersTO usersTO = new UsersTO();

		CollectionServiceResponse<Clinician> response = userManagementService.searchForClinicians(form.getFirstName(), form.getLastName(), c.getStationNo());
		SMApiUtility.throwExceptionOnErrors(ErrorEnum.USER_MANAGEMENT_SERVICE_ERROR_906, response);
		Collection<Clinician> list = response.getCollection();
		
		//Page 1 with size 25 will result: start = 0; end = 25; or (0-24)
		long startElement = pageNumber.longValue()*pageSize.longValue();
		long endElement = (pageNumber.longValue()+1)*pageSize.longValue();
		long currElement = 0;
		
		List<UserTO> userArray = new ArrayList<UserTO>();
		for (Clinician c1 : list) {
			if( currElement >= startElement && currElement < endElement) {
				userArray.add(new ClinicianUserTO(c1));
			}
			currElement++;
		}
		usersTO.setUser(userArray);
		usersTO.setAvailable(new Long(list.size()));
		
		return usersTO;
	}
	
	@Override
	public ClinicianUserTO getClinician(Long userId) {
		Session session = sessionApiService.checkSession(mc);
		Clinician c = (Clinician) session.getClinician();

		//Validation?
		
		ServiceResponse<Clinician> response = authenticationService.fetchClinicianById(userId);
		SMApiUtility.throwExceptionOnErrors(ErrorEnum.AUTHENTICATION_SERVICE_ERROR_901, response);
		Clinician c1 = response.getPayload();

		if(null==c1) {
			SMApiUtility.throwException(ErrorEnum.USER_NOT_FOUND_105);
		}
		
		ServiceResponse<User> response2 = userManagementService.getClinicianDemographics(c1);
		SMApiUtility.throwExceptionOnErrors(ErrorEnum.USER_MANAGEMENT_SERVICE_ERROR_906, response2);
		User cd = response2.getPayload();

		return new ClinicianUserTO(c1.getId(), c1.getUsername(), cd.getFirstName(),
				cd.getLastName(), c1.getStationNo(),
				cd.getDepartment(),cd.getPhone());
	}
	
	@Override
	public ClinicianUserTO getCurrentClinicianInfo() {
		Session session = sessionApiService.checkSession(mc);
		Clinician c = (Clinician) session.getClinician();

		//Validation?
		
		ServiceResponse<Clinician> response = authenticationService.fetchClinicianById(c.getId());
		SMApiUtility.throwExceptionOnErrors(ErrorEnum.AUTHENTICATION_SERVICE_ERROR_901, response);
		Clinician c1 = response.getPayload();

		if(null==c1) {
			SMApiUtility.throwException(ErrorEnum.USER_NOT_FOUND_105);
		}
		
		ServiceResponse<User> response2 = userManagementService.getClinicianDemographics(c1);
		SMApiUtility.throwExceptionOnErrors(ErrorEnum.USER_MANAGEMENT_SERVICE_ERROR_906, response2);
		User cd = response2.getPayload();
		
		return new ClinicianUserTO(c1.getId(), c1.getUsername(), cd.getFirstName(),
				cd.getLastName(), c1.getStationNo(),
				cd.getDepartment(),cd.getPhone());
	}	


	public SessionAPIService getSessionApiService() {
		return sessionApiService;
	}
	public void setSessionApiService(SessionAPIService sessionService) {
		this.sessionApiService = sessionService;
	}
	public UserManagementService getUserManagementService() {
		return userManagementService;
	}
	public void setUserManagementService(UserManagementService userManagementService) {
		this.userManagementService = userManagementService;
	}
	public AuthenticationService getAuthenticationService() {
		return authenticationService;
	}
	public void setAuthenticationService(AuthenticationService authenticationService) {
		this.authenticationService = authenticationService;
	}

}
