package gov.va.med.mhv.sm.api.patient.impl;

import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.TreeMap;

import javax.annotation.Resource;
import javax.ws.rs.core.Response;

import org.apache.commons.lang.StringEscapeUtils;
import org.apache.commons.validator.EmailValidator;
import org.apache.cxf.jaxrs.ext.MessageContext;
import org.apache.cxf.jaxrs.impl.ResponseBuilderImpl;

import gov.va.med.mhv.foundation.service.response.ServiceResponse;
import gov.va.med.mhv.sm.api.common.SessionAPIService;
import gov.va.med.mhv.sm.api.enumeration.ErrorEnum;
import gov.va.med.mhv.sm.api.patient.PreferencesAPIService;
import gov.va.med.mhv.sm.api.transfer.NotificationStatusTO;
import gov.va.med.mhv.sm.api.transfer.Session;
import gov.va.med.mhv.sm.api.util.SMApiUtility;
import gov.va.med.mhv.sm.dao.SignatureDao;
import gov.va.med.mhv.sm.enumeration.EmailNotificationEnum;
import gov.va.med.mhv.sm.enumeration.MessageFilterEnum;
import gov.va.med.mhv.sm.model.Patient;
import gov.va.med.mhv.sm.model.Signature;
import gov.va.med.mhv.sm.service.UserManagementService;

public class PreferencesAPIServiceImpl implements PreferencesAPIService {

	private SessionAPIService sessionApiService;
	private UserManagementService userManagementService;
	private SignatureDao signatureDao;

	@Resource
	MessageContext mc;
	
	@Override
	public Response getNotificationPreference() {
		Session session = sessionApiService.checkSession(mc);
		Patient p = (Patient)session.getPatient();
		
		//Create Response Object
		NotificationStatusTO res = new NotificationStatusTO();
		if(p.getEmail() != null)
			res.setEmailAddress(p.getEmail());
		else
			res.setEmailAddress("");
		res.setNotifyMe(p.getEmailNotification().getName());
		
		ResponseBuilderImpl builder = new ResponseBuilderImpl();
		return builder.entity(res).status(200).build();
	}

	@Override
	public Response getNotificationPreferenceList() {
		Session session = sessionApiService.checkSession(mc);
		Patient p = (Patient)session.getPatient();
		
		//List<EmailNotificationEnum> emailFilterEnumList = EmailNotificationEnum.toList();
		Map<Long, String> emailFilterEnumList = new TreeMap<Long, String>(EmailNotificationEnum.EMAIL_NOTIFICATION_DETAIL);
		
		// Removing ON_ASSIGNEMNT(3l) from the list. this is for Clinician
		emailFilterEnumList.remove(3l);
		
		ResponseBuilderImpl builder = new ResponseBuilderImpl();
		return builder.entity(emailFilterEnumList).status(200).build();	
	
	}

	@Override
	public Response setNotificationPreference(NotificationStatusTO notiTO) {
		Session session = sessionApiService.checkSession(mc);
		Patient p = (Patient)session.getPatient();
		
		//test email Validation..if Empty proceed..its expected
		if(notiTO.getEmailAddress() != null && !EmailValidator.getInstance().isValid(notiTO.getEmailAddress())) {
			SMApiUtility.throwException(ErrorEnum.INVALID_EMAIL_ADDRESS_152);
		}
		if(notiTO.getNotifyMe() != null) {
			EmailNotificationEnum value = null;
//			//	p.setEmailNotification(EmailNotificationEnum. .valueOf(notiTO.getNotifyMe()));
//			Map<Long, String> emailFilterEnumList = new TreeMap<Long, String>(EmailNotificationEnum.EMAIL_NOTIFICATION_DETAIL);
//			for (Entry<Long, String> entry : emailFilterEnumList.entrySet()) {
//		        if (entry.getValue().equals(notiTO.getNotifyMe())) {
//		        	value = EmailNotificationEnum.EMAIL_NOTIFICATION_CODE.get(entry.getKey());
//		        }
//			}
//			
//			if(value != null)
//				p.setEmailNotification(value);
//			else {
//				SMApiUtility.throwException(ErrorEnum.DATA_INTEGRITY_ERROR_117);
//			}
			
			//Another Mechanism
			Long key = null;
			try{
				key = Long.parseLong(notiTO.getNotifyMe());
			} catch (NumberFormatException e ) {
				SMApiUtility.throwException(ErrorEnum.EMAIL_SIGNATURE_NOTIFY_ME_INVALID);
			}
			if(key.longValue() == 3l)
				SMApiUtility.throwException(ErrorEnum.EMAIL_SIGNATURE_NOTIFY_ME_INVALID);
			if(key != null )
				value = EmailNotificationEnum.EMAIL_NOTIFICATION_CODE.get(key);
        	
			if(value != null)
				p.setEmailNotification(value);
			else
				SMApiUtility.throwException(ErrorEnum.EMAIL_SIGNATURE_NOTIFY_ME_INVALID);
	        
		}	
		
		if(notiTO.getEmailAddress() != null)
			p.setEmail(notiTO.getEmailAddress());
		
		//save Preference
		ServiceResponse servRes = userManagementService.savePreferences(p);
		SMApiUtility.throwExceptionOnErrors(ErrorEnum.MESSAGE_SERVICE_ERROR_904, servRes);
		
		ResponseBuilderImpl builder = new ResponseBuilderImpl();
		return builder.status(200).build();	
	}

	@Override
	public Response getSignatureprefPreference() {
		Session session = sessionApiService.checkSession(mc);
		Patient p = (Patient)session.getPatient();
		
		//Create Response Object
		NotificationStatusTO res = new NotificationStatusTO();
		List signatureDaoList  =  signatureDao.getSignatureForUser(p.getId());
		Signature signature = null; 
		if(signatureDaoList != null && signatureDaoList.size() >0){
			signature = (Signature)signatureDaoList.get(0);
		}
		if(signature != null) {
			res.setSignatureName(signature.getName());
			res.setSignatureTitle(signature.getTitle());
			res.setIncludeSignature(signature.isActive());
		}
		else {
			SMApiUtility.throwException(ErrorEnum.EMAIL_SIGNATURE_NOT_EXIST_153);
		}
		
	
		ResponseBuilderImpl builder = new ResponseBuilderImpl();
		return builder.entity(res).status(200).build();
	}

	@Override
	public Response setSignaturePreference(NotificationStatusTO notiTO) {
		Session session = sessionApiService.checkSession(mc);
		Patient p = (Patient)session.getPatient();
		
		//Create Response Object
		NotificationStatusTO res = new NotificationStatusTO();
		List signatureDaoList  =  signatureDao.getSignatureForUser(p.getId());
		Signature signature = null; 
		
		if(signatureDaoList != null && signatureDaoList.size() >0){
			signature = (Signature)signatureDaoList.get(0);
		
		}
		
		if(notiTO.getIncludeSignature() != null && notiTO.getIncludeSignature()) {
			if(notiTO.getSignatureName() == null || notiTO.getSignatureName().trim().isEmpty() || notiTO.getSignatureTitle() == null || notiTO.getSignatureTitle().trim().isEmpty())
				SMApiUtility.throwException(ErrorEnum.EMAIL_SIGNATURE_NAME_TITLE_REQUIRED);
		}
		
		if(signature == null) {
			signature = new Signature();
			signature.setCreatedDate(new Date());
			signature.setUser(p);
		}
		
		if (notiTO.getSignatureName() != null)
			signature.setName(notiTO.getSignatureName());
		if (notiTO.getSignatureTitle() != null)
			signature.setTitle(notiTO.getSignatureTitle());

		if(notiTO.getIncludeSignature() != null && notiTO.getIncludeSignature()) {
			// IF TRUE then name and title should exist
			if(signature.getName() == null || signature.getName().trim().isEmpty() || signature.getTitle() == null || signature.getTitle().trim().isEmpty())
				SMApiUtility.throwException(ErrorEnum.EMAIL_SIGNATURE_NAME_TITLE_REQUIRED);
		}

		
		//By Default active will be true..this is expected.
		if (notiTO.getIncludeSignature() != null)
			signature.setActive(notiTO.getIncludeSignature());
		else
			SMApiUtility.throwException(ErrorEnum.DATA_INTEGRITY_ERROR_117);
		
		ServiceResponse<Boolean> response = userManagementService.addUserSignature(signature);
		SMApiUtility.throwExceptionOnErrors(ErrorEnum.MESSAGE_SERVICE_ERROR_904, response);
		
		ResponseBuilderImpl builder = new ResponseBuilderImpl();
		return builder.status(200).build();
	}

	@Override
	public Response getParticipationStatus() {
		// TODO: DO WE NEED THIS API...
		return null;
	}

	@Override
	public Response setParticipationStatus() {
		// TODO: DO WE NEED THIS API... ..we can NOT set Participation Status
		return null;
	}

	public SessionAPIService getSessionApiService() {
		return sessionApiService;
	}

	public void setSessionApiService(SessionAPIService sessionApiService) {
		this.sessionApiService = sessionApiService;
	}

	public SignatureDao getSignatureDao() {
		return signatureDao;
	}

	public void setSignatureDao(SignatureDao signatureDao) {
		this.signatureDao = signatureDao;
	}

	public UserManagementService getUserManagementService() {
		return userManagementService;
	}

	public void setUserManagementService(UserManagementService userManagementService) {
		this.userManagementService = userManagementService;
	}

}
