package gov.va.med.mhv.sm.escalation;

import java.util.Collection;
import java.util.List;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.springframework.context.ApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;
import org.springframework.orm.hibernate3.SessionFactoryUtils;
import org.springframework.orm.hibernate3.SessionHolder;
import org.springframework.transaction.support.TransactionSynchronizationManager;

import gov.va.med.mhv.foundation.service.response.BaseServiceResponse;
import gov.va.med.mhv.foundation.service.response.BaseServiceResponseUtils;
import gov.va.med.mhv.foundation.service.response.CollectionServiceResponse;
import gov.va.med.mhv.foundation.service.response.ServiceResponse;
import gov.va.med.mhv.sm.dao.MessageDao;
import gov.va.med.mhv.sm.dao.TriageGroupDao;
import gov.va.med.mhv.sm.model.TriageGroup;
import gov.va.med.mhv.sm.service.EmailService;
import gov.va.med.mhv.sm.service.EscalationService;

public class Escalate {

	@SuppressWarnings("unused")
	private static final Log log = LogFactory.getLog(Escalate.class);
	
	private ApplicationContext applicationContext;
	private EscalationService escalationService;
	private SessionFactory sessionFactory;
	private TriageGroupDao triageGroupDao;
	private EmailService   emailService;
	
	public Escalate(){
		
		this.applicationContext =
			   new ClassPathXmlApplicationContext(getConfigLocations());
		escalationService = (EscalationService)applicationContext.getBean("escalationService");
		sessionFactory = (SessionFactory) applicationContext.getBean("sessionFactory");
		triageGroupDao = (TriageGroupDao)applicationContext.getBean("triageGroupDao");
		emailService = (EmailService)applicationContext.getBean("emailService");
		
	}

	private String[] getConfigLocations() {
		return new String[] { 
				"gov/va/med/mhv/sm/persist-context.xml",
				"gov/va/med/mhv/sm/resource-context-ext.xml", 				
				"gov/va/med/mhv/sm/service-context.xml",
				"gov/va/med/mhv/sm/utility-context.xml",
				"gov/va/med/mhv/sm/dao/dao-sms-context.xml",
				"gov/va/med/mhv/sm/service/service-sms-context.xml",
				"gov/va/med/mhv/sm/wsclient-context.xml",
				"gov/va/med/mhv/sm/ad-context.xml"};
	}
	
	
	public void escalate(){
		Session session = null;
		try {
			session = openSession(sessionFactory);
			
			// Escalate messages
			ServiceResponse<Integer> escalationResponse = escalationService.escalate();
			if (BaseServiceResponseUtils.hasError(escalationResponse)) {
				BaseServiceResponseUtils.logMessages(escalationResponse, log, null);
				return;
			}

			int count = escalationResponse.getPayload();
			log.debug(count + " messages escalated.");

			// Notify people that there are escalated messages
			// 
			// Even if there are no messages escalated, we still need to
			// see if there are any notifications that failed last time
			// that need to be retried
			// 
			CollectionServiceResponse<TriageGroup> tgr = escalationService.getTriageGroupsToNotify();
			if (BaseServiceResponseUtils.hasError(tgr)) {
				BaseServiceResponseUtils.logMessages(tgr, log, null);
				//return;
			}

			Collection<TriageGroup> groups = tgr.getCollection();
			if (CollectionUtils.isEmpty(groups)) {
				log.debug("No triage groups need to be notified of escalated messages");
				//return;
			}

			ServiceResponse<Integer> notificationResponse = null;
			for (TriageGroup tg : groups) {
				notificationResponse =
					escalationService.notifyTriageGroupOnEscalation(tg);
				if (BaseServiceResponseUtils.hasError(notificationResponse)) {
					BaseServiceResponseUtils.logMessages(notificationResponse, log, null);
				}
				log.debug(tg.getName() + " notified about " +
						  notificationResponse.getPayload().intValue() + " escalated messages");
			}
			
			ServiceResponse<Boolean> serviceResponse = escalationService.notifyFacilityAdminAfterEscalatedXDays();
		

		} catch (Exception e) {
			log.error("Premature exit from escalate", e);
		} finally {
			closeSession(session);
		}
	}
	
	
	public static void main(String[] args) {
	
		Escalate e = new Escalate();
		e.escalate();
	}

	private Session openSession(SessionFactory sessionFactory)
	{
		Session session = SessionFactoryUtils.getSession(sessionFactory, true);
		TransactionSynchronizationManager.bindResource(sessionFactory,
													   new SessionHolder(session));
		return session;
	}



	private void closeSession(Session session)
	{
		if (session == null) {
			return;
		}
		TransactionSynchronizationManager.unbindResource(sessionFactory);
		session.flush();
		SessionFactoryUtils.closeSession(session);
	}
}
