package gov.va.med.mhv.foundation.web.filter;

import gov.va.med.mhv.foundation.web.util.HttpResponseHeaderHandler;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

public class ResponseHeaderFilter implements Filter {
	
	private static final Log LOG = LogFactory.getLog(ResponseHeaderFilter.
		class);
	
	private List<HttpResponseHeaderHandler> handlers = 
		new ArrayList<HttpResponseHeaderHandler>();  

	@SuppressWarnings("unchecked")
	public void init(FilterConfig filterConfiguration) throws ServletException {
		if (filterConfiguration != null) {
			LOG.debug("Found " + describe(filterConfiguration));
			for (Enumeration<String> parameters = filterConfiguration.
					getInitParameterNames(); // "unchecked"
				 parameters.hasMoreElements();) 
			{
				String handlerName = parameters.nextElement();
				String className = filterConfiguration.getInitParameter(
					handlerName);
				try {
					HttpResponseHeaderHandler handler = 
						(HttpResponseHeaderHandler) Class.forName(className).
						newInstance();
					handler.init();
					handlers.add(handler);
				} catch (InstantiationException e) {
					LOG.error("Problem instantiating response handler '" + 
						className + "'", e);
				} catch (IllegalAccessException e) {
					LOG.error("Problem instantiating response handler '" + 
						className + "'", e);
				} catch (ClassNotFoundException e) {
					LOG.error("Problem instantiating response handler '" + 
						className + "'", e);
				}
			}
		} else {
			LOG.warn("No Response Header Filter configuration found.");
		}
	}

	public void destroy() {
		for (HttpResponseHeaderHandler handler: handlers) {
			handler.destroy();
		}
	}

	public void doFilter(ServletRequest servletRequest, 
		ServletResponse servletResponse, FilterChain chain) throws IOException, 
		ServletException 
	{
		HttpServletResponse response = (HttpServletResponse) servletResponse;
		for (HttpResponseHeaderHandler handler: handlers) {
			handler.apply(response);
		}
		chain.doFilter(servletRequest, servletResponse);
	}

	
	@SuppressWarnings("unchecked")
	private String describe(FilterConfig filterConfiguration) {
		StringBuilder builder = new StringBuilder();
		builder.append("ResponseHeaderFilters{");
		for (Enumeration<String> parameters = filterConfiguration.
				getInitParameterNames(); // "unchecked"
			 parameters.hasMoreElements();) 
		{
			String parameter = parameters.nextElement();
			builder.append("\n\t").append(parameter).append("=").append( 
				filterConfiguration.getInitParameter(parameter));
		}
		builder.append("\n}");
		return builder.toString();
	}
	
}




