package gov.va.med.mhv.foundation.web.xwork;

import gov.va.med.mhv.foundation.model.PersistentObject;
import gov.va.med.mhv.foundation.service.response.BaseServiceResponse;
import gov.va.med.mhv.foundation.service.response.BaseServiceResponseUtils;
import gov.va.med.mhv.foundation.service.response.messages.Message;
import gov.va.med.mhv.foundation.service.response.messages.MessageExtUtils;
import gov.va.med.mhv.foundation.service.response.messages.MessageHandler;
import gov.va.med.mhv.foundation.service.response.messages.MessageUtils;
import gov.va.med.mhv.foundation.service.response.messages.Messages;
import gov.va.med.mhv.foundation.service.response.messages.MessagesUtils;
import gov.va.med.mhv.foundation.service.response.messages.Severity;
import gov.va.med.mhv.foundation.service.response.messages.SeverityUtils;
import gov.va.med.mhv.foundation.util.DescriptionBuilder;
import gov.va.med.mhv.foundation.util.Precondition;

import java.util.ArrayList;
import java.util.Collection;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

import org.apache.commons.lang.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.struts2.ServletActionContext;
import org.apache.struts2.interceptor.ServletRequestAware;
import org.hibernate.NonUniqueObjectException;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.springframework.orm.hibernate3.HibernateSystemException;
import org.springframework.orm.hibernate3.SessionFactoryUtils;
import org.springframework.web.context.WebApplicationContext;
import org.springframework.web.context.support.WebApplicationContextUtils;

import com.opensymphony.xwork2.ActionSupport;
import com.opensymphony.xwork2.Preparable;

@SuppressWarnings("serial")
public abstract class AbstractBaseActionSupport extends ActionSupport
	implements ServletRequestAware, Preparable, MessageHandler
{

	protected static final Message UNKNOWN_ERROR = MessageUtils.
		createErrorMessage("unknown.exception.occurred", null, null);

    @SuppressWarnings("unchecked")
    public static boolean hasCause(Throwable t, Class type) {
        Precondition.assertAssignableFrom("type", type, Throwable.class);
        for(;t != null; t = t.getCause()) {
            if (type.isAssignableFrom(t.getClass())) {
                return true;
            }
        }
        return false;
    }

	private HttpServletRequest request = null;

	public void prepare() throws Exception {
		// TODO: Load the default response support from Spring
		// application context in prepare method rather than hard coded
		// default. Make member non-final and use something like:
		//     getBean("defaultActionSupportResponseSupport");
		// This requires checks (e.g. assert statements) when value is used
		// Use protected overridable method to get bean name, such that
		// concrete action classes can easily override the bean being used
	}

	public void setServletRequest(HttpServletRequest httpServletRequest) {
		this.request = httpServletRequest;
	}
	public HttpServletRequest getServletRequest() {
		return request;
	}

	public Log getLog() {
		return LOG;
	}

	public boolean canHandleSeverity(Severity severity) {
		return true;
	}

	public boolean canHandleMessage(Message message) {
		return true;
	}

	public boolean handleMessages(BaseServiceResponse response) {
		return BaseServiceResponseUtils.handleMessages(response, this);
	}

	public boolean handleMessages(Messages messages) {
		return MessagesUtils.handleMessages(messages, this);
	}

	public boolean handleMessages(Collection<Message> messages) {
		return MessageExtUtils.handleMessages(messages, this);

	}

	public boolean handleMessage(Message message) {
		if (MessageExtUtils.isError(message)) {
			addActionError(getMessageText(message));
		} else if (MessageExtUtils.isInfo(message)) {
			addActionMessage(getMessageText(message));
		} else {
			log().error("Undhandled message: " + getMessageText(message),
				new RuntimeException());
		}
		return MessageExtUtils.isError(message);
	}

	public String getMessageText(Message message) {
		if ((message == null) || StringUtils.isBlank(message.getKey())) {
			return null;
		}
		String text = getText(message.getKey(), null,
			MessageExtUtils.getInsertsAsList(message));
		if (text != null) {
			return text;
		}
		String defaultText = getDefaultMessageText(message.getSeverity());
		log().warn("Failed to find message text for message key '" +
			message.getKey() + "'. Using default text '" + defaultText +
			"' instead");
		return defaultText;
	}

	public String getMessageText(Collection<Message> messages) {
		return MessageExtUtils.describe(messages);
	}

	public String getDefaultMessageText(Severity severity) {
		if (SeverityUtils.isError(severity)) {
			return getDefaultErrorText();
		}
		if (SeverityUtils.isInformational(severity)) {
			return getDefaultInfoText();
		}
		return null;
	}

	protected HttpServletRequest getRequest() {
		return getRequest(true);
	}
	protected HttpServletRequest getRequest(boolean isRequired) {
		HttpServletRequest request = getServletRequest();
		if ((request == null) && isRequired) {
			throw new IllegalStateException("No request found");
		}
		return request;
	}
	protected HttpSession getSession() {
		return getSession(true);
	}
	protected HttpSession getSession(boolean isRequired) {
		HttpSession session = (request != null) ? request.getSession() : null;
		if ((session == null) && isRequired) {
			throw new IllegalStateException("No session found");
		}
		return session;
	}

	protected final Log log() {
		Log log = getLog();
		return (log != null) ? log : LOG;
	}

	/**
	 * Gets a named bean from the Spring Framework Web Application Content
	 * @param name The name of the bean
	 * @return The bean
	 * @throws IllegalArgumentException When the bean is not found
	 */
    protected Object getBean(String name) {
    	return getBean(name, true);
    }
	/**
	 * Gets a named bean from the Spring Framework Web Application Content
	 * @param name The name of the bean
	 * @param required Indicates whether the bean must be found
	 * @return The bean
	 * @throws IllegalArgumentException When the bean is not found and
	 * 	is required
	 */
    protected Object getBean(String name, boolean required) {
    	Precondition.assertNotBlank("name", name);
		WebApplicationContext context = WebApplicationContextUtils.
			getWebApplicationContext(ServletActionContext.getServletContext());
		Object bean = context.getBean(name);
    	if (required) {
    		Precondition.assertNotNull("context bean[" + name + "]", bean);
    	}
    	return bean;
    }

    /**
     * Denotes whether a given service response contains any error messages.
     * @param response The response to check
     * @return true, when response contains at least one error message;
     * 	otherwise false.
     */
	protected boolean hasError(BaseServiceResponse response) {
		return BaseServiceResponseUtils.hasError(response);
	}
	protected boolean hasError(BaseServiceResponse response, String key) {
		return BaseServiceResponseUtils.hasError(response, key);
	}

	protected boolean hasInfo(BaseServiceResponse response) {
		return BaseServiceResponseUtils.hasInfo(response);
	}
	protected boolean hasInfo(BaseServiceResponse response, String key) {
		return BaseServiceResponseUtils.hasInfo(response, key);
	}

	protected final String getErrorText(BaseServiceResponse response) {
		return BaseServiceResponseUtils.getErrorText(response, this);
	}

	protected final String getInfoText(BaseServiceResponse response) {
		return BaseServiceResponseUtils.getInfoText(response, this);
	}

	protected String getDefaultErrorText() {
		return MessageExtUtils.DEFAULT_ERROR_TEXT;
	}

	protected String getDefaultInfoText() {
		return MessageExtUtils.DEFAULT_INFO_TEXT;
	}

	protected boolean useExceptionWhenHandlingError() {
		return false;
	}

	protected String handleError(Message message) {
		handleMessage(message);
		return ERROR;
	}

	protected String handleUnexpectedError() {
		handleMessage(UNKNOWN_ERROR);
		return ERROR;
	}

	/**
     * Reattaches a given collection of object to the Hibernate session.
     * The Hibernate session is obtained via the WebApplicationContext.
     * Should be used before the objects have been loaded, or a
     * org.hibernate.NonUniqueObjectException will be thrown.
     * @param detachedObjects The objects to reattach
     * @see #getHibernateSession()
     */
    protected Collection<PersistentObject> reattach(
        Collection<? extends PersistentObject> detachedObjects)
    {
        Collection<PersistentObject> reattachedObjects = null;
        if (detachedObjects == null) {
            return reattachedObjects;
        }
        reattachedObjects = new ArrayList<PersistentObject>();
        Session session = getHibernateSession();

        for (PersistentObject detachedObject: detachedObjects) {
            if (detachedObject != null) {
                reattachedObjects.add(updateInSession(session, detachedObject));
            }
        }
        return reattachedObjects;
    }

    /**
     * Reattaches a given object to the Hibernate session.
     * Should be used before the object has been loaded, or a
     * org.hibernate.NonUniqueObjectException will be thrown.
     * @param detachedObject The object to reattach.
     * @see #getHibernateSession()
     */
    protected PersistentObject reattach(PersistentObject detachedObject) {
        return updateInSession(getHibernateSession(), detachedObject);
    }

    /**
     * Gets the Hibernate session from the web application context.
     * Uses method getHibernateSessionFactoryBeanName() to obtain access
     * to the session factory and through the factory access to the session.
     * @return The Hibernate session
     * @see #getHibernateSessionFactoryBeanName()
     * @see #reattach(Object)
     * @see #reattach(Collection)
     */
    protected Session getHibernateSession() {
        WebApplicationContext wac = WebApplicationContextUtils.
        	getRequiredWebApplicationContext(getServletRequest().getSession().
        	getServletContext());
        Precondition.assertNotNull("wac", wac);
        return SessionFactoryUtils.getSession((SessionFactory) wac.
            getBean(getHibernateSessionFactoryBeanName()), false);
    }

    /**
     * Gets the name of the Hibernate session factory bean.
     * @return The name of the bean
     * @see #getHibernateSession()
     */
    protected String getHibernateSessionFactoryBeanName() {
        return "sessionFactory";
    }

    private PersistentObject updateInSession(Session session,
    		PersistentObject detachedObject)
    {
    	Precondition.assertNotNull("session", session);
    	assert detachedObject != null;
    	if (LOG.isDebugEnabled()) {
    		LOG.debug("Updating " + describe(detachedObject) +
    		" in Hibernate session");
    	}
    	boolean isNonUnique = false;
    	try {
    		session.update(detachedObject);
    	} catch (NonUniqueObjectException e) {
    		isNonUnique = true;
    	} catch (HibernateSystemException e) {
    		if (!hasCause(e, NonUniqueObjectException.class)) {
    			throw e;
    		}
    		isNonUnique = true;
    	}
    	if (isNonUnique) {
    		if (LOG.isDebugEnabled()) {
    			LOG.debug("Using " + describe(detachedObject) +
    			" from the Hibernate session instead");
    		}
    		detachedObject = (PersistentObject) session.get(session.
   				getEntityName(detachedObject), detachedObject.getId());
    	}
    	return detachedObject;
    }

    private String describe(Object object) {
        return new DescriptionBuilder().header(object).toString();
    }}
