package gov.va.med.mhv.foundation.crypto;

import gov.va.med.mhv.foundation.util.Precondition;

import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

public class ChecksumCalculator {
	
	private static final Log LOG = LogFactory.getLog(ChecksumCalculator.class);

	/**
	 * Default checksum algorithm (MD5).
	 */
	public static final String DEFAULT_ALGORITHM = "MD5";

	private final String salt;
	private final MessageDigest digester;

	public ChecksumCalculator(String salt) throws NoSuchAlgorithmException {
		this(salt, DEFAULT_ALGORITHM);
	}
	public ChecksumCalculator(String salt, String algorithm) 
		throws NoSuchAlgorithmException 
	{
		Precondition.assertNotNull("salt", salt);
		Precondition.assertNotNull("algorithm", algorithm);
		this.salt = salt;
		this.digester = MessageDigest.getInstance(algorithm);
		if (LOG.isDebugEnabled()) {
			LOG.debug("Generate checksums with salt = [" + salt + "]");
		}
	}
	
	public String calculate(String input) {
		String result = null;
		try {
			input = input + salt;
			if (LOG.isDebugEnabled()) {
				LOG.debug("Create checksum from [" + input + "]");
			}
			digester.update(input.getBytes());
			byte[] hash = digester.digest();
			StringBuffer buffer = new StringBuffer();
			for (int i = 0; i < hash.length; i++) {
				buffer.append(Integer.toHexString(hash[i]));
			}
			result = buffer.toString();
		} catch (Exception e) {
			LOG.error("Unable to create checksum from input [" + input + "]", 
				e);
			result = null;
		}
		return result;
	}
	
}
