package gov.va.med.mhv.foundation.service.delegate;

import gov.va.med.mhv.foundation.service.response.BaseServiceResponse;
import gov.va.med.mhv.foundation.service.response.BaseServiceResponseUtils;
import gov.va.med.mhv.foundation.service.response.messages.Message;
import gov.va.med.mhv.foundation.service.response.messages.MessageExtUtils;
import gov.va.med.mhv.foundation.service.response.messages.MessageHandler;
import gov.va.med.mhv.foundation.service.response.messages.Messages;
import gov.va.med.mhv.foundation.service.response.messages.MessagesUtils;
import gov.va.med.mhv.foundation.service.response.messages.Severity;
import gov.va.med.mhv.foundation.service.response.messages.SeverityUtils;

import java.util.Collection;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

public class AbstractBaseServiceDelegate implements MessageHandler {

	private static final long serialVersionUID = -2187982922346569977L;
	
	private static final Log LOG = LogFactory.getLog(
		AbstractBaseServiceDelegate.class);
	
	protected AbstractBaseServiceDelegate() {
	}
	
	public Log getLog() {
		return LOG;
	}

	public boolean canHandleSeverity(Severity severity) {
		Log log = log();
		return (severity != null) 
			&& ((SeverityUtils.isError(severity) && log.isErrorEnabled())
				|| (SeverityUtils.isInformational(severity) && log.
						isInfoEnabled()));
	}

	public boolean canHandleMessage(Message message) {
		return (message != null) && canHandleSeverity(message.getSeverity());
	}

	public boolean handleMessages(BaseServiceResponse response) {
		return BaseServiceResponseUtils.handleMessages(response, this);
	}

	public boolean handleMessages(Messages messages) {
		return MessagesUtils.handleMessages(messages, this);
	}

	public boolean handleMessages(Collection<Message> messages) {
		return MessageExtUtils.handleMessages(messages, this);
	}

	public boolean handleMessage(Message message) {
		MessageExtUtils.log(message, getLog(), (useExceptionWhenLoggingError()) 
			? new RuntimeException() : null);
		return MessageExtUtils.isError(message);
	}

	public String getMessageText(Message message) {
		return MessageExtUtils.describe(message);
	}

	public String getMessageText(Collection<Message> messages) {
		return MessageExtUtils.describe(messages);
	}

	public String getDefaultMessageText(Severity severity) {
		if (SeverityUtils.isError(severity)) {
			return getDefaultErrorMessageText();
		}
		if (SeverityUtils.isInformational(severity)) {
			return getDefaultInfoMessageText();
		}
		return null;
	}

	public boolean useExceptionWhenLoggingError() {
		return false;
	}

	protected boolean hasError(BaseServiceResponse response) {
		return BaseServiceResponseUtils.hasError(response);
	}

	protected boolean hasError(BaseServiceResponse response, String key) {
		return BaseServiceResponseUtils.hasError(response);
	}

	protected boolean hasInfo(BaseServiceResponse response) {
		return BaseServiceResponseUtils.hasInfo(response);
	}

	protected boolean hasInfo(BaseServiceResponse response, String key) {
		return BaseServiceResponseUtils.hasInfo(response, key);
	}
	
	protected final Log log() {
		Log log = getLog();
		return (log != null) ? log : LOG;
	}

	protected final String getErrorText(BaseServiceResponse response) {
		return getErrorText(response, getDefaultMessageSeparator());
	}
	protected final String getErrorText(BaseServiceResponse response,
		String separator) 
	{
		return BaseServiceResponseUtils.getErrorText(response, this, separator);
	}

	protected final String getInfoText(BaseServiceResponse response) {
		return getInfoText(response, getDefaultMessageSeparator());
	}
	protected final String getInfoText(BaseServiceResponse response, 
		String separator) 
	{
		return BaseServiceResponseUtils.getInfoText(response, this, separator);
	}

	protected String getDefaultErrorMessageText() {
		return MessageExtUtils.DEFAULT_ERROR_TEXT;
	}

	protected String getDefaultInfoMessageText() {
		return MessageExtUtils.DEFAULT_INFO_TEXT;
	}

	protected String getDefaultMessageSeparator() {
		return null;
	}

	protected void copyMessages(BaseServiceResponse fromResponse,
		BaseServiceResponse toResponse)
	{
		if ((fromResponse == null) || (toResponse == null)) {
			return;
		}
		if (BaseServiceResponseUtils.hasError(fromResponse)) {
			for (Message message: BaseServiceResponseUtils.getErrorMessages(
				fromResponse))
			{
				toResponse.getMessages().addMessage(message);
			}
		}
		if (BaseServiceResponseUtils.hasInfo(fromResponse)) {
			for (Message message: BaseServiceResponseUtils.getInfoMessages(
					fromResponse))
			{
				toResponse.getMessages().addMessage(message);
			}
		}
	}
}
