package gov.va.med.mhv.foundation.service.response;

import java.util.Collection;

import gov.va.med.mhv.foundation.service.response.BaseServiceResponse;
import gov.va.med.mhv.foundation.service.response.messages.Message;
import gov.va.med.mhv.foundation.service.response.messages.MessageExtUtils;
import gov.va.med.mhv.foundation.service.response.messages.MessageHandler;
import gov.va.med.mhv.foundation.service.response.messages.MessagesUtils;
import gov.va.med.mhv.foundation.util.Precondition;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.commons.logging.Log;

public final class BaseServiceResponseUtils {
	
	public static boolean hasMessages(BaseServiceResponse response) {
		return (response != null) && (response.getMessages() != null);
	}

	public static boolean hasError(BaseServiceResponse response) {
		return (response != null) && MessagesUtils.hasError(response.
			getMessages());
	}
	public static boolean hasError(BaseServiceResponse response, String key) {
		if (StringUtils.isBlank(key) || !hasError(response)) {
			return false;
		}
		for (Message message: getErrorMessages(response)) { // "unchecked"
			if (MessageExtUtils.isError(message)) {
				return true;
			}
		}
		return false;
	}
	public static boolean hasError(BaseServiceResponse response, 
		Collection<String> keys) 
	{
		if (CollectionUtils.isEmpty(keys) || !hasError(response)) {
			return false;
		}
    	for (Message message: BaseServiceResponseUtils.getErrorMessages(
    			response))
    	{
    		for (String key: keys) {
    			if (MessageExtUtils.isError(message, key)) {
    				return true;
    			}
    		}
    	}
		return false;
	}
	
	public static boolean hasInfo(BaseServiceResponse response) {
		return (response != null) && MessagesUtils.hasInfo(response.
			getMessages());
	}
	public static boolean hasInfo(BaseServiceResponse response, String key) {
		if (StringUtils.isBlank(key) || !hasInfo(response)) {
			return false;
		}
		for (Message message: getInfoMessages(response)) {  // "unchecked"
			if (MessageExtUtils.isError(message)) {
				return true;
			}
		}
		return false;
	}
	public static boolean hasInfo(BaseServiceResponse response, 
		Collection<String> keys) 
	{
		if (CollectionUtils.isEmpty(keys) || !hasInfo(response)) {
			return false;
		}
    	for (Message message: BaseServiceResponseUtils.getInfoMessages(
    			response))
    	{
    		for (String key: keys) {
    			if (MessageExtUtils.isInfo(message, key)) {
    				return true;
    			}
    		}
    	}
		return false;
	}
		

	public static Collection<Message> getErrorMessages(
		BaseServiceResponse response) 
	{
		return MessagesUtils.getErrorMessages((response != null) 
			? response.getMessages() : null);
	}

	public static Collection<Message> getInfoMessages(
		BaseServiceResponse response) 
	{
		return MessagesUtils.getInfoMessages((response != null) 
			? response.getMessages() : null);
	}

	public static boolean handleMessages(BaseServiceResponse response, 
		MessageHandler handler) 
	{
		return (BaseServiceResponseUtils.hasMessages(response)) 
			&& MessagesUtils.handleMessages(response.getMessages(), handler);
	}

	public static String getErrorText(BaseServiceResponse response, 
		MessageHandler handler) 
	{
		return getErrorText(response, handler, null);
	}
	public static String getErrorText(BaseServiceResponse response, 
		MessageHandler handler, String separator) 
	{
		Precondition.assertNotNull("handler", handler);
		if (!BaseServiceResponseUtils.hasMessages(response)) {
			return null;
		}
		return MessageExtUtils.getText(BaseServiceResponseUtils.
			getErrorMessages(response), handler, separator);
	}

	public static String getInfoText(BaseServiceResponse response, 
		MessageHandler handler) 
	{
		return getInfoText(response, handler, null);
	}
	public static String getInfoText(BaseServiceResponse response, 
		MessageHandler handler, String separator) 
	{
		if (!BaseServiceResponseUtils.hasMessages(response)) {
			return null;
		}
		Precondition.assertNotNull("handler", handler);
		if (!BaseServiceResponseUtils.hasMessages(response)) {
			return null;
		}
		return MessageExtUtils.getText(BaseServiceResponseUtils.
			getInfoMessages(response), handler, separator);
	}

	public static void logMessages(BaseServiceResponse response, Log log, 
		Exception e) 
	{
		if (response == null) {
			return;
		}
		if (response.getMessages() == null) {
			return;
		}
		MessagesUtils.log(response.getMessages(), log, e);
	}

	public static boolean copyMessages(BaseServiceResponse fromResponse,
		BaseServiceResponse toResponse)
	{
		if (fromResponse == null) {
			return false;
		}
		if (toResponse == null) {
			return false;
		}
		boolean hasError = hasError(fromResponse); 
		if (hasError) {
			for (Message message: getErrorMessages(fromResponse)) {
				toResponse.getMessages().addMessage(message);
			}
		}
		if (hasInfo(fromResponse)) {
			for (Message message: getInfoMessages(fromResponse)) {
				toResponse.getMessages().addMessage(message);
			}
		}
		return hasError;
	}

	protected BaseServiceResponseUtils() {
	}

}
