package gov.va.med.mhv.foundation.service.response.messages;

import gov.va.med.mhv.foundation.util.Loggable;

import java.io.Serializable;

/**
 * Handles message for a given target.
 * @author Rob Proper
 */
public interface MessageHandler extends Loggable, Serializable {

	
	/**
	 * Indicates if this MessageHandler handles messages of the given severity.
	 * it handles messages of the given severity. 
	 * @return True, if this handler does for the given target; false 
	 * otherwise.
	 */
	boolean canHandleSeverity(Severity severity);

	/**
	 * Indicates if this MessageHandler handles messages of the given severity.
	 * it handles messages of the given severity. 
	 * @return True, if this handler does for the given target; false 
	 * otherwise.
	 */
	boolean canHandleMessage(Message message);

	/**
	 * Handles a given message. 
	 * @param message The message to handle.
	 * @return True, if error message; false otherwise.
	 * @see {@link #getMessageText()}
	 */
	boolean handleMessage(Message message);
	
	/**
	 * Returns the message text for a given target.  
	 * @param message The message to find the text for.
	 * @see {@link #getMessageText()}
	 * @see {@link MessageHandlerTarget#getDefaultErrorMessageText()}
	 * @see {@link MessageHandlerTarget#getDefaultInfoText()}
	 */
	String getMessageText(Message message);
	
	/**
	 * Returns the default text used with a given severity.  
	 * @param target The instance for which to determine the default text. 
	 * @param severity The severity for which to find the default text.
	 * @see {@link #getMessageText()}
	 * @see {@link MessageHandlerTarget#getDefaultErrorMessageText()}
	 * @see {@link MessageHandlerTarget#getDefaultInfoText()}
	 */
	String getDefaultMessageText(Severity severity); 

}
