package gov.va.med.mhv.foundation.service.response.messages;

import gov.va.med.mhv.foundation.util.Precondition;

import java.util.Collection;
import java.util.Collections;

import org.apache.commons.logging.Log;

public final class MessagesUtils {
	
	public static final boolean hasError(Messages messages) {
		return (messages != null) && messages.hasErrorMessages();
	}
	public static final boolean hasError(Messages messages, String key) {
		if (!(MessageExtUtils.isKey(key) && hasError(messages))) {
			return false;
		}
		for (Message message: getErrorMessages(messages)) { // "unchecked"
			if (MessageExtUtils.isError(message, key)) {
				return true;
			}
		}
		return false;
	}

	public static final boolean hasInfo(Messages messages) {
		return (messages != null) && messages.hasInformationalMessages();
	}
	public static final boolean hasInfo(Messages messages, String key) {
		if (!(MessageExtUtils.isKey(key) && hasInfo(messages))) {
			return false;
		}
		for (Message message: getInfoMessages(messages)) { // "unchecked"
			if (MessageExtUtils.isInfo(message, key)) {
				return true;
			}
		}
		return false;
	}

	public static final boolean hasSeverity(Messages messages, 
		Severity severity) 
	{
		return (SeverityUtils.isError(severity) && hasError(messages)) 
			|| (SeverityUtils.isInformational(severity) && hasInfo(messages));

	}
	public static final boolean hasKey(Messages messages, Severity severity,
		String key) 
	{
		if (!(MessageExtUtils.isKey(key) && hasSeverity(messages, 
			severity))) 
		{
			return false;
		}
		for (Message message: getMessages(messages, severity)) { // "unchecked"
			if (MessageExtUtils.hasKey(message, key)) {
				return true;
			}
		}
		return false;
	}

	@SuppressWarnings("unchecked")
	public static final Collection<Message> getErrorMessages(
		Messages messages) 
	{
		return (messages != null) ? messages.getErrorMessages() 
			: Collections.EMPTY_LIST;
	}

	@SuppressWarnings("unchecked")
	public static final Collection<Message> getInfoMessages(Messages messages) {
		return (messages != null) ? messages.getInformationalMessages()
			: Collections.EMPTY_LIST;
	}

	@SuppressWarnings("unchecked")
	public static final Collection<Message> getMessages(Messages messages, 
		Severity severity) 
	{
		if (SeverityUtils.isError(severity)) {
			return getErrorMessages(messages);
		}
		if (SeverityUtils.isInformational(severity)) {
			return getInfoMessages(messages);
		}
		return Collections.EMPTY_LIST;
	}

	public static boolean canHandleError(MessageHandler handler) {
		return (handler != null)  && handler.canHandleSeverity(SeverityUtils.
			ERROR_SEVERITY);
	}

	public static boolean canHandleInfo(MessageHandler handler) {
		return (handler != null)  && handler.canHandleSeverity(SeverityUtils.
			INFORMATIONAL_SEVERITY);
	}

	public static boolean handleMessages(Messages messages, 
		MessageHandler handler) 
	{
		Precondition.assertNotNull("handler", handler);
		if (messages == null) {
			return false;
		}
		final boolean hasError = MessagesUtils.hasError(messages);
		if (hasError && canHandleError(handler)) {
			MessageExtUtils.handleMessages(MessagesUtils.getErrorMessages(
				messages), handler); 
		}
		if (MessagesUtils.hasInfo(messages) && canHandleInfo(handler)) {
			MessageExtUtils.handleMessages(MessagesUtils.getInfoMessages(
				messages), handler);
		}
		return hasError;
	}
	
	public static void log(Messages messages, Log log, Exception e) {
		if (messages == null) {
			return;
		}
		MessageExtUtils.log(getErrorMessages(messages), log, e);
		MessageExtUtils.log(getInfoMessages(messages), log, e);
	}

	protected MessagesUtils() {
	}
	
}
