package gov.va.med.mhv.foundation.util;

import java.util.Calendar;
import java.util.Date;

public final class CalendarExtUtils {

	/**
	 * Convert a date to a Calendar instance with the same time
	 * @param date The date to convert, null allowed
	 * @return The calendar instance; null when given date is null
	 */
	public static Calendar toCalendar(Date date) {
		if (date == null) {
			return null;
		}
		Calendar calendar = Calendar.getInstance();
		calendar.setTime(date);
		return calendar;
	}

	/**
	 * Create a new date with the given amount of given field added.
	 * @param date The date to add to, null allowed
	 * @param field The field to add.
	 * @param amount The amount to add.
	 * @return The new date; null is the given date is null.
	 * @see Calendar#add(int, int)
	 */
	public static Date add(Date date, int field, int amount) {
		if (date == null) {
			return null;
		}
		Calendar calendar = toCalendar(date);
		calendar.add(field, amount);
		return calendar.getTime();
	}

	/**
	 * Create a date with the given amount of given field added to the 
	 * current time.
	 * @param milliseconds The number of milliseconds since 
	 * January 1, 1970, 00:00:00 GMT.
	 * @param field The field to add.
	 * @param amount The amount to add.
	 * @return The new date; null is the given date is null.
	 * @see Calendar#add(int, int)
	 */
	public static Date add(int field, int amount) {
		Calendar calendar = Calendar.getInstance();
		calendar.setTimeInMillis(System.currentTimeMillis());
		calendar.add(field, amount);
		return calendar.getTime();
	}

	/**
	 * Create a date with the given amount of given field added to the 
	 * timeInMilliseconds.
	 * @param milliseconds The number of milliseconds since 
	 * January 1, 1970, 00:00:00 GMT.
	 * @param field The field to add.
	 * @param amount The amount to add.
	 * @return The new date; null is the given date is null.
	 * @see Calendar#add(int, int)
	 */
	public static Date add(long milliseconds, int field, int amount) {
		Calendar calendar = Calendar.getInstance();
		calendar.setTimeInMillis(milliseconds);
		calendar.add(field, amount);
		return calendar.getTime();
	}

	private CalendarExtUtils() {
	}
}
