package gov.va.med.mhv.sm.web.utils;

import gov.va.med.mhv.foundation.util.ConfigurationManager;
import gov.va.med.mhv.foundation.util.LogSanitizer;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.util.Properties;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.owasp.esapi.ESAPI;

public class SMHelpUtil {

	private static final Log log = LogFactory.getLog(SMHelpUtil.class);
	public static final String HELP_CONTENT_ATTRIBUTE_NAME = "HELP_CONTENT";
	private static String HELP_PREFIX = null;
	private static final String PROP_FILE_NAME = File.separator + "sm-web.properties";
	private static final String HELP_PREFIX_KEY = "sm.web.help.url.prefix";
	
	public static String getContent(String pagetoDisplay) throws IOException {
		if ( HELP_PREFIX == null ) {
			Properties props = ConfigurationManager.getConfiguration(PROP_FILE_NAME);
			if ( props == null ) {
				if(log.isErrorEnabled()){
					log.error("Could not load properties file: " + PROP_FILE_NAME);
				}
			} else {
				HELP_PREFIX = props.getProperty(HELP_PREFIX_KEY);
				if ( HELP_PREFIX == null ) {
					if(log.isErrorEnabled()){
						log.error("Could not find property " + HELP_PREFIX_KEY + " in " + PROP_FILE_NAME);
					}
				}
			}
		}
		if ( HELP_PREFIX == null ) return "SM Help Configuration error";
		StringBuilder buffer = null;
		InputStream in = null;
		try {
			//read and download the page text
			URL url = new URL(getHelpUrlPrefix() + pagetoDisplay);
			in = url.openStream();    	
			byte[] data = new byte[1024];
			buffer = new StringBuilder();
			for(int length = in.read(data); length != -1; length = in.read(data) ) {
				buffer.append(new String(data, 0, length));
			}
		}catch (Exception e) {
			if(log.isErrorEnabled()){
				log.error(LogSanitizer.cleanText("Requested help file "+pagetoDisplay+ " could not be processed."), e);
			}
			buffer = new StringBuilder("Unable to retrieve content for " + ESAPI.encoder().encodeForHTML(pagetoDisplay));
		} finally {           
			if (in != null) in.close();       	
		}
		return (buffer != null) ? buffer.toString() : null;
	}

	protected static String getHelpUrlPrefix() throws IOException {
		return HELP_PREFIX;
	}
	
}
